/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.api.analyse.node;

import com.abssqr.plat.common.facade.enums.AnalyseServiceEnum;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.core.service.util.AnalyseReportUtil;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;

/**
 * 整体逾期率
 * @author hanfei
 * @version $Id: AnalyseChartAna101Node.java, v 0.1 2019-10-09 11:26 PM hanfei Exp $
 */
@Component
public class AnalyseChartAna101Node extends AnalyseChartNodeBase {
    
    /**
     * 放款池、承接池报表的逾期剩余本金和 (WK012) / 放款池、承接池资产剩余本金和 (WK017)
     * @param context
     * @return
     */
    @Override
    protected Void handle(AnalyseChartContext context) {
        List<ReportField> amtFields = Lists.newArrayList(
                WkAstReportFields.WK012,
                WkAstReportFields.WK017
        );
        super.handleDayTrendChart(context, amtFields, true, (acctDate, itemMap) -> {
            // 计算逾期率 = 放款池、承接池报表的逾期剩余本金和 / 放款池、承接池资产剩余本金和
            BigDecimal denominator = AnalyseReportUtil.getItemBig(itemMap, WkAstReportFields.WK017);
            return CalcUtil.dividePercRate(
                    AnalyseReportUtil.getItemBig(itemMap, WkAstReportFields.WK012), denominator);
        });
        return null;
    }
    
    @Override
    protected AnalyseServiceEnum getServiceEnum(){
        return AnalyseServiceEnum.DAY_OVD;
    }
}
