/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.api.analyse.node;

import com.abssqr.plat.common.facade.enums.AnalyseServiceEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.core.service.util.AnalyseReportUtil;
import com.general.system.common.model.Money;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;

/**
 * 日闲置率
 * @author hanfei
 * @version $Id: AnalyseChartAna204Node.java, v 0.1 2019-10-09 11:26 PM hanfei Exp $
 */
@Component
public class AnalyseChartAna204Node extends AnalyseChartNodeBase {
    
    /**
     * 日闲置率
     * 放款池、承接池 日可用资金总额/ 放款池 + 承接池信托规模
     * @param context
     * @return
     */
    @Override
    protected Void handle(AnalyseChartContext context) {
        List<ReportField> amtFields = Lists.newArrayList(
                WkAstReportFields.WK001
        );
        // 查询对应受让池计划
        List<PlanBase> planBases = analyseSpvReportManager.getPlanBySpv(context.getPlanBase().getPlanNo());
        // 统计图获取数据
        super.handleDayTrendChart(context, amtFields, true, (acctDate, itemMap) -> {
            // 放款池、承接池 日可用资金总额
            BigDecimal numerator = AnalyseReportUtil.getItemBig(itemMap, WkAstReportFields.WK001);
            // 获取放款池、承接池信托计划规模合计
            Money denominator = AnalyseReportUtil.getPlanTotalAmtBySpvPlanNo(context.getPlanBase(), planBases);
            return CalcUtil.dividePercRate(numerator, denominator.getAmount());
        });
        return null;
    }
    
    @Override
    protected AnalyseServiceEnum getServiceEnum(){
        return AnalyseServiceEnum.DAY_IDLE;
    }
}
