/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.api.ast;

import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.dal.mysql.auto.dao.LoanBakDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.LoanBakDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.LoanBakListPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PropCntStat;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAst;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.param.loan.LoanPageQryParam;
import com.abssqr.plat.common.facade.param.loan.LoanQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.ast.LoanDelVO;
import com.abssqr.plat.common.facade.result.ast.LoanListVO;
import com.abssqr.plat.common.facade.service.ast.LoanService;
import com.abssqr.plat.common.model.domain.contract.LoanContract;
import com.abssqr.plat.common.model.repo.ast.LoanInstalRepository;
import com.abssqr.plat.common.model.repo.ast.LoanRepository;
import com.abssqr.plat.common.model.repo.contract.LoanContractRepository;
import com.abssqr.plat.common.model.repo.plan.PlanAstRepository;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.model.utils.DescUtil;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.abssqr.plat.core.service.util.RptBuilderUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 资产
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.api.ast: LoanServiceImpl.java, v 0.1 2019-05-30 19:45 zhenxuan.luo Exp $ 
 */
@Component
public class LoanServiceImpl extends ServiceSupport implements LoanService {
    
    @Autowired
    private LoanRepository         loanRepository;
    @Autowired
    private LoanInstalRepository   loanInstalRepository;
    @Autowired
    private LoanContractRepository loanContractRepository;
    @Autowired
    private AstSyncTaskCtrRepo     astSyncTaskCtrRepo;
    @Autowired
    private PlanManager            planManager;
    @Autowired
    private PlanAstRepository      planAstRepository;
    @Autowired
    private LoanBakDAO             loanBakDAO;
    @Autowired
    private OrgAcctDayComponent    orgAcctDayComponent;
    
    @Override
    public BasePage<LoanListVO> getOrgLoanList(LoanPageQryParam param) {
        return this.doQry(param, () -> {
            // 获取计划
            PlanBase planBase = planManager.getPlanFullByNo(param.getOwnOrg());
            // 查询最近模型转换完成日期
            Date maxDate = astSyncTaskCtrRepo.getTaskLogMaxOrgDate(
                    SyncTaskTypeEnum.AST_MOD, orgAcctDayComponent.getOrgCode(planBase).getCode());
            
            LoanBakListPage listPage = loanRepository.getPageByOwnOrg(param, maxDate);
            BasePage<LoanListVO> res = new BasePage<>();
            res.setCurrPageNo(listPage.getCurrPageNo());
            res.setTotal(listPage.getTotal());
            res.setLimit(listPage.getLimit());
            res.setDatas(buildListVos(listPage.getDatas()));
            return res;
        });
    }
    
    @Override
    public LoanDelVO getPlanAstDetail(LoanQryParam param) {
        PlanBase planBase = planManager.getPlanFullByNo(param.getPlanNo());
        
        VarChecker.checkNotNull(planBase, "不存在该计划[{0}]", param.getPlanNo());
     
        // 查询最近模型转换完成日期
        Date maxDate = astSyncTaskCtrRepo.getTaskLogMaxOrgDate(
                SyncTaskTypeEnum.AST_MOD, orgAcctDayComponent.getOrgCode(planBase).getCode());
        
        LoanDelVO loanDelVO = new LoanDelVO();
        //计划编号
        loanDelVO.setPlanNo(planBase.getPlanNo());
        //计划名称
        loanDelVO.setPlanName(planBase.getPlanName());
        //本金规模
        loanDelVO.setTotalAmt(RptBuilderUtil.getInvestedAmt(planBase));
    
        PlanAst planAst = planAstRepository.getByPlanNo(planBase.getPlanNo());
    
        // 根据会计日获取loanbak 统计数据
        PropCntStat planLoanStat = loanBakDAO.statPlanBal(maxDate, planBase.getPlanNo());
    
        loanDelVO.setAstAmt((planAst == null) ? new Money() : planAst.getAccGrant());
        loanDelVO.setAstCurrentBal((planAst == null) ? new Money() : planAst.getCash());
    
        loanDelVO.setAstCurrentAmt((planLoanStat == null) ? new Money() : planLoanStat.getAmt());
        loanDelVO.setAstCurrentCnt((planLoanStat == null) ? BigDecimal.ZERO : new BigDecimal(planLoanStat.getCnt()));
        return loanDelVO;
    }
    
    /**
     * 构建资产列表模型
     * @param loans
     * @return
     */
    private List<LoanListVO> buildListVos(List<LoanBakDO> loans) {
        List<LoanListVO> loanListVOS = Lists.newArrayList();
        
        if (CollectionUtils.isEmpty(loans)) {
            return loanListVOS;
        }
        
        // 1. 查找关联的合同信息
        List<String> astNos = loans.stream().map(LoanBakDO::getAstNo).collect(Collectors.toList());
        List<LoanContract> contracts = loanContractRepository.getByAstNos(astNos);
        
        Map<String, LoanContract> contractMap = contracts.stream().collect(
                Collectors.toMap(LoanContract::getAstNo, item -> item));
        
        // 2. 转化领域模型到VO
        for (LoanBakDO loanBakDO : loans) {
            loanListVOS.add(buildListVo(loanBakDO, contractMap.get(loanBakDO.getAstNo())));
        }
        
        return loanListVOS;
    }
    
    private LoanListVO buildListVo(LoanBakDO loan, LoanContract loanContract) {
        
        boolean isContNull = loanContract == null;
        Money zeroMoney = new Money(0);
        LoanListVO listVO = new LoanListVO();
        listVO.setAstNo(loan.getAstNo());
        listVO.setContractNo(loanContract.getOutContNo());
        listVO.setOutAstNo(loan.getOutAstNo());
        listVO.setOutProdNo(loan.getOutProdNo());
        listVO.setManagerOrgCode(loan.getManagerOrgCode());
        listVO.setLenderOrgCode(loan.getLenderOrgCode());
        listVO.setInDate(loan.getInDate());
        listVO.setDeptrName(isContNull ? "" : loanContract.getLoanCustInfo().getName());
        listVO.setDistAmout(isContNull ? zeroMoney : loanContract.getDisbAmount());
        listVO.setRepayMethod(isContNull ? null : loanContract.getLoanProperty().getRepayMethod());
        listVO.setRateDesc(DescUtil.buildRateDesc(loanContract));
        listVO.setStartDate(loanContract.getContractPeriod().getStartDate());
        listVO.setEndDate(loanContract.getContractPeriod().getEndDate());
        listVO.setTermCnt(loan.getTermCnt());
        listVO.setCurTerm(loan.getCurTerm());
        listVO.setPrinBal(loan.getPrinBal().add(loan.getOvdPrinBal()));
        listVO.setIntrBal(loan.getIntrBal().add(loan.getOvdIntrBal()));
        listVO.setPenalBal(loan.getPrinPenalBal().add(loan.getIntrPenalBal()));
        listVO.setOvdDays(loan.getOvdDays());
        listVO.setStatus(LoanStatusEnum.getByCode(loan.getStatus()));
        listVO.setCost(loan.getCost());
        //        listVO.setRiskCl(loan.getLoanRisk().getRiskClass());
        return listVO;
    }
    
}