/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.api.ast;

import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.dal.mysql.auto.dao.LoanDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.LoanDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.LoanListPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstScale;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.LoanPropCntStat;
import com.abssqr.plat.common.facade.enums.CapitalAstMatchEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TitleTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAcctTitle;
import com.abssqr.plat.common.facade.model.plan.PlanAst;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.param.loan.LoanAstQryParam;
import com.abssqr.plat.common.facade.param.loan.LoanAstTermQryParam;
import com.abssqr.plat.common.facade.param.loan.LoanPageQryParam;
import com.abssqr.plat.common.facade.param.loan.LoanQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.PlanConst;
import com.abssqr.plat.common.facade.result.ast.LoanDelVO;
import com.abssqr.plat.common.facade.result.ast.LoanDetail;
import com.abssqr.plat.common.facade.result.ast.LoanInstalDetail;
import com.abssqr.plat.common.facade.result.ast.LoanListVO;
import com.abssqr.plat.common.facade.service.ast.SpvLoanService;
import com.abssqr.plat.common.model.convertor.vo.LoanVOConvert;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.contract.LoanContract;
import com.abssqr.plat.common.model.repo.ast.LoanBakRepo;
import com.abssqr.plat.common.model.repo.ast.LoanRepository;
import com.abssqr.plat.common.model.repo.contract.LoanContractRepository;
import com.abssqr.plat.common.model.repo.plan.PlanAcctTitleRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAstRepository;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.ast.LoanAstAssembler;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.abssqr.plat.core.service.util.RptBuilderUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 放款池项目计划及资产服务
 *
 * @author hanfei
 * @version $Id: SpvLoanServiceImpl.java, v 0.1 2019-12-12 5:21 PM hanfei Exp $
 */
@Service
public class SpvLoanServiceImpl extends ServiceSupport implements SpvLoanService {

    @Autowired
    private LoanAstAssembler       loanAstAssembler;
    @Autowired
    private LoanRepository         loanRepository;
    @Autowired
    private PlanManager            planManager;
    @Autowired
    private PlanAstRepository      planAstRepository;
    @Autowired
    private LoanContractRepository loanContractRepository;
    @Autowired
    private PlanRepo               planRepo;
    @Autowired
    private LoanDAO                loanDAO;
    @Autowired
    private LoanBakRepo            loanBakRepo;
    @Autowired
    private PlanAcctTitleRepo      planAcctTitleRepo;
    @Autowired
    private AstSyncTaskCtrRepo     astSyncTaskCtrRepo;
    @Autowired
    private OrgAcctDayComponent    orgAcctDayComponent;
    
    /**
     * 计划详情
     *
     * @param param
     * @return
     */
    @Override
    public LoanDelVO getSpvDetail(LoanQryParam param) {
        return this.doQry(param, () -> {
            PlanBase planBase = planManager.getPlanFullByNo(param.getPlanNo());
            VarChecker.checkNotNull(planBase, "不存在该计划[{0}]", param.getPlanNo());

            LoanDelVO loanDelVO = new LoanDelVO();
            //计划编号
            loanDelVO.setPlanNo(planBase.getPlanNo());
            //计划名称
            loanDelVO.setPlanName(planBase.getPlanName());
            //本金规模
            loanDelVO.setTotalAmt(RptBuilderUtil.getInvestedAmt(planBase));

            // 获取水位
            PlanAst planAst = planAstRepository.getByPlanNo(planBase.getPlanNo());
            loanDelVO.setAstAmt((planAst == null) ? new Money() : planAst.getAccGrant());
            loanDelVO.setAstCurrentBal((planAst == null) ? new Money() : planAst.getCash());

            // 获取loan 统计数据
            LoanPropCntStat planLoanStat = loanDAO.statSpvBal(planBase.getPlanNo());
            loanDelVO.setAstCurrentAmt((planLoanStat == null) ? new Money() : planLoanStat.getAmt());
            loanDelVO
                    .setAstCurrentCnt((planLoanStat == null) ? BigDecimal.ZERO : new BigDecimal(planLoanStat.getCnt()));
            // 资金剩余规模
            Money capitalBal = new Money();
            loanDelVO.setCapitalBal(capitalBal);
            // 获取本金科目
            List<PlanAcctTitle> planAcctTitleList = planAcctTitleRepo.getByEntityNo(param.getPlanNo());
            planAcctTitleList.forEach(item -> {
                if (item.getTitleType() == TitleTypeEnum.INVESTED || item.getTitleType() == TitleTypeEnum.PRIOR_INVESTED || item.getTitleType() == TitleTypeEnum.SECND_INVESTED) {
                    if (StringUtils.isEmpty(param.getCapitalNo()) || param.getCapitalNo().equals(item.getRefNo())) {
                        capitalBal.addTo(item.getBal());
                    }

                }
            });
    
            // 根据机构编码获取
            Date maxDate = astSyncTaskCtrRepo.getTaskLogMaxOrgDate(
                    SyncTaskTypeEnum.AST_MOD, orgAcctDayComponent.getOrgCode(planBase).getCode());
            // 匹配规模
            List<AstScale> astScale = loanBakRepo.getAstScale(maxDate, param.getPlanNo());
            Money astMatchAmt = new Money();
            loanDelVO.setAstMatchAmt(astMatchAmt);
            astScale.forEach(item -> {
                if ((!CapitalAstMatchEnum.DEFAULT.getCode().equals(item.getCapitalNo())) && (StringUtils.isEmpty(param.getCapitalNo()) || param.getCapitalNo().equals(item.getCapitalNo()))) {
                    astMatchAmt.addTo(item.getPrinBal());
                }
            });

            return loanDelVO;
        });
    }

    /**
     * 资产列表
     *
     * @param param
     * @return
     */
    @Override
    public BasePage<LoanListVO> queryLoanPage(LoanPageQryParam param) {
        return this.doQry(param, () -> {
            LoanListPage listPage = loanRepository.queryLoanPage(param);

            BasePage<LoanListVO> res = new BasePage<>();
            res.setCurrPageNo(listPage.getCurrPageNo());
            res.setTotal(listPage.getTotal());
            res.setLimit(listPage.getLimit());

            if (ToolUtil.isNotEmpty(listPage.getDatas())) {
                // 查找关联的合同信息
                List<String> astNos = listPage.getDatas().stream().map(LoanDO::getAstNo).collect(Collectors.toList());
                List<LoanContract> contracts = loanContractRepository.getByAstNos(astNos);
                res.setDatas(LoanVOConvert.buildDetailList(listPage.getDatas(), contracts));
            }
            return res;
        });
    }

    /**
     * 获取资产详情
     *
     * @param param
     * @return
     */
    @Override
    public LoanDetail getLoanDetail(LoanAstQryParam param) {
        return this.doQry(param, () -> {
            this.initPlan();
            LoanDetail detail = LoanVOConvert.buildDetail(getLoan(param.getAstNo()));
            return detail;
        });
    }


    /**
     * 分期资产收益详情
     *
     * @param param
     * @return
     */
    @Override
    public LoanInstalDetail getLoanInstalDetail(LoanAstTermQryParam param) {
        return this.doQry(param, () -> {
            this.initPlan();
            Loan loan = getLoan(param.getAstNo());
            LoanInstalDetail detail = LoanVOConvert.buildDetail(loan, param.getTermNo());
            return detail;
        });
    }

    /**
     * 取资产
     *
     * @param astNo
     * @return
     */
    private Loan getLoan(String astNo) {
        Loan loan = loanRepository.getByAstNo(astNo);
        VarChecker.checkNotNull(loan, "错误的资产编号[{0}]", astNo);
        loanAstAssembler.assembleLoan(loan, null);
        return loan;
    }

    /**
     * 初始化计划信息，方便后面取信息
     */
    private void initPlan() {
        // 查询计划
        List<PlanBase> planBaseList = planRepo.getPlansByStatusList(null,
                Lists.newArrayList(PlanStatusEnum.ONGOING, PlanStatusEnum.PREPARING));

        for (PlanBase planBase : planBaseList) {
            if (!PlanConst.getAllPlan().containsKey(planBase.getPlanNo())) {
                PlanConst.getAllPlan().put(planBase.getPlanNo(), planBase);
            }
        }
    }
}
