/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.api.plan;

import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.enums.AllocTypeEmun;
import com.abssqr.plat.common.facade.enums.PlanAllocStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.TitleTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAcctTitle;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanAst;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocAssignedOprParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocCfrmCalcParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocDelParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocDetailQryParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocEditParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocQryParam;
import com.abssqr.plat.common.facade.param.plan.alloc.TitleNameQryParam;
import com.abssqr.plat.common.facade.result.plan.PlanAllocVO;
import com.abssqr.plat.common.facade.result.plan.TitleNameVO;
import com.abssqr.plat.common.facade.service.plan.PlanAllocService;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.model.repo.plan.PlanAcctTitleRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAllocRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAstRepository;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.utils.ApproveChangeUtil;
import com.abssqr.plat.common.util.utils.BeanCopierUtil;
import com.abssqr.plat.core.service.plan.PlanAllocAmtCalcManager;
import com.abssqr.plat.core.service.plan.PlanAllocManager;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import com.general.system.facade.current.IdentityUtil;
import com.general.system.tool.util.ToolUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 计划资产分配服务
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.api.plan: PlanAllocServiceImpl.java, v 0.1 2019-06-03 14:24 bangis.wangdf Exp $
 */
@Service
public class PlanAllocServiceImpl extends ServiceSupport implements PlanAllocService {
    @Autowired
    PlanAllocManager        planAllocManager;
    @Autowired
    PlanAcctTitleRepo       planAcctTitleRepo;
    @Autowired
    PlanRepo                planRepo;
    @Autowired
    PlanAstRepository       planAstRepository;
    @Autowired
    PlanAllocRepo           planAllocRepo;
    @Autowired
    PlanAllocAmtCalcManager planAllocAmtCalcManager;
    
    /**
     * 获取计划分配方案
     *
     * @param param
     */
    @Override
    public List getPlanAlloc(@Valid PlanAllocQryParam param) {
        return super.doQry(param, () -> {
            List<PlanAlloc> planAllocList = planAllocManager.queryAllocByEntityNo(param.getPlanNo());
            if (ToolUtil.isEmpty(planAllocList)) {
                return planAllocList;
            }
            List<Map> newMapList = planAllocList.stream().map(one -> {
                Map allocMap = BeanCopierUtil.beanToMap(one);
                allocMap.put("operate", ApproveChangeUtil.allowEdit(one.getApproveStatus()));
                return allocMap;
            }).collect(Collectors.toList());
            return newMapList;
        });
    }
    
    /**
     * 获取分配计划详情
     *
     * @param param
     * @return
     */
    @Override
    public PlanAllocVO getPlanAllocDetail(@Valid PlanAllocDetailQryParam param) {
        return super.doQry(param, () -> {
            // 分配计划详情
            PlanAlloc alloc = null;
            // 工单查询
            if (param.getWork()) {
                alloc = planAllocManager.getDataChangePlanAllocBy(param.getAllocNo());
            } else {
                alloc = planAllocManager.getPlanAllocDetail(param.getAllocNo());
            }
            
            // 查询水位
            PlanAst ast = planAstRepository.getByPlanNo(alloc.getPlanNo());
    
            PlanBase planBase = planRepo.getPlanByNo(alloc.getPlanNo());
            List<PlanAcctTitle> planAcctTitles = planAcctTitleRepo.getByEntityNo(alloc.getPlanNo());
            // 试算
            planAllocAmtCalcManager.calcCfrmAmtByAmt(planBase, new Money(), alloc, planAcctTitles);
            // 转换模型
            PlanAllocVO allocVO = new PlanAllocVO();
            allocVO.convert(planBase, ast, alloc);
            
            if (!param.getWork()) {
                // 设置控制页面 按钮
                this.setViewBtn(allocVO, alloc);
            }
            return allocVO;
        });
    }
    
    /**
     * 获取会计科目 分配事项内容
     *
     * @param param
     * @return
     */
    @Override
    public List<TitleNameVO> getTitleNames(@Valid TitleNameQryParam param) {
        return super.doQry(param, () -> {
            List<TitleTypeEnum> titleTypeEnums = TitleTypeEnum.convert(AllocTypeEmun.getByCode(param.getAllocType()));
            List<PlanAcctTitle> planAcctTitles = planAcctTitleRepo.getTitleNames(param.getPlanNo(), titleTypeEnums);
            if (CollectionUtils.isNotEmpty(planAcctTitles)) {
                return planAcctTitles.stream().map(planAcctTitle -> {
                    TitleNameVO vo = new TitleNameVO();
                    vo.setTitleName(planAcctTitle.getTitleName());
                    vo.setTitleNo(planAcctTitle.getTitleNo());
                    return vo;
                }).collect(Collectors.toList());
            }
            return null;
        });
    }
    
    /**
     * 编辑分配计划详情
     *
     * @param param
     * @return
     */
    @Override
    public PlanAllocVO edit(@Valid PlanAllocEditParam param, PlanTypeEnum planType, boolean submit) {
        return super.doOpr(param, () -> {
            //如果分配编号为null则创建
            PlanBase planBase = planRepo.getPlanByNo(param.getPlanNo());
            VarChecker.checkArgument(planBase.getStatus() != PlanStatusEnum.COMPLETED,
                    "计划状态为[{0}],无法编辑", planBase.getStatus().getDesc());
            // 查询水位
            PlanAst ast = planAstRepository.getByPlanNo(param.getPlanNo());
            
            VarChecker.checkNotNull(ast, "该计划[{0}]水位不存在，非审批通过后无法生成对应水位", param.getPlanNo());
            
            // 更新保存分配计划
            PlanAlloc alloc = planAllocManager.addOrUpdate(planBase,
                    param, IdentityUtil.getUserCode(), IdentityUtil.getUserOrgCode(), submit);
            
            // 转换模型
            PlanAllocVO allocVO = new PlanAllocVO();
            allocVO.convert(planBase, ast, alloc);
            // 设置控制页面 按钮
            this.setViewBtn(allocVO, alloc);
            return allocVO;
        });
    }
    
    /**
     * 根据分配金额进行试算
     *
     * @param param
     * @return
     */
    @Override
    public PlanAllocVO calcCfrmByAmt(@Valid PlanAllocCfrmCalcParam param, PlanTypeEnum planType) {
        return super.doQry(param, () -> {
            PlanBase planBase = planRepo.getPlanByNo(param.getPlanNo());
            VarChecker.checkNotNull(planBase, "计划[{0}]不存在", param.getPlanNo());
            // 查询水位
            PlanAst ast = planAstRepository.getByPlanNo(param.getPlanNo());
            
            VarChecker.checkNotNull(ast, "该计划[{0}]水位不存在，非审批通过后无法生成对应水位", param.getPlanNo());
            
            PlanAlloc alloc = planAllocRepo.getPlanAllocDetail(param.getAllocNo());
            VarChecker.checkNotNull(alloc, "分配计划[{0}]不存在", param.getAllocNo());
            
            List<PlanAcctTitle> planAcctTitles = planAcctTitleRepo.getByEntityNo(param.getPlanNo());
            // 试算
            planAllocAmtCalcManager.calcCfrmAmtByAmt(planBase, param.getAllocAmt(),
                    alloc, planAcctTitles);
            
            // 转换模型
            PlanAllocVO allocVO = new PlanAllocVO();
            allocVO.convert(planBase, ast, alloc);
            
            // 设置控制页面 按钮
            this.setViewBtn(allocVO, alloc);
            
            return allocVO;
        });
    }
    
    /**
     * 删除分配方案
     *
     * @param param
     * @return
     */
    @Override
    public Boolean delete(@Valid PlanAllocDelParam param) {
        return super.doOpr(param, () -> planAllocManager.delete(param.getAllocNo()));
    }
    
    /**
     * 分配方案调整为已分配
     *
     * @param param
     * @return
     */
    @Override
    public Boolean planAllocDetailAssigned(@Valid PlanAllocAssignedOprParam param) {
        return this.doOpr(param, () -> planAllocManager.planAllocDetailAssigned(param));
    }
    
    /**
     * 设置页面按钮
     *
     * @param allocVO
     * @param alloc
     */
    private void setViewBtn(PlanAllocVO allocVO, PlanAlloc alloc) {
        List<PlanAlloc> planAllocList = planAllocManager.queryAllocByEntityNo(alloc.getPlanNo());
        if (ToolUtil.isEmpty(planAllocList)) {
            return;
        }
        boolean isFirst = false;
        for (PlanAlloc planAlloc : planAllocList) {
            if (planAlloc.getStatus() == PlanAllocStatusEnum.ASSIGNED ||
                planAlloc.getStatus() == PlanAllocStatusEnum.PENDING) {
                continue;
            }
            // 判断是否是第一个
            if (StringUtils.equals(planAlloc.getAllocNo(), alloc.getAllocNo())) {
                isFirst = true;
            } else {
                break;
            }
        }
        
        // 判断是否有试算按钮 - 第一个未分配的分配方案
        // 判断是否有提交按钮 - 第一个未分配的分配方案
        allocVO.setFirst(isFirst);
        if (isFirst) {
            // 设置为 Y 时有
            allocVO.setCalc(AbssqrConstants.YES_FLAG);
            allocVO.setSubmit(AbssqrConstants.YES_FLAG);
        }
        // 判断是否有支付 当前时间 大于等于兑付日
        if (ApproveChangeUtil.callBackSucceed(alloc.getApproveStatus()) &&
            alloc.getStatus() == PlanAllocStatusEnum.UNASSIGNE) {
            // 设置为 Y 时有
            allocVO.setPay(AbssqrConstants.YES_FLAG);
        }
    }
}
