/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.biz.api.report;

import com.abssqr.plat.biz.shared.report.wk.WkManagerReportBuilder;
import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.DailyReportDateField;
import com.abssqr.plat.common.facade.model.report.MonthReportDateField;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.wk.WkManagerReport;
import com.abssqr.plat.common.facade.param.base.CommonPlanQryParam;
import com.abssqr.plat.common.facade.param.reportForm.ReportFormAstParam;
import com.abssqr.plat.common.facade.param.reportForm.ReportFormMgrParam;
import com.abssqr.plat.common.facade.service.report.ReportService;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.SystemDateUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 *
 * @author yangcheng
 * @version ReportServiceImpl.java, v0.12019-07-0114:19 yangchengExp$
 */
@Component
public class ReportServiceImpl extends ServiceSupport implements ReportService {
    //打印日志问题:需要打印错误日志，开启服务日志，日志服务需要展示什么参数，然后在去查询，拼接业务数据然后返回;，并且起事务；
    //设置时间段问题当前日期情况
    
    private static final Logger         LOGGER = LoggerFactory.getLogger(ReportServiceImpl.class);
    @Autowired
    private         PlanReportRepo         planReportRepo;
    
    @Autowired
    private         WkManagerReportBuilder wkManagerReportBuilder;
    
    @Autowired
    private         PlanRepo               planRepo;
    
    /**
     * 计划报表
     * @param param
     * @return
     */
    @Override
    public WkManagerReport getReportFormPlan(@Valid ReportFormMgrParam param) {
        return  this.doQry(param, () -> {
        
        String begin = param.getBeginTime();
        String end   = param.getEndTime();
        
        //检查能否解析日期字段
        MonthReportDateField beginField = new MonthReportDateField();
        beginField.parse(begin);
        MonthReportDateField endField = new MonthReportDateField();
        endField.parse(end);
        
        Date beginDate = DateTimeUtil.parseInTolerance(begin);
        Date endDate = DateTimeUtil.parseInTolerance(end);
    
        dateJudge(beginDate, endDate);
    
        //当判断通过的时候打印日志
        LogUtil.info(LOGGER, "计划报表统计:计划编号={0}, 开始时间={1}, 结束时间={2},",
                param.getPlanNo(), begin, end);
        
        PlanBase planBase = planRepo.getPlanByNo(param.getPlanNo());
    
        return wkManagerReportBuilder.getReport(planBase,begin,end);
        });
    }
    
    /**
     * 资产报表
     * @param param
     * @return
     */
    @Override
    public List<PlanReport> getReportFormAst(@Valid ReportFormAstParam param) {
        
        return this.doQry(param, () -> {
            Date beginDate = DateTimeUtil.parseInTolerance(param.getBeginDate());
            Date endDate = DateTimeUtil.parseInTolerance(param.getEndDate());
            
            dateJudge(beginDate, endDate);
            
            LogUtil.info(LOGGER, "资产报表统计:计划编号={0}, 开始时间={1}, 结束时间={2},",
                    param.getPlanNo(), param.getBeginDate(), param.getEndDate());
            
            DailyReportDateField beginField = new DailyReportDateField();
            beginField.setRptDate(beginDate);
            DailyReportDateField endField = new DailyReportDateField();
            endField.setRptDate(endDate);
            
            return planReportRepo.getPlansByDate(param.getPlanNo(), beginField.desc(), endField.desc(),
                    ReportTypeEnum.RPT001);
        });
    }
    
    @Override
    public List<PlanReport> getReportFormMonthEnd(@Valid CommonPlanQryParam param) {
        return this.doQry(param, () -> {
            LogUtil.info(LOGGER, "资产报表统计:计划编号={0}", param.getPlanNo());
            
            PlanBase planBase = planRepo.getPlanByNo(param.getPlanNo());
            
            Date effectDate = planBase.getEffectDate();
            Date curDate = SystemDateUtil.getSystemDate();
            
            List<String> monthEndDate = Lists.newArrayList();
            Date startDate = DateTimeUtil.parseFromYMD(
                    DateTimeUtil.format(effectDate, DateTimeUtil.FORMAT_DATE_YYYYMM) + "01");
            // 当前日期所在月，肯定没有月报产生
            while (DateTimeUtil.calMonthDiff(startDate, curDate) > 0) {
                startDate = DateTimeUtil.addMonths(startDate, 1);
                monthEndDate.add(DateTimeUtil.formatYMD(DateTimeUtil.addDays(startDate, -1)));
            }
            
            if (CollectionUtils.isEmpty(monthEndDate)) {
                throw new AbssqrBizException(
                        MessageUtil.formatMsg("计划生效日期[{0}]和当前日期[{1}]间，无法生成月末报表", effectDate, curDate));
            }
            
            return planReportRepo.getPlansByDateList(param.getPlanNo(), ReportTypeEnum.RPT001, monthEndDate);
        });
    }
    
    private boolean dateJudge(Date begin, Date end) {
        Date nowTime = DateTimeUtil.getStandardDate(SystemDateUtil.getSystemDate());
        
        if (DateTimeUtil.calcDayDiff(end, begin) < 0) {
            String errorText = MessageUtil.formatMsg("无法生成报表：结束时间{0}小于开始时间{1}", end, begin);
            LogUtil.error(LOGGER, errorText);
            throw new AbssqrBizException(errorText, CommonErrorCodeEnum.INVALID_PARAM);
        }
        
        if (DateTimeUtil.calcDayDiff(end, nowTime) >= 0) {
            String errorText = MessageUtil.formatMsg("无法生成报表：结束时间[{0}]必须小于当前时间[{1}]，否则报表还未生成", end, nowTime);
            LogUtil.error(LOGGER, errorText);
            throw new AbssqrBizException(errorText, CommonErrorCodeEnum.INVALID_PARAM);
        }
        
        return true;
    }
}
