/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.api.utility;

import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.param.utility.PlanVaTaxCalcParam;
import com.abssqr.plat.common.facade.service.utility.UtilService;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;

/**
 * 计算税金工具服务
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.interceptor: UtilServiceImpl.java, v 0.1 2019-06-05 01:42 zhenxuan.luo Exp $ 
 */
@Component
public class UtilServiceImpl extends ServiceSupport implements UtilService {
    
    @Autowired
    private PlanRepo planRepo;
    
    /**
     * 计算税金
     * @param param
     * @return
     */
    @Override
    public Money calcVatTaxForNet(@Valid PlanVaTaxCalcParam param) {
        return this.doQry(param, () -> {
            PlanBase planBase = planRepo.getPlanByNo(param.getPlanNo());
            VarChecker.checkNotNull(planBase, "计划[{0}]不存在", param.getPlanNo());
            
            BigDecimal taxRate = planBase.getVatRate();
            return CalcUtil.calcVatTaxForNet(param.getAmt(), taxRate);
        });
    }
}