/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.biz.validate.validator;

import com.abssqr.plat.biz.validate.AFieldValidator;
import com.abssqr.plat.biz.validate.FieldError;
import com.abssqr.plat.biz.validate.ValidateService;
import com.abssqr.plat.common.facade.validation.VldDt;
import com.general.system.common.util.LogUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**
 * 注解@{@link VldDt}的校验类实现
 *
 * @author zhenxuan.luo
 * @version $Id: DtValidator.java, v 0.1 2015年5月19日 上午11:29:42 zhenxuan.luo Exp $
 */
public final class DtValidator extends AFieldValidator<VldDt> {
    
    private final static Logger LOGGER = LoggerFactory.getLogger(
            DtValidator.class);
    
    /**
     * 默认构造函数
     */
    public DtValidator() {
        super(VldDt.class);
    }
    
    /**
     * @param annotationClass
     */
    public DtValidator(Class<VldDt> annotationClass) {
        super(annotationClass);
    }
    
    @Override
    public boolean isValid(ValidateService validateService, VldDt annotationObj,
                           Object validTargetFieldObj, FieldError errorConext) {
        boolean isValid = false;
        try {
            return isValid = validate(annotationObj, validTargetFieldObj);
        } finally {
            if (!isValid) {
                errorConext.setErrorMsg(errorConext.getErrorMsg() + annotationObj.message());
            }
        }
    }
    
    private boolean validate(VldDt annotationObj, Object validTargetFieldObj) {
        
        if (validTargetFieldObj == null) {
            return (annotationObj.nullable()) ? true : false;
        }
        
        if (validTargetFieldObj instanceof String) {
            String dtText = (String) validTargetFieldObj;
            
            // 先校验长度
            if (dtText.length() != annotationObj.dtFormat().length()) {
                return false;
            }
            
            // 再校验日期格式
            DateFormat dateFormat = new SimpleDateFormat(annotationObj.dtFormat());
            
            try {
                dateFormat.parse(dtText);
            } catch (Throwable e) {
                LogUtil.warn(LOGGER, "对于dt的参数校验失败，格式不匹配: [dtText={0}, format={1}]", dtText,
                        annotationObj.dtFormat());
                return false;
            }
            
            return true;
        }
        return false;
    }
    
}
