/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.biz.validate.validator;

import com.abssqr.plat.biz.validate.AFieldValidator;
import com.abssqr.plat.biz.validate.FieldError;
import com.abssqr.plat.biz.validate.ValidateService;
import com.abssqr.plat.common.facade.validation.VldRange;

import java.math.BigDecimal;

/**
 * 注解@{@link VldRange}的校验类实现
 * @author zhenxuan.luo
 * @version $Id: RangeValidator.java, v 0.1 2015年5月19日 上午10:56:18 zhenxuan.luo Exp $
 */
public final class RangeValidator extends AFieldValidator<VldRange> {
    /**
     * 默认构造器
     */
    public RangeValidator() {
        super(VldRange.class);
    }
    
    /**
     * 校验value的范围是否在 annotationObj 范围内
     * @param range 范围注解 {@link VldRange}
     * @param value 需要校验的值
     * @return true:校验成功;反之返回false
     */
    private boolean valid(VldRange range, long value) {
        long max = range.max();
        long min = range.min();
        return value >= min && value <= max;
    }
    
    private boolean validDecimal(VldRange range, BigDecimal value) {
        BigDecimal max = new BigDecimal(range.max());
        BigDecimal min = new BigDecimal(range.min());
        return value.compareTo(min) >= 0 && value.compareTo(max) <= 0;
    }
    
    @Override
    public boolean isValid(ValidateService validateService, VldRange annotationObj,
                           Object validTargetFieldObj, FieldError errorConext) {
        boolean isValid = false;
        try {
            return isValid = validate(annotationObj, validTargetFieldObj);
        } finally {
            if (!isValid) {
                errorConext.setErrorMsg(annotationObj.message());
            }
        }
    }
    
    /**
     * 校验
     * @param annotationObj 校验注解对象
     * @param validTargetFieldObj 需要校验的字段对象
     * @return true:校验通过,反之返回false
     */
    private boolean validate(VldRange annotationObj, Object validTargetFieldObj) {
        if (validTargetFieldObj == null) {
            return (annotationObj.nullable()) ? true : false;
        }
        
        if (validTargetFieldObj instanceof Integer) {
            int value = (Integer) validTargetFieldObj;
            return valid(annotationObj, value);
        } else if (validTargetFieldObj instanceof Long) {
            long value = (Long) validTargetFieldObj;
            return valid(annotationObj, value);
        } else if (validTargetFieldObj instanceof BigDecimal) {
            BigDecimal value = (BigDecimal) validTargetFieldObj;
            return validDecimal(annotationObj, value);
        }
        return false;
    }
}
