/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.astclient.wkloan.adapter;

import com.abssqr.plat.biz.shared.astclient.base.FieldTranslator;
import com.abssqr.plat.common.facade.enums.AstContractTypeEnum;
import com.abssqr.plat.common.facade.enums.AstTypeEnum;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.GrantFundTypeEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.LoanTypeEnum;
import com.abssqr.plat.common.facade.enums.PeriodTypeEnum;
import com.abssqr.plat.common.facade.enums.RateTypeEnum;
import com.abssqr.plat.common.facade.enums.RepayMethodEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.general.system.common.util.MessageUtil;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 字段转译组装工具
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.astclient.wkloan.adapter: WkFieldTranslater.java, v 0.1 2019-06-14 18:28 zhenxuan.luo Exp $ 
 */
@Component
public class WkFieldTranslator implements FieldTranslator {

    /**
     * 五矿资产状态转换
     * @param statusFields
     * @return
     */
    @Override
    public LoanStatusEnum transAstStatus(String... statusFields) {

        String status = statusFields[0];
        String form = statusFields[1];
        List<String> acptStatus = Lists.newArrayList("2", "3", "5", "6");

        if (!acptStatus.contains(status)) {
            throw new AbssqrBizException(MessageUtil.formatMsg("不支持的贷款状态[{0}]", status),
                    CommonErrorCodeEnum.INVALID_PARAM, false);
        }

        if (StringUtils.equals("6", status)) {
            // 已结清
            return LoanStatusEnum.CLEAR;
        } else if (StringUtils.equals("3", status)) {
            // 已取消
            return LoanStatusEnum.REVERSE;
        } else {
            switch (form) {
                case "1":
                    return LoanStatusEnum.NORMAL;
                case "2":
                    return LoanStatusEnum.OVERDUE;
                case "5":
                    return LoanStatusEnum.NORMAL;
                default:
                    throw new AbssqrBizException(MessageUtil.formatMsg("不支持的贷款形态[{0}]", form),
                            CommonErrorCodeEnum.INVALID_PARAM, false);
            }
        }
    }

    @Override
    public LoanStatusEnum transTermStatus(String termStatus) {

        switch (termStatus) {
            case "1":
                return LoanStatusEnum.NORMAL;
            case "2":
                return LoanStatusEnum.OVERDUE;
            case "3":
                return LoanStatusEnum.REVERSE;
            case "4":
                return LoanStatusEnum.CANCEL;
            case "5":
                return LoanStatusEnum.CLEAR;
            case "6":
                return LoanStatusEnum.NORMAL;
            default:
                throw new AbssqrBizException(MessageUtil.formatMsg("不支持的分期形态[{0}]", termStatus),
                        CommonErrorCodeEnum.INVALID_PARAM, false);
        }
    }

    @Override
    public LoanTypeEnum transLoanType(String loanType) {
        if (StringUtils.isBlank(loanType)) {
            return LoanTypeEnum.LT03;
        }
        return LoanTypeEnum.getByCode(loanType);
    }

    @Override
    public AstTypeEnum transAstType(String astType) {
        return AstTypeEnum.LOAN;
    }

    @Override
    public AstContractTypeEnum transContType(String contType) {
        return AstContractTypeEnum.CT01;
    }

    @Override
    public GrantFundTypeEnum transGrantType(String grantType) {
        return GrantFundTypeEnum.CT01;
    }

    @Override
    public RepayMethodEnum transRepayMethod(String repayMethod) {
        if (StringUtils.isBlank(repayMethod)) {
            return RepayMethodEnum.RM02;
        }
        switch (repayMethod) {
            case "01":
                return RepayMethodEnum.RM02;
            case "02":
                return RepayMethodEnum.RM01;
//            case "1":
//                return RepayMethodEnum.RM02;
//            case "2":
//                return RepayMethodEnum.RM01;
        }
        throw new AbssqrBizException(MessageUtil.formatMsg("不支持的还款方式[{0}]", repayMethod),
                CommonErrorCodeEnum.INVALID_PARAM, false);
    }

    @Override
    public PeriodTypeEnum transPeriodType(String periodType) {
        if (StringUtils.isBlank(periodType)) {
            return PeriodTypeEnum.Month;
        }
        return PeriodTypeEnum.getByCode(periodType);
    }

    @Override
    public RateTypeEnum transRateType(String rateType) {
        if (StringUtils.isBlank(rateType)) {
            return RateTypeEnum.RT02;
        }

        switch (rateType) {
            case "10":
                return RateTypeEnum.RT01;
            case "20":
                return RateTypeEnum.RT02;
            case "30":
                return RateTypeEnum.RT03;
            default:
                throw new AbssqrBizException(MessageUtil.formatMsg("不支持的利率类型[{0}]", rateType),
                        CommonErrorCodeEnum.INVALID_PARAM, false);
        }

    }
}