/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.astclient.wkloan.adapter;

import com.abssqr.plat.biz.shared.astclient.wkloan.entity.WkMfsLoanEntity;
import com.abssqr.plat.common.dal.sync.auto.dao.WkMfsLnPactDAO;
import com.abssqr.plat.common.dal.sync.auto.dao.WkMfsOpenRepayDAO;
import com.abssqr.plat.common.dal.sync.auto.dao.WkMfsRepayInfoDAO;
import com.abssqr.plat.common.dal.sync.auto.dao.WkMfsRepayPlanDAO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsDueBillDO;
import com.abssqr.plat.common.model.wk.domain.sync.WkMfsSyncCtr;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.core.service.wk.ast.sync.WkMfsAstManager;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.VarChecker;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.astclient.wkloan.adapter: WkAstAssemply.java, v 0.1 2019-06-14 01:35 zhenxuan
 * .luo Exp $
 */
@Component
public class WkMfsAstAssembly {

    @Autowired
    WkMfsAstManager   wkMfsAstManager;
    @Autowired
    WkMfsOpenRepayDAO wkMfsOpenRepayDAO;
    @Autowired
    WkMfsLnPactDAO    wkMfsLnPactDAO;
    @Autowired
    WkMfsRepayInfoDAO wkMfsRepayInfoDAO;
    @Autowired
    WkMfsRepayPlanDAO wkMfsRepayPlanDAO;
    @Autowired
    AbsConfig         absConfig;
    
    /**
     * 组装原始数据
     * @param ctrl
     * @return
     */
    public WkMfsLoanEntity assembly(WkMfsSyncCtr ctrl) {
        WkMfsLoanEntity wkLoanEntity = new WkMfsLoanEntity();

        Date syncDate = DateTimeUtil.getStandardDate(ctrl.getAbsSyncDate());
        WkMfsDueBillDO wkMfsDueBillDO = wkMfsAstManager.getWkMfsDueBillDO(ctrl);
        VarChecker.checkNotNull(wkMfsDueBillDO, "五矿资产同步[{0}]找不到对应的借据信息", ctrl);

        wkLoanEntity.setOutAstNo(wkMfsDueBillDO.getLoanId());
        wkLoanEntity.setManagerOrgCode(wkMfsDueBillDO.getOrgNo());
        wkLoanEntity.setLenderOrgCode(wkMfsDueBillDO.getProjNo());
        wkLoanEntity.setAccountDate(DateTimeUtil.addDays(syncDate, -1));
        wkLoanEntity.setSyncDate(syncDate);
        wkLoanEntity.setProdNo(wkMfsDueBillDO.getProductId());

        wkLoanEntity.setWkMfsDueBillDO(wkMfsDueBillDO);

        wkLoanEntity.setWkMfsRepayPlanDOS(wkMfsAstManager.getWkMfsRepayPlanDOS(ctrl));

        wkLoanEntity.setWkMfsLnPactDO(wkMfsAstManager.getWkMfsLnPactDO(ctrl));

        wkLoanEntity.setWkMfsOpenRepayDO(wkMfsAstManager.getWkMfsOpenRepayDO(ctrl));

        wkLoanEntity.setWkMfsRepayInfoDOS(wkMfsAstManager.getWkMfsRepayInfoDOS(ctrl));

        wkLoanEntity.check(ctrl.getLoanId(), DateTimeUtil.formatYMD(wkLoanEntity.getAccountDate()));
        return wkLoanEntity;
    }
    
    /**
     * 同步数据到业务库中 便于做数据订正
     * @param ctrl
     */
    public void syncData2Local(WkMfsSyncCtr ctrl){
        wkMfsAstManager.syncData2Local(ctrl);
    }
}