/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.astclient.wkloan.adapter;

import com.abssqr.plat.biz.shared.astclient.base.AstClientAdapter;
import com.abssqr.plat.biz.shared.astclient.wkloan.entity.WkMfsLoanEntity;
import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanBak;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.astSync.BaseTsfRes;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncBill;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlan;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlanTrans;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncTrans;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.common.model.repo.ast.LoanRepository;
import com.abssqr.plat.common.model.repo.pool.AstPoolRepository;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import com.abssqr.plat.core.service.action.AstTransAction;
import com.abssqr.plat.core.service.action.CommonTransActionFactory;
import com.abssqr.plat.core.service.ast.LoanAstAssembler;
import com.abssqr.plat.core.service.ast.LoanAstRepoManager;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.astclient.wkloan.adapter: WkAstClientAdapter.java, v 0.1 2019-06-14 01:34
 * zhenxuan.luo Exp $
 */
@Component
public class WkMfsAstClientAdapter implements AstClientAdapter<WkMfsLoanEntity> {
    
    @Autowired
    private LoanAstAssembler         loanAstAssembler;
    @Autowired
    private LoanRepository           loanRepository;
    @Autowired
    private WkMfsLoanTranslator      wkMfsLoanTranslator;
    @Autowired
    private CommonTransActionFactory commonTransActionFactory;
    @Autowired
    private LoanAstRepoManager       loanAstRepoManager;
    @Autowired
    private AstPoolRepository        astPoolRepository;
    
    /**
     * 原始模型转换同步模型，进行业务处理
     *
     * @param astEntity
     * @return
     */
    @Override
    @PerformTrace(name = "WkMfsAstClientAdapter")
    public boolean process(WkMfsLoanEntity astEntity) {
        
        // 1. 确定同步交易：如果系统内不存在，则是数据同步，如果系统内已存在，则进一步处理放款和还款
        Loan loan = loanAstAssembler.lockByOutAstNo(astEntity.getOutAstNo(), astEntity.getManagerOrgCode(),
                astEntity.getAccountDate());
        // 资产同步模型转换并准备交易模型
        BaseTsfRes baseTsfRes = wkMfsLoanTranslator.translateLoan(astEntity, loan);
        
        if (baseTsfRes.isIgnore()) {
            // 跳过处理
            return false;
        }
        
        List<LoanBak> loanBaks = Lists.newArrayList();
        
        // 初始同步资产交易处理
        if (baseTsfRes.getInitLoanAstSyncTrans() != null) {
            // 资产增量初始化
            LoanAstSyncTrans initAstSyncTrans = baseTsfRes.getInitLoanAstSyncTrans();
            // 根据初始化交易流水执行后续处理 see: InitSyncAction
            AstTransAction action = commonTransActionFactory.getAction(initAstSyncTrans.getEventCode().getSceneCode());
            loan = action.executeAction(baseTsfRes, initAstSyncTrans, loan);

            //进行挂费处理
            processSync(baseTsfRes, EventCodeEnum.HANG_OTHER_FEE, loan);

            // 还款补偿处理  see: LoanAstRepoManager
            loanAstRepoManager.process(baseTsfRes, baseTsfRes.getRepoComponsateAstSyncTrans(), loan);
            loanBaks.addAll(LoanTransDetail.buildBaks(loan, loan.getLastUpdateDate()));
            loanAstAssembler.store(loan, astEntity.getAccountDate(), loan.getStatus().isActive());
            
            if (loan.getStatus().isActive()) {
                // 补一条结息操作
                loanAstAssembler.initForTrans(loan, astEntity.getDayBeginAccountDate());
                // 处理结息-首日同步的补偿
                processSync(baseTsfRes, EventCodeEnum.SETTLE, loan);
                loanAstAssembler.store(loan, astEntity.getDayBeginAccountDate(), false);
                loanBaks.addAll(LoanTransDetail.buildBaks(loan, astEntity.getDayBeginAccountDate()));
            }

            
        } else {
            // 资产存量处理
            int transCnt = 0;
            // 依次处理 结息、减免、放款冲正、还款
            // 结息 SettleSyncAction  回款户+'
            if (baseTsfRes.getSettleCompnsateSyncTrans() != null
                    && CollectionUtils.isNotEmpty(baseTsfRes.getSettleCompnsateSyncTrans().getRepayPlanTransList())) {
                doProcessSync(baseTsfRes, EventCodeEnum.SETTLE_COMPONSATE.getSceneCode(), loan,
                        Lists.newArrayList(baseTsfRes.getSettleCompnsateSyncTrans()));
            }
            
            // 减免 ExemptSyncAction  回款户-
            if (baseTsfRes.getExemptCompnsateSyncTrans() != null
                    && CollectionUtils.isNotEmpty(baseTsfRes.getExemptCompnsateSyncTrans().getRepayPlanTransList())) {
                doProcessSync(baseTsfRes, EventCodeEnum.EXMPT.getSceneCode(), loan,
                        Lists.newArrayList(baseTsfRes.getExemptCompnsateSyncTrans()));
            }
            
            // 放款冲正 GrantReverseSyncAction  放款户+
            transCnt += processSync(baseTsfRes, TransCodeEnum.GRANT_REVERSE, loan);
            // 期次调整
            transCnt += processSync(baseTsfRes, TransCodeEnum.ADJ_TERM, loan);

            //挂费
            transCnt += processSync(baseTsfRes,TransCodeEnum.HANG_FEE,loan);

            // 还款 RepoSyncAction    回款户+
            transCnt += processSync(baseTsfRes, TransCodeEnum.REPO, loan);
            // 处理核销
            transCnt += processSync(baseTsfRes, EventCodeEnum.WRITE_OFF, loan);

            loanAstAssembler.store(loan, astEntity.getAccountDate(), loan.getStatus().isActive());
            loanBaks.addAll(LoanTransDetail.buildBaks(loan, astEntity.getAccountDate()));
            // 贷款已结清，不再重复处理结息
            if (loan.getStatus().isActive()) {
                // 正常与逾期补充结息处理
                loanAstAssembler.initForTrans(loan, astEntity.getDayBeginAccountDate());
                // 处理结息
                transCnt += processSync(baseTsfRes, EventCodeEnum.SETTLE, loan);
                loanAstAssembler.store(loan, astEntity.getDayBeginAccountDate(), false);
                loanBaks.addAll(LoanTransDetail.buildBaks(loan, astEntity.getDayBeginAccountDate()));
            }
            
            VarChecker
                    .checkEquals(transCnt, baseTsfRes.getLoanAstSyncTransList().size(), "资产同步处理：实际处理数[{0}]和流水数[{1}]不相等",
                            transCnt, baseTsfRes.getLoanAstSyncTransList().size());
        }
        loanRepository.backup(loanBaks);
        checkEqual(baseTsfRes, loan);
        
        return true;
    }
    
    /**
     * 对应TransCodeEnum场景的同步处理操作
     *
     * @param baseTsfRes
     * @param eventCode
     * @param loan
     * @return
     */
    private int processSync(BaseTsfRes baseTsfRes, EventCodeEnum eventCode, Loan loan) {
        
        List<LoanAstSyncTrans> trans = baseTsfRes.getLoanAstSyncTransList().stream()
                                               .filter(item -> item.getEventCode() == eventCode
                                               ).collect(Collectors.toList());
        
        doProcessSync(baseTsfRes, eventCode.getSceneCode(), loan, trans);
        
        return trans.size();
    }
    
    private int processSync(BaseTsfRes baseTsfRes, TransCodeEnum transCode, Loan loan) {
        
        List<LoanAstSyncTrans> trans = baseTsfRes.getLoanAstSyncTransList().stream()
                                               .filter(item -> item.getEventCode().getSceneCode() == transCode
                                               ).collect(Collectors.toList());
        
        doProcessSync(baseTsfRes, transCode, loan, trans);
        
        return trans.size();
    }
    
    private void doProcessSync(BaseTsfRes baseTsfRes, TransCodeEnum transCode, Loan loan,
                               List<LoanAstSyncTrans> trans) {
        AstTransAction action = commonTransActionFactory.getAction(transCode);
        VarChecker.checkNotNull(action, "资产同步不支持的事件类型", transCode);
        
        for (LoanAstSyncTrans loanAstSyncTrans : trans) {
            action.executeAction(baseTsfRes, loanAstSyncTrans, loan);
        }
    }
    
    private void checkEqual(BaseTsfRes astEntity, Loan loan) {
        
        LoanAstSyncBill syncBill = astEntity.getLoanAstSyncBill();
        List<LoanAstSyncRepayPlan> syncRepayPlans = astEntity.getRepayPlans();

        if(loan.getLoanRisk().getWtfStatus()== YNEnum.Y){
            return;
        }

        if (loan.getStatus() == LoanStatusEnum.REVERSE) {
            VarChecker.checkEquals(new DebtMoney(), loan.getBal(),
                    "[syncCheckFail][{0}]数据同步账单余额不相等sync=[{1}], loan=[{2}]",
                    syncBill.getLoanNo(), new DebtMoney(), loan.getBal());
        } else {
            VarChecker.checkEquals(syncBill.getBal(), loan.getBal(),
                    "[syncCheckFail][{0}]数据同步账单余额不相等sync=[{1}], loan=[{2}]",
                    syncBill.getLoanNo(), syncBill.getBal(), loan.getBal());
        }
        
        VarChecker.checkEquals(syncBill.getStatus(), loan.getStatus(),
                "[syncCheckFail][{0}]数据同步账单状态不相等sync=[{1}], loan=[{2}]", syncBill.getLoanNo(), syncBill.getStatus(),
                loan.getStatus());
        
        int repayPlanSize = syncRepayPlans.stream().filter(item -> item.getTermNo() != 0).collect(Collectors.toList())
                                    .size();
        int instalSize = loan.getLoanInstalList().stream().filter(item -> item.getTermNo() != 0)
                                 .collect(Collectors.toList()).size();
        
        VarChecker.checkEquals(repayPlanSize, instalSize,
                "[syncCheckFail][{0}]数据同步分期数不相等sync=[{1}], loan=[{2}]", syncBill.getLoanNo(), repayPlanSize,
                instalSize);
        Map<Integer, LoanInstalDomain> loanInstalMap = loan.getLoanInstalMap();
        
        Map<Integer, LoanAstSyncRepayPlanTrans> repoCompTrans = Maps.newHashMap();
        
        for (LoanAstSyncRepayPlan syncRepayPlan : syncRepayPlans) {
            LoanInstalDomain loanInstal = loanInstalMap.get(syncRepayPlan.getTermNo());
            
            if (syncRepayPlan.getTermNo() == 0) {
                VarChecker.checkArgument(loanInstal.getBal().isZero(),
                        "[syncCheckFail][{0}-{1}]数据同步0分期为提前还款分期，余额必须清0 [{2}]", syncRepayPlan.getLoanNo(),
                        syncRepayPlan.getTermNo(), loanInstal.getBal());
                VarChecker.checkArgument(loanInstal.getStatus() == LoanStatusEnum.CLEAR,
                        "[syncCheckFail][{0}-{1}]数据同步0分期为提前还款分期，状态必须清结清 [{2}]", syncRepayPlan.getLoanNo(),
                        syncRepayPlan.getTermNo(), loanInstal.getStatus());
                continue;
            }
            
            if (loan.getStatus() == LoanStatusEnum.REVERSE) {
                VarChecker.checkEquals(new DebtMoney(), loanInstal.getBal(),
                        "[syncCheckFail][{0}-{1}]数据同步分期余额不相等sync=[{2}], loan=[{3}]",
                        syncRepayPlan.getLoanNo(), syncRepayPlan.getTermNo(), new DebtMoney(),
                        loanInstal.getBal());
            } else {
                VarChecker.checkEquals(syncRepayPlan.getCheckBal(), loanInstal.getBal(),
                        "[syncCheckFail][{0}-{1}]数据同步分期余额不相等sync=[{2}], loan=[{3}]",
                        syncRepayPlan.getLoanNo(), syncRepayPlan.getTermNo(), syncRepayPlan.getCheckBal(),
                        loanInstal.getBal());
            }
            
            VarChecker.checkEquals(syncRepayPlan.getCheckStatus(), loanInstal.getStatus(),
                    "[syncCheckFail][{0}-{1}]数据同步分期状态不相等sync=[{2}], loan=[{3}]",
                    syncRepayPlan.getLoanNo(), syncRepayPlan.getTermNo(), syncRepayPlan.getCheckStatus(),
                    loanInstal.getStatus());
        }
    }
}
