/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.astclient.wkloan.adapter;

import cn.hutool.core.util.NumberUtil;
import com.abssqr.plat.biz.shared.astclient.wkloan.entity.WkMfsLoanEntity;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsDueBillDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsLnPactDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsOpenRepayDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsRepayInfoDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsRepayPlanDO;
import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.enums.CertTypeEnum;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.RateTypeEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.model.domain.ast.AstPeriod;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.astSync.BaseTsfRes;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncBill;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncContract;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlan;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlanTrans;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncTrans;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.utils.DecimalCalcUtil;
import com.abssqr.plat.common.model.utils.UserUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.MoneyUtil;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 资产模型及交易处理 模型转换
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.astclient.wkloan.adapter: WkLoanTranslator.java, v 0.1 2019-06-14 15:09
 * zhenxuan.luo Exp $
 */
@Component
public class WkMfsLoanTranslator {
    private final static Logger LOGGER           = LoggerFactory.getLogger(WkMfsLoanTranslator.class);
    private final static Money  ZERO             = new Money();
    /**
     * 冲正标记
     */
    private final static String REPO_REVERSE_EVT = "16";

    @Autowired
    private WkFieldTranslator wkFieldTranslator;

    /**
     * 资产数据转型
     *
     * @param wkLoanEntity
     * @param loan
     * @return
     */
    public BaseTsfRes translateLoan(WkMfsLoanEntity wkLoanEntity, Loan loan) {
        //全局异常
        Exception exception = null;
        BaseTsfRes baseTsfRes = new BaseTsfRes();
        baseTsfRes.setLoanAstSyncBill(buildBillSync(wkLoanEntity));
        String bizCurDt = DateTimeUtil.formatYMD(wkLoanEntity.getAccountDate());
        LogUtil.info(LOGGER, "构建账单后产品编码为[{0}]", baseTsfRes.getLoanAstSyncBill().getProdCode());
        try {
            if (loan == null) {
                //已结清，冲正，取消的资产，不用再同步了
                // 冲正账单，直接返回
                if (baseTsfRes.getLoanAstSyncBill().getStatus() == LoanStatusEnum.REVERSE) {
                    baseTsfRes.setIgnore(true);
                    copyCheckBal(baseTsfRes.getRepayPlans());
                    return baseTsfRes;
                }

                if (baseTsfRes.getLoanAstSyncBill().getStatus() == LoanStatusEnum.CLEAR
                    || baseTsfRes.getLoanAstSyncBill().getStatus() == LoanStatusEnum.CANCEL) {

                    // 当前会计日期大于放款日期，说明不是上日结清，非当天还款当天放款的场景，直接跳过
                    if (DateTimeUtil.calcDayDiff(baseTsfRes.getLoanAstSyncBill().getAccountDate(),
                        baseTsfRes.getLoanAstSyncBill().getDisbDate()) > 0) {
                        baseTsfRes.setIgnore(true);
                        copyCheckBal(baseTsfRes.getRepayPlans());
                        return baseTsfRes;
                    }
                }

                // 包含了放款和资产同步
                baseTsfRes.setLoanContract(buildContract(baseTsfRes.getLoanAstSyncBill(), wkLoanEntity));
                // 组装还款计划
                baseTsfRes.setRepayPlans(buildExistRepayPlans(wkLoanEntity, baseTsfRes.getLoanAstSyncBill()));
                // 设置状态
                resetLoanStatus(baseTsfRes);

                WkMfsOpenRepayDO openRepayDO = wkLoanEntity.getWkMfsOpenRepayDO();
                // 生成放款资产交易流水信息
                LoanAstSyncTrans loanAstSyncTrans = new LoanAstSyncTrans();
                loanAstSyncTrans.setBizNo(openRepayDO.getSeqNo());
                loanAstSyncTrans.setAccountDate(wkLoanEntity.getAccountDate());
                loanAstSyncTrans.setTransTime(DateTimeUtil.parseInTolerance(openRepayDO.getDeducData()));
                loanAstSyncTrans.setEventCode(EventCodeEnum.SYNC);
                // 生成放款资产分期交易流水信息
                for (LoanAstSyncRepayPlan loanAstSyncRepayPlan : baseTsfRes.getRepayPlans()) {
                    LoanAstSyncRepayPlanTrans loanAstSyncRepayPlanTrans = new LoanAstSyncRepayPlanTrans();
                    loanAstSyncRepayPlanTrans.setTermNo(loanAstSyncRepayPlan.getTermNo());
                    loanAstSyncRepayPlanTrans.setOutBizNo(openRepayDO.getSeqNo());
                    loanAstSyncRepayPlanTrans.setTransTime(loanAstSyncTrans.getTransTime());
                    loanAstSyncRepayPlanTrans.setEventCode(loanAstSyncTrans.getEventCode());
                    loanAstSyncRepayPlanTrans
                        .setPrinAmt(loanAstSyncRepayPlan.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN));
                    loanAstSyncRepayPlanTrans
                        .setIntrAmt(loanAstSyncRepayPlan.getBal().getByFieldType(DebtMoneyFieldEnum.INTR));
                    loanAstSyncRepayPlanTrans
                        .setPrinPenalAmt(
                            loanAstSyncRepayPlan.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
                    loanAstSyncRepayPlanTrans.setLoanNo(loanAstSyncRepayPlan.getLoanNo());
                    loanAstSyncRepayPlanTrans.setSyncDate(loanAstSyncRepayPlan.getSyncDate());
                    loanAstSyncRepayPlanTrans.setAccountDate(wkLoanEntity.getAccountDate());

                    loanAstSyncTrans.getRepayPlanTransList().add(loanAstSyncRepayPlanTrans);
                }

                baseTsfRes.setInitLoanAstSyncTrans(loanAstSyncTrans);

                // 处理还款补偿
                List<WkMfsRepayInfoDO> repayInfoDOS = filterRepayInfos(wkLoanEntity.getWkMfsRepayInfoDOS(), bizCurDt);
                // 按期分组
                Map<Integer, List<WkMfsRepayInfoDO>> repayPlanInfoGrp = repayInfoDOS.stream().collect(
                    Collectors.groupingBy(WkMfsRepayInfoDO::getTermNo));

                List<WkMfsRepayInfoDO> combinedRepayInfoDOs = Lists.newArrayList();
                // 按期汇总还款金额
                for (int termNo = 0; termNo <= baseTsfRes.getRepayPlans().size(); termNo++) {
                    List<WkMfsRepayInfoDO> termRepayInfoDOS = repayPlanInfoGrp.get(termNo);
                    if (CollectionUtils.isEmpty(termRepayInfoDOS)) {
                        continue;
                    }
                    // 汇总还款金额
                    combinedRepayInfoDOs.add(combileRepayInfos(termRepayInfoDOS));
                }

                // 处理还款补偿信息
                baseTsfRes.setRepoComponsateAstSyncTrans(
                    buildTransForRepayComponsate(combinedRepayInfoDOs, baseTsfRes.getLoanAstSyncBill()));

                copyCheckBal(baseTsfRes.getRepayPlans());
                // 覆盖处理还款补偿对应分期同步模型的余额
                recoverInstalBalForRepayComponsate(baseTsfRes, openRepayDO, combinedRepayInfoDOs);
                if (ToolUtil.isNotEmpty(combinedRepayInfoDOs)){
                    //提前还款明细费挂账补偿
                    buildHangFeeTransForPreRepay(combileRepayInfos(combinedRepayInfoDOs), baseTsfRes.getRepoComponsateAstSyncTrans(),baseTsfRes);
                }
                // 补空结息操作，生成当日流水和备份
                baseTsfRes.getLoanAstSyncTransList().add(processDummySettle(baseTsfRes.getLoanAstSyncBill(),
                    baseTsfRes.getRepayPlans()));
            } else {
                if (loan.getStatus() == LoanStatusEnum.CLEAR || loan.getStatus() == LoanStatusEnum.REVERSE) {
                    if (baseTsfRes.getLoanAstSyncBill().getStatus() == loan.getStatus()) {
                        baseTsfRes.setIgnore(true);
                        copyCheckBal(baseTsfRes.getRepayPlans());
                        return baseTsfRes;
                    } else {
                        throw new AbssqrBizException(MessageUtil.formatMsg(
                            "[资产同步校验失败]：[{0}]-当前状态为已结清或者冲正[{1}]，五矿状态为[{2}], 存在冲突", loan.getOutAstNo(),
                            loan.getStatus(), baseTsfRes.getLoanAstSyncBill().getStatus()),
                            CommonErrorCodeEnum.INVALID_INTERNAL_MODEL);
                    }
                }
                // 如果是新增，那么只同步贷款数据就行，不用在处理还款信息
                LoanAstSyncBill loanAstSyncBill = baseTsfRes.getLoanAstSyncBill();
                // 组装还款计划
                baseTsfRes.setRepayPlans(buildExistRepayPlans(wkLoanEntity, loanAstSyncBill));

                // 处理放款冲正
                if (baseTsfRes.getLoanAstSyncBill().getStatus() == LoanStatusEnum.REVERSE) {
                    if (loan.getStatus() != LoanStatusEnum.REVERSE) {
                        baseTsfRes.getLoanAstSyncTransList().add(
                            buildTransForGrantReverse(loanAstSyncBill, baseTsfRes.getRepayPlans()));
                    }
                    copyCheckBal(baseTsfRes.getRepayPlans());
                    return baseTsfRes;
                }

                // 重置借据状态（应对宽限期内借据状态为正常的情况）
                resetLoanStatus(baseTsfRes);

                // 处理还款流水 及 还款冲正流水
                List<WkMfsRepayInfoDO> repayInfoDOS = filterRepayInfos(wkLoanEntity.getWkMfsRepayInfoDOS(), bizCurDt);
                // 生成资产还款交易流水
                LoanAstSyncTrans repoSyncTrans = new LoanAstSyncTrans();
                if (CollectionUtils.isNotEmpty(repayInfoDOS)) {
                    WkMfsRepayInfoDO repayInfoDO = repayInfoDOS.get(0);
                    Map<Integer, List<WkMfsRepayInfoDO>> repayPlanInfoGrp = repayInfoDOS.stream().collect(
                        Collectors.groupingBy(WkMfsRepayInfoDO::getTermNo));
                    // 还款资产交易流水
                    repoSyncTrans.setLoanNo(loanAstSyncBill.getLoanNo());
                    repoSyncTrans.setBizNo(repayInfoDO.getSeqNo());
                    repoSyncTrans.setAccountDate(loanAstSyncBill.getAccountDate());
                    repoSyncTrans.setTransTime(DateTimeUtil.parseInTolerance(repayInfoDO.getTranDate()));
                    repoSyncTrans.setEventCode(EventCodeEnum.DUE_REPAY);

                    // 生成资产分期还款交易流水
                    for (int termNo = 1; termNo <= baseTsfRes.getRepayPlans().size(); termNo++) {
                        List<WkMfsRepayInfoDO> termRepayInfoDOS = repayPlanInfoGrp.get(termNo);
                        if (CollectionUtils.isEmpty(termRepayInfoDOS)) {
                            continue;
                        }
                        // 汇总还款金额 进行到期还款
                        this.buildTransForTermRepay(combileRepayInfos(termRepayInfoDOS), repoSyncTrans,
                            baseTsfRes.getRepayPlans());
                    }
                    // 判断是否有提前还款信息 termNo = 0为提前还款
                    List<WkMfsRepayInfoDO> term0RepayInfoDOS = repayPlanInfoGrp.get(0);
                    if (CollectionUtils.isNotEmpty(term0RepayInfoDOS)) {
                        // 汇总还款金额 进行提前还款
                        WkMfsRepayInfoDO wkMfsRepayInfoDO = combileRepayInfos(term0RepayInfoDOS);
                        //处理提前还款
                        this.buildTransForPreRepay(wkMfsRepayInfoDO, repoSyncTrans, baseTsfRes.getRepayPlans(), loan);
                    }
                    if(CollectionUtils.isNotEmpty(repayInfoDOS)) {
                        // 汇总还款金额 进行提前还款挂费交易组装
                        this.buildHangFeeTransForPreRepay(combileRepayInfos(repayInfoDOS), repoSyncTrans, baseTsfRes);
                    }
                    baseTsfRes.getLoanAstSyncTransList().add(repoSyncTrans);

                }

                // 生成结息补偿流水
                processSettleComponsate(baseTsfRes,
                    baseTsfRes.getRepayPlans(), loan, baseTsfRes.getLoanAstSyncTransList());
                // 处理到期和逾期结息
                LoanAstSyncTrans settleTrans = processDueSettleAndOvd(baseTsfRes, loan);
                if (settleTrans != null) {
                    baseTsfRes.getLoanAstSyncTransList().add(settleTrans);
                }
                // 处理分期调整
                processAdjTerm(baseTsfRes, loan, repoSyncTrans);
                // 核销处理
                processWtf(baseTsfRes, loan, wkLoanEntity);
                copyCheckBal(baseTsfRes.getRepayPlans());
            }
            return baseTsfRes;
        } catch (Exception e) {
            exception = e;
            LogUtil.error(LOGGER, e, "translateLoan模型同步异常：" + e.getMessage());
            throw e;
        } finally {
            if (exception == null) {
                check(baseTsfRes, wkLoanEntity);
            }
        }
    }

    private void copyCheckBal(List<LoanAstSyncRepayPlan> repayPlans) {
        for (LoanAstSyncRepayPlan repayPlan : repayPlans) {
            repayPlan.setCheckBal(repayPlan.getBal().cloneAll());
            repayPlan.setCheckStatus(repayPlan.getStatus());
        }
    }

    private void check(BaseTsfRes baseTsfRes, WkMfsLoanEntity wkLoanEntity) {

        if (baseTsfRes.isIgnore()) {
            return;
        }

        // 校验状态
        LoanAstSyncBill syncBill = baseTsfRes.getLoanAstSyncBill();
        WkMfsDueBillDO wkMfsDueBill = wkLoanEntity.getWkMfsDueBillDO();

        if (StringUtils.equals(wkMfsDueBill.getLoanForm(), "3")) {
            // 3: 已冲正
            VarChecker.checkEquals(syncBill.getStatus(), LoanStatusEnum.REVERSE,
                "[资产同步校验失败]：[{0}]-转化后状态为[{1}]，五矿状态为已冲正[{2}]", syncBill.getLoanNo(), syncBill.getStatus(),
                wkMfsDueBill.getLoanForm());
            return;
        }

        if (syncBill.getStatus() == LoanStatusEnum.REVERSE) {
            // 3: 已冲正
            VarChecker.checkArgument(StringUtils.equals(wkMfsDueBill.getLoanStatus(), "3"),
                "[资产同步校验失败]：[{0}]-转化后状态为已冲正[{1}]，五矿状态为[{2}]", syncBill.getLoanNo(), syncBill.getStatus(),
                wkMfsDueBill.getLoanForm());
            return;
        }

        // 校验账单余额，不校验分期余额，因为分期上的利息余额不准
        VarChecker.checkEquals(syncBill.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN),
            new Money(wkMfsDueBill.getPrinBal()),
            "[资产同步校验失败]：[{0}]-转化后本金不相等：转化后[{1}]，五矿[{2}]", syncBill.getLoanNo(),
            syncBill.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN), wkMfsDueBill.getPrinBal());
        VarChecker.checkEquals(syncBill.getBal().getByFieldType(DebtMoneyFieldEnum.OVD_PRIN),
            new Money(wkMfsDueBill.getOvdPrinBal()),
            "[资产同步校验失败]：[{0}]-转化后逾期本金不相等：转化后[{1}]，五矿[{2}]", syncBill.getLoanNo(),
            syncBill.getBal().getByFieldType(DebtMoneyFieldEnum.OVD_PRIN), wkMfsDueBill.getOvdPrinBal());
        VarChecker.checkEquals(syncBill.getBal().getByFieldType(DebtMoneyFieldEnum.INTR),
            new Money(wkMfsDueBill.getIntBal()),
            "[资产同步校验失败]：[{0}]-转化后利息不相等：转化后[{1}]，五矿[{2}]", syncBill.getLoanNo(),
            syncBill.getBal().getByFieldType(DebtMoneyFieldEnum.INTR), wkMfsDueBill.getIntBal());
        VarChecker.checkEquals(syncBill.getBal().getByFieldType(DebtMoneyFieldEnum.OVD_INTR),
            new Money(wkMfsDueBill.getOvdIntBal()),
            "[资产同步校验失败]：[{0}]-转化后逾期利息不相等：转化后[{1}]，五矿[{2}]", syncBill.getLoanNo(),
            syncBill.getBal().getByFieldType(DebtMoneyFieldEnum.OVD_INTR), wkMfsDueBill.getOvdIntBal());
        VarChecker.checkEquals(syncBill.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY),
            new Money(wkMfsDueBill.getPnltIntBal()),
            "[资产同步校验失败]：[{0}]-转化后罚息不相等：转化后[{1}]，五矿[{2}]", syncBill.getLoanNo(),
            syncBill.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY), wkMfsDueBill.getPnltIntBal());

        // 校验转换后分期余额汇总和账单相等
        DebtMoney totalRepayPlanBal = new DebtMoney();
        for (LoanAstSyncRepayPlan repayPlan : baseTsfRes.getRepayPlans()) {
            totalRepayPlanBal.addAll(repayPlan.getBal());
        }
        VarChecker.checkEquals(syncBill.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY),
            new Money(wkMfsDueBill.getPnltIntBal()),
            "[资产同步校验失败]：[{0}]-转化后账单余额[{1}]不等于分期余额汇总[{2}]", syncBill.getLoanNo(),
            syncBill.getBal(), totalRepayPlanBal);

        Map<Integer, LoanAstSyncRepayPlan> syncRepayPlanMap = baseTsfRes.getRepayPlans().stream().collect(
            Collectors.toMap(LoanAstSyncRepayPlan::getTermNo, item -> item));

        Map<Integer, LoanAstSyncRepayPlanTrans> repoCompTrans = Maps.newHashMap();

        for (WkMfsRepayPlanDO repayPlanDO : wkLoanEntity.getWkMfsRepayPlanDOS()) {
            if (repayPlanDO.getTermNo() == 0) {
                continue;
            }

            LoanAstSyncRepayPlan repayPlan = syncRepayPlanMap.get(repayPlanDO.getTermNo());

            Money prinBal = StringUtils.equals("4", repayPlanDO.getTermStatus()) ?
                ZERO :
                new Money(repayPlanDO.getPrinTotal().subtract(repayPlanDO.getPrinRepay()));
            Money ckPrinBal = repayPlan.getCheckBal().getByFieldType(DebtMoneyFieldEnum.PRIN);
            Money ckPrinPnyBal = repayPlan.getCheckBal().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY);

            VarChecker.checkEquals(ckPrinBal, prinBal,
                "[资产同步校验失败]：[{0}-{1}]-分期转化后本金不相等：转化后[{2}]，五矿[{3}]", repayPlan.getLoanNo(),
                repayPlan.getTermNo(), ckPrinBal, prinBal);

            Money prinPnyBal = StringUtils.equals("4", repayPlanDO.getTermStatus()) ?
                ZERO :
                new Money(repayPlanDO.getPnltIntTotal().subtract(repayPlanDO.getPnltIntRepay()));

            VarChecker.checkEquals(ckPrinPnyBal, prinPnyBal,
                "[资产同步校验失败]：[{0}-{1}]-分期转化后罚息不相等：转化后[{2}]，五矿[{3}]", repayPlan.getLoanNo(),
                repayPlan.getTermNo(), ckPrinPnyBal, prinPnyBal);
        }
    }

    /**
     * 汇总还款金额
     *
     * @param repayInfoDOs
     * @return
     */
    private WkMfsRepayInfoDO combileRepayInfos(List<WkMfsRepayInfoDO> repayInfoDOs) {
        Collections.sort(repayInfoDOs, Comparator.comparing(WkMfsRepayInfoDO::getSeqNo));
        Collections.reverse(repayInfoDOs);

        WkMfsRepayInfoDO res = repayInfoDOs.get(0);
        for (int i = 1; i < repayInfoDOs.size(); i++) {
            WkMfsRepayInfoDO var = repayInfoDOs.get(i);
            res.setTotalAmt(NumberUtil.add(res.getTotalAmt(), var.getTotalAmt()));
            res.setIncomeAmt(NumberUtil.add(res.getIncomeAmt(), var.getIncomeAmt()));
            res.setPrinAmt(NumberUtil.add(res.getPrinAmt(), var.getPrinAmt()));
            res.setIntAmt(NumberUtil.add(res.getIntAmt(), var.getIntAmt()));
            res.setPnltIntAmt(NumberUtil.add(res.getPnltIntAmt(), var.getPnltIntAmt()));
            res.setTrustIntAmtAcc(NumberUtil.add(res.getTrustIntAmtAcc(), var.getTrustIntAmtAcc()));
            res.setTrustPnltIntAmtAcc(NumberUtil.add(res.getTrustPnltIntAmtAcc(), var.getTrustPnltIntAmtAcc()));
            res.setRepayViolateAmt(NumberUtil.add(res.getRepayViolateAmt(), var.getRepayViolateAmt()));
            res.setTrustRepayViolateAmtAcc(NumberUtil.add(res.getTrustRepayViolateAmtAcc(), var.getTrustRepayViolateAmtAcc()));
        }

        return res;
    }

    private void resetLoanStatus(BaseTsfRes baseTsfRes) {
        LoanAstSyncBill bill = baseTsfRes.getLoanAstSyncBill();
        int maxOvdDays = 0;
        for (LoanAstSyncRepayPlan repayPlan : baseTsfRes.getRepayPlans()) {
            if (repayPlan.getStatus().isActive()) {
                if (repayPlan.getOvdDays() > 0) {
                    if (repayPlan.getStatus() == LoanStatusEnum.NORMAL) {
                        repayPlan.setStatus(LoanStatusEnum.OVERDUE);
                    }
                    maxOvdDays = Math.max(maxOvdDays, repayPlan.getOvdDays());
                }
            } else {
                // 结清，冲正
                repayPlan.setOvdDays(0);
            }
            if (repayPlan.getStatus() == LoanStatusEnum.OVERDUE && bill.getStatus() == LoanStatusEnum.NORMAL) {
                bill.setStatus(LoanStatusEnum.OVERDUE);
                break;
            }
        }
    }

    private LoanAstSyncBill buildBillSync(WkMfsLoanEntity wkLoanEntity) {

        WkMfsDueBillDO wkMfsDueBillDO = wkLoanEntity.getWkMfsDueBillDO();

        LoanAstSyncBill loanAstSyncBill = new LoanAstSyncBill();
        //  根据放款机构查询发起机构，并填入
        loanAstSyncBill.setSponserOrgCode("");
        loanAstSyncBill.setLoanNo(wkMfsDueBillDO.getLoanId());
        loanAstSyncBill.setSyncDate(wkLoanEntity.getSyncDate());
        loanAstSyncBill.setAccountDate(wkLoanEntity.getAccountDate());
        loanAstSyncBill.setAstType(wkFieldTranslator.transAstType(""));
        loanAstSyncBill.setLenderOrgCode(wkLoanEntity.getLenderOrgCode());
        loanAstSyncBill.setManagerOrgCode(wkLoanEntity.getManagerOrgCode());
        loanAstSyncBill.setProdCode(wkLoanEntity.getProdNo());

        loanAstSyncBill.setStartDate(DateTimeUtil.parseFromYMD(wkMfsDueBillDO.getStartDate()));
        loanAstSyncBill.setEndDate(DateTimeUtil.parseFromYMD(wkMfsDueBillDO.getEndDate()));
        loanAstSyncBill.setDisbDate(DateTimeUtil.parseFromYMD(wkMfsDueBillDO.getPayoutDate()));
        if (wkMfsDueBillDO.getCurTerm() != null) {
            loanAstSyncBill.setCurTerm(wkMfsDueBillDO.getCurTerm());
        }
        loanAstSyncBill.setTermCnt(wkMfsDueBillDO.getTotalTerms());

        // 先转状态，
        loanAstSyncBill.setStatus(
            wkFieldTranslator.transAstStatus(wkMfsDueBillDO.getLoanStatus(), wkMfsDueBillDO.getLoanForm()));
        loanAstSyncBill.getBal().setByFieldType(DebtMoneyFieldEnum.PRIN,
                new Money(wkMfsDueBillDO.getPrinBal() == null ? BigDecimal.ZERO : wkMfsDueBillDO.getPrinBal()));
        loanAstSyncBill.getBal().setByFieldType(DebtMoneyFieldEnum.OVD_PRIN,
                new Money(wkMfsDueBillDO.getOvdPrinBal() == null ? BigDecimal.ZERO :wkMfsDueBillDO.getOvdPrinBal()));
        loanAstSyncBill.getBal().setByFieldType(DebtMoneyFieldEnum.INTR,
                new Money(wkMfsDueBillDO.getIntBal() == null ? BigDecimal.ZERO : wkMfsDueBillDO.getIntBal()));
        loanAstSyncBill.getBal().setByFieldType(DebtMoneyFieldEnum.OVD_INTR,
                new Money(wkMfsDueBillDO.getOvdIntBal() == null ? BigDecimal.ZERO : wkMfsDueBillDO.getOvdIntBal()));
        // 因为没有区分本罚利罚，罚息都计入本罚
        loanAstSyncBill.getBal().setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY,
                new Money(wkMfsDueBillDO.getPnltIntBal() == null ? BigDecimal.ZERO : wkMfsDueBillDO.getPnltIntBal()));

        return loanAstSyncBill;
    }

    private LoanAstSyncContract buildContract(LoanAstSyncBill loanAstSyncBill, WkMfsLoanEntity wkLoanEntity) {
        WkMfsLnPactDO wkMfsLnPactDO = wkLoanEntity.getWkMfsLnPactDO();
        WkMfsOpenRepayDO wkMfsOpenRepayDO = wkLoanEntity.getWkMfsOpenRepayDO();
        if (null == wkMfsLnPactDO || null == wkMfsOpenRepayDO) {
            throw new AbssqrBizException(
                MessageUtil.formatMsg("构造贷款合同信息，但是模型数据不全，没有合同或者账单信息.[{0}]", loanAstSyncBill.getLoanNo()),
                CommonErrorCodeEnum.INVALID_PARAM, false);
        }

        LoanAstSyncContract loanAstSyncContract = new LoanAstSyncContract();
        loanAstSyncContract.setProdNo(wkLoanEntity.getProdNo());
        loanAstSyncContract.setContNo(wkMfsLnPactDO.getPactNo());
        loanAstSyncContract.setContType(wkFieldTranslator.transContType(""));
        loanAstSyncContract.setMobileNo("");
        loanAstSyncContract.setUserName(wkMfsOpenRepayDO.getCustName());
        loanAstSyncContract.setCertNo(wkMfsOpenRepayDO.getCertNo());
        loanAstSyncContract.setCertType(CertTypeEnum.getByCode(wkMfsOpenRepayDO.getCertType()));
        loanAstSyncContract
            .setUserId(UserUtil.buildUserId(loanAstSyncContract.getCertType(), loanAstSyncContract.getCertNo()));
        // 截取身份证号前三位为省份
        loanAstSyncContract.setProvince(StringUtils.substring(loanAstSyncContract.getCertNo(), 0, 3));
        // 截取身份证号4-6位为城市
        loanAstSyncContract.setCity(StringUtils.substring(loanAstSyncContract.getCertNo(), 3, 6));
        loanAstSyncContract.setLoanType(wkFieldTranslator.transLoanType(wkMfsLnPactDO.getLnType()));
        loanAstSyncContract.setPurpose(wkMfsLnPactDO.getLnUse());
        loanAstSyncContract.setGrantType(wkFieldTranslator.transGrantType(""));
        loanAstSyncContract.setApplyLoc(wkMfsLnPactDO.getAppArea());
        loanAstSyncContract.setDisbAmt(new Money(wkMfsOpenRepayDO.getEncashAmt()));
        loanAstSyncContract.setDisbDate(DateTimeUtil.parseFromYMD(wkMfsOpenRepayDO.getStartDate()));
        loanAstSyncContract.setRepayMethod(wkFieldTranslator.transRepayMethod(wkMfsOpenRepayDO.getRepayMode()));
        loanAstSyncContract.setRepayPeriod(1);
        // 期限单位
        loanAstSyncContract.setRepayPeriodType(wkFieldTranslator.transPeriodType(""));
        // 固定月利率  不进行转换 wkFieldTranslator.transRateType(wkMfsDueBillDO.getRateType())
        loanAstSyncContract.setRateType(RateTypeEnum.RT02);
        // 默认为月利率  FIXME 五矿招联利率问题 新增同步机构的时候重点关注
        loanAstSyncContract.setDaysCoef(30);

        loanAstSyncContract.setInterRate(wkMfsLnPactDO.getLnRate());
        loanAstSyncContract.setPenalRate(wkMfsLnPactDO.getOverRate());

        loanAstSyncContract.setTermCnt(wkMfsOpenRepayDO.getTotalTerms());
        loanAstSyncContract.setStartDate(DateTimeUtil.parseFromYMD(wkMfsOpenRepayDO.getStartDate()));
        loanAstSyncContract.setEndDate(DateTimeUtil.parseFromYMD(wkMfsOpenRepayDO.getEndDate()));
        loanAstSyncContract.setLoanNo(loanAstSyncBill.getLoanNo());
        loanAstSyncContract.setSyncDate(loanAstSyncBill.getSyncDate());
        loanAstSyncContract.setAccountDate(loanAstSyncBill.getAccountDate());

        return loanAstSyncContract;
    }

    /**
     * 组装还款计划
     *
     * @param wkLoanEntity
     * @param loanAstSyncBill
     * @return
     */
    private List<LoanAstSyncRepayPlan> buildExistRepayPlans(WkMfsLoanEntity wkLoanEntity,
                                                            LoanAstSyncBill loanAstSyncBill) {
        List<LoanAstSyncRepayPlan> loanAstSyncRepayPlanList = Lists.newArrayList();

        Date curStartDate = loanAstSyncBill.getStartDate();
        // 按期次排序
        Collections.sort(wkLoanEntity.getWkMfsRepayPlanDOS(), Comparator.comparing(WkMfsRepayPlanDO::getTermNo));

        // 非初始化的情况，只关注分期的最终余额即可，不需要再考虑实际的已还。五矿的已还数据已经不准。
        for (WkMfsRepayPlanDO wkMfsRepayPlanDO : wkLoanEntity.getWkMfsRepayPlanDOS()) {
            if (wkMfsRepayPlanDO.getTermNo() == 0) {
                continue;
            }
            LoanAstSyncRepayPlan loanAstSyncRepayPlan = buildRepayPlan(wkMfsRepayPlanDO, curStartDate, loanAstSyncBill);
            curStartDate = DateTimeUtil.addDays(loanAstSyncRepayPlan.getAstPeriod().getEndDate(), 1);
            loanAstSyncRepayPlanList.add(loanAstSyncRepayPlan);
        }

        List<WkMfsRepayInfoDO> repayInfoDOS = wkLoanEntity.getWkMfsRepayInfoDOS();
        Collections.sort(repayInfoDOS, Comparator.comparing(WkMfsRepayInfoDO::getCurDate));
        Collections.reverse(repayInfoDOS);

        Optional<WkMfsRepayInfoDO> latestPreRepay = repayInfoDOS.stream().filter(item -> item.getTermNo() == 0)
            .findFirst();

        for (LoanAstSyncRepayPlan loanAstSyncRepayPlan : loanAstSyncRepayPlanList) {
            if (loanAstSyncRepayPlan.getStatus() == LoanStatusEnum.CANCEL) {
                if (!latestPreRepay.isPresent()) {
                    LogUtil.error(LOGGER, "[{0}]存在提前还款分期，但是找不到提前还款流水", loanAstSyncBill);
                    throw new AbssqrBizException(MessageUtil.formatMsg("[{0}]存在提前还款分期，但是找不到提前还款流水", loanAstSyncBill),
                        CommonErrorCodeEnum.INVALID_PARAM);
                }

                WkMfsRepayInfoDO repayInfoDO = latestPreRepay.get();

                loanAstSyncRepayPlan.setStatus(LoanStatusEnum.CLEAR);
                // 结清日期可能存在不准确（多次提前还款，每次都提前结清部分期次）但是应该不存在实际数据
                loanAstSyncRepayPlan.getAstPeriod().setClearDate(DateTimeUtil.parseFromYMD(repayInfoDO.getCurDate()));
                loanAstSyncRepayPlan.getBal().setByFieldType(DebtMoneyFieldEnum.PRIN, new Money());
            }
        }

        return loanAstSyncRepayPlanList;
    }

    private LoanAstSyncRepayPlan buildRepayPlan(WkMfsRepayPlanDO wkMfsRepayPlanDO, Date curStartDate,
                                                LoanAstSyncBill loanAstSyncBill) {
        int termNo = wkMfsRepayPlanDO.getTermNo();
        //分期状态转换
        LoanStatusEnum status = wkFieldTranslator.transTermStatus(wkMfsRepayPlanDO.getTermStatus());
        //分期
        LoanAstSyncRepayPlan repayPlan = new LoanAstSyncRepayPlan();
        repayPlan.setTermNo(termNo);

        //资产周期组装
        repayPlan.getAstPeriod().setStartDate(curStartDate);

        repayPlan.getAstPeriod().setEndDate(DateTimeUtil.parseFromYMD(wkMfsRepayPlanDO.getStartDate()));
        repayPlan.getAstPeriod().setGraceEndDate(DateTimeUtil.parseFromYMD(wkMfsRepayPlanDO.getEndDate()));

        //宽限期内 招联为正常，需转为逾期
        if (status == LoanStatusEnum.NORMAL &&
            DateTimeUtil.calcDayDiff(loanAstSyncBill.getDayBeginAccountDate(), repayPlan.getAstPeriod().getEndDate())
                > 0) {
            status = LoanStatusEnum.OVERDUE;
        } else if (loanAstSyncBill.getStatus() == LoanStatusEnum.REVERSE) {
            status = LoanStatusEnum.REVERSE;
        }

        repayPlan.setStatus(status);

        repayPlan.getAstPeriod().setClearDate(DateTimeUtil.parseFromYMD(wkMfsRepayPlanDO.getClearDate()));
        repayPlan.getAstPeriod().setInDate(loanAstSyncBill.getStartDate());

        //设置本金余额
        repayPlan.setDueIntrAmt(new Money(wkMfsRepayPlanDO.getIntTotal()));
        Money prinBal = new Money(wkMfsRepayPlanDO.getPrinTotal().subtract(wkMfsRepayPlanDO.getPrinRepay()));
        repayPlan.getBal().setByFieldType(DebtMoneyFieldEnum.PRIN, prinBal);

        //设置利息余额
        if ((repayPlan.getStatus() == LoanStatusEnum.OVERDUE || repayPlan.getStatus() == LoanStatusEnum.NORMAL)) {
            // 不考虑结清，结清余额默认就是0了，不需要再处理
            if (DateTimeUtil
                .calcDayDiff(loanAstSyncBill.getDayBeginAccountDate(), repayPlan.getAstPeriod().getEndDate())
                > 0) {
                // 对于逾期的，直接从分期上取余额
                Money intrBal = new Money(wkMfsRepayPlanDO.getIntTotal().subtract(wkMfsRepayPlanDO.getIntRepay()));
                repayPlan.getBal().setByFieldType(DebtMoneyFieldEnum.INTR, intrBal);
                //本罚
                Money intrPenalBal = new Money(
                    wkMfsRepayPlanDO.getPnltIntTotal().subtract(wkMfsRepayPlanDO.getPnltIntRepay()));
                repayPlan.getBal()
                    .setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, intrPenalBal);
            } else if (repayPlan.getAstPeriod().isInTerm(loanAstSyncBill.getDayBeginAccountDate())) {
                // 对于当期的，从借据取余额
                Money intrBal = loanAstSyncBill.getBal().getByFieldType(DebtMoneyFieldEnum.INTR).clone();
                repayPlan.getBal().setByFieldType(DebtMoneyFieldEnum.INTR, intrBal);
                loanAstSyncBill.setIntrSet(true);
            } else if (DateTimeUtil.calcDayDiff(loanAstSyncBill.getDayBeginAccountDate(),
                repayPlan.getAstPeriod().getStartDate()) < 0) {
                if (!loanAstSyncBill.isIntrSet()) {
                    // 未来期的第一期：当期结清，利息归在第二期
                    Money intrBal = loanAstSyncBill.getBal().getByFieldType(DebtMoneyFieldEnum.INTR).clone();
                    repayPlan.getBal().setByFieldType(DebtMoneyFieldEnum.INTR, intrBal);
                    loanAstSyncBill.setIntrSet(true);
                }
            }
        }

        repayPlan.getPaid()
            .setByFieldType(DebtMoneyFieldEnum.PRIN, new Money(wkMfsRepayPlanDO.getPrinRepay()));
        repayPlan.getPaid()
            .setByFieldType(DebtMoneyFieldEnum.INTR, new Money(wkMfsRepayPlanDO.getIntRepay()));
        repayPlan.getPaid()
            .setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, new Money(wkMfsRepayPlanDO.getPnltIntRepay()));

        int ovdDays = (int)DateTimeUtil.calcDayDiff(loanAstSyncBill.getDayBeginAccountDate(),
            repayPlan.getAstPeriod().getEndDate());
        repayPlan.setOvdDays((ovdDays > 0) ? ovdDays : 0);
        repayPlan.setLoanNo(loanAstSyncBill.getLoanNo());
        repayPlan.setSyncDate(loanAstSyncBill.getSyncDate());
        repayPlan.setAccountDate(loanAstSyncBill.getAccountDate());

        return repayPlan;
    }

    /**
     * ======================================== 交易流水构造
     */
    /**
     * 放款冲正交易流水
     *
     * @param loanAstSyncBill
     * @param loanAstSyncRepayPlans
     * @return
     */
    private LoanAstSyncTrans buildTransForGrantReverse(LoanAstSyncBill loanAstSyncBill,
                                                       List<LoanAstSyncRepayPlan> loanAstSyncRepayPlans) {
        LoanAstSyncTrans loanAstSyncTrans = new LoanAstSyncTrans();
        loanAstSyncTrans.setLoanNo(loanAstSyncBill.getLoanNo());
        loanAstSyncTrans.setBizNo(loanAstSyncBill.getLoanNo());
        loanAstSyncTrans.setAccountDate(loanAstSyncBill.getAccountDate());
        loanAstSyncTrans.setTransTime(loanAstSyncBill.getAccountDate());
        loanAstSyncTrans.setEventCode(EventCodeEnum.GRANT_REVERSE);

        for (LoanAstSyncRepayPlan repayPlan : loanAstSyncRepayPlans) {
            LoanAstSyncRepayPlanTrans trans = new LoanAstSyncRepayPlanTrans();
            trans.setTermNo(repayPlan.getTermNo());
            trans.setOutBizNo(loanAstSyncTrans.getBizNo());
            trans.setTransTime(loanAstSyncTrans.getAccountDate());
            trans.setEventCode(loanAstSyncTrans.getEventCode());
            trans.setPrinAmt(repayPlan.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN));
            trans.setLoanNo(loanAstSyncBill.getLoanNo());
            trans.setSyncDate(loanAstSyncBill.getSyncDate());
            trans.setAccountDate(loanAstSyncTrans.getAccountDate());

            loanAstSyncTrans.getRepayPlanTransList().add(trans);
        }

        return loanAstSyncTrans;
    }

    /**
     * 提前还款交易流水
     *
     * @param wkMfsRepayInfoDO
     * @param loanAstSyncTrans
     * @param loanAstSyncRepayPlans
     * @param loan
     * @return
     */
    private LoanAstSyncTrans buildTransForPreRepay(WkMfsRepayInfoDO wkMfsRepayInfoDO,
                                                   LoanAstSyncTrans loanAstSyncTrans,
                                                   List<LoanAstSyncRepayPlan> loanAstSyncRepayPlans, Loan loan) {

        Map<Integer, LoanInstalDomain> loanInstalMap = loan.getLoanInstalMap();

        Money totalPrinAmt = new Money();
        Money totalIntrAmt = new Money();
        Money totalIntrFundAmt = new Money();
        Money totalPnyIntrAmt = new Money();
        Money totalPnyIntrFundAmt = new Money();

        Money totalPrinPaid = new Money(wkMfsRepayInfoDO.getPrinAmt());
        Money totalIntrPaid = new Money(wkMfsRepayInfoDO.getIntAmt());
        Money totalPnyIntrPaid = new Money(wkMfsRepayInfoDO.getPnltIntAmt());
        Money totalIntrFundPaid = new Money(wkMfsRepayInfoDO.getTrustIntAmtAcc());
        Money totalPnyIntrFundPaid = new Money(wkMfsRepayInfoDO.getTrustPnltIntAmtAcc());

        Map<Integer, LoanAstSyncRepayPlanTrans> repayPlanTrans = loanAstSyncTrans.getRepayPlanTransList().stream()
            .collect(Collectors
                .toMap(LoanAstSyncRepayPlanTrans::getTermNo,
                    item -> item));
        BigDecimal intrDecimal = wkMfsRepayInfoDO.getIntAmt();
        BigDecimal intrFundDecimal = wkMfsRepayInfoDO.getTrustIntAmtAcc();
        BigDecimal pnyIntrDecimal = wkMfsRepayInfoDO.getPnltIntAmt();
        BigDecimal pnyIntrFundDecimal = wkMfsRepayInfoDO.getTrustPnltIntAmtAcc();
        LoanAstSyncRepayPlanTrans curRepayPlanTrans = null;
        LoanAstSyncRepayPlanTrans lastOdueRepayPlanTrans = null;


        //确定提前还款费是否产生
        boolean repoGenOtherFee = wkMfsRepayInfoDO.getRepayViolateAmt() != null
            && wkMfsRepayInfoDO.getRepayViolateAmt().compareTo(BigDecimal.ZERO) > 0;

        // 正序遍历，把利息放在最早的分期
        for (int i = 0; i < loanAstSyncRepayPlans.size(); i++) {
            if (totalIntrPaid.isZero() && totalPnyIntrPaid.isZero() && totalPrinPaid.isZero()) {
                break;
            }

            LoanAstSyncRepayPlan repayPlan = loanAstSyncRepayPlans.get(i);
            LoanInstal instal = loanInstalMap.get(repayPlan.getTermNo());
            LoanAstSyncRepayPlanTrans repayTrans = repayPlanTrans.get(repayPlan.getTermNo());

            Money intrAmt = new Money();
            Money intrFundAmt = new Money();
            Money pnyIntrAmt = new Money();
            Money pnyIntrFundAmt = new Money();

            Money todayPaidPrin = new Money();
            Money todayPaidIntr = new Money();
            Money todayPaidPnyIntr = new Money();
            if (null != repayTrans) {
                todayPaidPrin = repayTrans.getPrinAmt().clone();
                todayPaidIntr = repayTrans.getIntrAmt().clone();
                todayPaidPnyIntr = repayTrans.getPrinPenalAmt().clone();
            }

            // 处理本金：本金处理直接拿上日本金-今日本金计算发生额，扣除在分期还款中已经处理的本金发生
            Money prePrin = instal.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN);
            Money aftPrin = repayPlan.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN);
            Money prinDiff = prePrin.subtract(aftPrin).subtract(todayPaidPrin);
            Money prinAmt = new Money();
            if (prinDiff.isGreaterThanZero()) {
                prinAmt = MoneyUtil.min(prinDiff, totalPrinPaid);
                VarChecker.checkArgument(!prinAmt.isNegtive(),
                    "还款流水[{0}]模型转换后，本金发生额为负[{1}]", wkMfsRepayInfoDO, prinAmt);
                totalPrinAmt = totalPrinAmt.add(prinAmt);
            }

            // 处理利息-只针对到期和逾期：利息处理默认分摊期次早的分期，将发生额分完为止
            if (instal.getAstPeriod().isInTerm(loanAstSyncTrans.getAccountDate())
                || instal.getStatus() == LoanStatusEnum.OVERDUE) {

                // 利息处理
                if (totalIntrPaid.isGreaterThanZero() && intrDecimal.compareTo(BigDecimal.ZERO) > 0) {
                    intrAmt = instal.getBal().getByFieldType(DebtMoneyFieldEnum.INTR).subtract(todayPaidIntr);
                    intrAmt = intrAmt.isGreaterThanZero() ? intrAmt : new Money();
                    intrAmt = MoneyUtil.min(intrAmt, totalIntrPaid);
                    totalIntrAmt = totalIntrAmt.add(intrAmt);

                    if ((totalIntrPaid.subtract(intrAmt)).isZero()) {
                        // 在这次摊销里摊完利息发生，将剩余的实收全被给当期
                        intrFundAmt = totalIntrFundPaid.clone();
                    } else {
                        // 按比例摊销实收
                        intrFundAmt = new Money(intrAmt.getAmount().multiply(intrFundDecimal).divide(intrDecimal,
                            AbssqrConstants.DECIMAL_SCALE, AbssqrConstants.DECIMAL_ROUNDING_MODE));
                        intrFundAmt = MoneyUtil.min(intrFundAmt, totalIntrFundPaid);
                    }
                    totalIntrFundAmt = totalIntrFundAmt.add(intrFundAmt);
                    totalIntrFundPaid = totalIntrFundPaid.subtract(intrFundAmt);
                }

                // 罚息处理
                if (totalPnyIntrPaid.isGreaterThanZero() && (pnyIntrDecimal.compareTo(BigDecimal.ZERO) > 0)) {
                    pnyIntrAmt = instal.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY)
                        .subtract(todayPaidPnyIntr);
                    pnyIntrAmt = pnyIntrAmt.isGreaterThanZero() ? pnyIntrAmt : new Money();
                    pnyIntrAmt = MoneyUtil.min(pnyIntrAmt, totalPnyIntrPaid);
                    totalPnyIntrAmt = totalPnyIntrAmt.add(pnyIntrAmt);

                    if ((totalPnyIntrPaid.subtract(pnyIntrAmt)).isZero()) {
                        // 在这次摊销里摊完罚息息发生，将剩余的实收全被给当期
                        pnyIntrFundAmt = totalPnyIntrFundPaid.clone();
                    } else {
                        // 按比例摊销实收  分期本罚余额 * 信托方分润罚息 / 罚息发生额
                        pnyIntrFundAmt = new Money(
                            pnyIntrAmt.getAmount().multiply(pnyIntrFundDecimal).divide(pnyIntrDecimal,
                                AbssqrConstants.DECIMAL_SCALE, AbssqrConstants.DECIMAL_ROUNDING_MODE));
                        // 按比例摊销实收 与 信托方分润罚息 取小
                        pnyIntrFundAmt = MoneyUtil.min(pnyIntrFundAmt, totalPnyIntrFundPaid);
                    }
                    // 汇总校验
                    totalPnyIntrFundAmt = totalPnyIntrFundAmt.add(pnyIntrFundAmt);
                    totalPnyIntrFundPaid = totalPnyIntrFundPaid.subtract(pnyIntrFundAmt);
                }
            }
            // 只要没分配完，就生成流水，避免类似按期付息到期还本的资产，无法结清本金为0的期次
            if (totalIntrPaid.isGreaterThanZero() || totalPnyIntrPaid.isGreaterThanZero() || totalPrinPaid
                .isGreaterThanZero()) {
                if (null == repayTrans) {
                    // 生成还款交易流水
                    repayTrans = buildRepayPlanTrans(repayPlan, loanAstSyncTrans, EventCodeEnum.PRE_REPAY,
                        new DebtMoney(), DateTimeUtil.parseInTolerance(wkMfsRepayInfoDO.getTranTime()));
                    repayTrans.setOutBizNo(wkMfsRepayInfoDO.getSeqNo());
                    repayPlanTrans.put(instal.getTermNo(), repayTrans);
                }

                if (instal.getAstPeriod().isInTerm(loanAstSyncTrans.getAccountDate())) {
                    curRepayPlanTrans = repayTrans;
                } else if (instal.getStatus() == LoanStatusEnum.OVERDUE) {
                    lastOdueRepayPlanTrans = repayTrans;
                }
                repayTrans.getPrinAmt().addTo(prinAmt);
                repayTrans.getPrinFundAmt().addTo(prinAmt);
                repayTrans.getIntrAmt().addTo(intrAmt);
                repayTrans.getIntrFundAmt().addTo(intrFundAmt);
                repayTrans.getPrinPenalAmt().addTo(pnyIntrAmt);
                repayTrans.getPrinPenalFundAmt().addTo(pnyIntrFundAmt);
                //计算是否会产生提前还款费
                if(checkIsPreRepay(repayPlan, instal) && repoGenOtherFee){
                    repayTrans.setRepoGenOtherFeePrinAmt(repayTrans.getRepoGenOtherFeePrinAmt().add(prinAmt));
                }
            }

            totalPrinPaid = totalPrinPaid.subtract(prinAmt);
            totalIntrPaid = totalIntrPaid.subtract(intrAmt);
            totalPnyIntrPaid = totalPnyIntrPaid.subtract(pnyIntrAmt);

        }

        if (curRepayPlanTrans != null) {
            // 如果利息没分摊完，则全部给当前期
            curRepayPlanTrans.getIntrAmt().addTo(totalIntrPaid);
            curRepayPlanTrans.getIntrFundAmt().addTo(totalIntrFundPaid);
            totalIntrAmt.addTo(totalIntrPaid);
            totalIntrFundAmt.addTo(totalIntrFundPaid);
        }

        if (lastOdueRepayPlanTrans != null) {
            // 因罚息先摊早期逾期的分期，如果罚息没分摊完，则说明早期的罚息余额已经分摊完毕。
            // 默认给最晚逾期的分期。若余额分摊出现异常，待余额校验报错。
            lastOdueRepayPlanTrans.getPrinPenalAmt().addTo(totalPnyIntrPaid);
            lastOdueRepayPlanTrans.getPrinPenalFundAmt().addTo(totalPnyIntrFundPaid);
            totalPnyIntrAmt.addTo(totalPnyIntrPaid);
            totalPnyIntrFundAmt.addTo(totalPnyIntrFundPaid);
        }
        VarChecker.checkArgument(totalPrinPaid.isZero(),
            "提前还款[{0}]组装, 分摊后剩余本金发生额[{1}]不为0", wkMfsRepayInfoDO.getLoanId(), totalPrinPaid);
        VarChecker.checkEquals(totalPrinAmt, new Money(wkMfsRepayInfoDO.getPrinAmt()),
            "提前还款[{0}]组装, 组装后流水本金发生额汇总[{1}]和实际发生[{2}]不一致", wkMfsRepayInfoDO.getLoanId(), totalPrinAmt,
            new Money(wkMfsRepayInfoDO.getPrinAmt()));
        VarChecker.checkEquals(totalIntrAmt, new Money(wkMfsRepayInfoDO.getIntAmt()),
            "提前还款[{0}]组装, 组装后流水利息发生额汇总[{1}]和实际发生[{2}]不一致", wkMfsRepayInfoDO.getLoanId(), totalIntrAmt,
            new Money(wkMfsRepayInfoDO.getIntAmt()));
        VarChecker.checkEquals(totalIntrFundAmt, new Money(wkMfsRepayInfoDO.getTrustIntAmtAcc()),
            "提前还款[{0}]组装, 组装后流水利息实收发生额汇总[{1}]和实际发生[{2}]不一致", wkMfsRepayInfoDO.getLoanId(), totalIntrFundAmt,
            new Money(wkMfsRepayInfoDO.getTrustIntAmtAcc()));
        VarChecker.checkEquals(totalPnyIntrAmt, new Money(wkMfsRepayInfoDO.getPnltIntAmt()),
            "提前还款[{0}]组装, 组装后流水罚息发生额汇总[{1}]和实际发生[{2}]不一致", wkMfsRepayInfoDO.getLoanId(), totalPnyIntrAmt,
            new Money(wkMfsRepayInfoDO.getPnltIntAmt()));
        VarChecker.checkEquals(totalPnyIntrFundAmt, new Money(wkMfsRepayInfoDO.getTrustPnltIntAmtAcc()),
            "提前还款[{0}]组装, 组装后流水罚息实收发生额汇总[{1}]和实际发生[{2}]不一致", wkMfsRepayInfoDO.getLoanId(), totalPnyIntrFundAmt,
            new Money(wkMfsRepayInfoDO.getTrustPnltIntAmtAcc()));
        return loanAstSyncTrans;
    }

    /**
     * 组织挂费交易
     *
     * @param wkMfsRepayInfoDO
     * @param repoSyncTrans
     * @param baseTsfRes
     */
    private LoanAstSyncTrans buildHangFeeTransForPreRepay(WkMfsRepayInfoDO wkMfsRepayInfoDO,
                                                          LoanAstSyncTrans repoSyncTrans,
                                                          BaseTsfRes baseTsfRes) {
        LoanAstSyncTrans hangFeeSyncTrans = new LoanAstSyncTrans();

        LoanAstSyncBill loanAstSyncBill = baseTsfRes.getLoanAstSyncBill();
        // 还款资产交易流水
        hangFeeSyncTrans.setLoanNo(loanAstSyncBill.getLoanNo());
        hangFeeSyncTrans.setBizNo(MessageUtil.formatMsg("{0}-{1}-hangFee", loanAstSyncBill.getLoanNo(),
            DateTimeUtil.formatYMD(DateTimeUtil.getStandardDate(loanAstSyncBill.getAccountDate()))));
        hangFeeSyncTrans.setAccountDate(loanAstSyncBill.getAccountDate());
        hangFeeSyncTrans.setTransTime(DateTimeUtil.parseInTolerance(wkMfsRepayInfoDO.getTranDate()));
        hangFeeSyncTrans.setEventCode(EventCodeEnum.HANG_OTHER_FEE);

        //1. 总提前还款服务费
        Money totalFeeAmt = new Money(wkMfsRepayInfoDO.getRepayViolateAmt() == null
                                              ? BigDecimal.ZERO : wkMfsRepayInfoDO.getRepayViolateAmt());
        //    机构实收提前还款费
        Money totalFundFeeAmt = new Money(wkMfsRepayInfoDO.getTrustRepayViolateAmtAcc() == null
                                                  ? BigDecimal.ZERO : wkMfsRepayInfoDO.getTrustRepayViolateAmtAcc());
        if(totalFeeAmt.isZero()){
            return hangFeeSyncTrans;
        }

        Money preRepayPrinForFeeAmt = new Money();
        Map<Integer, BigDecimal> termMoneyMap = Maps.newHashMap();
        //2 组装挂费
        // 2.1 过滤出需要计算提前还款费 交易（期次号+ LoanAstSyncRepayPlanTrans）
        //                                      还款交易指令
        Map<Integer, LoanAstSyncRepayPlanTrans> repoPlanTransMap = Maps.newHashMap();
        for (LoanAstSyncRepayPlanTrans repoRepayPlanTrans : repoSyncTrans.getRepayPlanTransList()) {
            //存在提前还引起挂费本金 > 0
            if (repoRepayPlanTrans.getRepoGenOtherFeePrinAmt().isGreaterThanZero()) {
                //每一期还本金
                Money prinAmt = repoRepayPlanTrans.getRepoGenOtherFeePrinAmt();
                //根据替换还款校验对应期次是否符合提前还款处理
                preRepayPrinForFeeAmt.addTo(prinAmt);
                termMoneyMap.put(repoRepayPlanTrans.getTermNo(), prinAmt.getAmount());
                repoPlanTransMap.put(repoRepayPlanTrans.getTermNo(), repoRepayPlanTrans);
            }
        }
        //计算入账比例
        Map<Integer, BigDecimal> ratios = new HashMap<>();
        DecimalCalcUtil.calcSplitFactor(ratios, termMoneyMap, false);

        Map<Integer, Money> termFeeMap = calcPreRepayFee(totalFeeAmt,ratios,
            wkMfsRepayInfoDO.getLoanId());
        Map<Integer, LoanAstSyncRepayPlan> repayPlanMap = baseTsfRes.getRepayPlans().stream().collect(
            Collectors.toMap(LoanAstSyncRepayPlan::getTermNo, item -> item));
        //3 组织挂费交易
        //   先设置最大期次
        int termFundFeeZeroNo = termFeeMap.keySet().stream().min(Integer::compareTo).get();
        Money remainFundFeeAmt = totalFundFeeAmt.clone();
        for (Integer termNo : termFeeMap.keySet()) {
            Money termFeeAmt = termFeeMap.get(termNo);
            // 生成还款交易流水
            LoanAstSyncRepayPlanTrans repayFeeTrans = buildRepayPlanTrans(repayPlanMap.get(termNo),
                hangFeeSyncTrans, EventCodeEnum.HANG_OTHER_FEE,
                new DebtMoney(), DateTimeUtil.parseInTolerance(wkMfsRepayInfoDO.getTranTime()));

            repayFeeTrans.setOutBizNo(hangFeeSyncTrans.getBizNo());
            //机构资产挂费时需要根据这里获取本金
            repayFeeTrans.setPrinAmt(new Money(termMoneyMap.get(termNo)));
            repayFeeTrans.setOtherFeeAmt(termFeeAmt);

            //4 回写还款指令 补充还款处理提前还款费部分
            LoanAstSyncRepayPlanTrans repoRepayPlanTrans = repoPlanTransMap.get(termNo);
            repoRepayPlanTrans.setOtherFeeAmt(termFeeAmt);
            //  4.1 根据比例计算机构实收费
            BigDecimal fundRatio = CalcUtil.divide(totalFundFeeAmt.getAmount(), totalFeeAmt.getAmount());
            Money termFundFeeAmt = termFeeAmt.multiply(fundRatio);
            if(termFundFeeAmt.isGreaterThanZero()) {
                termFundFeeAmt = MoneyUtil.min(termFundFeeAmt, remainFundFeeAmt);
                remainFundFeeAmt.subtractFrom(termFundFeeAmt);
                repoRepayPlanTrans.setOtherFeeFundAmt(termFundFeeAmt);
            }else{
                //出现多期没有分到，暂定随便给一期
                termFundFeeZeroNo=termNo;
            }
        }
        //         4.2 机构实收尾差处理
        if(remainFundFeeAmt.isGreaterThanZero()){
            LoanAstSyncRepayPlanTrans repoRepayPlanTrans = repoPlanTransMap.get(termFundFeeZeroNo);
            repoRepayPlanTrans.setOtherFeeFundAmt(repoRepayPlanTrans.getOtherFeeFundAmt().add(remainFundFeeAmt));
            remainFundFeeAmt.subtractFrom(remainFundFeeAmt);
        }

        //5 校验数据
        Money hangFeeDetAmt = new Money();
        for (LoanAstSyncRepayPlanTrans loanAstSyncRepayPlanTrans : hangFeeSyncTrans.getRepayPlanTransList()) {
            hangFeeDetAmt.addTo(loanAstSyncRepayPlanTrans.getOtherFeeAmt());
        }
        VarChecker.checkArgument(hangFeeDetAmt.equals(totalFeeAmt),
            "提前还款服务费[{0}]组装, 组装后实还发生额汇总用户实还分摊后金额不一致,待分摊总额{1},实际分摊金额{2}", wkMfsRepayInfoDO.getLoanId(),
            totalFeeAmt, hangFeeDetAmt);
        Money repoFeeDetAmt = new Money();
        Money repoFundFeeDetAmt = new Money();
        repoPlanTransMap.forEach((termNo, loanAstSyncRepayPlanTrans) -> {
            repoFeeDetAmt.addTo(loanAstSyncRepayPlanTrans.getOtherFeeAmt());
            repoFundFeeDetAmt.addTo(loanAstSyncRepayPlanTrans.getOtherFeeFundAmt());
        });
        VarChecker.checkArgument(repoFeeDetAmt.equals(totalFeeAmt),
            "提前还款服务费[{0}]组装, 组装后实还分摊后金额不一致,待分摊总额{1},实际分摊金额{2}", wkMfsRepayInfoDO.getLoanId(), totalFeeAmt,
            hangFeeDetAmt);
        VarChecker.checkArgument(repoFundFeeDetAmt.equals(totalFundFeeAmt), "提前还款服务费[{0}]组装, 组装后实收分摊后金额不一致,待分摊总额{1},实际分摊金额{2}",
            wkMfsRepayInfoDO.getLoanId(), repoFundFeeDetAmt, totalFundFeeAmt);
        baseTsfRes.getLoanAstSyncTransList().add(hangFeeSyncTrans);
        return hangFeeSyncTrans;
    }

    private Map<Integer, Money> calcPreRepayFee(Money totalFeeAmt, Map<Integer, BigDecimal> ratios,
                                                String loanId) {
        Money remainFeeAmt = totalFeeAmt.clone();
        Map<Integer, Money> termFeeMap = Maps.newHashMap();
        int maxTermNo = ratios.keySet().stream().max(Integer::compareTo).get();
        ratios.forEach((termNo, ratio) -> {
            //按比例计算提前还款服务费挂费金额
            Money termFee = CalcUtil.calcMoneyRatio(totalFeeAmt, ratio);
            termFee = MoneyUtil.min(termFee, remainFeeAmt);
            if(termFee.isGreaterThanZero()) {
                remainFeeAmt.subtractFrom(termFee);
                //记录费用
                termFeeMap.put(termNo, termFee);
            }
            if (remainFeeAmt.isZero()) {
                return;
            }
        });
        // 2.2 费用尾差处理， 给最大期次（便于校验）
        if (remainFeeAmt.isGreaterThanZero()) {
            Money termFeeAmt = termFeeMap.get(maxTermNo);
            if (termFeeAmt == null) {
                termFeeAmt = new Money();
                termFeeMap.put(maxTermNo, termFeeAmt);
            }
            termFeeAmt.addTo(remainFeeAmt);
            remainFeeAmt.subtractFrom(remainFeeAmt);
        }
        VarChecker.checkArgument(remainFeeAmt.isZero(), "提前还款服务费没有分摊完成{0}-剩余：{1}", loanId, remainFeeAmt);

        Money repayAllFeeAmt = new Money();
        for (Money amt : termFeeMap.values()) {
            repayAllFeeAmt.addTo(amt);
        }
        VarChecker.checkArgument(repayAllFeeAmt.equals(totalFeeAmt), "提前还款服务费分摊后金额不一致{0},待分摊总额{1},实际分摊金额{2}", loanId,
            totalFeeAmt, repayAllFeeAmt);
        return termFeeMap;
    }

    private boolean checkIsPreRepay(LoanAstSyncRepayPlan loanAstSyncRepayPlan, LoanInstal instal) {
        //如果分期资产是逾期则不是提前还款
        if (instal!=null&&LoanStatusEnum.OVERDUE == instal.getStatus()) {
            return false;
        }
        //还款日大于期次的提前还款返回true
        if (DateTimeUtil.calcDayDiff(loanAstSyncRepayPlan.getAccountDate(),
                loanAstSyncRepayPlan.getAstPeriod().getEndDate()) < 0) {
            return true;
        }
        return false;
    }

    /**
     * 还款补偿
     *
     * @param wkMfsRepayInfoDOs
     * @param loanAstSyncBill
     * @return
     */
    private LoanAstSyncTrans buildTransForRepayComponsate(List<WkMfsRepayInfoDO> wkMfsRepayInfoDOs,
                                                          LoanAstSyncBill loanAstSyncBill) {
        LoanAstSyncTrans loanAstSyncTrans = new LoanAstSyncTrans();
        loanAstSyncTrans.setLoanNo(loanAstSyncBill.getLoanNo());
        loanAstSyncTrans
            .setBizNo(loanAstSyncBill.getLoanNo() + "_" + DateTimeUtil.formatYMD(loanAstSyncBill.getAccountDate()));
        loanAstSyncTrans.setAccountDate(loanAstSyncBill.getAccountDate());
        loanAstSyncTrans.setTransTime(loanAstSyncBill.getAccountDate());
        loanAstSyncTrans.setEventCode(EventCodeEnum.REPAY_COMPONSATE);

        // 倒序遍历，为了把利息发生额放在最早的那期还款计划
        for (WkMfsRepayInfoDO wkMfsRepayInfoDO : wkMfsRepayInfoDOs) {

            LoanAstSyncRepayPlanTrans loanAstSyncRepayPlanTrans = new LoanAstSyncRepayPlanTrans();
            loanAstSyncRepayPlanTrans.setTermNo(wkMfsRepayInfoDO.getTermNo());
            loanAstSyncRepayPlanTrans.setOutBizNo(loanAstSyncTrans.getBizNo());
            loanAstSyncRepayPlanTrans.setTransTime(loanAstSyncTrans.getTransTime());
            loanAstSyncRepayPlanTrans.setEventCode(loanAstSyncTrans.getEventCode());
            loanAstSyncRepayPlanTrans.setPrinAmt(new Money(wkMfsRepayInfoDO.getPrinAmt()));
            loanAstSyncRepayPlanTrans.setPrinFundAmt(new Money(wkMfsRepayInfoDO.getPrinAmt()));
            loanAstSyncRepayPlanTrans.setIntrAmt(new Money(wkMfsRepayInfoDO.getIntAmt()));
            loanAstSyncRepayPlanTrans.setIntrFundAmt(new Money(wkMfsRepayInfoDO.getTrustIntAmtAcc()));
            loanAstSyncRepayPlanTrans.setPrinPenalAmt(new Money(wkMfsRepayInfoDO.getPnltIntAmt()));
            loanAstSyncRepayPlanTrans.setPrinPenalFundAmt(new Money(wkMfsRepayInfoDO.getTrustPnltIntAmtAcc()));
            //loanAstSyncRepayPlanTrans.setOtherFeeAmt(new Money(wkMfsRepayInfoDO.getRepayViolateAmt()));
            //loanAstSyncRepayPlanTrans.setOtherFeeFundAmt(new Money(wkMfsRepayInfoDO.getTrustRepayViolateAmtAcc()));
            loanAstSyncRepayPlanTrans.setLoanNo(loanAstSyncBill.getLoanNo());
            loanAstSyncRepayPlanTrans.setSyncDate(loanAstSyncBill.getSyncDate());
            loanAstSyncRepayPlanTrans.setAccountDate(loanAstSyncTrans.getAccountDate());
            boolean repoGenOtherFee = wkMfsRepayInfoDO.getRepayViolateAmt() != null
                && wkMfsRepayInfoDO.getRepayViolateAmt().compareTo(BigDecimal.ZERO) > 0;
            if(repoGenOtherFee) {
                loanAstSyncRepayPlanTrans.setRepoGenOtherFeePrinAmt(loanAstSyncRepayPlanTrans.getPrinAmt());
            }
            loanAstSyncTrans.getRepayPlanTransList().add(loanAstSyncRepayPlanTrans);
        }

        return loanAstSyncTrans;
    }

    /**
     * 到期还款交易流水
     *
     * @param wkMfsRepayInfoDO
     * @param loanAstSyncTrans
     * @param loanAstSyncRepayPlans
     * @return
     */
    private LoanAstSyncTrans buildTransForTermRepay(WkMfsRepayInfoDO wkMfsRepayInfoDO,
                                                    LoanAstSyncTrans loanAstSyncTrans,
                                                    List<LoanAstSyncRepayPlan> loanAstSyncRepayPlans) {
        // 取出对应期次还款计划
        Optional<LoanAstSyncRepayPlan> curTermRepayPlanOpt = loanAstSyncRepayPlans.stream()
            .filter(item -> item.getTermNo()
                .equals(wkMfsRepayInfoDO
                    .getTermNo()))
            .findFirst();
        VarChecker.checkArgument(curTermRepayPlanOpt.isPresent(), "生成还款流水失败，无法找到对应分期的还款计划[{0}]", wkMfsRepayInfoDO);
        LoanAstSyncRepayPlan curTermRepayPlan = curTermRepayPlanOpt.get();
        Date curDate = DateTimeUtil.parseFromYMD(wkMfsRepayInfoDO.getCurDate());
        EventCodeEnum eventCode;
        // 判断到期还款还是逾期还款
        if (DateTimeUtil.calcDayDiff(curTermRepayPlan.getAstPeriod().getEndDate(), curDate) >= 0) {
            eventCode = EventCodeEnum.DUE_REPAY;
        } else {
            eventCode = EventCodeEnum.ODUE_REPAY;
        }

        LoanAstSyncRepayPlanTrans loanAstSyncRepayPlanTrans = buildRepayPlanTrans(curTermRepayPlan, loanAstSyncTrans,
            eventCode, new DebtMoney(), DateTimeUtil.parseInTolerance(wkMfsRepayInfoDO.getTranDate()));
        loanAstSyncRepayPlanTrans.setOutBizNo(wkMfsRepayInfoDO.getSeqNo());

        loanAstSyncRepayPlanTrans.setPrinAmt(new Money(wkMfsRepayInfoDO.getPrinAmt()));
        loanAstSyncRepayPlanTrans.setPrinFundAmt(new Money(wkMfsRepayInfoDO.getPrinAmt()));
        // 不考虑先结息后还款的情况，结息在还款中处理
        loanAstSyncRepayPlanTrans.setIntrAmt(new Money(wkMfsRepayInfoDO.getIntAmt()));
        loanAstSyncRepayPlanTrans.setIntrFundAmt(new Money(wkMfsRepayInfoDO.getTrustIntAmtAcc()));
        loanAstSyncRepayPlanTrans.setPrinPenalAmt(new Money(wkMfsRepayInfoDO.getPnltIntAmt()));
        loanAstSyncRepayPlanTrans.setPrinPenalFundAmt(new Money(wkMfsRepayInfoDO.getTrustPnltIntAmtAcc()));

        //确定提前还款费是否产生本金
        boolean repoGenOtherFee = wkMfsRepayInfoDO.getRepayViolateAmt() != null
            && wkMfsRepayInfoDO.getRepayViolateAmt().compareTo(BigDecimal.ZERO) > 0;
        if(repoGenOtherFee) {
            loanAstSyncRepayPlanTrans.setRepoGenOtherFeePrinAmt(loanAstSyncRepayPlanTrans.getPrinAmt());
        }

        return loanAstSyncTrans;
    }

    /**
     * 处理结息和减免补偿
     *
     * @param baseTsfRes
     * @param loanAstSyncRepayPlans
     * @param loan
     * @param syncTrans
     * @return
     */
    private void processSettleComponsate(BaseTsfRes baseTsfRes,
                                         List<LoanAstSyncRepayPlan> loanAstSyncRepayPlans,
                                         Loan loan, List<LoanAstSyncTrans> syncTrans) {

        LoanAstSyncBill loanAstSyncBill = baseTsfRes.getLoanAstSyncBill();
        Date accountDate = DateTimeUtil.getStandardDate(loanAstSyncBill.getAccountDate());

        LoanAstSyncTrans settleSyncTrans = new LoanAstSyncTrans();
        settleSyncTrans.setLoanNo(loanAstSyncBill.getLoanNo());
        settleSyncTrans.setBizNo(MessageUtil.formatMsg("{0}-{1}-settleComp", loanAstSyncBill.getLoanNo(),
            DateTimeUtil.formatYMD(accountDate)));

        settleSyncTrans.setAccountDate(accountDate);
        settleSyncTrans.setTransTime(accountDate);
        settleSyncTrans.setEventCode(EventCodeEnum.SETTLE_COMPONSATE);
        baseTsfRes.setSettleCompnsateSyncTrans(settleSyncTrans);

        // 减免息费
        LoanAstSyncTrans exmptSyncTrans = new LoanAstSyncTrans();
        exmptSyncTrans.setLoanNo(loanAstSyncBill.getLoanNo());
        exmptSyncTrans.setBizNo(MessageUtil.formatMsg("{0}-{1}-exmptComp", loanAstSyncBill.getLoanNo(),
            DateTimeUtil.formatYMD(accountDate)));

        exmptSyncTrans.setAccountDate(accountDate);
        exmptSyncTrans.setTransTime(accountDate);
        exmptSyncTrans.setEventCode(EventCodeEnum.EXMPT);
        baseTsfRes.setExemptCompnsateSyncTrans(exmptSyncTrans);

        Map<Integer, LoanInstal> loanInstalMap = loan.getLoanInstalList().stream().collect(
            Collectors.toMap(LoanInstal::getTermNo, item -> item));

        // 上日结清的，处理结息补偿
        for (LoanAstSyncRepayPlan repayPlan : loanAstSyncRepayPlans) {

            LoanInstal loanInstal = loanInstalMap.get(repayPlan.getTermNo());
            if (null == loanInstal) {
                throw new AbssqrBizException(
                    MessageUtil.formatMsg("无法找到[{0}-{1}]对应的期次", repayPlan.getLoanNo(), repayPlan.getTermNo()),
                    CommonErrorCodeEnum.NO_DATA);
            }

            // 当前期结息
            // 1, 获取上日还款的利息发生额
            Money intrRepoAmt = new Money();
            Money prinPenalRepoAmt = new Money();
            syncTrans.forEach(buildedSyncTrans -> {
                if (buildedSyncTrans.getEventCode().getSceneCode() == TransCodeEnum.REPO) {
                    buildedSyncTrans.getRepayPlanTransList().forEach(repoPlanTrans -> {
                        if (repoPlanTrans.getTermNo() == loanInstal.getTermNo()) {
                            intrRepoAmt.addTo(repoPlanTrans.getIntrAmt());
                            prinPenalRepoAmt.addTo(repoPlanTrans.getPrinPenalAmt());
                        }
                    });
                }
            });

            // 3. 上日余额
            DebtMoney lastBal = loanInstal.getBal();

            Money intrAmt = intrRepoAmt.subtract(lastBal.getByFieldType(DebtMoneyFieldEnum.INTR));
            Money prinPenalAmt = prinPenalRepoAmt.subtract(lastBal.getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
            if (intrAmt.isGreaterThanZero() || prinPenalAmt.isGreaterThanZero()) {
                // 补结息不考虑是否结清，因为已还大于上日余额就无法入账
                LogUtil.info(LOGGER, "还款计划[{0}]触发结息补偿，上日余额[{1}]，今日已还intr:[{2}], prinPenal:[{3}]",
                    repayPlan, lastBal, intrRepoAmt, prinPenalRepoAmt);
                DebtMoney compSettleAmt = new DebtMoney();
                compSettleAmt.setByFieldType(DebtMoneyFieldEnum.INTR, MoneyUtil.max(intrAmt, new Money()));
                compSettleAmt
                    .setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, MoneyUtil.max(prinPenalAmt, new Money()));
                buildRepayPlanTrans(repayPlan, settleSyncTrans, settleSyncTrans.getEventCode(), compSettleAmt,
                    settleSyncTrans.getTransTime());
            } else if (intrAmt.isNegtive() || prinPenalAmt.isNegtive()) {
                // 非结清，不处理结息补偿
                if (repayPlan.getStatus() != LoanStatusEnum.CLEAR) {
                    continue;
                }

                // 如果是已结清，本地资产必须为 NORMAL OVERDUE
                if (!loanInstal.getStatus().isActive()) {
                    continue;
                }

                intrAmt = intrAmt.multiply(-1);
                prinPenalAmt = prinPenalAmt.multiply(-1);
                if (intrAmt.compareTo(new Money("0.01")) == 0 && prinPenalAmt.compareTo(new Money("0.01")) == 0) {
                    LogUtil.info(LOGGER, "还款计划[{0}]触发减免补偿，上日余额[{1}]，今日已还intr:[{2}], prinPenal:[{3}]",
                        repayPlan, lastBal, intrRepoAmt, prinPenalRepoAmt);
                } else {
                    LogUtil.warn(LOGGER, "还款计划[{0}]触发减免补偿, 减免金额大于一分钱, 需要关注，上日余额[{1}]，今日已还intr:[{2}], prinPenal:[{3}]",
                        repayPlan, lastBal, intrRepoAmt, prinPenalRepoAmt);
                }
                DebtMoney compExmptAmt = new DebtMoney();
                compExmptAmt.setByFieldType(DebtMoneyFieldEnum.INTR, MoneyUtil.max(intrAmt, new Money()));
                compExmptAmt
                    .setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, MoneyUtil.max(prinPenalAmt, new Money()));
                buildRepayPlanTrans(repayPlan, exmptSyncTrans, exmptSyncTrans.getEventCode(), compExmptAmt,
                    exmptSyncTrans.getTransTime());
            }
        }
    }

    /**
     * 处理到期和逾期结息
     *
     * @param baseTsfRes
     * @param loan
     * @return
     */
    private LoanAstSyncTrans processDueSettleAndOvd(BaseTsfRes baseTsfRes, Loan loan) {

        LoanAstSyncBill loanAstSyncBill = baseTsfRes.getLoanAstSyncBill();
        List<LoanAstSyncRepayPlan> loanAstSyncRepayPlans = baseTsfRes.getRepayPlans();
        List<LoanAstSyncTrans> syncTrans = baseTsfRes.getLoanAstSyncTransList();
        LoanAstSyncTrans exemptTrans = baseTsfRes.getExemptCompnsateSyncTrans();
        LoanAstSyncTrans settleCompTrans = baseTsfRes.getSettleCompnsateSyncTrans();

        Date settleAccountDate = DateTimeUtil.getStandardDate(loanAstSyncBill.getDayBeginAccountDate());

        LoanAstSyncTrans loanAstSyncTrans = new LoanAstSyncTrans();
        loanAstSyncTrans.setLoanNo(loanAstSyncBill.getLoanNo());
        loanAstSyncTrans.setBizNo(MessageUtil.formatMsg("{0}-{1}-{2}", loanAstSyncBill.getLoanNo(),
            DateTimeUtil.formatYMD(settleAccountDate), EventCodeEnum.SETTLE));

        loanAstSyncTrans.setAccountDate(settleAccountDate);
        loanAstSyncTrans.setTransTime(settleAccountDate);
        loanAstSyncTrans.setEventCode(EventCodeEnum.SETTLE);

        Map<Integer, LoanInstal> loanInstalMap = loan.getLoanInstalList().stream().collect(
            Collectors.toMap(LoanInstal::getTermNo, item -> item));

        Map<Integer, LoanAstSyncRepayPlanTrans> exemptRepayPlanTrans = exemptTrans.getRepayPlanTransList().stream()
            .collect(Collectors
                .toMap(LoanAstSyncRepayPlanTrans::getTermNo,
                    t -> t));
        Map<Integer, LoanAstSyncRepayPlanTrans> settleCompRepayPlanTrans = settleCompTrans.getRepayPlanTransList()
            .stream()
            .collect(Collectors
                .toMap(LoanAstSyncRepayPlanTrans::getTermNo,
                    t -> t));
        // 提取到期或者逾期且未结清的分期
        for (LoanAstSyncRepayPlan repayPlan : loanAstSyncRepayPlans) {
            // 非正常，逾期，结清，可能为已冲正，不处理结息
            if (!repayPlan.getStatus().isActive()) {
                continue;
            }
            // 获取资产分期
            LoanInstal loanInstal = loanInstalMap.get(repayPlan.getTermNo());
            if (null == loanInstal) {
                throw new AbssqrBizException(
                    MessageUtil.formatMsg("无法找到[{0}-{1}]对应的期次", repayPlan.getLoanNo(), repayPlan.getTermNo()),
                    CommonErrorCodeEnum.NO_DATA);
            }

            // 当前期结息
            // 1, 获取上日还款发生额
            Money intrRepoAmt = new Money();
            Money pnyIntrRepoAmt = new Money();
            syncTrans.forEach(buildedSyncTrans -> {
                // 判断是还款回收
                if (buildedSyncTrans.getEventCode().getSceneCode() == TransCodeEnum.REPO) {
                    // 获取还款交易流水
                    buildedSyncTrans.getRepayPlanTransList().forEach(repoPlanTrans -> {
                        if (repoPlanTrans.getTermNo() == loanInstal.getTermNo()) {
                            // 累计利息还款
                            intrRepoAmt.addTo(repoPlanTrans.getIntrAmt());
                            pnyIntrRepoAmt.addTo(repoPlanTrans.getPrinPenalAmt());
                        }
                    });
                }
            });

            Money settleIntrCompAmt = new Money();
            Money settlePnyCompAmt = new Money();
            if (settleCompRepayPlanTrans.containsKey(repayPlan.getTermNo())) {
                settleIntrCompAmt = settleCompRepayPlanTrans.get(repayPlan.getTermNo()).getIntrAmt();
                settlePnyCompAmt = settleCompRepayPlanTrans.get(repayPlan.getTermNo()).getPrinPenalAmt();
            }

            // 2. 获取利息余额
            Money curBal = repayPlan.getBal().getByFieldType(DebtMoneyFieldEnum.INTR);
            // 3. 上日利息余额
            Money lastBal = loanInstal.getBal().getByFieldType(DebtMoneyFieldEnum.INTR);
            // 计算出利息差额
            Money settleAmt = intrRepoAmt.add(curBal).subtract(settleIntrCompAmt).subtract(lastBal);

            // 判断逾期结息，考虑到跨天未结正常利息的场景，所以不能用else和上面结正常利息互斥
            long dayDiff = DateTimeUtil.calcDayDiff(settleAccountDate, repayPlan.getAstPeriod().getEndDate());
            Money settlePnyAmt = new Money();
            if (dayDiff > 0) {
                // 获取罚息余额
                Money curPnyBal = repayPlan.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY);
                // 获取上日罚息余额
                Money lastPnyBal = loanInstal.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY);
                // 计算罚息差额
                settlePnyAmt = pnyIntrRepoAmt.add(curPnyBal).subtract(settlePnyCompAmt).subtract(lastPnyBal);
            }

            // 如果逾期天数小于等于0，不判断结息金额是否为0，保证每天能有结息，从而触发备份, 否则，只在结息金额大于0的时候才触发
            LoanAstSyncRepayPlanTrans repayPlanTrans = buildRepayPlanTrans(repayPlan, loanAstSyncTrans,
                loanAstSyncTrans.getEventCode(), new DebtMoney(), loanAstSyncTrans.getTransTime());
            repayPlanTrans.setIntrAmt(MoneyUtil.max(settleAmt, new Money()));
            repayPlanTrans.setPrinPenalAmt(MoneyUtil.max(settlePnyAmt, new Money()));

            if (settleAmt.isNegtive() || settlePnyAmt.isNegtive()) {
                settleAmt = settleAmt.multiply(-1);
                settlePnyAmt = settlePnyAmt.multiply(-1);

                LoanAstSyncRepayPlanTrans exemptPlanTrans = exemptRepayPlanTrans.get(repayPlan.getTermNo());
                if (exemptPlanTrans == null) {
                    exemptPlanTrans = buildRepayPlanTrans(repayPlan, exemptTrans, exemptTrans.getEventCode(),
                        new DebtMoney(), exemptTrans.getTransTime());
                }

                exemptPlanTrans.setIntrAmt(MoneyUtil.max(settleAmt, new Money()));
                exemptPlanTrans.setPrinPenalAmt(MoneyUtil.max(settlePnyAmt, new Money()));
            }
        }

        if (CollectionUtils.isEmpty(loanAstSyncTrans.getRepayPlanTransList())) {
            return null;
        } else {
            return loanAstSyncTrans;
        }
    }

    /**
     * 处理分期本金调整
     *
     * @param baseTsfRes
     */
    private void processAdjTerm(BaseTsfRes baseTsfRes, Loan loan, LoanAstSyncTrans repaySyncTrans) {

        LoanAstSyncBill loanAstSyncBill = baseTsfRes.getLoanAstSyncBill();
        List<LoanAstSyncRepayPlan> loanAstSyncRepayPlans = baseTsfRes.getRepayPlans();
        Map<Integer, LoanInstalDomain> loanInstalMap = loan.getLoanInstalMap();

        LoanAstSyncTrans adjTermSyncTrans = new LoanAstSyncTrans();
        adjTermSyncTrans.setLoanNo(loanAstSyncBill.getLoanNo());
        adjTermSyncTrans.setBizNo(MessageUtil.formatMsg("{0}-{1}-{2}", loanAstSyncBill.getLoanNo(),
            DateTimeUtil.formatYMD(loanAstSyncBill.getAccountDate()), TransCodeEnum.ADJ_TERM.getCode()));
        adjTermSyncTrans.setAccountDate(loanAstSyncBill.getAccountDate());
        adjTermSyncTrans.setTransTime(loanAstSyncBill.getAccountDate());
        adjTermSyncTrans.setEventCode(EventCodeEnum.ADJ_TERM_IN);

        Map<Integer, LoanAstSyncRepayPlanTrans> repayPlanTrans = repaySyncTrans.getRepayPlanTransList().stream()
            .collect(Collectors
                .toMap(LoanAstSyncRepayPlanTrans::getTermNo,
                    item -> item));

        // 正序遍历，把利息放在最早的分期
        for (int i = 0; i < loanAstSyncRepayPlans.size(); i++) {

            LoanAstSyncRepayPlan repayPlan = loanAstSyncRepayPlans.get(i);
            LoanInstal instal = loanInstalMap.get(repayPlan.getTermNo());
            LoanAstSyncRepayPlanTrans repayTrans = repayPlanTrans.get(repayPlan.getTermNo());

            Money todayPaidPrin = new Money();
            if (null != repayTrans) {
                todayPaidPrin = repayTrans.getPrinAmt().clone();
            }

            // 处理本金：本金处理直接拿上日本金-今日本金计算发生额，扣除在分期还款中已经处理的本金发生
            Money prePrin = instal.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN);
            Money aftPrin = repayPlan.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN);
            Money prinDiff = prePrin.subtract(aftPrin).subtract(todayPaidPrin);

            if (prinDiff.isNegtive()) {
                // 分期本金变多，分期调入
                LoanAstSyncRepayPlanTrans abjRepayTrans = buildRepayPlanTrans(repayPlan, adjTermSyncTrans,
                    EventCodeEnum.ADJ_TERM_IN, new DebtMoney(), adjTermSyncTrans.getTransTime());
                abjRepayTrans.setPrinAmt(prinDiff.multiply(-1));
            } else if (prinDiff.isGreaterThanZero()) {
                LoanAstSyncRepayPlanTrans abjrepayTrans = buildRepayPlanTrans(repayPlan, adjTermSyncTrans,
                    EventCodeEnum.ADJ_TERM_OUT, new DebtMoney(), adjTermSyncTrans.getTransTime());
                abjrepayTrans.setPrinAmt(prinDiff);
            }
        }

        if (CollectionUtils.isNotEmpty(adjTermSyncTrans.getRepayPlanTransList())) {
            baseTsfRes.getLoanAstSyncTransList().add(adjTermSyncTrans);
        }
    }

    /**
     * 处理核销
     *
     * @param baseTsfRes
     * @param loan
     * @param wkLoanEntity
     */
    private void processWtf(BaseTsfRes baseTsfRes, Loan loan, WkMfsLoanEntity wkLoanEntity) {
        if (loan.getLoanRisk().getWtfStatus() == YNEnum.N && StringUtils.equals(
            wkLoanEntity.getWkMfsDueBillDO().getLoanForm(), "5")) {
            LoanAstSyncBill loanAstSyncBill = baseTsfRes.getLoanAstSyncBill();
            LoanAstSyncTrans wtfSyncTrans = new LoanAstSyncTrans();
            wtfSyncTrans.setLoanNo(loanAstSyncBill.getLoanNo());
            wtfSyncTrans.setBizNo(MessageUtil.formatMsg("{0}-{1}-{2}", loanAstSyncBill.getLoanNo(),
                DateTimeUtil.formatYMD(loanAstSyncBill.getAccountDate()), TransCodeEnum.ST_CHG.getCode()));
            wtfSyncTrans.setAccountDate(loanAstSyncBill.getAccountDate());
            wtfSyncTrans.setTransTime(loanAstSyncBill.getAccountDate());
            wtfSyncTrans.setEventCode(EventCodeEnum.WRITE_OFF);
            baseTsfRes.getLoanAstSyncTransList().add(wtfSyncTrans);
            // 提取到期或者逾期且未结清的分期
            for (LoanAstSyncRepayPlan repayPlan : baseTsfRes.getRepayPlans()) {
                // 非正常，逾期，不处理结息
                if (!repayPlan.getStatus().isActive() || repayPlan.getTermNo() == 0) {
                    continue;
                }

                //
                buildRepayPlanTrans(repayPlan, wtfSyncTrans, wtfSyncTrans.getEventCode(), new DebtMoney(),
                    wtfSyncTrans.getTransTime());
            }

        }
    }

    /**
     * 生成空结息流水，供新增资产同步用
     *
     * @param loanAstSyncBill
     * @param loanAstSyncRepayPlans
     * @return
     */
    private LoanAstSyncTrans processDummySettle(LoanAstSyncBill loanAstSyncBill,
                                                List<LoanAstSyncRepayPlan> loanAstSyncRepayPlans) {

        Date settleAccountDate = DateTimeUtil.getStandardDate(loanAstSyncBill.getDayBeginAccountDate());

        LoanAstSyncTrans loanAstSyncTrans = new LoanAstSyncTrans();
        loanAstSyncTrans.setLoanNo(loanAstSyncBill.getLoanNo());
        loanAstSyncTrans.setBizNo(MessageUtil.formatMsg("{0}-{1}-{2}", loanAstSyncBill.getLoanNo(),
            DateTimeUtil.formatYMD(settleAccountDate), EventCodeEnum.SETTLE));

        loanAstSyncTrans.setAccountDate(settleAccountDate);
        loanAstSyncTrans.setTransTime(settleAccountDate);
        loanAstSyncTrans.setEventCode(EventCodeEnum.SETTLE);

        // 提取到期或者逾期且未结清的分期
        for (LoanAstSyncRepayPlan repayPlan : loanAstSyncRepayPlans) {
            // 非正常，逾期，不处理结息
            if (!repayPlan.getStatus().isActive() || repayPlan.getTermNo() == 0) {
                continue;
            }

            // 生成空结息交易流水
            buildRepayPlanTrans(repayPlan, loanAstSyncTrans, loanAstSyncTrans.getEventCode(), new DebtMoney(),
                loanAstSyncTrans.getTransTime());
        }

        return loanAstSyncTrans;
    }

    private void recoverInstalBalForRepayComponsate(BaseTsfRes baseTsfRes, WkMfsOpenRepayDO openRepayDO,
                                                    List<WkMfsRepayInfoDO> wkMfsRepayInfoDOs) {
        if (CollectionUtils.isEmpty(wkMfsRepayInfoDOs)) {
            return;
        }
        LoanAstSyncBill loanAstSyncBill = baseTsfRes.getLoanAstSyncBill();

        LoanAstSyncTrans loanAstSyncTrans = baseTsfRes.getInitLoanAstSyncTrans();

        List<LoanAstSyncRepayPlanTrans> repayPlanTranses = loanAstSyncTrans.getRepayPlanTransList();

        List<LoanAstSyncRepayPlan> repayPlans = baseTsfRes.getRepayPlans();
        Map<Integer, LoanAstSyncRepayPlan> repayPlanMap = repayPlans.stream().collect(
            Collectors.toMap(LoanAstSyncRepayPlan::getTermNo, item -> item));
        Map<Integer, LoanAstSyncRepayPlanTrans> repayPlanTransMap =
            repayPlanTranses.stream().collect(Collectors.toMap(LoanAstSyncRepayPlanTrans::getTermNo, item -> item));

        // 倒序遍历，为了把利息发生额放在最早的那期还款计划
        for (WkMfsRepayInfoDO repayInfoDO : wkMfsRepayInfoDOs) {
            LoanAstSyncRepayPlan repayPlan = repayPlanMap.get(repayInfoDO.getTermNo());

            if (null == repayPlan) {
                repayPlan = new LoanAstSyncRepayPlan();
                repayPlan.setTermNo(repayInfoDO.getTermNo());
                AstPeriod astPeriod = new AstPeriod();
                astPeriod.setStartDate(loanAstSyncTrans.getAccountDate());
                astPeriod.setEndDate(loanAstSyncTrans.getAccountDate());
                astPeriod.setInDate(loanAstSyncBill.getStartDate());
                astPeriod.setGraceEndDate(loanAstSyncTrans.getAccountDate());
                // fixbug 为放款日
                astPeriod.setDisbDate(loanAstSyncBill.getDisbDate());

                repayPlan.setAstPeriod(astPeriod);
                repayPlan.setDueIntrAmt(new Money(repayInfoDO.getIntAmt()));
                repayPlan.setStatus(LoanStatusEnum.NORMAL);
                repayPlan.setOvdDays(0);
                repayPlan.setLoanNo(loanAstSyncBill.getLoanNo());
                repayPlan.setUserId(loanAstSyncBill.getUserId());
                repayPlan.setSyncDate(loanAstSyncBill.getSyncDate());
                repayPlan.setAccountDate(loanAstSyncBill.getAccountDate());

                repayPlanMap.put(repayPlan.getTermNo(), repayPlan);
                repayPlans.add(repayPlan);
            } else {
                repayPlan.setStatus(LoanStatusEnum.NORMAL);

                repayPlan.getPaid().subByFieldType(DebtMoneyFieldEnum.PRIN, new Money(repayInfoDO.getPrinAmt()));
                repayPlan.getPaid().subByFieldType(DebtMoneyFieldEnum.INTR, new Money(repayInfoDO.getIntAmt()));
                repayPlan.getPaid()
                    .subByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, new Money(repayInfoDO.getPnltIntAmt()));
            }

            repayPlan.getBal().addByFieldType(DebtMoneyFieldEnum.PRIN, new Money(repayInfoDO.getPrinAmt()));
            repayPlan.getBal().addByFieldType(DebtMoneyFieldEnum.INTR, new Money(repayInfoDO.getIntAmt()));
            repayPlan.getBal().addByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, new Money(repayInfoDO.getPnltIntAmt()));

            LoanAstSyncRepayPlanTrans repayPlanTrans = repayPlanTransMap.get(repayInfoDO.getTermNo());
            if (null == repayPlanTrans) {
                repayPlanTrans = new LoanAstSyncRepayPlanTrans();
                repayPlanTrans.setTermNo(repayInfoDO.getTermNo());
                repayPlanTrans.setOutBizNo(openRepayDO.getSeqNo());
                repayPlanTrans.setTransTime(loanAstSyncTrans.getTransTime());
                repayPlanTrans.setEventCode(EventCodeEnum.SYNC_COMPONSATE);

                repayPlanTrans.setLoanNo(loanAstSyncBill.getLoanNo());
                repayPlanTrans.setUserId(loanAstSyncBill.getUserId());
                repayPlanTrans.setSyncDate(loanAstSyncBill.getSyncDate());
                repayPlanTrans.setAccountDate(loanAstSyncBill.getAccountDate());

                repayPlanTransMap.put(repayInfoDO.getTermNo(), repayPlanTrans);
                repayPlanTranses.add(repayPlanTrans);
            }
            repayPlanTrans.getPrinAmt().addTo(new Money(repayInfoDO.getPrinAmt()));
            repayPlanTrans.getIntrAmt().addTo(new Money(repayInfoDO.getIntAmt()));
            repayPlanTrans.getPrinPenalAmt().addTo(new Money(repayInfoDO.getPnltIntAmt()));
        }
    }


    /**
     * 生成资产分期 交易流水
     *
     * @param repayPlan
     * @param loanAstSyncTrans
     * @param eventCode
     * @param amt
     * @param transTime
     * @return
     */
    private LoanAstSyncRepayPlanTrans buildRepayPlanTrans(LoanAstSyncRepayPlan repayPlan,
                                                          LoanAstSyncTrans loanAstSyncTrans, EventCodeEnum eventCode,
                                                          DebtMoney amt, Date transTime) {
        LoanAstSyncRepayPlanTrans repayPlanTrans = buildBaseRepayPlanTrans(repayPlan,
            loanAstSyncTrans, eventCode, transTime);
        // 只要逾期，就处理，不管罚息发生额是否大于0.因为要更新逾期天数
        repayPlanTrans.setPrinAmt(amt.getByFieldType(DebtMoneyFieldEnum.PRIN));
        repayPlanTrans.setIntrAmt(amt.getByFieldType(DebtMoneyFieldEnum.INTR));
        repayPlanTrans.setPrinPenalAmt(amt.getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        return repayPlanTrans;
    }

    /**
     * 不包含发生额的分期交易流水
     *
     * @param repayPlan
     * @param loanAstSyncTrans
     * @param eventCode
     * @param transTime
     * @return
     */
    private LoanAstSyncRepayPlanTrans buildBaseRepayPlanTrans(LoanAstSyncRepayPlan repayPlan,
                                                              LoanAstSyncTrans loanAstSyncTrans,
                                                              EventCodeEnum eventCode, Date transTime) {
        LoanAstSyncRepayPlanTrans repayPlanTrans = new LoanAstSyncRepayPlanTrans();

        repayPlanTrans.setTermNo(repayPlan.getTermNo());
        repayPlanTrans.setOutBizNo(loanAstSyncTrans.getBizNo());
        repayPlanTrans.setTransTime(transTime);
        repayPlanTrans.setEventCode(eventCode);
        repayPlanTrans.setLoanNo(loanAstSyncTrans.getLoanNo());
        repayPlanTrans.setSyncDate(repayPlan.getSyncDate());
        repayPlanTrans.setAccountDate(loanAstSyncTrans.getAccountDate());

        loanAstSyncTrans.getRepayPlanTransList().add(repayPlanTrans);
        return repayPlanTrans;
    }

    /**
     * 对于包含同一期还款及还款冲正进行过滤，找出还款数据
     *
     * @param repayInfoDOS
     * @param bizCurDt
     * @return
     */
    private List<WkMfsRepayInfoDO> filterRepayInfos(List<WkMfsRepayInfoDO> repayInfoDOS, String bizCurDt) {
        // 还款冲正
        List<WkMfsRepayInfoDO> revInfos = repayInfoDOS.stream()
            .filter(item ->
                StringUtils.equals(bizCurDt, item.getCurDate()) &&
                    StringUtils.equals(item.getEvent(), REPO_REVERSE_EVT))
            .collect(Collectors.toList());
        // 还款
        List<WkMfsRepayInfoDO> effInfos = repayInfoDOS.stream()
            .filter(item ->
                StringUtils.equals(bizCurDt, item.getCurDate()) &&
                    !StringUtils
                        .equals(item.getEvent(), REPO_REVERSE_EVT))
            .collect(Collectors.toList());

        Iterator<WkMfsRepayInfoDO> effInfoIter = effInfos.iterator();
        while (effInfoIter.hasNext()) {
            WkMfsRepayInfoDO effInfo = effInfoIter.next();

            Iterator<WkMfsRepayInfoDO> revInfoIter = revInfos.iterator();
            while (revInfoIter.hasNext()) {
                // 过滤当天冲正的repayInfo
                WkMfsRepayInfoDO revInfo = revInfoIter.next();
                // 判断还款与 还款冲正数据是否一致
                if (isRepayInfoAmtEqual(effInfo, revInfo)) {
                    Date revRepayDate = DateTimeUtil.parseInTolerance(revInfo.getCurDate());
                    Date effRepayDate = DateTimeUtil.parseInTolerance(effInfo.getCurDate());
                    // 只考虑当天冲正的场景
                    if (DateTimeUtil.calcDayDiff(revRepayDate, effRepayDate) == 0) {
                        // 对应冲正流水要同步删除，避免有多个正常repayInfo匹配的情况
                        revInfoIter.remove();
                        effInfoIter.remove();
                        break;
                    }
                }
            }
        }

        return effInfos;
    }

    private boolean isRepayInfoAmtEqual(WkMfsRepayInfoDO left, WkMfsRepayInfoDO right) {
        if (!left.getTermNo().equals(right.getTermNo())) {
            return false;
        }

        if (left.getPrinAmt().equals(right.getPrinAmt())
            && left.getIntAmt().equals(right.getIntAmt())
            && left.getPnltIntAmt().equals(right.getPnltIntAmt())) {
            return true;
        } else {
            return false;
        }
    }
}