/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.astclient.wkloan.entity;

import cn.hutool.core.util.NumberUtil;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsDueBillDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsLnPactDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsOpenRepayDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsRepayInfoDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsRepayPlanDO;
import com.abssqr.plat.common.model.domain.astSync.BaseAstEntity;
import com.general.system.common.util.StringUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;

import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.astclient.wkloan.entity: WkLoan.java, v 0.1 2019-06-07 04:26 zhenxuan.luo Exp $ 
 */
public class WkMfsLoanEntity extends BaseAstEntity {
    /**
     * 借据
     */
    private WkMfsDueBillDO         wkMfsDueBillDO;
    /**
     * 还款计划
     */
    private List<WkMfsRepayPlanDO> wkMfsRepayPlanDOS = Lists.newArrayList();
    /**
     * 合同
     */
    private WkMfsLnPactDO          wkMfsLnPactDO;
    /**
     * 放款信息
     */
    private WkMfsOpenRepayDO       wkMfsOpenRepayDO;
    /**
     * 还款信息
     */
    private List<WkMfsRepayInfoDO> wkMfsRepayInfoDOS = Lists.newArrayList();

    public void check(String loanId, String curDt) {
        VarChecker.checkNotNull(this.wkMfsDueBillDO, "[{0}]数据模型同步：账单数据为空", loanId);

        VarChecker.checkArgument(CollectionUtils.isNotEmpty(this.wkMfsRepayPlanDOS),
                "[{0}]数据模型同步：还款计划为空", loanId);

        if (CollectionUtils.isNotEmpty(wkMfsRepayInfoDOS)) {
            List<WkMfsRepayInfoDO> curRepayInfos = wkMfsRepayInfoDOS.stream()
                                                           .filter(item -> StringUtil.equals(curDt, item.getCurDate()))
                                                           .collect(Collectors.toList());

            for (WkMfsRepayInfoDO curRepayInfo : curRepayInfos) {
                BigDecimal incomeAmt = curRepayInfo.getIncomeAmt();
                BigDecimal checkIncomeAmt = NumberUtil.add(
                        curRepayInfo.getPrinAmt(),
                        curRepayInfo.getTrustIntAmtAcc(),
                        curRepayInfo.getTrustPnltIntAmtAcc(),
                        curRepayInfo.getTrustRepayViolateAmtAcc());

                VarChecker.checkArgument(incomeAmt.compareTo(checkIncomeAmt) == 0,
                        "[{0}]数据模型同步：还款流水总实收[{1}]和各项金额[{2}]不相等",
                        loanId, incomeAmt, checkIncomeAmt);
            }
        }

    }

    public WkMfsDueBillDO getWkMfsDueBillDO() {
        return wkMfsDueBillDO;
    }

    public void setWkMfsDueBillDO(WkMfsDueBillDO wkMfsDueBillDO) {
        this.wkMfsDueBillDO = wkMfsDueBillDO;
    }

    public List<WkMfsRepayPlanDO> getWkMfsRepayPlanDOS() {
        return wkMfsRepayPlanDOS;
    }

    public void setWkMfsRepayPlanDOS(
            List<WkMfsRepayPlanDO> wkMfsRepayPlanDOS) {
        this.wkMfsRepayPlanDOS = wkMfsRepayPlanDOS;
    }

    public WkMfsLnPactDO getWkMfsLnPactDO() {
        return wkMfsLnPactDO;
    }

    public void setWkMfsLnPactDO(WkMfsLnPactDO wkMfsLnPactDO) {
        this.wkMfsLnPactDO = wkMfsLnPactDO;
    }

    public WkMfsOpenRepayDO getWkMfsOpenRepayDO() {
        return wkMfsOpenRepayDO;
    }

    public void setWkMfsOpenRepayDO(WkMfsOpenRepayDO wkMfsOpenRepayDO) {
        this.wkMfsOpenRepayDO = wkMfsOpenRepayDO;
    }

    public List<WkMfsRepayInfoDO> getWkMfsRepayInfoDOS() {
        return wkMfsRepayInfoDOS;
    }

    public void setWkMfsRepayInfoDOS(
            List<WkMfsRepayInfoDO> wkMfsRepayInfoDOS) {
        this.wkMfsRepayInfoDOS = wkMfsRepayInfoDOS;
    }
}