/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.buyBackMatch;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherExecutorTemp;
import com.abssqr.plat.biz.shared.handler.LoanAstBuyBackMatchHandler;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * 资产回购匹配执行器
 */
@Component
public class LoanAstBuyBackMatchExecutor extends DispatcherExecutorTemp<CommonTask> {

    private final static Logger LOGGER       = LoggerFactory.getLogger(LoanAstBuyBackMatchExecutor.class);

    @Autowired
    private CommonTaskRepository       commonTaskRepository;
    @Autowired
    private LoanAstBuyBackMatchHandler loanAstBuyBackMatchHandler;

    @Override
    public CommonTask doExecute(CommonTask node) {
        LogUtil.info(LOGGER, "执行回购匹配任务[{0}]-start", node.getContext());
        loanAstBuyBackMatchHandler.execute(node.getContext(), status -> {
            if (TaskStatusEnum.SUCCESS == status) {
                node.setTaskStatus(status);
                markSuccess(node);
            } else {
                markFail(node, "执行回购匹配任务失败");
            }
            LogUtil.info(LOGGER, "执行回购匹配任务[{0}]-end,status=[{1}]", node.getContext(), status);
        });

        return node;
    }

    @Override
    protected CommonTask markSuccess(CommonTask node) {
        commonTaskRepository.updateTaskStatus(node.getId(), TaskStatusEnum.SUCCESS, TaskStatusEnum.EXECUTING, "");
        return node;
    }

    @Override
    protected CommonTask markFail(CommonTask node, String failMessage) {
        // 因异常引起的重试
        Date nextTime = DateTimeUtil.addSecond(SystemDateUtil.getSystemDate(), 60);
        String msg = failMessage.length() > 2000 ? failMessage.substring(0, 2000) : failMessage;
        commonTaskRepository.updateTaskExep(nextTime, node.getId(), msg);
        return node;
    }

    @Override
    protected CommonTask markException(CommonTask node, String failMessage) {
        return markFail(node, failMessage);
    }


    @Override
    protected String getName() {
        return LoanAstBuyBackMatchStarter.JOB_NAME;
    }
}
