/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.buyBackMatch;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.core.service.ast.LoanAstBuyBackManager;
import com.abssqr.plat.core.service.host.HostManager;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.ThreadUtil;
import com.general.system.common.util.VarChecker;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 资产转让
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.task.task: AstTrTaskLoader.java, v 0.1 2018-01-08 00:59 hongwei.wang Exp $
 */
@Component
public class LoanAstBuyBackMatchLoader implements DispatcherLoader<CommonTask> {

    private final static Logger                LOGGER = LoggerFactory.getLogger(LoanAstBuyBackMatchLoader.class);
    @Autowired
    private              TransactionTemplate   transactionTemplate;
    @Autowired
    private              JobControlRepository  jobControlRepository;
    @Autowired
    private              CommonTaskRepository  commonTaskRepository;
    @Autowired
    private              HostManager           hostManager;


    private static long sleepTime = 1000 * 60;

    /**
     * 装载待分发节点
     *
     * @param size
     * @return
     */
    @Override
    public List<CommonTask> loadToDoNodes(int size) {
        ThreadUtil.sleep(sleepTime, LOGGER);
        return transactionTemplate.execute(transactionStatus -> {
            // job加锁
            jobControlRepository.lockByJobName(LoanAstBuyBackMatchStarter.JOB_NAME);
            // 获取回购筛选任务
            List<CommonTask> existsTask = commonTaskRepository.selectUnifinishTaskByType(CommonTaskTypeEnum.BUY_BACK_MATCH);
            // 过滤 排除非初始化的和下次执行时间大于当前时间的任务
            existsTask = existsTask.stream()
                    .filter(item -> item.getTaskStatus() == TaskStatusEnum.INIT
                            && SystemDateUtil.getSystemDate().compareTo(item.getNextExecTime()) > 0)
                    .collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(existsTask)) {
                int cnt = commonTaskRepository
                        .updateTaskExec(SystemDateUtil.getSystemDate(), hostManager.getHostInfo().getHostName(),
                                existsTask);
                VarChecker.checkArgument(cnt==existsTask.size(),"[LoanAstBuyBackMatchLoader] - 更新条数与预期不符,可能存在并发问题");
            }
            return existsTask;
        });
    }

    /**
     * 设置休眠时间  测试用
     * @param time
     */
    public void setSleepTime(Long time){
        sleepTime = time;
    }
}
