/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.transfer;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.common.dal.mysql.auto.dao.TrTaskDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.TrTaskDO;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.convertor.TrTaskConvert;
import com.abssqr.plat.common.model.domain.task.TrCtrTask;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.abssqr.plat.common.model.repo.tr.TrCtrTaskRepo;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.host.HostManager;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Date;
import java.util.List;

/**
 * 资产转让
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.task.task: AstTrTaskLoader.java, v 0.1 2018-01-08 00:59 hongwei.wang Exp $
 */
@Component
public class AstTrTaskLoader implements DispatcherLoader<TrTask> {

    private final static Logger LOGGER = LoggerFactory.getLogger(AstTrTaskLoader.class);

    @Autowired
    private TrTaskDAO           trTaskDAO;
    @Autowired
    private TransactionTemplate transactionTemplate;
    @Autowired
    private HostManager         hostManager;
    @Autowired
    private TrCtrTaskRepo       trCtrTaskRepo;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;

    /**
     * 装载待分发节点
     *
     * @param size
     * @return
     */
    @Override
    public List<TrTask> loadToDoNodes(int size) {

        Date curDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
        TrCtrTask trCtrTask = trCtrTaskRepo.getByStatus(curDate, TaskStatusEnum.INIT, AstPackTradeTypeEnum.AST_TRF_BIZ);
        if (null == trCtrTask) {
            return Lists.newArrayList();
        }

        return transactionTemplate.execute(transactionStatus -> {
            TrCtrTask lockedCtrTask = trCtrTaskRepo.lockById(trCtrTask.getId());
            if (lockedCtrTask.getTaskStatus() != TaskStatusEnum.INIT) {
                return Lists.newArrayList();
            }
            trCtrTaskRepo.updateExecTms(lockedCtrTask.getId(), lockedCtrTask.getExecTimes() + 1);

            Long undoCnt = trTaskDAO.cntInitTaskByBatchNo(lockedCtrTask.getBatchNo());
            if (undoCnt == 0) {
                LogUtil.info(LOGGER, "转让控制[{0}]无待转任务，标记结束", lockedCtrTask);
                trCtrTaskRepo.updateStatus(lockedCtrTask.getId(), TaskStatusEnum.SUCCESS);
                return Lists.newArrayList();
            }

            List<TrTaskDO> taskDOS = trTaskDAO.selectInitTask(SystemDateUtil.getSystemDate(), Long.valueOf(size),
                    lockedCtrTask.getBatchNo(), AstPackTradeTypeEnum.AST_TRF_BIZ.getCode());
            if (CollectionUtils.isEmpty(taskDOS)) {
                return Lists.newArrayList();
            }

            int count = trTaskDAO.updateTaskExec(SystemDateUtil.getSystemDate(),
                    hostManager.getHostInfo().getHostName(),
                    taskDOS);
            VarChecker.checkEquals(count, taskDOS.size(), "[TaskLoader.loadToDoNodes]更新任务数跟期望数不一致，实际更新：{0}，期望更新：{1}",
                    count,
                    taskDOS.size());

            LogUtil.info(LOGGER, "转让控制[{0}]捞取[{1}]待执行任务", lockedCtrTask, count);
            return TrTaskConvert.convertDTOs(taskDOS);
        });
    }
}
