/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.transfer;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherExecutor;
import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.biz.shared.dispatcher.DispatcherStarter;
import com.abssqr.plat.common.dal.mysql.auto.dao.TrTaskDAO;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.abssqr.plat.core.service.host.HostManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.task.task: AstTrTaskDispatcherManager.java, v 0.1 2018-01-08 02:42 hongwei
 * .wang Exp $
 */
@Component
public class AstTrTaskStarter extends DispatcherStarter<TrTask> {

    @Autowired
    private AstTrTaskExecutor astTrTaskExecutor;
    @Autowired
    private AstTrTaskLoader astTrTaskLoader;
    @Autowired
    private HostManager hostManager;
    @Autowired
    private TrTaskDAO   trTaskDAO;

    @Value("${dispatcher.tr.coreSize}")
    private int coreSize;

    @Value("${dispatcher.tr.maxSize}")
    private int maxSize;
    
    @Value("${dispatcher.tr.hungrySize}")
    private int hungrySize;
    
    @Value("${dispatcher.tr.queueSize}")
    private int queueSize = 200;

    @Override
    protected void recovery() {
        trTaskDAO.updateHangTasksByIp(hostManager.getHostInfo().getHostName(), AstPackTradeTypeEnum.AST_TRF_BIZ.getCode());
    }

    @Override
    public String getName() {
        return "trTaskDispatcher";
    }

    @Override
    protected int getCoreSize() {
        return coreSize;
    }

    @Override
    protected int getMaxSize() {
        return maxSize;
    }

    @Override
    protected int getQueueSize() {
        return queueSize;
    }

    @Override
    protected int getHungrySize() {
        return hungrySize;
    }

    @Override
    protected DispatcherLoader<TrTask> getLoader() {
        return astTrTaskLoader;
    }

    @Override
    protected DispatcherExecutor<TrTask> getExecutor() {
        return astTrTaskExecutor;
    }

}
