/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.wkAstSync.mfs;

import com.abssqr.plat.biz.shared.astclient.wkloan.adapter.WkMfsAstAssembly;
import com.abssqr.plat.biz.shared.astclient.wkloan.adapter.WkMfsAstClientAdapter;
import com.abssqr.plat.biz.shared.astclient.wkloan.entity.WkMfsLoanEntity;
import com.abssqr.plat.biz.shared.dispatcher.DispatcherExecutorTemp;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.wk.domain.sync.WkMfsSyncCtr;
import com.abssqr.plat.common.model.wk.repo.WkMfsSyncCtrRepo;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 文件同步后账单 转换为内部模型存贮
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.dispatcher.syncDispatcher: LoanAstSyncDispatcherExecutor.java, v 0.1 2018-01-08
 * 上午01:45 zhenxuan.luo Exp $
 */
@Component
public class WkMfsAstSyncDispatcherExecutor extends DispatcherExecutorTemp<WkMfsSyncCtr> {

    private final static int DEFAULT_STEP = 5;
    private final static Logger LOGGER = LoggerFactory.getLogger(WkMfsAstSyncDispatcherExecutor.class);
    @Autowired
    WkMfsAstAssembly      wkMfsAstAssembly;
    @Autowired
    WkMfsAstClientAdapter wkMfsAstClientAdapter;
    @Autowired
    WkMfsSyncCtrRepo      wkMfsSyncCtrRepo;

    @Override
    public WkMfsSyncCtr doExecute(WkMfsSyncCtr node) {
        WkMfsSyncCtr lockNode = wkMfsSyncCtrRepo.lock(node.getId());
        if (lockNode.getTaskStatus() != TaskStatusEnum.EXECUTING) {
            LogUtil.info(LOGGER, "任务[{0}]状态不是执行中，不执行", lockNode);
            return node;
        }
        // 组装外部模型
        WkMfsLoanEntity wkMfsLoanEntity = wkMfsAstAssembly.assembly(lockNode);
        // 执行资产交易
        wkMfsAstClientAdapter.process(wkMfsLoanEntity);
        markSuccess(lockNode);
        return node;
    }

    @Override
    protected WkMfsSyncCtr markSuccess(WkMfsSyncCtr node) {
        node.setTaskStatus(TaskStatusEnum.SUCCESS);
        wkMfsSyncCtrRepo.updateTaskStatus(node.getId(), TaskStatusEnum.EXECUTING, TaskStatusEnum.SUCCESS);
        return node;
    }

    @Override
    protected WkMfsSyncCtr markFail(WkMfsSyncCtr node, String failMessage) {
        // 资产同步，不能设置为失败，必须成功
        markException(node, failMessage);
        return node;
    }

    @Override
    protected WkMfsSyncCtr markException(WkMfsSyncCtr node, String failMessage) {
        wkMfsSyncCtrRepo
            .updateTaskExep(
                DateTimeUtil.addSecond(SystemDateUtil.getSystemDate(), calcExeSleepTime(node.getExecTimes())),
                node.getId(), TaskStatusEnum.FAILURE);
        wkMfsAstAssembly.syncData2Local(node);
        return node;
    }

    private int calcExeSleepTime(Long execTimes) {
        if (execTimes == null) {
            execTimes = 0L;
        }
        int result = DEFAULT_STEP;
        if (execTimes > 100) {
            //休息两小时
            result = 60 * 60;
        } else if (execTimes > 50) {
            //休息半小时
            result = 60;
        }
        return result;
    }

    @Override
    protected String getName() {
        return "WkMfsAstSyncDispatcherExecutor";
    }
}
