/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.wkAstSync.mfs;

import java.util.List;
import java.util.stream.Collectors;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.biz.shared.dispatcher.TaskGroupService;
import com.abssqr.plat.common.model.wk.domain.sync.WkMfsSyncCtr;
import com.abssqr.plat.common.model.wk.repo.WkMfsSyncCtrRepo;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import com.abssqr.plat.core.service.host.HostManager;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * 转换为内部模型存贮任务分发
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.dispatcher.syncDispatcher: LoanAstSyncDispatcherLoader.java, v 0.1 2018-01-08
 * 上午01:33 zhenxuan.luo Exp $
 */
@Component
public class WkMfsAstSyncDispatcherLoader implements DispatcherLoader<WkMfsSyncCtr> {

    private final static Logger LOGGER = LoggerFactory.getLogger(
        WkMfsAstSyncDispatcherLoader.class);

    @Autowired
    TransactionTemplate           transactionTemplate;
    @Autowired
    WkMfsSyncCtrRepo              wkMfsSyncCtrRepo;
    @Autowired
    HostManager                   hostManager;

    /**
     * Split形式的load数据的方式
     */
    @Override
    @PerformTrace(name = "WkMfsAstSyncDispatcherLoader")
    public List<WkMfsSyncCtr> loadToDoNodes(TaskGroupService firstTaskGroupService,
                                            TaskGroupService spareTaskGroupService, int size) {
        List<WkMfsSyncCtr> result = Lists.newArrayList();

        transactionTemplate.execute(new TransactionCallbackWithoutResult() {
            @Override
            protected void doInTransactionWithoutResult(TransactionStatus status) {
                loadTasksUseTaskRange(firstTaskGroupService, result, size);
                loadTasksUseTaskRange(spareTaskGroupService, result, size);
                //判断当前是否符合
                if (CollectionUtils.isNotEmpty(result)) {
                    List<Long> taskIds = result.stream().map(WkMfsSyncCtr::getId).collect(Collectors.toList());
                    int upCnt = wkMfsSyncCtrRepo
                        .updateTaskExec(SystemDateUtil.getSystemDate(), hostManager.getHostInfo().getHostName(),
                            taskIds);
                    VarChecker.checkEquals(upCnt, result.size(), "更新数量不一致");
                    LogUtil.info(LOGGER, "WKMFS-资产同步任务加载：捞取[{0}]条待执行任务]", taskIds.size());
                    LogUtil.info(LOGGER, "WKMFS-资产同步任务加载IDS={0}", taskIds);
                }

            }
        });
        return result;
    }

    /**
     * 优先使用第一个任务区间去找
     *
     * @param taskGroupService
     * @param result
     * @param size
     */
    private void loadTasksUseTaskRange(TaskGroupService taskGroupService, List<WkMfsSyncCtr> result, int size) {
        List taskRange = taskGroupService.getTasks();
        if (CollectionUtils.isNotEmpty(taskRange) && taskRange.size() == 2 && size > result.size()) {
            Long minId = (Long)taskRange.get(0);
            Long maxId = (Long)taskRange.get(1);
            List<WkMfsSyncCtr> tasksWithRange = wkMfsSyncCtrRepo.selectInitTaskWithRange(size - result.size(), minId,
                maxId);
            if (CollectionUtils.isNotEmpty(tasksWithRange)) {
                result.addAll(tasksWithRange);
            } else {
                if (!wkMfsSyncCtrRepo.isHasRemainTask(minId, maxId)) {
                    LogUtil.info(LOGGER, "WKMFS-资产同步任务{0},{1}结束]", minId, maxId);
                    //标记当前任务区间为处理完成
                    taskGroupService.finish();
                }
            }
        }
    }

}