/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.wkAstSync.mfs;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherSplitor;
import com.abssqr.plat.common.dal.sync.auto.resultmap.MfsBillCtrIdRange;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskGroupTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.TaskGroup;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtrLog;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.common.model.wk.repo.WkMfsSyncCtrRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.host.HostManager;
import org.apache.commons.collections.CollectionUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.dispatcher.wkAstSync.mfs: WkMfsAstSyncDispatcherSplitor.java, v 0.1 2020-03-14
 * 10:53 bangis.wangdf Exp $
 */
@Component
public class WkMfsAstSyncDispatcherSplitor extends DispatcherSplitor<Long> {

    @Value("${dispatcher.sync.split.limit:5000}")
    private Long   limit;
    @Value("${dispatcher.sync.split.cronExpr:0/5 * * * * ?}")
    private String cronExpr;

    @Autowired
    AstSyncTaskCtrRepo   astSyncTaskCtrRepo;
    @Autowired
    WkMfsSyncCtrRepo     wkMfsSyncCtrRepo;
    @Autowired
    HostManager          hostManager;
    @Autowired
    CommonTaskRepository commonTaskRepository;
    @Autowired
    JobControlRepository jobControlRepository;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;

    /**
     *
     * @return
     */
    @Override
    public List<TaskGroup> split() {
        List<TaskGroup> splitTaskGroup = new ArrayList<>();
        List<AstSyncTaskCtr> astSyncTaskCtrs = astSyncTaskCtrRepo.getAstSyncTaskCtrs();
        if (CollectionUtils.isEmpty(astSyncTaskCtrs)) {
            return splitTaskGroup;
        }
        // 切分数据
        this.splitData(splitTaskGroup);
        // 转让会计日
        Day execDate = orgAcctDayComponent.getTrfAcctDay();
        //完成任务
        for (AstSyncTaskCtr astSyncTaskCtr : astSyncTaskCtrs) {
            String orgCodeStr = astSyncTaskCtr.getOrgCode();

            OrgCodeEnum orgCode = OrgCodeEnum.getByCode(orgCodeStr);
            // 机构会计日
            Day orgDate = orgAcctDayComponent.getAstAcctDay(orgCode);

            //同步已完成
            AstSyncTaskCtrLog syncLog = astSyncTaskCtrRepo
                .getTaskLogByOrgDate(orgDate.getStandardDate(), SyncTaskTypeEnum.AST_SYNC, orgCodeStr);
            if (syncLog != null && syncLog.getStatus() == TaskStatusEnum.SUCCESS) {
                AstSyncTaskCtrLog modelLog = astSyncTaskCtrRepo
                    .getTaskLogByOrgDate(orgDate.getStandardDate(), SyncTaskTypeEnum.AST_MOD, orgCodeStr);
                //模型转换未完成
                if (modelLog == null || modelLog.getStatus() != TaskStatusEnum.SUCCESS) {
                    if (!wkMfsSyncCtrRepo.isHasRemainTask(orgCodeStr)) {
                        astSyncTaskCtrRepo.insertSyncLog(execDate.getStandardDate(), orgDate, astSyncTaskCtr,
                            SyncTaskTypeEnum.AST_MOD,
                            TaskStatusEnum.SUCCESS, hostManager.getHostInfo().getHostName());

                        /**
                         // bangis 创建一个模型同步后检查是否有遗漏资产同步的CommonTask 已排进技改任务
                         String commonBizNo = getOrgCode()+execDate.toString();
                         CommonTask commonTask = CommonTaskUtil.buildCommonTaskWithContext(commonBizNo,
                         CommonTaskTypeEnum.AST_MOD_CHECK, getOrgCode());
                         commonTaskRepository.create(commonTask);
                         **/
                    }else{
                        //所有数据处理完成后
                        if(CollectionUtils.isEmpty(splitTaskGroup)){
                            //忽略的数据直接删除，错误原因在详情里说明
                            wkMfsSyncCtrRepo.deleteFailTasks(orgCodeStr);
                        }
                    }
                }
            }
        }


        return splitTaskGroup;

    }

    @Override
    public String getTaskJobName() {
        return "AST_MOD_MFS";
    }

    private void splitData(List<TaskGroup> splitTaskGroup) {
        Long taskCnt = wkMfsSyncCtrRepo.getTaskCnt();
        int groupAmt = new Long((taskCnt / limit + 1)).intValue();
        groupAmt = groupAmt > 400 ? 400 : groupAmt;
        long minId = 0L;
        for (int i = 0; i < groupAmt; i++) {
            MfsBillCtrIdRange mfsBillCtrIdRange = wkMfsSyncCtrRepo.selectMaxIdAndMinIdOfInitTask(limit, minId);
            if (null != mfsBillCtrIdRange && mfsBillCtrIdRange.getMinId() != null
                && mfsBillCtrIdRange.getMaxId() != null) {
                TaskGroup taskGroup = new TaskGroup();
                taskGroup.setJobName(getTaskJobName());
                taskGroup.setTaskType(TaskGroupTypeEnum.ID_RANGE);
                taskGroup.setTasks(Lists.newArrayList(mfsBillCtrIdRange.getMinId(), mfsBillCtrIdRange.getMaxId()));
                minId = mfsBillCtrIdRange.getMaxId();
                splitTaskGroup.add(taskGroup);
            } else {
                break;
            }
        }
    }

    @Override
    protected String getCronExpr() {
        return cronExpr;
    }
}
