/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.wkAstSync.mfs;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherExecutor;
import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.biz.shared.dispatcher.DispatcherSplitor;
import com.abssqr.plat.biz.shared.dispatcher.DispatcherStarter;
import com.abssqr.plat.common.model.wk.domain.sync.WkMfsSyncCtr;
import com.abssqr.plat.common.model.wk.repo.WkMfsSyncCtrRepo;
import com.abssqr.plat.core.service.host.HostManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.dispatcher.wkAstSync.mfs: WkMfsAstSyncDispatcherStarter.java, v 0.1 2020-03-16
 * 10:16 bangis.wangdf Exp $
 */

@Component
@Configuration
public class WkMfsAstSyncDispatcherStarter extends DispatcherStarter<WkMfsSyncCtr> {

    @Autowired
    WkMfsAstSyncDispatcherLoader   wkMfsAstSyncDispatcherLoader;
    @Autowired
    WkMfsAstSyncDispatcherExecutor wkMfsAstSyncDispatcherExecutor;
    @Autowired
    WkMfsAstSyncDispatcherSplitor  wkMfsAstSyncDispatcherSplitor;
    @Autowired
    WkMfsSyncCtrRepo               wkMfsSyncCtrRepo;
    @Autowired
    HostManager                    hostManager;

    @Value("${dispatcher.sync.coreSize}")
    private int coreSize;

    @Value("${dispatcher.sync.maxSize}")
    private int maxSize;

    @Value("${dispatcher.sync.hungrySize}")
    private int hungrySize;

    @Value("${dispatcher.sync.queueSize}")
    private int queueSize = 200;

    @Override
    protected void recovery() {
        wkMfsSyncCtrRepo.updateHangTasksByIp(hostManager.getHostInfo().getHostName());
    }

    @Override
    public String getName() {
        return "WkMfsAstSyncDispatcher";
    }

    @Override
    protected int getCoreSize() {
        return coreSize;
    }

    @Override
    protected int getMaxSize() {
        return maxSize;
    }

    @Override
    protected int getQueueSize() {
        return queueSize;
    }

    @Override
    protected int getHungrySize() {
        return hungrySize;
    }

    @Override
    protected DispatcherLoader<WkMfsSyncCtr> getLoader() {
        return wkMfsAstSyncDispatcherLoader;
    }

    @Override
    protected DispatcherExecutor<WkMfsSyncCtr> getExecutor() {
        return wkMfsAstSyncDispatcherExecutor;
    }

    @Override
    protected DispatcherSplitor getSplitor() {
        return wkMfsAstSyncDispatcherSplitor;
    }
}