/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.handler;

import com.abssqr.plat.common.dal.mysql.auto.paging.PactsAstMatchPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstPoolItem;
import com.abssqr.plat.common.facade.enums.AstPackStatusEnum;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.ast.Pack;
import com.abssqr.plat.common.model.domain.task.TrCtrTask;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.repo.pool.AstPoolRepository;
import com.abssqr.plat.common.model.repo.tr.TrCtrTaskRepo;
import com.abssqr.plat.common.model.repo.tr.TrTaskRepo;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.util.loggers.CommonLoggers;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.ast.AstPackManager;
import com.abssqr.plat.core.service.rule.AstScreenRuleComponent;
import com.general.system.common.model.Money;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MoneyUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import static com.abssqr.plat.common.model.enums.IdTypeEnum.ASTSCREENINGRULE;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.handler: PlanAstMatchHandler.java, v 0.1 2019-11-12 11:45 bangis.wangdf Exp $
 */
@Component
public class PlanAstMatchHandler {
    protected final Logger LOGGER = CommonLoggers.AST_MATCH;
    @Autowired
    AstPackManager         astPackManager;
    @Autowired
    AstPoolRepository      astPoolRepository;
    @Autowired
    TrTaskRepo             trTaskRepo;
    @Autowired
    TrCtrTaskRepo          trCtrTaskRepo;
    @Autowired
    AstScreenRuleComponent astScreenRuleComponent;
    @Autowired
    TransactionTemplate    transactionTemplate;
    @Autowired
    OrgAcctDayComponent    orgAcctDayComponent;

    private int queryLimit     = 2000;
    private int batchInsertCnt = 500;

    public void execute(String spvPlanNo, List<Pack> packs, Consumer<TaskStatusEnum> consumer) {
        //清理历史TrTask
        Map<String, TrCtrTask> trCtrTaskMap = Maps.newHashMap();
        VarChecker.checkNotEmpty(packs, "[PlanAstMatchHandler]-[{0}]筛选启动失败,没有对应的Packs");
        Date curDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
        //清理历史转让流水数据
        this.clearHisTrTask(spvPlanNo, curDate);

        String batchNo = SequenceUtil.genId(ASTSCREENINGRULE);
        LogUtil.info(LOGGER, "开始筛选[{0}]放款计划，批次号=[{1}]", spvPlanNo, batchNo);
        PactsAstMatchPage pactsAstMatchPage = new PactsAstMatchPage();
        pactsAstMatchPage.setLimit(queryLimit);
        pactsAstMatchPage.setOwnOrg(spvPlanNo);
        pactsAstMatchPage.setSysDate(curDate);
        do {
            LogUtil.info(LOGGER, "[PlanAstMatchHandler]-[{0}]开始筛选[{0}]放款计划第[{1}]页", spvPlanNo,
                    pactsAstMatchPage.getCurrentPage());
            this.astMatch(pactsAstMatchPage, packs, batchNo, curDate, trCtrTaskMap);
            pactsAstMatchPage.setCurrPageNo(pactsAstMatchPage.getCurrentPage() + 1);
        } while (pactsAstMatchPage.hasNextPage());
        transactionTemplate.execute(status -> {
            astPackManager.updatePackStatus(packs, AstPackStatusEnum.SCEEN_FIS, AstPackStatusEnum.INIT);
            trTaskRepo.updateStatus(batchNo, TaskStatusEnum.INIT, TaskStatusEnum.IGNORE);
            //插入转让控制表，可以开始转让
            if (MapUtils.isNotEmpty(trCtrTaskMap)) {
                trCtrTaskRepo.insertBatch(Lists.newArrayList(trCtrTaskMap.values()));
            }
            consumer.accept(TaskStatusEnum.SUCCESS);
            return status;
        });
    }

    private void clearHisTrTask(String spvPlanNo, Date acctDate) {
        //清理 spvPlanNo 会计日前30天数据
        //清理 spvPlanNo 对应非成功的
        trTaskRepo.clearHisTrTask(spvPlanNo, acctDate, AstPackTradeTypeEnum.AST_TRF_BIZ);
    }

    private void astMatch(PactsAstMatchPage pactAstMatch, List<Pack> packs, String batchNo, Date curDate,
                          Map<String, TrCtrTask> trCtrTaskMap) {
        // 支持切片 Packs
        List<Pack> slicePacks = packs.stream().filter(pack -> pack.getTransferRule().getSlice()).collect(
                Collectors.toList());

        PactsAstMatchPage pactAstMatchPage = astPoolRepository.queryForAstMatch(pactAstMatch);
        List<AstPoolItem> astPools = pactAstMatchPage.getDatas();
        List<TrTask> trTasks = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(astPools)) {
            for (AstPoolItem astPoolItem : astPools) {
                TrTask trTask = null;
                //未转让过
                if (!astPoolItem.isSlice()) {
                    // 总体随机排序
                    Collections.shuffle(packs);
                    trTask = astPoolMatch(packs, batchNo, curDate, trCtrTaskMap, astPoolItem);
                }
                //切片筛选
                if (trTask == null && CollectionUtils.isNotEmpty(slicePacks)) {
                    List<AstPoolItem> astPoolItems = astPoolRepository.getAstPoolItems(curDate, astPoolItem.getAstNo());
                    VarChecker.checkNotEmpty(astPoolItems, "资产筛选:[{0},找不到未转让的分期]", astPoolItem.getAstNo());
                    Collections.sort(astPoolItems, (l, r) -> r.getTermNo().compareTo(l.getTermNo()));
                    for (AstPoolItem astPoolTermItem : astPoolItems) {
                        Collections.shuffle(slicePacks);
                        if (astPoolTermItem.getPrinBal().isGreaterThanZero()) {
                            trTask = astPoolMatch(slicePacks, batchNo, curDate, trCtrTaskMap, astPoolTermItem);
                            if (trTask != null) {
                                break;
                            }
                        }
                    }
                }
                if (trTask != null) {
                    VarChecker.checkArgument(trTask.getTfrPrinBal().isGreaterThanZero(), "Pack筛选到资产-待转金额为0,trTask{0}",
                            trTask);
                    trTasks.add(trTask);
                    LogUtil.info(LOGGER, "[{0}]Pack筛选到资产，astNo=[{1}],batchNo=[{2}]", trTask.getTfrPack(),
                            trTask.getAstNo(), batchNo);
                    trySaveTrTask(trTasks, batchInsertCnt);
                }
            }
            trySaveTrTask(trTasks, 0);
        }
    }

    private TrTask astPoolMatch(List<Pack> packs, String batchNo, Date acctDate, Map<String, TrCtrTask> trCtrTaskMap,
                                AstPoolItem astPoolItem) {
        for (Pack pack : packs) {
            //筛选规则匹配
            boolean isMatch = astScreenRuleComponent.astMatchTransferRule(acctDate, pack, () -> astPoolItem);
            //规模匹配
            if (isMatch) {
                TrTask task = this.dealTrCtrTask(batchNo, acctDate, trCtrTaskMap, pack, astPoolItem);
                if (task != null) {
                    return task;
                }
            }
        }
        return null;
    }

    /**
     * 尝试保存
     *
     * @param trTasks
     * @param minSize
     */
    private void trySaveTrTask(List<TrTask> trTasks, int minSize) {
        if (CollectionUtils.isNotEmpty(trTasks) && trTasks.size() >= minSize) {
            trTaskRepo.insertBatch(trTasks);
            trTasks.clear();
        }
    }

    /**
     * 初始化转让控制任务
     *
     * @param batchNo
     * @param acctDate
     * @param trCtrTaskMap
     * @param pack
     * @param astPoolItem
     * @return 满足
     */
    private TrTask dealTrCtrTask(String batchNo, Date acctDate, Map<String, TrCtrTask> trCtrTaskMap, Pack pack,
                                 AstPoolItem astPoolItem) {
        TrCtrTask trCtrTask = trCtrTaskMap.get(pack.getPackCode());
        if (trCtrTask == null) {
            trCtrTask = new TrCtrTask();
            trCtrTask.setTfrDate(acctDate);
            trCtrTask.setBatchNo(batchNo);
            trCtrTask.setPackCode(pack.getPackCode());
            trCtrTask.setTaskStatus(TaskStatusEnum.INIT);
            trCtrTask.setManagerOrgCode(pack.getManagerOrgCode());
            trCtrTask.setPackType(AstPackTradeTypeEnum.AST_TRF_BIZ);
            trCtrTaskMap.put(pack.getPackCode(), trCtrTask);
        }
        Money tmpTrMoney = trCtrTask.getPreTrAmt().add(astPoolItem.getPrinBal());

        if (MoneyUtil.compare(tmpTrMoney, pack.getAvalAmt()) < 1) {
            LogUtil.info(LOGGER,
                    "dealTrCtrTask-false [{0}]Pack筛选到资产，AstNo:{1}-AstPrin:{2}-Pack.tmpTrAmt:{3}-PackAvalAmt:{4}-{5}]",
                    pack.getPackCode(), astPoolItem.getAstNo(), astPoolItem.getPrinBal(), tmpTrMoney, pack.getAvalAmt(),
                    batchNo);
            trCtrTask.addPreTrAmt(astPoolItem.getPrinBal());
            return dealTrTask(astPoolItem, batchNo, acctDate, pack);
        }
        // 日志输出量太大，debug时输出
        LogUtil.debug(LOGGER, "dealTrCtrTask-false [{0}]Pack筛选到资产，AstNo:{1}-AstPrin:{2}-Pack.tmpTrAmt:{3}-PackAvalAmt:{4}-{5}]",
                pack.getPackCode(), astPoolItem.getAstNo(), astPoolItem.getPrinBal(), tmpTrMoney, pack.getAvalAmt(), batchNo);
        return null;

    }

    private TrTask dealTrTask(AstPoolItem astPool, String batchNo, Date acctDate, Pack pack) {
        TrTask task = new TrTask();
        task.setExecTimes(0L);
        task.setTfrDt(acctDate);
        task.setBatchNo(batchNo);
        task.setNextExecTime(SystemDateUtil.getSystemDate());
        task.setAstNo(astPool.getAstNo());
        task.setTfrPack(pack.getPackCode());
        task.setTermNo(astPool.getTermNo());
        task.setAstDueDt(astPool.getEndDate());
        task.setTfrPrinBal(astPool.getPrinBal());
        task.setTaskStatus(TaskStatusEnum.IGNORE);
        task.setTfrOut(astPool.getLenderOrgCode());
        task.setSeqNo(SequenceUtil.genId(IdTypeEnum.SEQ));
        task.setPackType(AstPackTradeTypeEnum.AST_TRF_BIZ);
        return task;
    }
}
