/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.report.wk;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.NumberUtil;
import com.abssqr.plat.biz.shared.report.PlanAstStatAmtUtil;
import com.abssqr.plat.biz.shared.report.ReportItemFilter;
import com.abssqr.plat.biz.shared.report.ReportStatMap;
import com.abssqr.plat.common.dal.mysql.auto.dao.LoanBakDAO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.LoanAstProfitStat;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.LoanBakFormationItem;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanAstReportStat;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanStatItem;
import com.abssqr.plat.common.facade.enums.AstFlowTypeEnum;
import com.abssqr.plat.common.facade.enums.AstRiskClassEnum;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.ReportAstStatTypeEnum;
import com.abssqr.plat.common.facade.enums.ReportDimTypeEnum;
import com.abssqr.plat.common.facade.enums.ReportTitleGroupEnum;
import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.enums.StageChangeTypeEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.DailyReportDateField;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.model.report.ReporterItemCodeKey;
import com.abssqr.plat.common.facade.model.report.Statistics;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.model.domain.plan.PlanAcctTitleLog;
import com.abssqr.plat.common.model.enums.OvdDaysTypeEnum;
import com.abssqr.plat.common.model.enums.TraderTypeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.loanTrans.LoanAstProfitLogRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.core.service.report.ReportItemHolder;
import com.abssqr.plat.core.service.report.calc.impl.CommonItemCalc;
import com.abssqr.plat.core.service.util.RptBuilderUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 日报表统计项生成
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.report: WkReportBuilder.java, v 0.1 2019-06-28 02:19 zhenxuan.luo Exp $
 */
@Component
public class WkAstReportBuilder {
    
    @Autowired
    private ReportItemFilter     reportItemFilter;
    @Autowired
    private PlanReportRepo       planReportRepo;
    @Autowired
    private LoanBakDAO           loanBakDAO;
    @Autowired
    private LoanAstProfitLogRepo loanAstProfitLogRepo;
    
    /**
     * 资产日报
     *
     * @param planBase
     * @param lastReport
     * @param reportField
     * @param statLoanBakBal
     * @param statAmt
     * @param planStatItems
     * @param acctTitleLogMap
     * @param spvDPlanNos spv计划对应的受让计划编号
     * @return
     */
    public List<PlanReportItem> buildReport(PlanBase planBase, PlanReport lastReport,
                                            DailyReportDateField reportField,
                                            List<PlanAstReportStat> statLoanBakBal,
                                            List<PlanAstReportStat> statAmt,
                                            List<PlanAstReportStat> statAccrued,
                                            List<PlanStatItem> planStatItems,
                                            Map<ReportTitleGroupEnum, List<PlanAcctTitleLog>> acctTitleLogMap,
                                            List<String> spvDPlanNos) {
        // 统计会计日
        Date accountDate = reportField.getAccountDate();
        // 余额
        ReportStatMap statLoanBakBalMap = ReportStatMap.buildReportMap(statLoanBakBal);
        // 发生额
        ReportStatMap statAmtMap = ReportStatMap.buildReportMap(statAmt);
        // 每日结息项统计
        ReportStatMap statAccruedMap = ReportStatMap.buildReportMap(statAccrued);
        
        // 清空初始化
        ReportItemHolder.clearHolder();
        
        //受让池报表放款池报表，不同区分开方便以后分开维护，目前能复用考虑的暂时考虑复用
        if (planBase.getPlanType() == PlanTypeEnum.PLAN) {
            //资金分析
            ReportItemHolder.add(this.buildPlanCapitalReport(planBase, accountDate, planStatItems, lastReport));
            
            //交易贷款分析
            ReportItemHolder.add(
                    this.buildTfrPlanReport(planBase.getPlanNo(), accountDate, lastReport, statAmtMap));
        } else if (planBase.getPlanType() == PlanTypeEnum.SPV) {
            //查询受让计划报表
            Map<ReporterItemCodeKey, Statistics> reportEntityMap = Maps.newHashMap();
            if (CollectionUtils.isNotEmpty(spvDPlanNos)) {
                spvDPlanNos.forEach(planNo -> {
                    PlanReport report = planReportRepo.getReport(planNo, ReportTypeEnum.RPT001, reportField.desc());
                    VarChecker.checkNotNull(report,"[{0}]SPV日报生成失败，找不到对应受让计划[{1}].[{2}]日报",
                            planBase.getPlanNo(),planNo,reportField.desc());
                    RptBuilderUtil.recordedData(report.getReportItems(), reportEntityMap);
                });
            }
            //资金分析
            ReportItemHolder.add(
                    this.buildSpvCapitalReport(planBase, accountDate, planStatItems, lastReport, statAmtMap, reportEntityMap, spvDPlanNos));
            //交易贷款分析
            ReportItemHolder.add(
                    this.buildTfrSpvReport(planBase.getPlanNo(), accountDate, lastReport, statAmtMap, reportEntityMap, spvDPlanNos));
        }
        
        // 资产分析  +核销分析  +逾期分析
        ReportItemHolder.add(
                this.buildAstReport(planBase.getPlanNo(), accountDate, lastReport, statLoanBakBalMap));
        
        // 费用及兑付分析
        ReportItemHolder.add(
                this.buildFeeReport(planBase.getPlanNo(), accountDate, lastReport, acctTitleLogMap));
        // 还款分析
        ReportItemHolder.add(
                this.buildRepoReport(planBase.getPlanNo(), accountDate, lastReport, statAmtMap, statAccruedMap));
        // 五级分类余额统计
        ReportItemHolder.add(
                this.buildAstClzReport(planBase.getPlanNo(), accountDate, statLoanBakBalMap));
        
        // 其余统计项计算
        ReportItemHolder.add(
                this.buildRestItemReport(planBase, accountDate, lastReport, acctTitleLogMap));
        
        // 资产结构统计
        ReportItemHolder.add(
                this.buildAstFormationReport(planBase, accountDate));
        
        return ReportItemHolder.getHolder();
    }
    
    /**
     * 受让池资金分析
     *
     * @param planEntity
     * @param accountDate
     * @param planStatItems
     * @param lastReport
     * @return
     */
    private List<PlanReportItem> buildPlanCapitalReport(PlanBase planEntity, Date accountDate,
                                                        List<PlanStatItem> planStatItems,
                                                        PlanReport lastReport) {
        // 当日交易贷款资金发生额
        BigDecimal WK003Amt = BigDecimal.ZERO;
        // 当日还款资金发生额
        BigDecimal WK004Amt = BigDecimal.ZERO;
        // 兑付流出资金发生额
        BigDecimal WK005Amt = BigDecimal.ZERO;
        // 其他流出资金发生额
        BigDecimal WK006Amt = BigDecimal.ZERO;
        // 其他流入资金发生额
        BigDecimal WK007Amt = BigDecimal.ZERO;
        // 总流出资金发生额
        BigDecimal WK008Amt = BigDecimal.ZERO;
        // 总流入资金发生额
        BigDecimal WK009Amt = BigDecimal.ZERO;
        // 支付应付利息
        BigDecimal WK100Amt = BigDecimal.ZERO;
        // 放款冲正
        BigDecimal WK101Amt = BigDecimal.ZERO;
        // 冻结资金
        BigDecimal WK000Amt = BigDecimal.ZERO;
        // 资金流水上的回购资金流出
        BigDecimal WK146Amt = BigDecimal.ZERO;
        // 资金流水上的回购资金流入
        BigDecimal WK147Amt = BigDecimal.ZERO;
        
        // review一下金额计算逻辑 -- 应该是没影响，
        for (PlanStatItem planStatItem : planStatItems) {
            if (planStatItem.getPlanNo() == null) {
                continue;
            }
            TransCodeEnum transCode = TransCodeEnum.getByCode(planStatItem.getTransCode());
            BigDecimal frozenCashAmt = planStatItem.getFrozenCashAmt().getAmount();
            //累计冻结发生额
            WK000Amt = WK000Amt.add(frozenCashAmt);
            // 获取变更水位完成交易的实际发生额，在途的交易金额 发生额都为0
            BigDecimal amt = planStatItem.getCashAmt().getAmount();
            if (BigDecimal.ZERO.compareTo(amt) == 0) {
                continue;
            }
            if (transCode.getFundFlow() == AstFlowTypeEnum.FLOW_OUT) {
                WK008Amt = WK008Amt.add(amt);
                switch (transCode) {
                    case BUY_AST:
                        WK003Amt = WK003Amt.add(amt);
                        break;
                    case ALLOC:
                        WK005Amt = WK005Amt.add(amt);
                        break;
                    case PAY_HANG_INTR:
                        WK100Amt = WK100Amt.add(amt);
                        break;
                    case CAP_ADJ_SUB:
                        WK006Amt = WK006Amt.add(amt);
                        break;
                    case BUY_BACK_IN:
                        // 回购资产流入，资金流出
                        WK146Amt = WK146Amt.add(amt);
                        break;
                    default:
                        throw new AbssqrBizException(
                                MessageUtil.formatMsg("无法识别的计划[{0}]资金处理交易[{1}]", planEntity.getPlanNo(), transCode),
                                CommonErrorCodeEnum.INVALID_INTERNAL_MODEL);
                }
            } else if (transCode.getFundFlow() == AstFlowTypeEnum.FLOW_IN) {
                WK009Amt = WK009Amt.add(amt);
                switch (transCode) {
                    case REPO:
                        WK004Amt = WK004Amt.add(amt);
                        break;
                    case GRANT_REVERSE:
                        WK101Amt = WK101Amt.add(amt);
                        break;
                    case SELL_AST:
                        break;
                    case CAP_ADJ_ADD:
                    case CAP_INVEST:
                        WK007Amt = WK007Amt.add(amt);
                        break;
                    case BUY_BACK_OUT:
                        // 回购资产流出，资金流入
                        WK147Amt = WK147Amt.add(amt);
                        break;
                    default:
                        throw new AbssqrBizException(
                                MessageUtil.formatMsg("无法识别的计划[{0}]资金处理交易[{1}]", planEntity.getPlanNo(), transCode),
                                CommonErrorCodeEnum.INVALID_INTERNAL_MODEL);
                }
            }
        }
        // 上日闲置资金余额
        BigDecimal lastWK001Amt = RptBuilderUtil.getReportVal(lastReport, WkAstReportFields.WK001.getCode());
        // 闲置资金余额 = 上日闲置资金余额 + 总流入 - 总流出
        BigDecimal WK001Amt = lastWK001Amt.add(WK009Amt).subtract(WK008Amt);
        // 冻结金额 - 兑付流出资金发生额 = 冻结金额余额
        WK000Amt = WK000Amt.subtract(WK005Amt);
        List<PlanReportItem> ret = Lists.newArrayList();
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK000, 0L, WK000Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK001, 0L, WK001Amt));
        
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK003, 0L, WK003Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK004, 0L, WK004Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK005, 0L, WK005Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK006, 0L, WK006Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK007, 0L, WK007Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK008, 0L, WK008Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK009, 0L, WK009Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK100, 0L, WK100Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK101, 0L, WK101Amt));
        
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK135, 0L, WK005Amt, lastReport));
        // 弃用： 转让当日应计挂帐利息
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK132, 0L, BigDecimal.ZERO));
        
        // 回购金额的流出
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK146, 0L, WK146Amt));
        // 回购金额的流入
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK147, 0L, WK147Amt));
        return ret;
    }
    
    /**
     * 放款池资金分析
     *
     * @param planEntity
     * @param accountDate
     * @param planStatItems
     * @param lastReport
     * @return
     */
    private List<PlanReportItem> buildSpvCapitalReport(PlanBase planEntity, Date accountDate,
                                                       List<PlanStatItem> planStatItems,
                                                       PlanReport lastReport, ReportStatMap statAmtMap,
                                                       Map<ReporterItemCodeKey, Statistics> reportEntityMap,
                                                       List<String> planNos) {
        
        // 冻结资金
        BigDecimal WK000Amt = BigDecimal.ZERO;
        //贷款还款资金发生额
        BigDecimal WK004Amt = BigDecimal.ZERO;
        //兑付流出资金发生额
        BigDecimal WK005Amt = BigDecimal.ZERO;
        //其他流出资金发生额
        BigDecimal WK006Amt = BigDecimal.ZERO;
        //其他流入资金发生额
        BigDecimal WK007Amt = BigDecimal.ZERO;
        // 支付应付利息
        BigDecimal WK100Amt = BigDecimal.ZERO;
        // 放款冲正
        BigDecimal WK101Amt = BigDecimal.ZERO;
        //转让贷款资金发生额
        BigDecimal WK112Amt = BigDecimal.ZERO;
        //新增资产发生额
        BigDecimal WK111Amt = BigDecimal.ZERO;
        // 回购资金发生额
        BigDecimal WK146Amt = BigDecimal.ZERO;
        for (PlanStatItem planStatItem : planStatItems) {
            if (ToolUtil.isOneEmpty(planStatItem, planStatItem.getPlanNo())) {
                continue;
            }
            StageChangeTypeEnum changeType = StageChangeTypeEnum.getByCode(planStatItem.getChangeType());
            TraderTypeEnum traderType = TraderTypeEnum.getByCode(planStatItem.getTraderType());
            TransCodeEnum transCode = TransCodeEnum.getByCode(planStatItem.getTransCode());
            //累计冻结发生额
            WK000Amt = WK000Amt.add(planStatItem.getFrozenCashAmt().getAmount());
            // 获取变更水位完成交易的实际发生额，在途的交易金额 发生额都为0
            BigDecimal amt = planStatItem.getCashAmt().getAmount();
            if (BigDecimal.ZERO.compareTo(amt) == 0) {
                continue;
            }
            switch (transCode) {
                case BUY_AST:
                    // 放款池 资产交易兑付时只记增加
                    if (changeType == StageChangeTypeEnum.ADD) {
                        WK112Amt = WK112Amt.add(amt);
                    }
                    break;
                case ALLOC:
                    WK005Amt = WK005Amt.add(amt);
                    break;
                case PAY_HANG_INTR:
                    if (changeType == StageChangeTypeEnum.ADD) {
                        WK100Amt = WK100Amt.add(amt);
                    }
                    break;
                case CAP_ADJ_SUB:
                    WK006Amt = WK006Amt.add(amt);
                    break;
                case REPO:
                    if (changeType == StageChangeTypeEnum.ADD) {
                        // 放款池还款总金额 包含承接池还款金额
                        WK004Amt = WK004Amt.add(amt);
                    }else if (changeType == StageChangeTypeEnum.SUB
                              && traderType == TraderTypeEnum.COUNTERPARTY) {
                        // 还款扣减并且为对手方时，还款兑付给承接池需要进行扣减还款金额
                        WK004Amt = WK004Amt.subtract(amt);
                    }
                    break;
                //冲正发生额放款池无水位记录
                case GRANT_REVERSE:
                    break;
                case SELL_AST:
                    break;
                case CAP_ADJ_ADD:
                case CAP_INVEST:
                    WK007Amt = WK007Amt.add(amt);
                    break;
//                case BUY_BACK_IN:
//                    // 针对放款池 从放款户进行回购资金交易，不产生资金流水，不进行统计处理
//                    break;
                default:
                    throw new AbssqrBizException(
                            MessageUtil.formatMsg("无法识别的计划[{0}]资金处理交易[{1}]", planEntity.getPlanNo(), transCode),
                            CommonErrorCodeEnum.INVALID_INTERNAL_MODEL);
            }
        }
        //放款池新增资产统计
        PlanAstReportStat newStat = reportItemFilter.filter(statAmtMap,
                EventCodeEnum.SYNC, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        // 还款补偿
        PlanAstReportStat repoComponsateStat = reportItemFilter.filter(statAmtMap,
                EventCodeEnum.REPAY_COMPONSATE, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        // 发放冲正
        PlanAstReportStat grantStat = reportItemFilter.filter(statAmtMap,
                EventCodeEnum.GRANT_REVERSE, ReportAstStatTypeEnum.ALL,CommonItemCalc.COMMON_VAL);
        // 放款池回购登记 ReportEntity
        PlanAstReportStat buybackStat = reportItemFilter.filter(statAmtMap,
                EventCodeEnum.BACK_OUT, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        
        PlanAstReportStat allNewStat = reportItemFilter.combine(repoComponsateStat, newStat);
        
        //新增贷款统计
        WK111Amt = PlanAstStatAmtUtil.getPrin(allNewStat);
        //放款冲统计
        WK101Amt = PlanAstStatAmtUtil.getPrin(grantStat);
        // 回购买入资产资金流出发生额
        WK146Amt = PlanAstStatAmtUtil.getCost(buybackStat);
        
        //计算总流入= 还款资金发生额+转让贷款资金发生额+其他流入资金发生额+放款冲正+应付利息
        //计算总流入= 还款资金发生额+转让贷款资金发生额+其他流入资金发生额+放款冲正+转让资产应计应收利息+还款服务费
        BigDecimal WK009Amt = WK112Amt.add(WK100Amt).add(WK007Amt).add(WK101Amt).add(WK004Amt);
        //计算总流出=分配兑付+其他流出+放款 + 回购买入资产发生额
        BigDecimal WK008Amt = WK005Amt.add(WK006Amt.add(WK111Amt).add(WK146Amt));
        
        //冻结金额=上一日冻结金额+冻结发生额-分配兑付金额
        WK000Amt = RptBuilderUtil.getReportVal(lastReport, WkAstReportFields.WK000.getCode()).add(WK000Amt).subtract(WK005Amt);
        
        //统计回转服付金额=放款池还款+转让发生额+其他流入+应付利息-预冻结金额
        BigDecimal WK110Amt = WK004Amt.add(WK100Amt).add(WK112Amt).subtract(WK000Amt);
        
        List<PlanReportItem> ret = Lists.newArrayList();
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK000, 0L, WK000Amt));
        
        //闲置资金余额=上一日闲置余额+总流入-总流出
        BigDecimal lastWK001Amt = RptBuilderUtil.getReportVal(lastReport, WkAstReportFields.WK001.getCode());
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK001,
                0L, lastWK001Amt.add(WK009Amt).subtract(WK008Amt)));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK004, 0L, WK004Amt));
        // 兑付流出资金发生额
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK005, 0L, WK005Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK006, 0L, WK006Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK007, 0L, WK007Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK008, 0L, WK008Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK009, 0L, WK009Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK100, 0L, WK100Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK101, 0L, WK101Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK110, 0L, WK110Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK112, 0L, WK112Amt));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK111, 0L, WK111Amt));
        
        // 弃用： 转让当日应计未收利息和 = 各承接池转让当日应计未付挂帐利息和
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK132, reportEntityMap, planNos));
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK135, 0L, WK005Amt, lastReport));
    
        // 回购买入资产资金流出发生额
        ret.add(buildReportItem(planEntity.getPlanNo(), accountDate, WkAstReportFields.WK146, 0L, WK146Amt));
        return ret;
    }
    
    /**
     * 资产分析  +核销分析  +逾期分析
     *
     * @param planNo
     * @param accountDate
     * @param lastPlanReport
     * @param statBalMap  余额
     * @return
     */
    private List<PlanReportItem> buildAstReport(String planNo, Date accountDate, PlanReport lastPlanReport,
                                                ReportStatMap statBalMap) {
        // 正常贷款统计
        PlanAstReportStat statusNorStat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.STATUS,
                LoanStatusEnum.NORMAL.getCode());
        
        // 逾期贷款统计
        PlanAstReportStat statusOvdStat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.STATUS,
                LoanStatusEnum.OVERDUE.getCode());
        
        // 核销贷款统计
        PlanAstReportStat statusWrtStat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.WTF_STATUS,
                YNEnum.Y.getCode());
        
        // 逾期91天以上且核销贷款统计
        PlanAstReportStat ovdWtfStat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.OVD_WTF_STATUS,
                CommonItemCalc.COMMON_VAL);
        // 总贷款
        PlanAstReportStat allStat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.ALL,
                CommonItemCalc.COMMON_VAL);
        // 1-3 本金
        PlanAstReportStat ovd01Stat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.OVD_DAYS,
                OvdDaysTypeEnum.OD01.getCode());
        // 4-30 本金
        PlanAstReportStat ovd02Stat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.OVD_DAYS,
                OvdDaysTypeEnum.OD02.getCode());
        // 31-60 本金
        PlanAstReportStat ovd03Stat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.OVD_DAYS,
                OvdDaysTypeEnum.OD03.getCode());
        // 逾期61天-90天(含)本金余额
        PlanAstReportStat ovd04Stat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.OVD_DAYS,
                OvdDaysTypeEnum.OD04.getCode());
        // 逾期91天-179天本金余额
        PlanAstReportStat ovd05Stat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.OVD_DAYS,
                OvdDaysTypeEnum.OD05.getCode());
        // 逾期180天及以上本金余额
        PlanAstReportStat ovd06Stat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.OVD_DAYS,
                OvdDaysTypeEnum.OD06.getCode());
        
        // 获取当日统计新增放款或转入贷款收益金额 通过loanBak的cost 获取所有资产成本
        PlanAstReportStat profitAmt = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.ALL,
                ReportDimTypeEnum.ACCRUED.getCode());
        
        List<PlanReportItem> ret = Lists.newArrayList();
        // 正常贷款本金余额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK010,
                statusNorStat.getAstChg(), PlanAstStatAmtUtil.getPrin(statusNorStat)));
        // 正常贷款利息余额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK011,
                statusNorStat.getAstChg(), PlanAstStatAmtUtil.getAllIntr(statusNorStat)));
        // 逾期贷款本金余额
        PlanReportItem ovdPrin = buildReportItem(planNo, accountDate, WkAstReportFields.WK012,
                statusOvdStat.getAstChg(), PlanAstStatAmtUtil.getPrin(statusOvdStat));
        ret.add(ovdPrin);
        // 逾期贷款利息余额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK013,
                statusOvdStat.getAstChg(), PlanAstStatAmtUtil.getAllIntr(statusOvdStat)));
        // 逾期贷款罚息余额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK014,
                statusOvdStat.getAstChg(), PlanAstStatAmtUtil.getAllPny(statusOvdStat)));
        // 核销贷款本金余额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK015,
                statusWrtStat.getAstChg(), PlanAstStatAmtUtil.getPrin(statusWrtStat)));
        // 核销贷款利息余额（含罚息）
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK016,
                statusWrtStat.getAstChg(), PlanAstStatAmtUtil.getAllIntrWithPny(statusWrtStat)));
        // 核销贷款利息余额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WKExt002,
                statusWrtStat.getAstChg(), PlanAstStatAmtUtil.getAllIntr(statusWrtStat)));
        // 核销贷款罚息余额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WKExt003,
                statusWrtStat.getAstChg(), PlanAstStatAmtUtil.getAllPny(statusWrtStat)));
        // 总贷款本金余额
        PlanReportItem allPrin = buildReportItem(planNo, accountDate, WkAstReportFields.WK017,
                allStat.getAstChg(), PlanAstStatAmtUtil.getPrin(allStat));
        ret.add(allPrin);
        // 总贷款利息余额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK018,
                allStat.getAstChg(), PlanAstStatAmtUtil.getAllIntr(allStat)));
        // 总贷款罚息余额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK019,
                allStat.getAstChg(), PlanAstStatAmtUtil.getAllPny(allStat)));
        
        // 累计资产应计年化收益额
        PlanReportItem profitItem = buildReportItem(planNo, accountDate, WkAstReportFields.WK128,
                profitAmt.getAstChg(), PlanAstStatAmtUtil.getAllIntr(profitAmt));
        ret.add(profitItem);
        // 加权收益率 = WK128 Σ（每笔借据本金*该笔借据年化收益率）/ 对应资产总本金
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK129, 0L,
                CalcUtil.divide(profitItem.getAmt(), profitAmt.getPrinAmt().getAmount())));
        
        // 逾期1天-3天（含）本金余额
        PlanReportItem ovd01StatPrin = buildReportItem(planNo, accountDate, WkAstReportFields.WK021,
                ovd01Stat.getAstChg(), PlanAstStatAmtUtil.getPrin(ovd01Stat));
        ret.add(ovd01StatPrin);
        // 逾期1天-30天(含)本金余额  1-3 加 4- 30
        PlanReportItem ovd02StatPrin = buildReportItem(planNo, accountDate, WkAstReportFields.WK022,
                NumberUtil.add(ovd01Stat.getAstChg(), ovd02Stat.getAstChg()).longValue(),
                NumberUtil.add(PlanAstStatAmtUtil.getPrin(ovd01Stat), PlanAstStatAmtUtil.getPrin(ovd02Stat)));
        ret.add(ovd02StatPrin);
        // 逾期31天-60天(含)本金余额
        PlanReportItem ovd03StatPrin = buildReportItem(planNo, accountDate, WkAstReportFields.WK023,
                ovd03Stat.getAstChg(), PlanAstStatAmtUtil.getPrin(ovd03Stat));
        ret.add(ovd03StatPrin);
        // 逾期61天-90天(含)本金余额
        PlanReportItem ovd04StatPrin =buildReportItem(planNo, accountDate, WkAstReportFields.WK024,
                ovd04Stat.getAstChg(), PlanAstStatAmtUtil.getPrin(ovd04Stat));
        ret.add(ovd04StatPrin);
        
        // 逾期91天及以上本金余额  91-179 加 180以上
        PlanReportItem ovd05StatPrin = buildReportItem(planNo, accountDate, WkAstReportFields.WK025,
                NumberUtil.add(ovd05Stat.getAstChg(), ovd06Stat.getAstChg()).longValue(),
                NumberUtil.add(PlanAstStatAmtUtil.getPrin(ovd05Stat), PlanAstStatAmtUtil.getPrin(ovd06Stat)));
        ret.add(ovd05StatPrin);
        // 逾期180天及以上本金余额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK118,
                ovd06Stat.getAstChg(), PlanAstStatAmtUtil.getPrin(ovd06Stat)));
        // 逾期91天及以上非核销本金余额
        PlanReportItem ovdWtfStatPrin = buildReportItem(planNo, accountDate, WkAstReportFields.WKExt001,
                ovdWtfStat.getAstChg(), PlanAstStatAmtUtil.getPrin(ovdWtfStat));
        ret.add(ovdWtfStatPrin);
        
        /*** 计算分级逾期率 ***/
        // 整体逾期率
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK020, ovdPrin.getAstCnt(),
                CalcUtil.divide(ovdPrin.getAmt(), allPrin.getAmt())));
        // M3+以上逾期率
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK026, ovd05StatPrin.getAstCnt(),
                CalcUtil.divide(ovd05StatPrin.getAmt(), allPrin.getAmt())));
        // 整体逾期率（不含宽限期）
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK120, ovdPrin.getAstCnt(),
                CalcUtil.divide(ovdPrin.getAmt().subtract(ovd01StatPrin.getAmt()), allPrin.getAmt())));
        // m0逾期率
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK121, ovd01StatPrin.getAstCnt(),
                CalcUtil.divide(ovd01StatPrin.getAmt(), allPrin.getAmt())));
        // m1逾期率
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK122, ovd02StatPrin.getAstCnt(),
                CalcUtil.divide(ovd02StatPrin.getAmt(), allPrin.getAmt())));
        // m1+逾期率
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK123,
                NumberUtil.add(ovd03StatPrin.getAstCnt(), ovd04StatPrin.getAstCnt(), ovd05StatPrin.getAstCnt()).longValue(),
                CalcUtil.divide(NumberUtil.add(ovd03StatPrin.getAmt(), ovd04StatPrin.getAmt(), ovd05StatPrin.getAmt()),
                        allPrin.getAmt())));
        // m2逾期率
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK124, ovd03StatPrin.getAstCnt(),
                CalcUtil.divide(ovd03StatPrin.getAmt(), allPrin.getAmt())));
        // m3逾期率
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK125, ovd04StatPrin.getAstCnt(),
                CalcUtil.divide(ovd04StatPrin.getAmt(), allPrin.getAmt())));
        // M6+逾期率
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK126, ovd06Stat.getAstChg(),
                CalcUtil.divide(PlanAstStatAmtUtil.getPrin(ovd06Stat), allPrin.getAmt())));
        
        /**
         * 发生额 = 今日余额 - 昨日报表余额
         */
        // 当日逾期本金发生额
        ret.add(RptBuilderUtil.buildDiffReportItem(planNo, accountDate, WkAstReportFields.WK049,
                statusOvdStat.getAstChg(), PlanAstStatAmtUtil.getPrin(statusOvdStat), lastPlanReport));
        // 当日逾期利息发生额
        ret.add(RptBuilderUtil.buildDiffReportItem(planNo, accountDate, WkAstReportFields.WK050,
                statusOvdStat.getAstChg(), PlanAstStatAmtUtil.getAllIntr(statusOvdStat), lastPlanReport));
        // 当日核销本金发生额
        ret.add(RptBuilderUtil.buildDiffReportItem(planNo, accountDate, WkAstReportFields.WK051,
                statusWrtStat.getAstChg(), PlanAstStatAmtUtil.getPrin(statusWrtStat), lastPlanReport));
        // 当日核销利息发生额（含罚息）
        ret.add(RptBuilderUtil.buildDiffReportItem(planNo, accountDate, WkAstReportFields.WK052,
                statusWrtStat.getAstChg(), PlanAstStatAmtUtil.getAllIntrWithPny(statusWrtStat), lastPlanReport));
        
        return ret;
    }
    
    /**
     * 受让池交易贷款分析
     *
     * @param planNo
     * @param accountDate
     * @param lastPlanReport
     * @param statAmtMap
     * @return
     */
    private List<PlanReportItem> buildTfrPlanReport(String planNo, Date accountDate, PlanReport lastPlanReport,
                                                    ReportStatMap statAmtMap) {
        PlanAstReportStat tfrStat = reportItemFilter.filter(statAmtMap,
                EventCodeEnum.TRANS_IN, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        /**
         * 由于 LoanAstTrManager 中 使用TRANS_OUT 作为转让事件码 所有由原 TRANS_IN 改为 TRANS_OUT
         * 兼容 如果 TRANS_OUT 没有时获取TRANS_IN
         */
        if (BigDecimal.ZERO.compareTo(PlanAstStatAmtUtil.getPrin(tfrStat)) == 0){
            tfrStat = reportItemFilter.filter(statAmtMap,
                    EventCodeEnum.TRANS_OUT, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        }
        
        List<PlanReportItem> ret = Lists.newArrayList();
        // 累计交易贷款发生
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK027, tfrStat.getAstChg(),
                PlanAstStatAmtUtil.getCost(tfrStat), lastPlanReport));
        // 当日交易贷款发生
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK028, tfrStat.getAstChg(),
                PlanAstStatAmtUtil.getCost(tfrStat)));
        
        // 累计交易贷款本金
        PlanReportItem totalTrf = buildReportItem(planNo, accountDate, WkAstReportFields.WK102, tfrStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(tfrStat), lastPlanReport);
        ret.add(totalTrf);
        // 当日交易贷款本金
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK103, tfrStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(tfrStat)));
    
        // 回购卖出资产 对应发生额与本金
        PlanAstReportStat buyBackStat = reportItemFilter.filter(statAmtMap,
                EventCodeEnum.BACK_OUT, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        // 累计回购贷款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK144, buyBackStat.getAstChg(),
                PlanAstStatAmtUtil.getCost(buyBackStat), lastPlanReport));
        // 当日回购贷款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK145, buyBackStat.getAstChg(),
                PlanAstStatAmtUtil.getCost(buyBackStat)));
        // 累计回购贷款本金
        ret.add( buildReportItem(planNo, accountDate, WkAstReportFields.WK142, buyBackStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(buyBackStat), lastPlanReport));
        // 当日回购贷款本金
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK143, buyBackStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(buyBackStat)));
        return ret;
    }
    
    /**
     * 放款池交易贷款分析
     *
     * @param spvNo
     * @param accountDate
     * @param lastPlanReport
     * @param statAmtMap
     * @param planNos       对应受让计划编号
     * @return
     */
    private List<PlanReportItem> buildTfrSpvReport(String spvNo, Date accountDate, PlanReport lastPlanReport,
                                                   ReportStatMap statAmtMap,
                                                   Map<ReporterItemCodeKey, Statistics> reportEntityMap,
                                                   List<String> planNos) {
        //增量同步
        PlanAstReportStat grantStat = reportItemFilter.filter(statAmtMap,
                EventCodeEnum.SYNC, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        //还款补偿，当天放款当天还款数据
        PlanAstReportStat repoComponsateStat = reportItemFilter.filter(statAmtMap,
                EventCodeEnum.REPAY_COMPONSATE, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        
        //新增贷款发生额统计 = 增量同步 + 还款补偿
        PlanAstReportStat allPreRepay = reportItemFilter.combine(grantStat, repoComponsateStat);
        
        List<PlanReportItem> ret = Lists.newArrayList();
        
        // 累计交易贷款发生
        ret.add(buildReportItem(spvNo, accountDate, WkAstReportFields.WK027, reportEntityMap, planNos));
        // 当日交易贷款发生
        ret.add(buildReportItem(spvNo, accountDate, WkAstReportFields.WK028, reportEntityMap, planNos));
        // 累计交易贷款本金
        ret.add(buildReportItem(spvNo, accountDate, WkAstReportFields.WK102, reportEntityMap, planNos));
        // 当日交易贷款本金
        ret.add(buildReportItem(spvNo, accountDate, WkAstReportFields.WK103, reportEntityMap, planNos));
        
        // 累计新增贷款
        PlanReportItem totalGrant = buildReportItem(spvNo, accountDate, WkAstReportFields.WK116, grantStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allPreRepay), lastPlanReport);
        ret.add(totalGrant);
        // 当日新增资产
        ret.add(buildReportItem(spvNo, accountDate, WkAstReportFields.WK115, grantStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allPreRepay)));
    
        // 回购买入资产 对应发生额与本金
        PlanAstReportStat buyBackStat = reportItemFilter.filter(statAmtMap,
                EventCodeEnum.BACK_OUT, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        // 累计回购贷款发生额
        ret.add(buildReportItem(spvNo, accountDate, WkAstReportFields.WK144, buyBackStat.getAstChg(),
                PlanAstStatAmtUtil.getCost(buyBackStat), lastPlanReport));
        // 当日回购贷款发生额
        ret.add(buildReportItem(spvNo, accountDate, WkAstReportFields.WK145, buyBackStat.getAstChg(),
                PlanAstStatAmtUtil.getCost(buyBackStat)));
        // 累计回购贷款本金
        ret.add( buildReportItem(spvNo, accountDate, WkAstReportFields.WK142, buyBackStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(buyBackStat), lastPlanReport));
        // 当日回购贷款本金
        ret.add(buildReportItem(spvNo, accountDate, WkAstReportFields.WK143, buyBackStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(buyBackStat)));
        return ret;
    }
    
    /**
     * 费用分析
     * @param planNo
     * @param accountDate
     * @param lastReport
     * @param acctTitleLogMap
     * @return
     */
    private List<PlanReportItem> buildFeeReport(String planNo, Date accountDate, PlanReport lastReport,
                                                Map<ReportTitleGroupEnum, List<PlanAcctTitleLog>> acctTitleLogMap) {
        List<PlanReportItem> ret = Lists.newArrayList();
        
        // 获取金融服务费
        Money finFeeAmt = RptBuilderUtil.getAcctTitleMoney(acctTitleLogMap, ReportTitleGroupEnum.FIN_SER_FEE);
        // 当日金融服务费-->笔数逻辑
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK108, finFeeAmt.isZero() ? 0L : 1L,
                finFeeAmt.getAmount()));
        // 累计金融服务费-->如果发生则记一笔，如果没发生则不记一笔
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK109, finFeeAmt.isZero() ? 0L : 1L,
                finFeeAmt.getAmount(), lastReport));
        
        // 还款服务费  针对放款池统计时会有 ， 承接池没有对应数据
        LoanAstProfitStat repoFee = loanAstProfitLogRepo.getRepayOrgDistPftAmt(accountDate, planNo, null);
        // 结息还款服务费
        Money repayIntrFeeAmt = repoFee != null ? repoFee.getRepayIntrFeeAmt() : new Money();
        // 实还还款服务费
        Money fundRepayIntrFeeAmt = repoFee != null ? repoFee.getFundRepayIntrFeeAmt() : new Money();
        // 当日结息还款服务费
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK136,   0L,
                repayIntrFeeAmt.getAmount()));
        // 累计结息还款服务费
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK137,  0L,
                repayIntrFeeAmt.getAmount(), lastReport));
        
        // 当日实还还款服务费
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK138,   0L,
                fundRepayIntrFeeAmt.getAmount()));
        // 累计实还还款服务费
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK139,  0L,
                fundRepayIntrFeeAmt.getAmount(), lastReport));
    
        return ret;
    }
    
    /**
     * 还款分析
     *
     * @param planNo
     * @param accountDate
     * @param lastPlanReport
     * @param statAmtMap
     * @return
     */
    private List<PlanReportItem> buildRepoReport(String planNo, Date accountDate, PlanReport lastPlanReport,
                                                 ReportStatMap statAmtMap, ReportStatMap statAccrued ) {
        // 到期还款
        PlanAstReportStat allDueRepayStat = reportItemFilter.filter(
                statAmtMap, EventCodeEnum.DUE_REPAY, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        // 逾期还款
        PlanAstReportStat allOvdRepayStat = reportItemFilter.filter(
                statAmtMap, EventCodeEnum.ODUE_REPAY, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        // 提前还款
        PlanAstReportStat allPreRepayStat = reportItemFilter.filter(
                statAmtMap, EventCodeEnum.PRE_REPAY, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        // 还款补偿发生不分类
        PlanAstReportStat allRepayComponsate = reportItemFilter.filter(
                statAmtMap, EventCodeEnum.REPAY_COMPONSATE, ReportAstStatTypeEnum.ALL, CommonItemCalc.COMMON_VAL);
        // 到期还款 核销
        PlanAstReportStat wrtDueRepayStat = reportItemFilter.filter(
                statAmtMap, EventCodeEnum.DUE_REPAY, ReportAstStatTypeEnum.WTF_STATUS, YNEnum.N.getCode());
        PlanAstReportStat wrtOvdRepayStat = reportItemFilter
                                                    .filter(statAmtMap, EventCodeEnum.ODUE_REPAY,
                                                            ReportAstStatTypeEnum.WTF_STATUS,
                                                            YNEnum.N.getCode());
        PlanAstReportStat wrtPreRepayStat = reportItemFilter
                                                    .filter(statAmtMap, EventCodeEnum.PRE_REPAY,
                                                            ReportAstStatTypeEnum.WTF_STATUS,
                                                            YNEnum.N.getCode());
        //到期发生统计--》结息。转让-->增量记结息，存量记结息,同时可能发生转让
        PlanAstReportStat allDueStat = reportItemFilter
                                               .filter(statAccrued, null,
                                                       ReportAstStatTypeEnum.DUE,
                                                       LoanStatusEnum.OVERDUE.getCode(),
                                                       LoanStatusEnum.NORMAL.getCode());
        
        //提前还款--内容组成:还款补偿+提前还款
        PlanAstReportStat allPreRepay = reportItemFilter.combine(allRepayComponsate, allPreRepayStat);
        //所有还款记录:到期还款,提前还款,逾期还款
        PlanAstReportStat allRepay = reportItemFilter.combine(allDueRepayStat, allOvdRepayStat, allPreRepay);
        //核销资产:到期还款，提前还款，逾期还款
        PlanAstReportStat allWrtRepay = reportItemFilter.combine(wrtDueRepayStat, wrtOvdRepayStat, wrtPreRepayStat);
        
        List<PlanReportItem> ret = Lists.newArrayList();
        // 累计已还本金发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK029, allRepay.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allRepay), lastPlanReport));
        // 累计已还利息发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK030, allRepay.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(allRepay), lastPlanReport));
        // 累计已还罚息发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK031, allRepay.getAstChg(),
                PlanAstStatAmtUtil.getAllPny(allRepay), lastPlanReport));
        
        // 累计提前还款本金发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK032, allPreRepay.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allPreRepay), lastPlanReport));
        // 累计提前还款利息发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK033, allPreRepay.getAstChg(),
                PlanAstStatAmtUtil.getAllIntrWithPny(allPreRepay), lastPlanReport));
        
        // 累计逾期本金还款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK034, allOvdRepayStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allOvdRepayStat), lastPlanReport));
        // 累计逾期利息还款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK035, allOvdRepayStat.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(allOvdRepayStat), lastPlanReport));
        // 累计逾期罚息还款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK036, allOvdRepayStat.getAstChg(),
                PlanAstStatAmtUtil.getAllPny(allOvdRepayStat), lastPlanReport));
        
        // 累计核销本金还款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK037, allWrtRepay.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allWrtRepay), lastPlanReport));
        // 累计核销利息还款发生额（含罚息）
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK038, allWrtRepay.getAstChg(),
                PlanAstStatAmtUtil.getAllIntrWithPny(allWrtRepay), lastPlanReport));
        
        // 当日已还本金发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK039, allRepay.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allRepay)));
        // 当日已还利息发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK040, allRepay.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(allRepay)));
        // 当日已还罚息发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK041, allRepay.getAstChg(),
                PlanAstStatAmtUtil.getAllPny(allRepay)));
        
        // 当日提前还款本金发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK042, allPreRepay.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allPreRepay)));
        // 当日提前还款利息发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK043, allPreRepay.getAstChg(),
                PlanAstStatAmtUtil.getAllIntrWithPny(allPreRepay)));
        
        // 当日逾期本金还款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK044, allOvdRepayStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allOvdRepayStat)));
        // 当日逾期利息还款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK045, allOvdRepayStat.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(allOvdRepayStat)));
        // 当日逾期罚息还款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK046, allOvdRepayStat.getAstChg(),
                PlanAstStatAmtUtil.getAllPny(allOvdRepayStat)));
        
        // 当日核销本金还款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK047, allWrtRepay.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allWrtRepay)));
        // 当日核销利息还款发生额（含罚息）
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK048, allWrtRepay.getAstChg(),
                PlanAstStatAmtUtil.getAllIntrWithPny(allWrtRepay)));
        
        //  当日计划应还本金
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK104, allDueStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allDueStat)));
        //  当日计划应还利息
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK105, allDueStat.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(allDueStat)));
        
        //  当日应还已还本金发生额--》应还应该归属到期本金发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK106, allDueRepayStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allDueRepayStat)));
        //  当日应还已还利息发生额--》应还应该归属到期利息发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK107, allDueRepayStat.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(allDueRepayStat)));
        
        // 当日核销利息还款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WKExt006, allWrtRepay.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(allWrtRepay)));
        // 当日核销罚息还款发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WKExt007, allWrtRepay.getAstChg(),
                PlanAstStatAmtUtil.getAllPny(allWrtRepay)));
        
        // 当日到期还款本金发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WKExt004, allDueRepayStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(allDueRepayStat)));
        // 当日到期还款利息发生额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WKExt005, allDueRepayStat.getAstChg(),
                PlanAstStatAmtUtil.getAllIntrWithPny(allDueRepayStat)));
    
        // 当日提前还款违约金
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK140, allPreRepay.getAstChg(),
                PlanAstStatAmtUtil.getOtherFeeAmt(allPreRepay)));
        // 累计提前还款违约金
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK141, allPreRepay.getAstChg(),
                PlanAstStatAmtUtil.getOtherFeeAmt(allPreRepay), lastPlanReport));
        return ret;
    }
    
    /**
     * 五级分类分析
     *
     * @param planNo
     * @param accountDate
     * @param statBalMap
     * @return
     */
    private List<PlanReportItem> buildAstClzReport(String planNo, Date accountDate, ReportStatMap statBalMap) {
        // 正常贷款统计
        PlanAstReportStat norlStat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.AST_CLZ,
                AstRiskClassEnum.NORMAL.getCode());
        // 关注
        PlanAstReportStat attnStat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.AST_CLZ,
                AstRiskClassEnum.ATTENTION.getCode());
        // 次级
        PlanAstReportStat secdStat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.AST_CLZ,
                AstRiskClassEnum.SECONDARY.getCode());
        // 可疑
        PlanAstReportStat suspStat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.AST_CLZ,
                AstRiskClassEnum.SUSPICIOUS.getCode());
        // 损失
        PlanAstReportStat lossStat = reportItemFilter.filter(statBalMap, null, ReportAstStatTypeEnum.AST_CLZ,
                AstRiskClassEnum.LOSS.getCode());
        
        List<PlanReportItem> ret = Lists.newArrayList();
        // 正常
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK911, norlStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(norlStat)));
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK912, norlStat.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(norlStat)));
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK913, norlStat.getAstChg(),
                PlanAstStatAmtUtil.getAllPny(norlStat)));
        // 关注
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK921, attnStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(attnStat)));
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK922, attnStat.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(attnStat)));
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK923, attnStat.getAstChg(),
                PlanAstStatAmtUtil.getAllPny(attnStat)));
        // 次级
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK931, secdStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(secdStat)));
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK932, secdStat.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(secdStat)));
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK933, secdStat.getAstChg(),
                PlanAstStatAmtUtil.getAllPny(secdStat)));
        // 可疑
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK941, suspStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(suspStat)));
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK942, suspStat.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(suspStat)));
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK943, suspStat.getAstChg(),
                PlanAstStatAmtUtil.getAllPny(suspStat)));
        // 损失
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK951, lossStat.getAstChg(),
                PlanAstStatAmtUtil.getPrin(lossStat)));
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK952, lossStat.getAstChg(),
                PlanAstStatAmtUtil.getAllIntr(lossStat)));
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK953, lossStat.getAstChg(),
                PlanAstStatAmtUtil.getAllPny(lossStat)));
        return ret;
    }
    
    /**
     * 其余统计项
     * 通过线程暂存统计结果进行计算得出
     * @param planBase
     * @param accountDate
     * @param lastPlanReport
     * @return
     */
    private List<PlanReportItem> buildRestItemReport(PlanBase planBase, Date accountDate, PlanReport lastPlanReport,
                                                     Map<ReportTitleGroupEnum, List<PlanAcctTitleLog>> acctTitleLogMap ) {
        String planNo = planBase.getPlanNo();
        PlanTypeEnum planType = planBase.getPlanType();
        List<PlanReportItem> ret = Lists.newArrayList();
        
        // 关于当日投入资本的兑付本金
        Money allocPrin = RptBuilderUtil.getAcctTitleMoney(acctTitleLogMap, ReportTitleGroupEnum.INVESTED_DAY);
        // 计划当日兑付本金金额
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WKPlan001,0L, allocPrin.getAmount()));
        // 计划累计兑付本金金额
        PlanReportItem WKPlan002AmtItem = buildReportItem(planNo, accountDate, WkAstReportFields.WKPlan002,
                0L, allocPrin.getAmount(), lastPlanReport);
        ret.add(WKPlan002AmtItem);
        
        // 不良率 = WK025逾期91天及以上本金余额 / 承接池：WK102累计交易贷款本金｜放款池：WK116累计新增贷款发生额
        BigDecimal denominator = ReportItemHolder.getPlanReportItemHolder(planNo,
                PlanTypeEnum.PLAN == planType ? WkAstReportFields.WK102 : WkAstReportFields.WK116);
        BigDecimal ovd05StatPrin = ReportItemHolder.getPlanReportItemHolder(planNo, WkAstReportFields.WK025);
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK127, 0L,
                CalcUtil.divide(ovd05StatPrin, denominator)));
        
//        // 加权收益率 = WK128 Σ（每笔借据本金*该笔借据年化收益率）/ 承接池：WK102累计交易贷款本金｜放款池：WK116累计新增贷款发生额
//        BigDecimal profitAmt = ReportItemHolder.getPlanReportItemHolder(planNo, WkAstReportFields.WK128);
//        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK129, 0L,
//                CalcUtil.divide(profitAmt, denominator)));
        
        // 实际闲置率 = WK001账户余额 /（层级累计规模 - WKPlan002计划累计兑付本金金额）
        BigDecimal WK001ItemAmt = ReportItemHolder.getPlanReportItemHolder(planNo, WkAstReportFields.WK001);
        Money investedBal = RptBuilderUtil.getInvestedBal(planBase, WKPlan002AmtItem.getAmt());
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK002,
                0L, CalcUtil.divide(WK001ItemAmt, investedBal.getAmount())));
        
        // 已实现收益额
        Money WK130Amt =  PlanTypeEnum.PLAN == planType ?
                                  RptBuilderUtil.getPlanProfitAmt(planBase) :
                                  RptBuilderUtil.getSpvProfitAmt(planBase);
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK130, 0L, WK130Amt.getAmount()));
        // 历史本金兑付流水
        List<PlanAcctTitleLog> acctTitleLogs = acctTitleLogMap.get(ReportTitleGroupEnum.INVESTED_ALL);
        // 废弃 ((已实现收益额-保证金）*365 + 保证金）/Σ（各信托层级*存续期）
        // 已实现收益率 改为 ((已实现收益额）*365）/Σ（(各信托层级-保证金)*对应层级存续期）
        ret.add(buildReportItem(planNo, accountDate, WkAstReportFields.WK131, 0L,
                CalcUtil.divide(WK130Amt.multiply(planBase.getCalcIntrType().getAnnualDays()).getAmount(),
                        RptBuilderUtil.getInvestedWeight(accountDate, planBase, acctTitleLogs, true).getAmount())));
        return ret;
    }
    
    /**
     * 资产结构统计
     * @param planBase
     * @param accountDate
     * @return
     */
    private List<PlanReportItem> buildAstFormationReport(PlanBase planBase, Date accountDate){
        List<PlanReportItem> ret = Lists.newArrayList();
        
        // 剩余结构
        List<LoanBakFormationItem> astItems = loanBakDAO.queryAstFormation(accountDate, planBase.getPlanNo());
        CollectionUtil.addAll(ret, RptBuilderUtil.convertFormation(
                planBase.getPlanNo(), accountDate, astItems, false));
        
        // 产品结构
        List<LoanBakFormationItem> prodItems =loanBakDAO.queryProdFormation(accountDate, planBase.getPlanNo());
        CollectionUtil.addAll(ret, RptBuilderUtil.convertFormation(
                planBase.getPlanNo(), accountDate, prodItems, true));
        return ret;
    }
    
    
    /**
     * 构建统计项
     *
     * @param planNo
     * @param accountDate
     * @param field
     * @param cnt
     * @param amt
     * @return
     */
    private PlanReportItem buildReportItem(String planNo, Date accountDate, ReportField field,
                                           Long cnt, BigDecimal amt) {
        return RptBuilderUtil.buildReportItem(planNo, accountDate, field, cnt, amt);
    }
    
    /**
     * 构建累计统计项
     *
     * @param planNo
     * @param accountDate
     * @param field
     * @param cnt
     * @param amt
     * @param lastPlanReport
     * @return
     */
    private PlanReportItem buildReportItem(String planNo, Date accountDate, ReportField field, long cnt,
                                           BigDecimal amt, PlanReport lastPlanReport) {
        
        if (lastPlanReport != null) {
            cnt = cnt + RptBuilderUtil.getReportCnt(lastPlanReport, field.getCode());
            amt = amt.add(RptBuilderUtil.getReportVal(lastPlanReport, field.getCode()));
        }
        return buildReportItem(planNo, accountDate, field, cnt, amt);
    }
    
    /**
     * 放款池对应承接池 统计项合并计算
     * @param spv
     * @param accountDate
     * @param field
     * @param reportEntityMap
     * @param plans
     * @return
     */
    private PlanReportItem buildReportItem(String spv, Date accountDate, ReportField field,
                                           Map<ReporterItemCodeKey, Statistics> reportEntityMap,
                                           List<String> plans) {
        BigDecimal amt = BigDecimal.ZERO;
        Long cnt = 0L;
        return buildReportItem(spv, accountDate, field, cnt, amt, reportEntityMap, plans, null);
    }
    
    /**
     * 放款池对应承接池 统计项合并计算并累计
     * @param spv
     * @param accountDate
     * @param field
     * @param reportEntityMap
     * @param plans
     * @return
     */
    private PlanReportItem buildReportItem(String spv, Date accountDate, ReportField field,
                                           long cnt, BigDecimal amt,
                                           Map<ReporterItemCodeKey, Statistics> reportEntityMap,
                                           List<String> plans, PlanReport lastPlanReport) {
        //1、需要构建key
        //2、get values组成amt和cnt
        //3、如果有多个受让计划，则需要循环求和
        //4、循环结束后进行add，
        //5、构建统计项
        if (CollectionUtils.isNotEmpty(plans)) {
            for (String plan : plans) {
                ReporterItemCodeKey reporterKey = new ReporterItemCodeKey(plan, field.getCode());
                Statistics value = reportEntityMap.get(reporterKey);
                if (value != null) {
                    amt = amt.add(value.getAmt());
                    cnt = cnt + value.getCnt();
                }
            }
        }
        if(lastPlanReport != null){
            return buildReportItem(spv, accountDate, field, cnt, amt, lastPlanReport);
        }else{
            return buildReportItem(spv, accountDate, field, cnt, amt);
        }
    }
}