/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.shared.report.wk;

import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.NumberUtil;
import com.abssqr.plat.common.dal.mysql.auto.dao.LoanBakDAO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PropCntStat;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PropDisbStat;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.IntrTypeEnum;
import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.enums.TitleTypeEnum;
import com.abssqr.plat.common.facade.model.plan.InvestStructure;
import com.abssqr.plat.common.facade.model.plan.PlanAcctTitle;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.manRpt.AmtPropCnt;
import com.abssqr.plat.common.facade.model.report.manRpt.AmtPropCntGroup;
import com.abssqr.plat.common.facade.model.report.manRpt.AstRpt;
import com.abssqr.plat.common.facade.model.report.manRpt.CashRpt;
import com.abssqr.plat.common.facade.model.report.manRpt.CommonPropStat;
import com.abssqr.plat.common.facade.model.report.manRpt.CommonPropStatGroup;
import com.abssqr.plat.common.facade.model.report.manRpt.FeeRpt;
import com.abssqr.plat.common.facade.model.report.manRpt.GrantRpt;
import com.abssqr.plat.common.facade.model.report.manRpt.InvestInfoRpt;
import com.abssqr.plat.common.facade.model.report.manRpt.PlanCommonRpt;
import com.abssqr.plat.common.facade.model.report.manRpt.RepoRpt;
import com.abssqr.plat.common.facade.model.report.manRpt.RiskClzRpt;
import com.abssqr.plat.common.facade.model.report.manRpt.TfrRpt;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.abssqr.plat.common.facade.model.report.wk.WkManReportFields;
import com.abssqr.plat.common.facade.model.report.wk.WkManagerReport;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.model.domain.plan.PlanAcctTitleLog;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.core.service.util.RptBuilderUtil;
import com.alibaba.fastjson.JSON;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.report.wk: WkManagerReportBuilder.java, v 0.1 2019-07-01 01:39 zhenxuan.luo Exp $ 
 */
@Component
public class WkManagerReportBuilder {
    
    private static List<Integer> AGE_SPLIT = Lists.newArrayList(22, 34, 44, 55);
    
    private static Integer LOC_CNT = 10;
    
    @Autowired
    private PlanReportRepo planReportRepo;
    @Autowired
    private LoanBakDAO loanBakDAO;
    
    /**
     * 供报表下载用，合并跨周期的计划报表
     *
     * @param planBase
     * @param startRptDt
     * @param endRptDt
     * @return
     */
    public WkManagerReport getReport(PlanBase planBase, String startRptDt, String endRptDt) {
        
        List<PlanReport> wkManagerReports = planReportRepo.getPlansByDate(planBase.getPlanNo(), startRptDt, endRptDt,
                ReportTypeEnum.RPT002);
        
        if (CollectionUtils.isEmpty(wkManagerReports)) {
            return null;
        }
        
        WkManagerReport combinedReport = new WkManagerReport();
        
        if (wkManagerReports.size() == 1) {
            combinedReport = JSON.parseObject(wkManagerReports.get(0).getDetail(), WkManagerReport.class);
        } else {
            Collections.sort(wkManagerReports, Comparator.comparing(item -> item.getRptDate().desc()));
            
            combinedReport.setPlanNo(planBase.getPlanNo());
            combinedReport.setPlanName(planBase.getPlanName());
            
            BigDecimal avgProfitTotal = BigDecimal.ZERO;
            BigDecimal dailyAvgPrinTotal = BigDecimal.ZERO;
            BigDecimal dailyAvgPrinExtTotal = BigDecimal.ZERO;
            
            for (int i = 0; i < wkManagerReports.size(); i++) {
                WkManagerReport wkManagerReport = JSON.parseObject(wkManagerReports.get(i).getDetail(),
                        WkManagerReport.class);
                //日均收益率    日均贷款余额（含核销贷款）日均贷款余额（不含核销贷款）
                avgProfitTotal = avgProfitTotal.add(wkManagerReport.getCommonRpt().getAvgProfit());
                dailyAvgPrinTotal = dailyAvgPrinTotal.add(wkManagerReport.getCommonRpt().getDailyAvgPrin());
                dailyAvgPrinExtTotal = dailyAvgPrinExtTotal.add(wkManagerReport.getCommonRpt().getDailyAvgPrinExt());
                
                //兑付信息
                Map<String, InvestInfoRpt> investInfoRptMap = wkManagerReport.getInvestRpts().stream().collect(
                        Collectors.toMap(InvestInfoRpt::getInvestNo, item -> item));
                
                //资金使用情况  总流入  总流出
                combinedReport.getCashRpt().combineAmt(wkManagerReport.getCashRpt());
                //转让情况  期间转让金额  期间转让笔数
                combinedReport.getTfrRpt().combineAmt(wkManagerReport.getTfrRpt());
                //放款情况  期间放款金额  期间放款笔数
                combinedReport.getGrantRpt().combineAmt(wkManagerReport.getGrantRpt());
                //回收情况  到期还款  提前还款  逾期还款   核销还款
                combinedReport.getRepoRpt().combineAmt(wkManagerReport.getRepoRpt());
                //费用支出    费用名称  费用编号  科目编号  支出时间  费用金额  增值税金额
                combinedReport.getFeeRpts().addAll(wkManagerReport.getFeeRpts());
                
                if (i == 0) {
                    
                    //报告开始日期
                    combinedReport.setRptStartDate(wkManagerReport.getRptStartDate());
                    //兑付信息
                    combinedReport.setInvestRpts(wkManagerReport.getInvestRpts());
                    
                    //资金使用情况   期初余额
                    combinedReport.getCashRpt().copyPre(wkManagerReport.getCashRpt());
                    //转让情况      期初累计转让金额  期初累计转让笔数
                    combinedReport.getTfrRpt().copyPre(wkManagerReport.getTfrRpt());
                    //放款情况      期初累计放款金额  期初累计放款笔数
                    combinedReport.getGrantRpt().copyPre(wkManagerReport.getGrantRpt());
                } else if (i == wkManagerReports.size() - 1) {
                    //结束时间
                    combinedReport.setRptEndDate(wkManagerReport.getRptEndDate());
                    //计划分析 合同不良率 静态不良率  报备不良率  实际不良率
                    combinedReport.getCommonRpt().copyAft(wkManagerReport.getCommonRpt());
                    //兑付信息
                    for (InvestInfoRpt infoRpt : combinedReport.getInvestRpts()) {
                        InvestInfoRpt lastInfoRpt = investInfoRptMap.get(infoRpt.getInvestNo());
                        infoRpt.combineAmt(lastInfoRpt);
                        infoRpt.copyAft(lastInfoRpt);
                    }
                    //资金使用情况   期末余额
                    combinedReport.getCashRpt().copyAft(wkManagerReport.getCashRpt());
                    //转让情况      期末累计转让金额   期末累计转让笔数
                    combinedReport.getTfrRpt().copyAft(wkManagerReport.getTfrRpt());
                    //放款情况      期末累计放款金额  期末累计放款笔数
                    combinedReport.getGrantRpt().copyAft(wkManagerReport.getGrantRpt());
                    //资产特征      正常    逾期    核销
                    combinedReport.setAstRpt(wkManagerReport.getAstRpt());
                    //五级分类
                    combinedReport.setRiskClzRpt(wkManagerReport.getRiskClzRpt());
                    
                    //统计属性集合
                    combinedReport.setCommonPropStat(wkManagerReport.getCommonPropStat());
                    //计数属性集合
                    combinedReport.setPropCnt(wkManagerReport.getPropCnt());
                } else {
                    for (InvestInfoRpt infoRpt : combinedReport.getInvestRpts()) {
                        InvestInfoRpt curInfoRpt = investInfoRptMap.get(infoRpt.getInvestNo());
                        infoRpt.combineAmt(curInfoRpt);
                    }
                }
            }
            long days = DateTimeUtil.calcDayDiff(combinedReport.getRptEndDate(), combinedReport.getRptStartDate());
            combinedReport.setRptDays(days);
            combinedReport.getCommonRpt().setAvgProfit(CalcUtil.divide(avgProfitTotal, new BigDecimal(days)));
            combinedReport.getCommonRpt().setDailyAvgPrin(CalcUtil.divide(dailyAvgPrinTotal, new BigDecimal(days)));
            combinedReport.getCommonRpt()
                    .setDailyAvgPrinExt(CalcUtil.divide(dailyAvgPrinExtTotal, new BigDecimal(days)));
        }
        
        combinedReport.sumInvestRpt();
        return combinedReport;
    }
    
    /**
     * 生成管理报表
     * @param plan
     * @param startDate
     * @param endDate
     * @param lastReport
     * @param planAcctTitles
     * @param titleLogs
     * @return
     */
    public WkManagerReport buildReport(PlanBase plan, Date startDate, Date endDate, WkManagerReport lastReport,
                                       List<PlanAcctTitle> planAcctTitles, List<PlanAcctTitleLog> titleLogs) {
        WkManagerReport report = new WkManagerReport();
        report.setPlanNo(plan.getPlanNo());
        report.setPlanName(plan.getPlanName());
        report.setCalcIntrType(plan.getCalcIntrType().getCode());
        report.setRptEndDate(endDate);
        report.setRptStartDate(startDate);
        report.setRptDays(DateTimeUtil.calcDayDiff(report.getRptEndDate(), report.getRptStartDate()) + 1);
        
        // 科目列表
        Map<String, InvestItem> acctTitles = buildInvestItem(planAcctTitles);
        // 科目流水
        Map<AcctLogKey, AcctAmt> titleLogAmtSum = buildAcctAmtSum(titleLogs);
        // 获取计划日期时间段报表数据
        List<PlanReportItem> dailyReports = planReportRepo.getItemsByDate(plan.getPlanNo(), report.getRptStartDate(),
                report.getRptEndDate());
        Map<String, PlanReportItem> sumReportItemMap = buildSumReportItem(dailyReports);
        Map<String, PlanReportItem> latestReportItemMap = buildReportItem(dailyReports, endDate);
        
        // 计划分析
        buildPlanCommonRpt(plan, report, sumReportItemMap, latestReportItemMap);
        
        // 计划兑付分析
        buildInvestRpt(plan, report, titleLogAmtSum, acctTitles, lastReport);
        
        // 资金使用情况
        buildCashRpt(plan, report, sumReportItemMap, lastReport);
        
        // 转让报表
        buildTfrReport(report, sumReportItemMap, lastReport);
    
        // 资产放款情况
        buildGrantReport(report, sumReportItemMap, lastReport);
        
        // 资产特征
        buildAstRpt(report, latestReportItemMap);
        
        // 资产回收情况
        buildRepoRpt(report, sumReportItemMap);
        
        // 五级分类
        buildRiskClzRpt(report, latestReportItemMap);
        
        // 费用
        buildFeeRpt(report, titleLogs);
        
        // 资产统计特征
        buildCommonStat(report, endDate);
        
        // 资产属性统计
        buildPropCntStat(report, endDate);
        
        return report;
    }
    
    /**
     * 计划分析
     *
     * @param plan
     * @param report
     * @param sumReportItemMap
     * @param latestReportItemMap
     */
    private void buildPlanCommonRpt(PlanBase plan, WkManagerReport report,
                                    Map<String, PlanReportItem> sumReportItemMap,
                                    Map<String, PlanReportItem> latestReportItemMap) {
        
        BigDecimal rptDays = new BigDecimal(report.getRptDays());
        
        PlanCommonRpt planCommonRpt = new PlanCommonRpt();
        // 总收益=最新应还利息+最新应还罚息+期间累计已还利息+期间累计已还罚息
        BigDecimal profit = RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK018.getCode())
                                    .add(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK019.getCode()))
                                    .add(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK040.getCode()))
                                    .add(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK041.getCode()));
        BigDecimal profitRate = CalcUtil.divide(profit, RptBuilderUtil.getInvestedAmt(plan).getAmount());
        planCommonRpt.setAvgProfit(CalcUtil.divide(profitRate, rptDays));
    
        /**
          当前月损失率=（近30天新增逾期90天以上的借据的未还剩余本金）/ 近30天平均剩余资产规模（包含核销）；
         （1-当前年化损失率） 当前年化损失率=当前月损失率*12
         */
        BigDecimal lossYearRate = NumberUtil.sub(1, NumberUtil.mul(CalcUtil.divide(
                RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK025.getCode()),
                        RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK017.getCode())), 12));
        // （1-闲置率）闲置率=从成立到当前，每日闲置率的平均值
        BigDecimal idleRate = NumberUtil.sub(1, CalcUtil.divide(
                RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK002.getCode()), rptDays));
        // 资产加权利率 累计维度，取准实时值 取最后一日的资产加权利率
        BigDecimal astRate = RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK129.getCode());
        // （1-招联金融服务费率）招联金融服务费费率：在项目录入时输入
        BigDecimal finSerFeeRate = NumberUtil.sub(1, plan.getFinSerFeeRate());
        /**
         * 信托实际收益率（税前） =（1-闲置率）*（1-当前年化损失率）*资产加权利率*（1-招联金融服务费率）+（1-当前年化损失率）
         */
        planCommonRpt.setProfitRate(NumberUtil.mul(idleRate, lossYearRate, astRate, finSerFeeRate).add(lossYearRate));
        
        // 报告期内，=sum（每日贷款余额（包含核销贷款））/统计天数
        BigDecimal prinSum = RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK017.getCode());
        planCommonRpt.setDailyAvgPrin(CalcUtil.divide(prinSum, rptDays));
        
        // 报告期内，=sum（每日贷款余额（不包含核销贷款））/统计天数
        BigDecimal prinExtSum = prinSum.subtract(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK015.getCode()));
        planCommonRpt.setDailyAvgPrinExt(CalcUtil.divide(prinExtSum, rptDays));
        
        // 91天以上逾期资产(含“181天以上核销”)
        BigDecimal oduePrin = RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK025.getCode());
        // 91天以上逾期资产(不含“181天以上核销”)
        BigDecimal odueExtPrin = RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WKExt001.getCode());
        // 贷款余额
        BigDecimal totalPrin = RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK017.getCode());
        // 累计放款金额
        BigDecimal totalAcctGrant = RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK027.getCode());
        
        // 91天以上逾期资产(不含“181天以上核销”)/累计放款金额
        planCommonRpt.setNplCont(CalcUtil.divide(odueExtPrin, totalAcctGrant));
        planCommonRpt.setNplStatic(CalcUtil.divide(oduePrin, totalAcctGrant));
        planCommonRpt.setNplReport(CalcUtil.divide(odueExtPrin, totalPrin));
        planCommonRpt.setNplReal(CalcUtil.divide(oduePrin, totalPrin));
    
        report.setCommonRpt(planCommonRpt);
    }
    
    /**
     * 投资结构和计划兑付信息
     * @param plan
     * @param report
     * @param titleLogAmtSum
     * @param acctTitles
     * @param lastReport
     */
    private void buildInvestRpt(PlanBase plan, WkManagerReport report, Map<AcctLogKey, AcctAmt> titleLogAmtSum,
                                Map<String, InvestItem> acctTitles,
                                WkManagerReport lastReport) {
        
        List<InvestInfoRpt> infoRpts = Lists.newArrayList();
        Map<String, InvestInfoRpt> lastInfoRptMap = Maps.newHashMap();
        
        if (null != lastReport) {
            lastInfoRptMap = lastReport.getInvestRpts().stream().collect(
                    Collectors.toMap(InvestInfoRpt::getInvestNo, item -> item));
        }
        // 根据投资结构 计算分配兑付情况
        for (InvestStructure investStrut : plan.getInvestStructureList()) {
            // 上个周期的月报，可能为null，需要判断null处理
            InvestInfoRpt lastInfoRpt = lastInfoRptMap.get(investStrut.getLayerNo());
            // 对应该投资结构层级的各个科目，不能为null，为null说明科目未初始化，为系统异常
            InvestItem investItem = acctTitles.get(investStrut.getLayerNo());
            VarChecker.checkNotNull(investItem, "投资结构[{0}]的科目为初始化", investStrut);
            
            InvestInfoRpt infoRpt = new InvestInfoRpt();
            infoRpt.setInvestNo(investStrut.getLayerNo());
            infoRpt.setInvestName(investStrut.getLayerName());
            infoRpt.setInvestCost(investStrut.getLayerAmt().getAmount());
            // 考虑无票面利率的情况
            if (investStrut.getIntrType() == IntrTypeEnum.NOTICKET) {
                infoRpt.setRate(null);
            } else {
                infoRpt.setRate(investStrut.getLayerIntr());
            }
            
            // 期初本金：上个报表的期末为这个报表的期初，默认为投入资本
            infoRpt.setPrePrin((null == lastInfoRpt) ? investStrut.getLayerAmt().getAmount() : lastInfoRpt.getAftPrin());
            // 期初收益：上个报表的期末为这个报表的期初，默认为0
            infoRpt.setPreProfit((null == lastInfoRpt) ? BigDecimal.ZERO : lastInfoRpt.getAftProfit());
            
            // 从科目流水中，汇总发生额，汇总结果可能为null（该月未发生相关交易），若为null，则作0处理
            AcctAmt prinAcctAmt = titleLogAmtSum.get(new AcctLogKey(investItem.investTitle, EventCodeEnum.TITLE_ALLOC));
            AcctAmt intrAllocAcctAmt = titleLogAmtSum.get(new AcctLogKey(investItem.intrTitle, EventCodeEnum.TITLE_ALLOC));
            AcctAmt intrSettleAcctAmt = titleLogAmtSum.get(new AcctLogKey(investItem.intrTitle, EventCodeEnum.TITLE_SETTLE));
            
            infoRpt.setAllocPrinAmt((prinAcctAmt == null) ? BigDecimal.ZERO : prinAcctAmt.amt);
            infoRpt.setAllocProfitAmt((intrAllocAcctAmt == null) ? BigDecimal.ZERO : intrAllocAcctAmt.amt);
            infoRpt.setAllocVtaxAmt((intrAllocAcctAmt == null) ? BigDecimal.ZERO : intrAllocAcctAmt.vtaxAmt);
            infoRpt.setAddProfitAmt((intrSettleAcctAmt == null) ? BigDecimal.ZERO : intrSettleAcctAmt.amt);
            
            // 期初 +/- 期间发生额 = 期末，不用考虑为null的情况
            infoRpt.setAftPrin(infoRpt.getPrePrin().subtract(infoRpt.getAllocPrinAmt()));
            infoRpt.setAftProfit(
                    infoRpt.getPreProfit().add(infoRpt.getAddProfitAmt()).subtract(infoRpt.getAllocProfitAmt()));
            // 汇总本金，收益的累计分配，若上日报表为null，则直接采用当期发生额，否则累加上日报表的汇总金额
            infoRpt.setTotAllocPrin((lastInfoRpt == null) ?
                                            infoRpt.getAllocPrinAmt() :
                                            infoRpt.getAllocPrinAmt().add(lastInfoRpt.getTotAllocPrin()));
            infoRpt.setTotAllocProfit((lastInfoRpt == null) ?
                                              infoRpt.getAllocProfitAmt() :
                                              infoRpt.getAllocProfitAmt().add(lastInfoRpt.getTotAllocProfit()));
            infoRpt.setTotAllocVtax((lastInfoRpt == null) ?
                                            infoRpt.getAllocVtaxAmt() :
                                            infoRpt.getAllocVtaxAmt().add(lastInfoRpt.getTotAllocVtax()));
            
            infoRpts.add(infoRpt);
        }
        report.setInvestRpts(infoRpts);
    }
    
    /**
     * 资金使用情况
     * @param plan
     * @param report
     * @param sumReportItemMap
     * @param lastReport
     */
    private void buildCashRpt(PlanBase plan, WkManagerReport report, Map<String, PlanReportItem> sumReportItemMap,
                              WkManagerReport lastReport) {
        
        CashRpt cashRpt = new CashRpt();
        cashRpt.setPreBal((lastReport == null) ?
                                  RptBuilderUtil.getInvestedAmt(plan).getAmount() : lastReport.getCashRpt().getAftBal());
        cashRpt.setInAmt(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK009.getCode()));
        cashRpt.setOutAmt(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK008.getCode()));
        cashRpt.setAftBal(cashRpt.getPreBal().add(cashRpt.getInAmt()).subtract(cashRpt.getOutAmt()));
        
        report.setCashRpt(cashRpt);
    }
    
    /**
     * 资产转让情况
     * @param report
     * @param sumReportItemMap
     * @param lastReport
     */
    private void buildTfrReport(WkManagerReport report, Map<String, PlanReportItem> sumReportItemMap,
                                WkManagerReport lastReport) {
        TfrRpt tfrRpt = new TfrRpt();
        
        tfrRpt.setPreAccGrant((lastReport == null) ? BigDecimal.ZERO : lastReport.getTfrRpt().getAftAccGrant());
        tfrRpt.setPreAccGrantCnt((lastReport == null) ? BigDecimal.ZERO : lastReport.getTfrRpt().getAftAccGrantCnt());
        tfrRpt.setGrantAmt(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK028.getCode()));
        tfrRpt.setGrantCnt(new BigDecimal(RptBuilderUtil.getReportCnt(sumReportItemMap, WkAstReportFields.WK028.getCode())));
        tfrRpt.setAftAccGrant(tfrRpt.getPreAccGrant().add(tfrRpt.getGrantAmt()));
        tfrRpt.setAftAccGrantCnt(tfrRpt.getPreAccGrantCnt().add(tfrRpt.getGrantCnt()));
        report.setTfrRpt(tfrRpt);
    }
    
    /**
     * 资产放款情况
     * @param report
     * @param sumReportItemMap
     * @param lastReport
     */
    private void buildGrantReport(WkManagerReport report, Map<String, PlanReportItem> sumReportItemMap,
                                WkManagerReport lastReport) {
        GrantRpt grantRpt = new GrantRpt();
    
        grantRpt.setPreAccGrant((lastReport == null) ? BigDecimal.ZERO : lastReport.getGrantRpt().getAftAccGrant());
        grantRpt.setPreAccGrantCnt((lastReport == null) ? BigDecimal.ZERO : lastReport.getGrantRpt().getAftAccGrantCnt());
        grantRpt.setGrantAmt(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK115.getCode()));
        grantRpt.setGrantCnt(new BigDecimal(RptBuilderUtil.getReportCnt(sumReportItemMap, WkAstReportFields.WK115.getCode())));
        grantRpt.setAftAccGrant(grantRpt.getPreAccGrant().add(grantRpt.getGrantAmt()));
        grantRpt.setAftAccGrantCnt(grantRpt.getPreAccGrantCnt().add(grantRpt.getGrantCnt()));
        report.setGrantRpt(grantRpt);
    }
    
    /**
     * 资产特征
     * @param report
     * @param latestReportItemMap
     */
    private void buildAstRpt(WkManagerReport report, Map<String, PlanReportItem> latestReportItemMap) {
        AstRpt astRpt = new AstRpt();
        
        AmtPropCnt normal = new AmtPropCnt();
        AmtPropCnt overdue = new AmtPropCnt();
        AmtPropCnt writeOff = new AmtPropCnt();
        
        normal.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(latestReportItemMap, WkAstReportFields.WK010.getCode())));
        normal.setPrin(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK010.getCode()));
        normal.setIntr(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK011.getCode()));
        normal.calTotal();
        
        overdue.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(latestReportItemMap, WkAstReportFields.WK012.getCode())));
        overdue.setPrin(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK012.getCode()));
        overdue.setIntr(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK013.getCode()));
        overdue.setPenl(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK014.getCode()));
        overdue.calTotal();
        
        writeOff.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(latestReportItemMap, WkAstReportFields.WK015.getCode())));
        writeOff.setPrin(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK015.getCode()));
        writeOff.setIntr(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WKExt002.getCode()));
        writeOff.setPenl(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WKExt003.getCode()));
        writeOff.calTotal();
        
        astRpt.setNormal(normal);
        astRpt.setOverdue(overdue);
        astRpt.setWriteoff(writeOff);
        
        report.setAstRpt(astRpt);
    }
    
    /**
     * 资产回收情况
     * @param report
     * @param sumReportItemMap
     */
    private void buildRepoRpt(WkManagerReport report, Map<String, PlanReportItem> sumReportItemMap) {
        RepoRpt repoRpt = new RepoRpt();
        
        AmtPropCnt dueRepo = new AmtPropCnt();
        AmtPropCnt preRepo = new AmtPropCnt();
        AmtPropCnt ovdRepo = new AmtPropCnt();
        AmtPropCnt wrtRepo = new AmtPropCnt();
        
        dueRepo.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(sumReportItemMap, WkAstReportFields.WKExt004.getCode())));
        dueRepo.setPrin(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WKExt004.getCode()));
        dueRepo.setIntr(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WKExt005.getCode()));
        dueRepo.calTotal();
        
        preRepo.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(sumReportItemMap, WkAstReportFields.WK042.getCode())));
        preRepo.setPrin(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK042.getCode()));
        preRepo.setIntr(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK043.getCode()));
        preRepo.setOtherFee(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK140.getCode()));
        preRepo.calTotal();
        
        ovdRepo.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(sumReportItemMap, WkAstReportFields.WK044.getCode())));
        ovdRepo.setPrin(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK044.getCode()));
        ovdRepo.setIntr(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK045.getCode()));
        ovdRepo.setPenl(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK046.getCode()));
        ovdRepo.calTotal();
        
        wrtRepo.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(sumReportItemMap, WkAstReportFields.WK047.getCode())));
        wrtRepo.setPrin(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WK047.getCode()));
        wrtRepo.setIntr(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WKExt006.getCode()));
        wrtRepo.setPenl(RptBuilderUtil.getReportVal(sumReportItemMap, WkAstReportFields.WKExt007.getCode()));
        wrtRepo.calTotal();
        
        repoRpt.setDueRepo(dueRepo);
        repoRpt.setPreRepo(preRepo);
        repoRpt.setOvdRepo(ovdRepo);
        repoRpt.setWrtRepo(wrtRepo);
        
        report.setRepoRpt(repoRpt);
    }
    
    /**
     * 五级分类统计
     *
     * @param report
     * @param latestReportItemMap
     */
    private void buildRiskClzRpt(WkManagerReport report, Map<String, PlanReportItem> latestReportItemMap) {
        
        RiskClzRpt riskClzRpt = new RiskClzRpt();
        
        AmtPropCnt normal = new AmtPropCnt();
        AmtPropCnt atten = new AmtPropCnt();
        AmtPropCnt second = new AmtPropCnt();
        AmtPropCnt suspic = new AmtPropCnt();
        AmtPropCnt loss = new AmtPropCnt();
        AmtPropCnt total = new AmtPropCnt();
        
        normal.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(latestReportItemMap, WkAstReportFields.WK911.getCode())));
        normal.setPrin(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK911.getCode()));
        normal.setIntr(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK912.getCode()));
        second.setPenl(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK913.getCode()));
        normal.calTotal();
        
        atten.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(latestReportItemMap, WkAstReportFields.WK921.getCode())));
        atten.setPrin(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK921.getCode()));
        atten.setIntr(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK922.getCode()));
        second.setPenl(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK923.getCode()));
        atten.calTotal();
        
        second.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(latestReportItemMap, WkAstReportFields.WK931.getCode())));
        second.setPrin(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK931.getCode()));
        second.setIntr(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK932.getCode()));
        second.setPenl(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK933.getCode()));
        second.calTotal();
        
        suspic.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(latestReportItemMap, WkAstReportFields.WK941.getCode())));
        suspic.setPrin(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK941.getCode()));
        suspic.setIntr(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK942.getCode()));
        suspic.setPenl(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK943.getCode()));
        suspic.calTotal();
        
        loss.setCnt(new BigDecimal(RptBuilderUtil.getReportCnt(latestReportItemMap, WkAstReportFields.WK951.getCode())));
        loss.setPrin(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK951.getCode()));
        loss.setIntr(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK952.getCode()));
        loss.setPenl(RptBuilderUtil.getReportVal(latestReportItemMap, WkAstReportFields.WK953.getCode()));
        loss.calTotal();
        
        total.addAllAmt(normal).addAllAmt(atten).addAllAmt(second).addAllAmt(suspic).addAllAmt(loss);
        normal.calcRatio(total);
        atten.calcRatio(total);
        second.calcRatio(total);
        suspic.calcRatio(total);
        loss.calcRatio(total);
        
        riskClzRpt.setNormal(normal);
        riskClzRpt.setAtten(atten);
        riskClzRpt.setSecond(second);
        riskClzRpt.setSuspic(suspic);
        riskClzRpt.setLoss(loss);
        riskClzRpt.setTotal(total);
        
        report.setRiskClzRpt(riskClzRpt);
    }
    
    /**
     * 费用报表
     *
     * @param report
     * @param acctTitleLogs
     */
    private void buildFeeRpt(WkManagerReport report, List<PlanAcctTitleLog> acctTitleLogs) {
        
        List<FeeRpt> feeRpts = Lists.newArrayList();
        
        Collections.sort(acctTitleLogs, Comparator.comparing(PlanAcctTitleLog::getAccountDate));
        
        for (PlanAcctTitleLog planAcctTitleLog : acctTitleLogs) {
            if (planAcctTitleLog.getTitleType() != TitleTypeEnum.FEE) {
                continue;
            }
            
            if (planAcctTitleLog.getEventCode() != EventCodeEnum.TITLE_ALLOC) {
                continue;
            }
            
            FeeRpt feeRpt = new FeeRpt();
            feeRpt.setFeeName(planAcctTitleLog.getTitleName());
            feeRpt.setFeeNo(planAcctTitleLog.getRefNo());
            feeRpt.setTitleNo(planAcctTitleLog.getTitleNo());
            feeRpt.setDt(DateTimeUtil.formatY_M_D(planAcctTitleLog.getAccountDate()));
            feeRpt.setFeeAmt(planAcctTitleLog.getAcctAmt().getAmount());
            feeRpt.setVtaxAmt(planAcctTitleLog.getvTaxAmt().getAmount());
            feeRpt.setMemo("");
            
            feeRpts.add(feeRpt);
        }
        
        report.setFeeRpts(feeRpts);
    }
    
    private Map<AcctLogKey, AcctAmt> buildAcctAmtSum(List<PlanAcctTitleLog> planAcctTitleLogs) {
        Map<AcctLogKey, AcctAmt> ret = Maps.newHashMap();
        
        for (PlanAcctTitleLog planAcctTitleLog : planAcctTitleLogs) {
            AcctLogKey key = new AcctLogKey(planAcctTitleLog.getTitleNo(), planAcctTitleLog.getEventCode());
            AcctAmt amt = ret.get(key);
            
            if (null == amt) {
                amt = new AcctAmt();
                ret.put(key, amt);
            }
            
            amt.amt = amt.amt.add(planAcctTitleLog.getAcctAmt().getAmount());
            amt.vtaxAmt = amt.vtaxAmt.add(planAcctTitleLog.getvTaxAmt().getAmount());
        }
        
        return ret;
    }
    
    private Map<String, PlanReportItem> buildSumReportItem(List<PlanReportItem> dailyReports) {
        return buildReportItem(dailyReports, null);
    }
    
    private Map<String, PlanReportItem> buildReportItem(List<PlanReportItem> dailyReports, Date accountDate) {
        Map<String, PlanReportItem> reportItemMap = Maps.newHashMap();
        
        for (PlanReportItem item : dailyReports) {
            if (accountDate != null && DateTimeUtil.calcDayDiff(accountDate, item.getAccountDate()) != 0) {
                continue;
            }
            PlanReportItem sumItem = reportItemMap.get(item.getRptItemCode());
            if (null == sumItem) {
                sumItem = new PlanReportItem();
                sumItem.setRptItemCode(sumItem.getRptItemCode());
                sumItem.setRptItemName(sumItem.getRptItemName());
                sumItem.setAstCnt(item.getAstCnt());
                sumItem.setAmt(item.getAmt());
                reportItemMap.put(item.getRptItemCode(), sumItem);
            }else{
                sumItem.setAstCnt(sumItem.getAstCnt() + item.getAstCnt());
                sumItem.setAmt(sumItem.getAmt().add(item.getAmt()));
            }
        }
        
        return reportItemMap;
    }
    
    private Map<String, InvestItem> buildInvestItem(List<PlanAcctTitle> planAcctTitles) {
        
        Map<String, InvestItem> ret = Maps.newHashMap();
        
        for (PlanAcctTitle acctTitle : planAcctTitles) {
            InvestItem item = ret.get(acctTitle.getRefNo());
            if (null == item) {
                item = new InvestItem();
                ret.put(acctTitle.getRefNo(), item);
            }
            
            switch (acctTitle.getTitleType()) {
                case INVEST_PROFIT:
                    item.intrTitle = acctTitle.getTitleNo();
                    break;
                case PRIOR_INVESTED:
                case SECND_INVESTED:
                case INVESTED:
                    item.investTitle = acctTitle.getTitleNo();
                    break;
                case FEE:
                    item.feeTitle = acctTitle.getTitleNo();
                    break;
            }
        }
        return ret;
    }
    
    /**
     * 资产特征分布
     * @param report
     * @param accountDate
     */
    private void buildCommonStat(WkManagerReport report, Date accountDate) {
        
        CommonPropStatGroup commonPropStatGroup = new CommonPropStatGroup();
        report.setCommonPropStat(commonPropStatGroup);
        
        // 资产合同金额特征
        CommonPropStat wkm001 = new CommonPropStat();
        commonPropStatGroup.getCommonPropStat().put(WkManReportFields.WKM001.getCode(), wkm001);
        //资产余额特征
        CommonPropStat wkm002 = new CommonPropStat();
        commonPropStatGroup.getCommonPropStat().put(WkManReportFields.WKM002.getCode(), wkm002);
        // 资产利率特征
        CommonPropStat wkm003 = new CommonPropStat();
        commonPropStatGroup.getCommonPropStat().put(WkManReportFields.WKM003.getCode(), wkm003);
        // 合同期限特征
        CommonPropStat wkm004 = new CommonPropStat();
        commonPropStatGroup.getCommonPropStat().put(WkManReportFields.WKM004.getCode(), wkm004);
        // 剩余期限特征
        CommonPropStat wkm005 = new CommonPropStat();
        commonPropStatGroup.getCommonPropStat().put(WkManReportFields.WKM005.getCode(), wkm005);
        // 账龄特征
        CommonPropStat wkm006 = new CommonPropStat();
        commonPropStatGroup.getCommonPropStat().put(WkManReportFields.WKM006.getCode(), wkm006);
        
        PropDisbStat propDisbStat = loanBakDAO.statDisb(accountDate, report.getPlanNo());
        if (propDisbStat != null) {
            wkm001.setMin(getStatAmt(propDisbStat.getMinContAmt()));
            wkm001.setMax(getStatAmt(propDisbStat.getMaxContAmt()));
            wkm001.setAvg(getStatAmt(propDisbStat.getAvgContAmt()));
            
            wkm002.setMin(getStatAmt(propDisbStat.getMinBal()));
            wkm002.setMax(getStatAmt(propDisbStat.getMaxBal()));
            wkm002.setAvg(getStatAmt(propDisbStat.getAvgBal()));
            
            wkm003.setMin(propDisbStat.getMinRate());
            wkm003.setMax(propDisbStat.getMaxRate());
            wkm003.setAvg(propDisbStat.getAvgRate());
            wkm003.setwAvg(propDisbStat.getWavgRate());
            
            wkm004.setMin(new BigDecimal(propDisbStat.getMinContPeriod()));
            wkm004.setMax(new BigDecimal(propDisbStat.getMaxContPeriod()));
            wkm004.setAvg(propDisbStat.getAvgContPeriod());
            wkm004.setwAvg(propDisbStat.getWavgContPeriod());
            
            wkm005.setMin(new BigDecimal(propDisbStat.getMinRemPeriod()));
            wkm005.setMax(new BigDecimal(propDisbStat.getMaxRemPeriod()));
            wkm005.setAvg(propDisbStat.getAvgRemPeriod());
            wkm005.setwAvg(propDisbStat.getWavgRemPeriod());
            
            wkm006.setMin(new BigDecimal(propDisbStat.getMinAcctPeriod()));
            wkm006.setMax(new BigDecimal(propDisbStat.getMaxAcctPeriod()));
            wkm006.setAvg(propDisbStat.getAvgAcctPeriod());
            wkm006.setwAvg(propDisbStat.getWavgAcctPeriod());
        }
    }
    
    private void buildPropCntStat(WkManagerReport report, Date accountDate) {
        
        AmtPropCntGroup amtPropCntGroup = new AmtPropCntGroup();
        report.setPropCnt(amtPropCntGroup);
        
        List<AmtPropCnt> locAmtProp = Lists.newArrayList();
        List<AmtPropCnt> ageAmtProp = Lists.newArrayList();
        List<AmtPropCnt> genAmtProp = Lists.newArrayList();
        
        /**
         ReportField WKM007 = new ReportField("WKM007", "性别");
         ReportField WKM008 = new ReportField("WKM008", "年龄");
         ReportField WKM009 = new ReportField("WKM009", "地区");
         */
        amtPropCntGroup.getPropCntMap().put(WkManReportFields.WKM007.getCode(), genAmtProp);
        amtPropCntGroup.getPropCntMap().put(WkManReportFields.WKM008.getCode(), ageAmtProp);
        amtPropCntGroup.getPropCntMap().put(WkManReportFields.WKM009.getCode(), locAmtProp);
        
        List<PropCntStat> ageStat = loanBakDAO.statAge(accountDate, report.getPlanNo());
        List<PropCntStat> locStat = loanBakDAO.statLocation(accountDate, report.getPlanNo());
        List<PropCntStat> genStat = loanBakDAO.statGender(accountDate, report.getPlanNo());
        
        // 性别
        AmtPropCnt maleProp = new AmtPropCnt();
        AmtPropCnt femaleProp = new AmtPropCnt();
        genAmtProp.add(maleProp);
        genAmtProp.add(femaleProp);
        Map<String, PropCntStat> genStatMap = genStat.stream()
                                                      .collect(Collectors.toMap(PropCntStat::getPropVal, item -> item));
        PropCntStat maleStat = genStatMap.get("male");
        PropCntStat femaleStat = genStatMap.get("female");
        
        maleProp.setCnt(getPropCntStatCnt(maleStat));
        maleProp.setPrin(getPropCntStatAmt(maleStat));
        femaleProp.setCnt(getPropCntStatCnt(femaleStat));
        femaleProp.setPrin(getPropCntStatAmt(femaleStat));
        
        // 年龄
        for (Integer age : AGE_SPLIT) {
            ageAmtProp.add(new AmtPropCnt());
        }
        ageAmtProp.add(new AmtPropCnt());
        
        for (PropCntStat ageCntStat : ageStat) {
            int age = Integer.parseInt(ageCntStat.getPropVal());
            
            int i = 0;
            AmtPropCnt curProp = null;
            for (; i < AGE_SPLIT.size(); i++) {
                if (age <= AGE_SPLIT.get(i)) {
                    curProp = ageAmtProp.get(i);
                    break;
                }
            }
            if (i == AGE_SPLIT.size()) {
                curProp = ageAmtProp.get(i);
            }
            curProp.setCnt(curProp.getCnt().add(getPropCntStatCnt(ageCntStat)));
            curProp.setPrin(curProp.getPrin().add(getPropCntStatAmt(ageCntStat)));
        }
        
        // 地区
        for (int i = 0; i < LOC_CNT; i++) {
            AmtPropCnt locProp = new AmtPropCnt();
            locAmtProp.add(locProp);
            
            if (i < locStat.size()) {
                PropCntStat propStat = locStat.get(i);
                String provinceCode = propStat.getPropVal();
                String province = IdcardUtil.getProvinceByIdCard(provinceCode + "1234567890123456");
                if (StringUtils.isBlank(province)) {
                    province = propStat.getPropVal();
                }
                locProp.setPropVal(province);
                locProp.setCnt(getPropCntStatCnt(propStat));
                locProp.setPrin(getPropCntStatAmt(propStat));
            }
        }
    }
    
    private BigDecimal getStatAmt(Money money) {
        if (null == money) {
            return BigDecimal.ZERO;
        } else {
            return money.getAmount();
        }
    }
    
    private BigDecimal getPropCntStatCnt(PropCntStat propCntStat) {
        if (null == propCntStat) {
            return BigDecimal.ZERO;
        } else {
            return new BigDecimal(propCntStat.getCnt());
        }
    }
    
    private BigDecimal getPropCntStatAmt(PropCntStat propCntStat) {
        if (null == propCntStat) {
            return BigDecimal.ZERO;
        } else {
            return propCntStat.getAmt().getAmount();
        }
    }
    
    private class InvestItem {
        String investTitle;
        
        String intrTitle;
        
        String feeTitle;
    }
    
    private class AcctAmt {
        /**
         *
         */
        BigDecimal amt = BigDecimal.ZERO;
        /**
         *
         */
        BigDecimal vtaxAmt = BigDecimal.ZERO;
    }
    
    private class AcctLogKey extends BaseEntity {
        /**
         * 科目编号
         */
        String titleNo;
        /**
         * 事件类型
         */
        EventCodeEnum eventCode;
        
        public AcctLogKey(String titleNo, EventCodeEnum eventCode) {
            this.titleNo = titleNo;
            this.eventCode = eventCode;
        }
    }
}