/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.ast;

import com.abssqr.plat.biz.shared.scheduler.BaseQuartzJob;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstPackDO;
import com.abssqr.plat.common.facade.enums.AccountAttributeEnum;
import com.abssqr.plat.common.facade.enums.AccountTypeEnum;
import com.abssqr.plat.common.facade.enums.AstPackStatusEnum;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanCapitalTransTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.model.convertor.PacKConvert;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.domain.task.TrCtrTask;
import com.abssqr.plat.common.model.repo.ast.PackRepository;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingLogRepo;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingOrderRepo;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalOrderRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.common.model.repo.tr.TrCtrTaskRepo;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.acct.AcctManager;
import com.abssqr.plat.core.service.ast.AstPackManager;
import com.abssqr.plat.core.service.util.CommonTaskUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.LogUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.PersistJobDataAfterExecution;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * 自动回购
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.scheduler.job.ast: AutoBackJob.java, v 0.1 2020-03-03 01:49 bangis.wangdf Exp $
 */
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
@Component
public class AutoBackJob extends BaseQuartzJob {
    private final static Logger LOGGER = LoggerFactory.getLogger(AutoBackJob.class);
    @Autowired
    PackRepository        packRepository;
    @Autowired
    CommonTaskRepository  commonTaskRepository;
    @Autowired
    TrCtrTaskRepo         trCtrTaskRepo;
    @Autowired
    AstPackManager        astPackManager;
    @Autowired
    PlanClearingLogRepo   planClearingLogRepo;
    @Autowired
    PlanClearingOrderRepo planClearingOrderRepo;
    @Autowired
    PlanCapitalOrderRepo  planCapitalOrderRepo;
    @Autowired
    AcctManager           acctManager;
    @Autowired
    AbsConfig             absConfig;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;
    
    @Override
    protected String getCronExpr() {
        return "0 1/5 * * * ?";
    }

    public static final String ORG_CODE   = "WKXT";
    public static final Money  reserveAmt = new Money("2000000");

    @Override
    protected void doExecute() {
        // 获取转让会计日
        Date curDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();

        List<CommonTask> commonTasks = commonTaskRepository.selectUnifinishTaskByType(
            CommonTaskTypeEnum.BUY_BACK_MATCH);

        //没有未完成的回购任务
        if (CollectionUtils.isNotEmpty(commonTasks)) {
            LogUtil.info(LOGGER, "AutoBackJob 不执行，存在待回购的 commonTask");
            return;
        }
        // 获取所有回购pack
        List<AstPackDO> astPackDOS = packRepository.selectPackByStatus(curDate,
            AstPackStatusEnum.WAIT,
            AstPackTradeTypeEnum.BUY_BACK_BIZ);
        if (CollectionUtils.isEmpty(astPackDOS)) {
            LogUtil.info(LOGGER, "AutoBackJob 不执行，没有待回购包");
            return;
        }
        List<TrCtrTask> activeBkCtrTasks = trCtrTaskRepo
            .getByManagerOrgCode(ORG_CODE, curDate, TaskStatusEnum.INIT, AstPackTradeTypeEnum.BUY_BACK_BIZ);

        if (CollectionUtils.isNotEmpty(activeBkCtrTasks)) {
            LogUtil.info(LOGGER, "AutoBackJob 不执行，存在待回购的TrTask");
            return;
        }

        //清算，结算，支付
        Long clearingLogCnt = planClearingLogRepo.cntInitClearingLog(TransCodeEnum.BUY_BACK_OUT);
        Long clearingOrderCnt = planClearingOrderRepo.cntInitClearingOrder(TransCodeEnum.BUY_BACK_OUT);
        Long activeCnt = planCapitalOrderRepo.getActiveCntByType(PlanCapitalTransTypeEnum.NT_BUY_BACK);
        if (clearingLogCnt + clearingOrderCnt + activeCnt > 0) {
            LogUtil.info(LOGGER, "AutoBackJob 不执行，支付未完成");
            return;
        }

        //判断账户余额是否满足
        AstPackDO astPackDO = astPackDOS.get(0);
        Money backAvalAmt = Money.createWithCent(astPackDO.getAvalAmt());
        // 1、获取计划对应资金账户 平台虚拟户
        List<Account> accounts = acctManager.getAcctByPlanNoAndType(astPackDO.getPlanNo(), AccountTypeEnum.PLAT_VIRTUAL,
            true);
        Money bankBal = new Money();
        if (CollectionUtils.isNotEmpty(accounts)) {
            for (Account account : accounts) {
                if (account.getActAttr() == AccountAttributeEnum.PAY) {
                    bankBal = account.getBankBal();
                }
            }
        }

        //银行余额 - 回购金额 - 预留金额 > 0
        if (bankBal.subtract(backAvalAmt).subtract(reserveAmt).isGreaterThanZero()) {
            //开启本次回购
            CommonTask commonTask = CommonTaskUtil.buildCommonTaskWithContext(astPackDO.getPackCode(),
                CommonTaskTypeEnum.BUY_BACK_MATCH, astPackDO.getPackCode());
            commonTaskRepository.create(commonTask);

            packRepository.updatePackStatus(Lists.newArrayList(PacKConvert.do2Domain(astPackDO)), AstPackStatusEnum.INIT, AstPackStatusEnum.WAIT);

        }else {
            LogUtil.info(LOGGER,"AutoBackJob 不执行，账户余额-[{0}]小于 筛选包金额[{1}] + 预留金额[{2}]",bankBal,backAvalAmt,reserveAmt);

        }

    }
}
