/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.ast;

import com.abssqr.plat.biz.shared.scheduler.BaseQuartzJob;
import com.abssqr.plat.common.dal.mysql.auto.dao.PlanTranScreenRelDAO;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAcctTitle;
import com.abssqr.plat.common.model.domain.plan.PlanAcctTitleTransDetail;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtrLog;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.loanTrans.LoanAstProfitLogRepo;
import com.abssqr.plat.common.model.repo.loanTrans.LoanInstalLogRepository;
import com.abssqr.plat.common.model.repo.plan.PlanAcctTitleRepo;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.host.HostManager;
import com.abssqr.plat.core.service.plan.PlanAcctTitleManager;
import com.abssqr.plat.core.service.plan.PlanSettleManager;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.StringUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * 招联金融服务费
 *
 * @author bangis.wangdf
 * @version $Id: PlanAstStatJob.java, v 0.1 2018-01-06 上午12:39   Exp $
 */
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
@Component
public class ZLFinSerFeeJob extends BaseQuartzJob {

    @Autowired
    HostManager             hostManager;
    @Autowired
    AstSyncTaskCtrRepo      astSyncTaskCtrRepo;
    @Autowired
    LoanInstalLogRepository loanInstalLogRepository;
    @Autowired
    LoanAstProfitLogRepo    loanAstProfitLogRepo;
    @Autowired
    TransactionTemplate     transactionTemplate;
    @Autowired
    PlanTranScreenRelDAO    planTranScreenRelDAO;
    @Autowired
    PlanAcctTitleRepo       planAcctTitleRepo;
    @Autowired
    PlanAcctTitleManager    planAcctTitleManager;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;
    
    //20191128 前所有金融服务费 查询instalLog，之后instalLog仅获取11条无法支持数据的金融服务费分摊
    private static Date     FEE_OFFSET_DAY = DateTimeUtil.parseFromYMD("20191127");
    /**
     * 每天 0 1/5 * * * ?
     */
    @Value("${sync.quartz.ast.ser.fee}")
    private String cron;

    @Value("${wkabs.astsync.zhaolian:1009}")
    private String zlOrgCode;

    @Override
    protected String getCronExpr() {
        return cron;
    }

    @Override
    public void doExecute() {
        
        AstSyncTaskCtr taskCtr = astSyncTaskCtrRepo.getAstSyncTaskCtr(zlOrgCode);
        VarChecker.checkNotNull(taskCtr, "没有匹配的同步控制任务={0}", zlOrgCode);
        OrgCodeEnum orgCodeEnum = OrgCodeEnum.getByCode(taskCtr.getOrgCode());
        VarChecker.checkNotNull(orgCodeEnum, "不匹配的机构编码={0}", taskCtr.getOrgCode());
        // 获取当前机构会计日
        Day orgDate = orgAcctDayComponent.getAstAcctDay(orgCodeEnum);
        // 判断对机构会计日的模型同步是否完成
        AstSyncTaskCtrLog astModTask = astSyncTaskCtrRepo.getTaskLogByOrgDate(
                orgDate.getStandardDate(), SyncTaskTypeEnum.AST_MOD, taskCtr.getOrgCode());
        if (astModTask != null && astModTask.getStatus() == TaskStatusEnum.SUCCESS) {
            if (astSyncTaskCtrRepo.checkCanRun(taskCtr.getOrgCode(),
                    taskCtr.getTaskCode(), SyncTaskTypeEnum.AST_SER_FEE, orgDate.getStandardDate())) {
                
//                Date accountDate = SystemDayUtil.getDay().getYesterday().getStandardDate();
                LogUtil.info(LOGGER, "[ZLFinSerFeeJob]-[{0}]招联金融服务费计算start", orgDate.getDateString());
                Set<String> spvPlanNos = taskCtr.getExtData().getSpvPlanNos();

                if (CollectionUtils.isNotEmpty(spvPlanNos)) {
                    List<PlanAcctTitleTransDetail> acctTitleTransDetails = Lists.newArrayList();
                    List<String> allPlanNos = Lists.newArrayList();
                    allPlanNos.addAll(spvPlanNos);
                    List<String> planNos = planTranScreenRelDAO.getPlanNosByManagerOrg(taskCtr.getOrgCode());
                    if (CollectionUtils.isNotEmpty(planNos)) {
                        allPlanNos.addAll(planNos);
                    }
                    //获取所有金融服务费科目
                    List<PlanAcctTitle> acctFinFeeTitles = planAcctTitleRepo.getAcctFinFeeTitles(allPlanNos);
                    if (CollectionUtils.isNotEmpty(acctFinFeeTitles)) {
                        acctFinFeeTitles.forEach(planAcctTitle -> {
                            Date beginDay = planAcctTitle.getLastSettleDate();
                            VarChecker.checkNotNull(beginDay, "招联金融服务费计算失败，未指定开始统计日期 acctTitle=[{0}]", planAcctTitle);
                            if (DateTimeUtil.calcDayDiff(orgDate.getStandardDate(), beginDay) > 0) {
                                LogUtil.info(LOGGER, "[ZLFinSerFeeJob]-[{0}]招联金融服务费计算-beginDay[{1}]", planAcctTitle,
                                    beginDay);
                                List<Date> calcAccountDates = Lists.newArrayList();
                                // beginDay = DateTimeUtil.addDays(beginDay, 1);
                                while (DateTimeUtil.calcDayDiff(orgDate.getStandardDate(), beginDay) > 0) {
                                    beginDay = DateTimeUtil.addDays(beginDay, 1);
                                    calcAccountDates.add(beginDay);
                                }
                                calcAccountDates.sort(Date::compareTo);
                                calcAccountDates.forEach(calcAccountDate -> {
                                    LogUtil.info(LOGGER, "[ZLFinSerFeeJob]-[{0}]招联金融服务费计算-calcAccountDate[{1}]",
                                        planAcctTitle, calcAccountDate);
                                    PlanAcctTitleTransDetail planAcctTitleTransDetail = PlanSettleManager
                                        .buildPlanAcctTitleTransDetail(
                                            calcAccountDate, planAcctTitle, true);

                                    Money finFeeAmt =new Money();
                                    if(DateTimeUtil.isBeforeOnDate(FEE_OFFSET_DAY,calcAccountDate)){
                                        if(StringUtil.equals(planAcctTitle.getPlanNo(), "9805102")) {
                                            finFeeAmt = loanInstalLogRepository.getOrgDistPftApporAmt(calcAccountDate,
                                                planAcctTitle.getPlanNo());
                                        }
                                        Money newFinFeeAmt = loanAstProfitLogRepo.getOrgDistPftAmt(calcAccountDate,planAcctTitle.getPlanNo());
                                        finFeeAmt.addTo(newFinFeeAmt);
                                    }else{
                                        //2019-11-28之前金融服务费 获取逻辑
                                        finFeeAmt = loanInstalLogRepository.getOrgDistPftAmt(calcAccountDate,planAcctTitle.getPlanNo());
                                    }
                                    planAcctTitleTransDetail.entry(finFeeAmt.getAmount());
                                    planAcctTitleTransDetail.setSettleDate(calcAccountDate);
                                    // set afterBal
                                    planAcctTitleTransDetail.processBeforeUpdate();

                                    acctTitleTransDetails.add(planAcctTitleTransDetail);
                                });
                            }
                        });
                    }
    
                    // 获取当前转让会计日
                    Day trfDate = orgAcctDayComponent.getTrfAcctDay();
                    
                    transactionTemplate.execute(status -> {
                        if (CollectionUtils.isNotEmpty(acctTitleTransDetails)) {
                            planAcctTitleManager.store(acctTitleTransDetails);
                        }
                        astSyncTaskCtrRepo.insertSyncLog(trfDate.getStandardDate(), orgDate, taskCtr,
                            SyncTaskTypeEnum.AST_SER_FEE,
                            TaskStatusEnum.SUCCESS, hostManager.getHostInfo().getHostName());
                        return status;
                    });

                    LogUtil.info(LOGGER, "[ZLFinSerFeeJob]-[{0}]招联金融服务费计算end", orgDate.getDateString());
                }
            } else {
                LogUtil.info(LOGGER, "[ZLFinSerFeeJob]-[{0}]招联金融服务费计算-ignore，已执行", taskCtr.getOrgCode());
            }
        } else {
            LogUtil.info(LOGGER, "[ZLFinSerFeeJob]-[{0}]招联金融服务费计算-ignore，AST_MOD未完成", taskCtr.getOrgCode());
        }
    }

    public void setZlOrgCode(String zlOrgCode) {
        this.zlOrgCode = zlOrgCode;
    }

}
