/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.keeper.node;

import com.abssqr.plat.common.dal.sync.auto.resultmap.MfsFailSyncBill;
import com.abssqr.plat.common.facade.enums.AbssqrNoticeTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.wk.repo.WkMfsSyncCtrRepo;
import com.general.system.common.model.IEnum;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Set;

/**
 * 资产同步模型转换 异常监控
 *
 * @author hanfei
 * @version $Id: AstSyncDispatcherKeeper.java, v 0.1 2019-07-26 2:31 PM hanfei Exp $
 */
@Component
public class AstModTransformKeeper extends AbstractKeeperNode {

    @Autowired
    private WkMfsSyncCtrRepo wkMfsSyncCtrRepo;

    @Value("${wkabs.org.zhaolian:1009}")
    private String zhaolianOrgCode;

    @Override
    protected Void doHandle(Date accountDate) {
        LogUtil.info(LOGGER, "资产模型转换任务监控-start");
        MfsFailSyncBill bill = wkMfsSyncCtrRepo.countFailSyncBill(null, execTimes, Lists.newArrayList(
                TaskStatusEnum.INIT.getCode(), TaskStatusEnum.EXECUTING.getCode(),TaskStatusEnum.FAILURE.getCode()));
        if (bill != null && bill.getCnt() > 0) {
            // 获取用户
            String msg = MessageUtil.formatMsg("模型转换异常，异常数量[{0}]，最大重试次数[{1}]", bill.getCnt(), bill.getMaxExecTimes());
            LogUtil.info(LOGGER, msg);
            // 获取用户
            Set<String> receivers = super.getAdminReceivers();
            sendSystemNotice("资产同步模型转换告警", zhaolianOrgCode, msg, receivers);
        }
        LogUtil.info(LOGGER, "资产模型转换任务监控-end");
        return null;
    }

    @Override
    protected IEnum getBizType() {
        return AbssqrNoticeTypeEnum.WkMfsModTransform;
    }

    @Override
    protected String getWatchJob() {
        return "Wk1009AstSyncDispatcher";
    }

}
