/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */

package com.abssqr.plat.biz.shared.scheduler.job.keeper.node;

import com.abssqr.plat.biz.shared.scheduler.job.capitalAssertMatch.CapitalAssertMatchJob;
import com.abssqr.plat.common.facade.enums.AbssqrNoticeTypeEnum;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.plan.PlanTaskLog;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.plan.PlanTaskLogRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.general.system.common.model.IEnum;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.SystemDateUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * 资金资产匹配监控
 *
 * @author xiachenxiang
 * @version com.abssqr.plat.biz.shared.scheduler.job.keeper.node: AstMatchKeeper.java, v 0.1 2020-02-07 1:47 PM xiachenxiang Exp $
 */
@Component
public class CapitalAstMatchKeeper extends AbstractKeeperNode {

    @Autowired
    private CommonTaskRepository commonTaskRepository;
    @Autowired
    private PlanTaskLogRepo      planTaskLogRepo;


    @Value("${wkabs.org.zhaolian:1009}")
    private String zhaolianOrgCode;
    @Value("${wkabs.zhaolian.capitalAstMatchKeeper.deadline:10 0 0}")
    private String deadline;
    private Long   second = 60L * 60L;

    @Override
    protected Void doHandle(Date date) {
        Date sysday = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
        // 判断当前时间是否大于10点
        Date currentTime = getCurrentStanderDate();
        Date deadlineTime = getDeadLineTime(currentTime, deadline);
        if (currentTime.compareTo(deadlineTime) > 0) {
            List<PlanTaskLog> tasks = planTaskLogRepo.getTasks(sysday, PlanTaskTypeEnum.CAP_AST_MATCH_KEEPER);
            if (CollectionUtils.isEmpty(tasks)) {
                LogUtil.info(LOGGER, "资金资产匹配监控 - start");
                StringBuffer sb = new StringBuffer();
                exec(sb, CommonTaskTypeEnum.SYNC_CAPTITALASSERT_MATCH);
                exec(sb, CommonTaskTypeEnum.UNMATCH_CAPTITALASSERT_MATCH);
                exec(sb, CommonTaskTypeEnum.CAPTITALASSERT_UNMATCH);
                if (StringUtils.isNotEmpty(sb)) {
                    // 获取用户
                    LogUtil.info(LOGGER, sb.toString());
                    // 获取用户
                    Set<String> receivers = super.getAdminReceivers();
                    sendSystemNotice("资金资产匹配监控告警", zhaolianOrgCode, sb.toString(), receivers);
                }

                PlanTaskLog planTaskLog = new PlanTaskLog();
                planTaskLog.setExecDate(sysday);
                planTaskLog.setPlanNo(zhaolianOrgCode);
                planTaskLog.setTaskType(PlanTaskTypeEnum.CAP_AST_MATCH_KEEPER);
                planTaskLogRepo.createTask(planTaskLog);
                LogUtil.info(LOGGER, "资金资产匹配监控 - end");
            }
        }
        return null;
    }

    /**
     * @param sb       提示消息
     * @param typeEnum 任务类型
     */
    private void exec(StringBuffer sb, CommonTaskTypeEnum typeEnum) {
        // 判断是否存在失败的任务
        List<CommonTask> commonTasks = commonTaskRepository.queryByTypesAndStatus(Lists.newArrayList(typeEnum.getCode()), Lists.newArrayList(TaskStatusEnum.FAILURE.getCode()));
        if (CollectionUtils.isNotEmpty(commonTasks)) {
            sb.append(MessageUtil.formatMsg("[{0}]存在{1}条失败任务\n", typeEnum.getDesc(), commonTasks.size()));
        }
        // 判断执行时间大于1小时
        Long count = commonTaskRepository.countByTypesAndStatusAndRange(second, Lists.newArrayList(typeEnum.getCode()), Lists.newArrayList(TaskStatusEnum.EXECUTING.getCode()));
        if (count.intValue() != 0) {
            sb.append(MessageUtil.formatMsg("[{0}]存在{1}条执行超时任务\n", typeEnum.getDesc(), count));
        }
        // 判断执行次数大于1
        count = commonTaskRepository.countUnFinishAndMultipleExecByTypes(Lists.newArrayList(typeEnum.getCode()));
        if (count.intValue() != 0) {
            sb.append(MessageUtil.formatMsg("[{0}]存在{1}条执行次数大于1的任务\n", typeEnum.getDesc(), count));
        }
    }

    @Override
    protected IEnum getBizType() {
        return AbssqrNoticeTypeEnum.CapitalAstMatch;
    }

    @Override
    protected String getWatchJob() {
        return CapitalAssertMatchJob.class.getName();
    }
}
