/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */

package com.abssqr.plat.biz.shared.scheduler.job.keeper.node;

import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstLogAmt;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.CapitalOrderAmt;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.ClearingLogCntAndId;
import com.abssqr.plat.common.facade.enums.AbssqrNoticeTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanCapitalTransTypeEnum;
import com.abssqr.plat.common.facade.enums.StageChangeTypeEnum;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.model.domain.job.JobControl;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingLogRepo;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingOrderRepo;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.model.repo.plan.PlanAstLogRepository;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalOrderRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.general.system.common.model.IEnum;
import com.general.system.common.model.Money;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.StringUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 内部校验
 *
 * @author xiachenxiang
 * @version com.abssqr.plat.biz.shared.scheduler.job.keeper.node: DataCheckKeeper.java, v 0.1 2020-01-06 4:40 PM xiachenxiang Exp $
 */
@Component
public class DataCheckKeeper extends AbstractKeeperNode {

    @Autowired
    private JobControlRepository    jobControlRepository;
    @Autowired
    private AstSyncTaskCtrRepo      astSyncTaskCtrRepo;
    @Autowired
    private PlanRepo                planRepo;
    @Autowired
    private PlanCapitalOrderRepo    planCapitalOrderRepo;
    @Autowired
    private PlanClearingLogRepo     planClearingLogRepo;
    @Autowired
    private PlanClearingOrderRepo   planClearingOrderRepo;
    @Autowired
    private PlanAstLogRepository    planAstLogRepository;
@Autowired
    private OrgAcctDayComponent orgAcctDayComponent;
    
    @Override
    protected Void doHandle(Date sysDate) {

        JobControl jobControl = jobControlRepository.selectByJobName("DataCheckKeeper");
        if (!jobControl.getIsEff().isYes()) {
            LogUtil.warn(LOGGER, "忽略[{0}--{1}]定时任务", "DataCheckKeeper", jobControl.getIsEff());
            return null;
        }
        Date curDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
        //获取已完成的机构
        List<String> orgCodeChecks = astSyncTaskCtrRepo.getLastStatusFinishedOrgCodes(curDate,
                SyncTaskTypeEnum.AST_MOD);
        if (CollectionUtils.isEmpty(orgCodeChecks)) {
            return null;
        }
        // 支付单数量为0
        if (!capitalCntIsZero()) {
            return null;
        }
        StringBuffer sb = new StringBuffer();
        boolean eq = true;
        LogUtil.info(LOGGER,"[DataCheckKeeper]-[{0}]数据核对start", curDate);
        for (String spvNo : orgCodeChecks) {
            eq &= checkData(spvNo, curDate, sb,TransCodeEnum.REPO);
            eq &=  checkData(spvNo, curDate, sb,TransCodeEnum.BUY_AST);

        }
        if (!eq && sb.length()>0){
            String msg = sb.toString();
            if (!StringUtil.isEmpty(msg)) {
                super.sendSystemNotice("内部数据核验", (String) getBizType().getCode(), msg, super.getAdminReceivers());
            }
        }
        LogUtil.info(LOGGER, "[DataCheckKeeper]-[{0}]数据核对end", curDate);
        return null;
    }

    /**
     * 还款校验
     * 清算金额 = 结算金额
     * 结算金额 = 支付单实际交易金额
     * 支付单交易金额 = 计划水位变更金额
     *
     * @param spvNo
     * @param date
     * @param sb
     * @param transCodeEnum
     */
    private boolean checkData(String spvNo, Date date, StringBuffer sb, TransCodeEnum transCodeEnum) {
        Map<String, List<String>> planMap = planRepo.getPlanBySpvs(Lists.newArrayList(spvNo));
        List<String> planList = planMap.get(spvNo);
        if (CollectionUtils.isEmpty(planList)){
            planList = Lists.newArrayList();
        }
        if (transCodeEnum == TransCodeEnum.REPO) {
            planList.add(spvNo);
        }
        Money planAdd = new Money();
        Money spvAdd = new Money();
        boolean eq = true;
        for (String planNo : planList) {
            // 清算 结算金额校验
            ClearingLogCntAndId start = planClearingLogRepo.getHisMaxIdAndCnt(date, planNo, transCodeEnum);
            Money clearAmt = new Money();
            if (transCodeEnum == TransCodeEnum.BUY_AST && planNo.equals(spvNo)) {
                clearAmt = planClearingLogRepo.getClearAmtByCntPtyCode(date, planNo, transCodeEnum);
            } else {
                clearAmt = planClearingLogRepo.getHisAmtByPlanNo(date, planNo, transCodeEnum);
            }
            Money clrOrderAmt = planClearingOrderRepo.getAmtByPlanNo(date, planNo, transCodeEnum);
            eq = checkData(clearAmt, clrOrderAmt, start, transCodeEnum, planNo, date, sb, "[{0}]清算金额与结算金额不一致,清算金额[{1}],结算金额[{2}],事件[{3}]");
            if (!eq) {
                return eq;
            }
            // 结算 支付
            CapitalOrderAmt capitalOrderAmt = planCapitalOrderRepo.getHisAmtByPlanNo(date, planNo,
                    PlanCapitalTransTypeEnum.getCapitalTransTypeByTransfer(transCodeEnum, !planNo.equals(spvNo)));
            Money payableAmt = Optional.ofNullable(capitalOrderAmt).map(CapitalOrderAmt::getPayableAmt).orElseGet(Money::new);
            eq = checkData(clrOrderAmt, payableAmt, start, transCodeEnum, planNo, date, sb, "[{0}]结算金额与支付金额不一致,结算金额[{1}],支付金额[{2}],事件[{3}]");
            if (!eq) {
                return eq;
            }
            // 支付 水位
            // 还款事件
            // 放款池水位减 = 放款池支付交易金额
            // 放款池水位加 = 放款池支付交易金额 + 关联受让池水位加
            // 放款池水位加 = 结算交易金额 + 关联受让池水位加
            List<AstLogAmt> astLogAmtList = planAstLogRepository.getAmtByPlanNo(date, planNo, transCodeEnum);
            if (CollectionUtils.isEmpty(astLogAmtList)) {
                LogUtil.warn(LOGGER, "[{0}]不存在事件[{1}]对应的水位记录", planNo, transCodeEnum.getCode());
                return false;
            }
            if (transCodeEnum == TransCodeEnum.REPO) {
                Optional<AstLogAmt> AstAddLogAmt = astLogAmtList.stream().filter(item -> item.getChangeType().equals(StageChangeTypeEnum.ADD.getCode())).findFirst();
                if (!AstAddLogAmt.isPresent()) {
                    LogUtil.error(LOGGER, "[{0}]不存在事件[{1}]对应的水位增加记录", planNo, transCodeEnum.getCode());
                }
                if (!planNo.equals(spvNo)) {
                    Money amt = AstAddLogAmt.get().getAmt();
                    Money transAmt = Optional.ofNullable(capitalOrderAmt).map(CapitalOrderAmt::getTransAmt).orElseGet(Money::new);
                    eq = checkData(transAmt, amt, start, transCodeEnum, planNo, date,
                            sb, "[{0}]交易金额与水位变化金额不一致,交易金额[{1}],水位变化金额[{2}],事件[{3}]");
                    if (!eq) {
                        return eq;
                    }
                    planAdd.addTo(amt);
                } else {
                    Money amt = AstAddLogAmt.get().getAmt();
                    spvAdd.addTo(amt);
                    planAdd.addTo(capitalOrderAmt.getTransAmt());
                }
                // 水位扣减
                Optional<AstLogAmt> AstSubLogAmt = astLogAmtList.stream().filter(item -> item.getChangeType().equals(StageChangeTypeEnum.SUB.getCode())).findFirst();
                if (AstSubLogAmt.isPresent()) {
                    Money amt = AstSubLogAmt.get().getAmt();
                    eq = checkData(capitalOrderAmt.getTransAmt(), amt, start, transCodeEnum, planNo, date,
                            sb, "[{0}]交易金额与水位变化金额不一致,交易金额[{1}],水位变化金额[{2}],事件[{3}]");
                    if (!eq) {
                        return eq;
                    }
                }
            } else {
                Money amt = astLogAmtList.get(0).getAmt();
                eq = checkData(capitalOrderAmt.getTransAmt(), amt, start, transCodeEnum, planNo, date,
                        sb, "[{0}]交易金额与水位变化金额不一致,交易金额[{1}],水位变化金额[{2}],事件[{3}]");
                if (!eq) {
                    return eq;
                }
            }
        }
        eq = spvAdd.subtract(planAdd).isZero();
        if (!eq) {
            sb.append(MessageUtil.formatMsg("[{0}]交易金额与水位变化金额不一致,交易金额[{1}],水位变化金额[{2}],事件[{3}]", spvNo, spvAdd, planAdd, transCodeEnum.getCode()));
        }
        return eq;
    }


    /**
     * 校验左右金额是否相等
     * 当金额不等 、 清算数据相等且支付单数量为0的时候加入错误信息
     *
     * @param left
     * @param right
     * @param start         开始时的数据
     * @param transCodeEnum 交易类型
     * @param planNo        计划编号
     * @param date          日期
     * @param sb
     * @param msg
     * @return
     */
    private boolean checkData(Money left, Money right, ClearingLogCntAndId start, TransCodeEnum transCodeEnum,
                              String planNo, Date date, StringBuffer sb, String msg) {
        left = left == null ? new Money() : left;
        right = right == null ? new Money() : right;
        boolean flag = left.subtract(right).isZero();
        if (!flag && checkClearEq(start, transCodeEnum, planNo, date) && this.capitalCntIsZero()) {
            sb.append(MessageUtil.formatMsg(msg, planNo, left, right, transCodeEnum.getCode()));
        }
        return flag;
    }

    /**
     * 校验清算数据是否一致
     *
     * @param start         开始时的数据
     * @param transCodeEnum 交易码
     * @param planNo        计划编号
     * @param date          会计日
     * @return
     */
    private boolean checkClearEq(ClearingLogCntAndId start, TransCodeEnum transCodeEnum, String planNo, Date date) {
        ClearingLogCntAndId end = planClearingLogRepo.getHisMaxIdAndCnt(date, planNo, transCodeEnum);
        if (start.getCnt() != null && end.getCnt() != null && start.getMaxId() != null && end.getMaxId() != null) {
            return start.getCnt().intValue() == end.getCnt().intValue()
                   && start.getMaxId().intValue() == end.getMaxId().intValue();
        }
        return false;
    }

    /**
     * 支付单是否为0
     *
     * @return
     */
    private boolean capitalCntIsZero() {
        return planCapitalOrderRepo.getActiveCntByType(null).intValue() == 0;
    }

    @Override
    protected IEnum getBizType() {
        return AbssqrNoticeTypeEnum.DataCheck;
    }

    @Override
    protected String getWatchJob() {
        return null;
    }
}
