/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.keeper.node;

import com.abssqr.plat.common.dal.mysql.auto.dao.AbsDataCheckDAO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.DataForCheck;
import com.abssqr.plat.common.dal.sync.auto.dao.WkDataCheckDAO;
import com.abssqr.plat.common.dal.sync.auto.resultmap.WKDataForCheck;
import com.abssqr.plat.common.facade.enums.AbssqrNoticeTypeEnum;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.model.domain.job.JobControl;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.core.service.host.HostManager;
import com.general.system.common.model.IEnum;
import com.general.system.common.model.Money;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.MoneyUtil;
import com.general.system.common.util.StringUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 五矿数据核验
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.scheduler.job.keeper.node: WkDataCheckKeeper.java, v 0.1 2019-09-27 11:57
 * bangis.wangdf Exp $
 */
@Component
public class WkDataCheckKeeper extends AbstractKeeperNode {

    @Autowired
    AbsConfig            absConfig;
    @Autowired
    AstSyncTaskCtrRepo   astSyncTaskCtrRepo;
    @Autowired
    PlanRepo             planRepo;
    @Autowired
    WkDataCheckDAO       wkDataCheckDAO;
    @Autowired
    AbsDataCheckDAO      absDataCheckDAO;
    @Autowired
    HostManager          hostManager;
    @Autowired
    JobControlRepository jobControlRepository;

    @Override
    protected Void doHandle(Date date) {
        JobControl jobControl = jobControlRepository.selectByJobName("WkDataCheckKeeper");
        if (!jobControl.getIsEff().isYes()) {
            LogUtil.warn(LOGGER, "忽略[{0}--{1}]定时任务", "WkDataCheckKeeper", jobControl.getIsEff());
            return null;
        }
        // 转让会计日
        Day trfDate = orgAcctDayComponent.getTrfAcctDay();

        // 获取已完成的机构
        List<String> orgCodeChecks = astSyncTaskCtrRepo.getLastStatusFinishedOrgCodes(trfDate.getStandardDate(), SyncTaskTypeEnum.AST_CHK);
        // 获取已完成的机构
        List<String> orgCodes = astSyncTaskCtrRepo.getLastStatusFinishedOrgCodes(trfDate.getStandardDate(), SyncTaskTypeEnum.AST_MOD);
        // 待核验的机构
        List<String> forCheckOrgs = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(orgCodes)) {
            orgCodes.forEach(org -> {
                if (!orgCodeChecks.contains(org)) {
                    forCheckOrgs.add(org);
                }
            });
        }

        if (CollectionUtils.isNotEmpty(forCheckOrgs)) {
            LogUtil.info(LOGGER, "[WkDataCheckKeeper]-[{0}]数据核对start", trfDate.getDateString());
            forCheckOrgs.forEach(orgCode -> {
                StringBuilder sb = new StringBuilder();
                OrgCodeEnum orgCodeEnum = OrgCodeEnum.getByCode(orgCode);
                // 机构会计日
                Day orgDate = orgAcctDayComponent.getAstAcctDay(orgCodeEnum);
                // 核验数据
                this.checkData(trfDate, orgCode, orgDate, sb);
                String msg = sb.toString();
                if (!StringUtil.isEmpty(msg)) {
                    super.sendSystemNotice("五矿数据核验", (String)getBizType().getCode(), msg, super.getAdminReceivers());
                }
                AstSyncTaskCtr astSyncTaskCtr = astSyncTaskCtrRepo.getAstSyncTaskCtr(orgCode);
                astSyncTaskCtrRepo.insertSyncLog(trfDate.getStandardDate(), orgDate, astSyncTaskCtr, SyncTaskTypeEnum.AST_CHK,
                    TaskStatusEnum.SUCCESS, hostManager.getHostInfo().getHostName());
            });
            LogUtil.info(LOGGER, "[WkDataCheckKeeper]-[{0}]数据核对end", trfDate.getDateString());
        }
        return null;
    }
    
    /**
     * 数据核验
     * @param trfDate
     * @param orgCode
     * @param orgDate
     * @param sb
     */
    public void checkData(Day trfDate, String orgCode, Day orgDate, StringBuilder sb) {
        List<OrganizationEntity> spvPlans = planRepo.getSpvByManagerOrgCode(orgCode);
        if (CollectionUtils.isNotEmpty(spvPlans)) {
            List<String> spvPlanNos = spvPlans.stream().map(OrganizationEntity::getPlanNo).collect(
                Collectors.toList());
//            Map<String, List<String>> planBySpvs = planRepo.getPlanBySpvs(spvPlanNos);
            //获取五矿数据情况
            List<WKDataForCheck> wkDataForChecks = wkDataCheckDAO.getDataForCheck(
                    orgCode, absConfig.getDbLink(), orgDate.getDayString(), absConfig.getWkxwDbName(), absConfig.getWkxwSynonymPre(), spvPlanNos);
            //获取到模型转换完数据
            Map<String, DataForCheck> spvDataForCheckMap = buildAllSpvData(trfDate.getStandardDate(), spvPlanNos, wkDataForChecks);
            //1. 放款计划数一致
            //if (wkDataForChecks.size() == spvDataForCheckMap.size()) {
            //2. 内容一致
            if (CollectionUtils.isNotEmpty(wkDataForChecks) && MapUtils.isNotEmpty(spvDataForCheckMap)) {
                wkDataForChecks.forEach(wkData -> {
                    DataForCheck dataForCheck = spvDataForCheckMap.get(wkData.getOwnOrg());
                    if (dataForCheck != null) {
                        checkData(dataForCheck, wkData, sb);
                    }
                });
            }
            //} else {
            //    sb.append(MessageUtil
            //        .formatMsg("五矿小微数据核验失败，{0}-数据同步失败-期望存在计划数{1},实际存在计划数{2}", orgCode, wkDataForChecks.size(),
            //            spvDataForCheckMap.size()));
            //}
        }
    }

    private void checkData(DataForCheck dataForCheck, WKDataForCheck wkData, StringBuilder sb) {
        checkData(wkData.getFkCnt(), dataForCheck.getFkCnt(), sb, dataForCheck.getOwnOrg() + "放款数量校验失败:[{0}]、[{1}]");
        checkData(wkData.getFkAmt(), dataForCheck.getFkAmt(), sb, dataForCheck.getOwnOrg() + "放款金额校验失败:[{0}]、[{1}]");
        //还款
        checkData(wkData.getRepayCnt(), dataForCheck.getRepayCnt(), sb,
            dataForCheck.getOwnOrg() + "还款金额校验失败:[{0}]、[{1}]");
        checkData(wkData.getRepayPrinAmt(), dataForCheck.getPrinAmt(), sb,
            dataForCheck.getOwnOrg() + "还款本金校验失败:[{0}]、[{1}]");
        checkData(wkData.getRepayIntAmt(), dataForCheck.getIntrAmt(), sb,
            dataForCheck.getOwnOrg() + "还款利息校验失败:[{0}]、[{1}]");
        checkData(wkData.getRepayPnltIntAmt(), dataForCheck.getPrinPenalAmt(), sb,
            dataForCheck.getOwnOrg() + "还款罚息校验失败:[{0}]、[{1}]");
        checkData(wkData.getRepayTrustIntAmt(), dataForCheck.getPaidIntrFundAmt(), sb,
            dataForCheck.getOwnOrg() + "五矿实收利息校验失败:[{0}]、[{1}]");
        checkData(wkData.getRepayTrustPnltIntAmt(), dataForCheck.getPaidPrinPenalFundAmt(), sb,
            dataForCheck.getOwnOrg() + "五矿实收罚息校验失败:[{0}]、[{1}]");
        //剩余资产
        checkData(wkData.getAstCnt(), dataForCheck.getAstCnt(), sb,
            dataForCheck.getOwnOrg() + "剩余资产笔数校验失败:[{0}]、[{1}]");
        checkData(wkData.getAstPrinBal(), dataForCheck.getAstPrinBal(), sb,
            dataForCheck.getOwnOrg() + "剩余资产本金校验失败:[{0}]、[{1}]");
        checkData(wkData.getAstIntBal(), dataForCheck.getAstIntrBal(), sb,
            dataForCheck.getOwnOrg() + "剩余资产利息校验失败:[{0}]、[{1}]");
        checkData(wkData.getAstPnltIntBal(), dataForCheck.getAstPrinPenalBal(), sb,
            dataForCheck.getOwnOrg() + "剩余资产罚息校验失败:[{0}]、[{1}]");

    }

    private void checkData(Long wkData, Long abssqrData, StringBuilder sb, String msgTemplate) {
        long wk = wkData == null ? 0L : wkData;
        long abs = abssqrData == null ? 0L : abssqrData;
        if (wk != abs) {
            sb.append(MessageUtil
                .formatMsg(msgTemplate, wk, abs));
        }
    }

    private void checkData(Money wkData, Money abssqrData, StringBuilder sb, String msgTemplate) {
        Money wk = wkData == null ? new Money() : wkData;
        Money abs = abssqrData == null ? new Money() : abssqrData;
        if (MoneyUtil.compare(wk, abs) != 0) {
            sb.append(MessageUtil
                .formatMsg(msgTemplate, wk, abs));
        }
    }

    private Map<String, DataForCheck> buildAllSpvData(Date curDate, List<String> spvPlanNos,
                                                      List<WKDataForCheck> wkDataForChecks) {
        Map<String, DataForCheck> spvDataForCheckMap = Maps.newHashMap();
        if (CollectionUtils.isNotEmpty(wkDataForChecks)) {
            List<DataForCheck> dataForChecks = absDataCheckDAO.getDataForCheck(curDate, spvPlanNos);
            if (CollectionUtils.isNotEmpty(dataForChecks)) {
                //模型转换完成后数据
                spvDataForCheckMap.putAll(buildAllSpvData(dataForChecks));
            }
        }
        return spvDataForCheckMap;
    }

    private Map<String, DataForCheck> buildAllSpvData(List<DataForCheck> dataForChecks) {
        Map<String, DataForCheck> spvDataForCheckMap = Maps.newHashMap();
        dataForChecks.forEach(data -> {
            spvDataForCheckMap.put(data.getLenderOrgCode(), data);
        });
        return spvDataForCheckMap;
    }

    @Override
    protected IEnum getBizType() {
        return AbssqrNoticeTypeEnum.WkDataCheck;
    }

    @Override
    protected String getWatchJob() {
        return null;
    }

}
