/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.sync.ast;

import com.abssqr.plat.biz.shared.scheduler.BaseQuartzJob;
import com.abssqr.plat.common.dal.sync.auto.dao.WkAbsMfsBillCtrDAO;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.host.HostManager;
import com.abssqr.plat.core.service.wk.ast.sync.ZLAstSyncTaskManager;
import com.general.system.common.util.LogUtil;
import org.apache.commons.lang3.StringUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 招联原始资产同步
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.scheduler.job: ZLOriginalAstSyncJob.java, v 0.1 2019-06-12 18:54 bangis.wangdf
 * Exp $
 */
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
@Component
public class ZLOriginalAstSyncJob extends BaseQuartzJob {
    @Autowired
    private HostManager          hostManager;
    @Autowired
    private AstSyncTaskCtrRepo   astSyncTaskCtrRepo;
    @Autowired
    private ZLAstSyncTaskManager zlAstSyncTaskManager;
    @Autowired
    private AbsConfig            absConfig;
    @Autowired
    private WkAbsMfsBillCtrDAO   wkAbsMfsBillCtrDAO;
    @Autowired
    private OrgAcctDayComponent  orgAcctDayComponent;
    
    
    @Value("${sync.quartz.ast.sync}")
    private String cronExpr;

    @Override
    public void afterPropertiesSet() throws Exception {
        super.afterPropertiesSet();
        astSyncTaskCtrRepo.updateSyncLogStatus(hostManager.getHostInfo().getHostName(), null, SyncTaskTypeEnum.AST_SYNC,
            TaskStatusEnum.FAILURE,
            TaskStatusEnum.EXECUTING);
    }

    @Override
    protected String getCronExpr() {
        return cronExpr;
    }

    @Override
    public void doExecute() {
        List<AstSyncTaskCtr> astSyncTaskCtrs = astSyncTaskCtrRepo.getAstSyncTaskCtrs();
        astSyncTaskCtrs.forEach(astSyncTaskCtr -> {
            try {
                if(astSyncTaskCtr.getEff()==true) {
                    //自动维护分区
                    autoPartition(astSyncTaskCtr);
                    //执行同步
                    doExecute(astSyncTaskCtr);
                }
            } catch (Throwable e) {
                LogUtil.error(LOGGER, e, "数据同步失败 orgCode=" + astSyncTaskCtr.getOrgCode());
            }
        });

    }

    public void autoPartition(AstSyncTaskCtr astSyncTaskCtr) {
        String partitionName = "BILL_CTR_"+astSyncTaskCtr.getOrgCode();
        String pName = wkAbsMfsBillCtrDAO.queryPartitionForCheck(partitionName);
        if(StringUtils.isBlank(pName)){
            wkAbsMfsBillCtrDAO.updateCreatePartition(astSyncTaskCtr.getOrgCode(),partitionName);
        }
    }

    public void doExecute(AstSyncTaskCtr astSyncTaskCtr) {
        OrgCodeEnum orgCode = OrgCodeEnum.getByCode(astSyncTaskCtr.getOrgCode());
        //ABS会计日
        Day absAcctDay = orgAcctDayComponent.getTrfAcctDay();
        // 获取对应机构会计日
        Day orgDay = orgAcctDayComponent.getAstAcctDay(orgCode);
        // 通过AstSyncTaskCtr 任务控制进行判断是否可以执行
        if (astSyncTaskCtr.getEff() == true) {
            if (StringUtils.isNotBlank(astSyncTaskCtr.getExtData().getCtrFlg())) {
                // 并判断今日同步任务是否已经执行
                if (astSyncTaskCtrRepo.checkCanRun(astSyncTaskCtr.getOrgCode(),
                    astSyncTaskCtr.getTaskCode(), SyncTaskTypeEnum.AST_SYNC, orgDay.getStandardDate())) {
                    zlAstSyncTaskManager.syncData(astSyncTaskCtr, orgDay, absAcctDay);
                }
            }
        }
    }
}
