package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstPackDO;
import java.util.List;
import java.util.Date;
import com.abssqr.plat.common.dal.mysql.auto.paging.ActivePackPage;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.mapper.AstPackDOMapper;

/**
* The Table ABS_AST_PACK.
* 资产包
*/
@Repository
public class AstPackDAO{

    @Autowired
    private AstPackDOMapper astPackDOMapper;

    /**
     * desc:插入表:ABS_AST_PACK.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(AstPackDO entity){
        return astPackDOMapper.insert(entity);
    }
    /**
     * desc:批量插入表:abs_account.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<AstPackDO> list){
        astPackDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:根据状态获取包.<br/>
     * @param scenDate scenDate
     * @param packType packType
     * @param packStatus packStatus
     * @return List<AstPackDO>
     */
    
    public List<AstPackDO> selectPackByStatus(Date scenDate,String packType,String packStatus){
        return astPackDOMapper.selectPackByStatus(scenDate, packType, packStatus);
    }
    /**
     * desc:根据packCode获取数据:ABS_AST_PACK.<br/>
     * @param packCode packCode
     * @return AstPackDO
     */
    @Deprecated
    public AstPackDO getByPackCode(String packCode){
        return astPackDOMapper.getByPackCode(packCode);
    }
    /**
     * desc:根据普通索引AbsTrCtrTask0获取数据:abs_tr_ctr_task.<br/>
     * @param scenDate scenDate
     * @param status status
     * @param packType packType
     * @param managerOrgCode managerOrgCode
     * @return List<AstPackDO>
     */
    
    public List<AstPackDO> getByManagerOrg(Date scenDate,String status,String packType,String managerOrgCode){
        return astPackDOMapper.getByManagerOrg(scenDate, status, packType, managerOrgCode);
    }
    /**
     * desc:更新表:ABS_AST_PACK.<br/>
     * @param packStatus packStatus
     * @param oldPackStatus oldPackStatus
     * @param packCodes packCodes
     * @return int
     */
    
    public int updatePackStatus(String packStatus,String oldPackStatus,List<String> packCodes){
    if(packCodes!=null && packCodes.size()>201){throw new RuntimeException("packCodes 超出201长度限制");}
        return astPackDOMapper.updatePackStatus(packStatus, oldPackStatus, packCodes);
    }
    /**
     * desc:根据packCode获取数据:ABS_AST_PACK.<br/>
     * @param planNo planNo
     * @param packType packType
     * @return List<AstPackDO>
     */
    
    public List<AstPackDO> getByPlanNo(String planNo,String packType){
        return astPackDOMapper.getByPlanNo(planNo, packType);
    }
    /**
     * desc:根据packCode获取数据:ABS_AST_PACK.<br/>
     * @param scenDate scenDate
     * @param planNo planNo
     * @param packType packType
     * @return AstPackDO
     */
    
    public AstPackDO getLastDayPackByPlanNo(Date scenDate,String planNo,String packType){
        return astPackDOMapper.getLastDayPackByPlanNo(scenDate, planNo, packType);
    }
    /**
     * desc:根据packCode获取数据:ABS_AST_PACK.<br/>
     * @param activePack activePack
     * @return ActivePackPage
     */
    @Deprecated
    public ActivePackPage queryByActivePacks(ActivePackPage activePack){
        int total = astPackDOMapper.queryByActivePacksCount(activePack);
        if(total>0){
            activePack.setDatas(astPackDOMapper.queryByActivePacksResult(activePack));
        }else{
            activePack.setDatas(new ArrayList());
        }
        activePack.setTotal(total);
        return activePack;
    }
    /**
     * desc:获取指定状态的资产包数量.<br/>
     * @param time time
     * @param status status
     * @param packType packType
     * @return Long
     */
    
    public Long countByStatusAndTime(Date time,String status,String packType){
        return astPackDOMapper.countByStatusAndTime(time, status, packType);
    }
    /**
     * desc:锁资产包.<br/>
     * @param packCode packCode
     * @return AstPackDO
     */
    
    public AstPackDO lockByPackCode(String packCode){
        return astPackDOMapper.lockByPackCode(packCode);
    }
}
