package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.LoanInstalDO;
import java.util.List;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.LoanInstalFormationItem;
import java.util.Date;
import com.abssqr.plat.common.dal.mysql.auto.mapper.LoanInstalDOMapper;

/**
* The Table abs_loan_instal.
* 还款计划明细
*/
@Repository
public class LoanInstalDAO{

    @Autowired
    private LoanInstalDOMapper loanInstalDOMapper;

    /**
     * desc:批量插入表:abs_loan_instal.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(LoanInstalDO entity){
        return loanInstalDOMapper.insert(entity);
    }
    /**
     * desc:根据唯一约束AbsLoanInstal更新表:abs_loan_instal.<br/>
     * @param entity entity
     * @return int
     */
    
    public int update(LoanInstalDO entity){
        return loanInstalDOMapper.update(entity);
    }
    /**
     * desc:根据唯一约束AbsLoanInstal获取数据:abs_loan_instal.<br/>
     * @param astNo astNo
     * @return List<LoanInstalDO>
     */
    
    public List<LoanInstalDO> getByAstNo(String astNo){
        return loanInstalDOMapper.getByAstNo(astNo);
    }
    /**
     * desc:获取对应计划的资产数量.<br/>
     * @param ownOrg ownOrg
     * @param list list
     * @return Long
     */
    
    public Long getPlanAstCount(String ownOrg,List<String> list){
    if(list!=null && list.size()>201){throw new RuntimeException("list 超出201长度限制");}
        return loanInstalDOMapper.getPlanAstCount(ownOrg, list);
    }
    /**
     * desc:获取资产当前剩余分期数量及余额总计.<br/>
     * @param ownOrg ownOrg
     * @return List<LoanInstalFormationItem>
     */
    
    public List<LoanInstalFormationItem> queryAstFormation(String ownOrg){
        return loanInstalDOMapper.queryAstFormation(ownOrg);
    }
    /**
     * desc:获取资产最大分期及余额总计.<br/>
     * @param ownOrg ownOrg
     * @return List<LoanInstalFormationItem>
     */
    
    public List<LoanInstalFormationItem> queryProdFormation(String ownOrg){
        return loanInstalDOMapper.queryProdFormation(ownOrg);
    }
    /**
     * desc:获取计划当前资产规模.<br/>
     * @param ownOrg ownOrg
     * @return Long
     */
    
    public Long getPlanAstScale(String ownOrg){
        return loanInstalDOMapper.getPlanAstScale(ownOrg);
    }
    /**
     * desc:获取计划时间区间内应还汇总金额.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param ownOrg ownOrg
     * @return Long
     */
    
    public Long getPlanAstShouldScaleByEndDate(Date endDate,Date startDate,String ownOrg){
        return loanInstalDOMapper.getPlanAstShouldScaleByEndDate(endDate, startDate, ownOrg);
    }
}
