package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanAllocDO;
import java.util.List;
import java.util.Date;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.mapper.PlanAllocDOMapper;

/**
* The Table abs_plan_alloc.
* 计划-分配计划
*/
@Repository
public class PlanAllocDAO{

    @Autowired
    private PlanAllocDOMapper planAllocDOMapper;

    /**
     * desc:插入表:abs_plan_alloc.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(PlanAllocDO entity){
        return planAllocDOMapper.insert(entity);
    }
    /**
     * desc:批量插入表:abs_plan_alloc.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<PlanAllocDO> list){
        planAllocDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:更新表:abs_plan_alloc.<br/>
     * @param entity entity
     * @return int
     */
    
    public int updateByAllocNo(PlanAllocDO entity){
        return planAllocDOMapper.updateByAllocNo(entity);
    }
    /**
     * desc:根据AllocNo删除数据:abs_plan_alloc.<br/>
     * @param allocNo allocNo
     * @return int
     */
    
    public int deleteByAllocNo(String allocNo){
        return planAllocDOMapper.deleteByAllocNo(allocNo);
    }
    /**
     * desc:根据主键获取数据:abs_plan_alloc.<br/>
     * @param allocNo allocNo
     * @return PlanAllocDO
     */
    
    public PlanAllocDO getByAllocNo(String allocNo){
        return planAllocDOMapper.getByAllocNo(allocNo);
    }
    /**
     * desc:更新状态.<br/>
     * @param status status
     * @param allocNo allocNo
     * @param oldStatus oldStatus
     * @return int
     */
    
    public int updateStatus(String status,String allocNo,String oldStatus){
        return planAllocDOMapper.updateStatus(status, allocNo, oldStatus);
    }
    /**
     * desc:根据主键获取数据:abs_plan_alloc.<br/>
     * @param planNo planNo
     * @return List<PlanAllocDO>
     */
    
    public List<PlanAllocDO> queryByPlanNo(String planNo){
        return planAllocDOMapper.queryByPlanNo(planNo);
    }
    /**
     * desc:根据计划编号获取时间段分配金额.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param planNo planNo
     * @param status status
     * @return List<PlanAllocDO>
     */
    
    public List<PlanAllocDO> queryAllocByPlanNo(Date endDate,Date startDate,String planNo,String status){
        return planAllocDOMapper.queryAllocByPlanNo(endDate, startDate, planNo, status);
    }
    /**
     * desc:根据兑付日获取查询最近计划:abs_plan_alloc.<br/>
     * @param planType planType
     * @return PlanAllocDO
     */
    
    public PlanAllocDO queryRecentlyPlans(String planType){
        return planAllocDOMapper.queryRecentlyPlans(planType);
    }
    /**
     * desc:查询相同结算日分配计划数量.<br/>
     * @param clearDate clearDate
     * @param planNo planNo
     * @param allocNo allocNo
     * @return int
     */
    
    public int querySameClearDate(Date clearDate,String planNo,String allocNo){
        return planAllocDOMapper.querySameClearDate(clearDate, planNo, allocNo);
    }
    /**
     * desc:根据主键获取数据:abs_plan_alloc.<br/>
     * @param allocNo allocNo
     * @return PlanAllocDO
     */
    
    public PlanAllocDO lockByAllocNo(String allocNo){
        return planAllocDOMapper.lockByAllocNo(allocNo);
    }
    /**
     * desc:获取所有已审批通过未解绑的兑付计划.<br/>
     * @param planNo planNo
     * @return List<PlanAllocDO>
     */
    
    public List<PlanAllocDO> queryUnMatchedApprovedAlloc(String planNo){
        return planAllocDOMapper.queryUnMatchedApprovedAlloc(planNo);
    }
    /**
     * desc:根据计划编号获取时间段分配金额.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param planNo planNo
     * @param status status
     * @param approveStatus approveStatus
     * @return Long
     */
    
    public Long queryStatAllocAmtByPlanNoAndPayDate(Date endDate,Date startDate,String planNo,String status,String approveStatus){
        return planAllocDOMapper.queryStatAllocAmtByPlanNoAndPayDate(endDate, startDate, planNo, status, approveStatus);
    }
    /**
     * desc:根据计划编号获取累计分配金额.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param status status
     * @param approveStatus approveStatus
     * @param listPlanNos listPlanNos
     * @return Long
     */
    
    public Long queryStatTotalAllocAmtByPlanNos(Date endDate,Date startDate,String status,String approveStatus,List<String> listPlanNos){
    if(listPlanNos!=null && listPlanNos.size()>201){throw new RuntimeException("listPlanNos 超出201长度限制");}
        return planAllocDOMapper.queryStatTotalAllocAmtByPlanNos(endDate, startDate, status, approveStatus, listPlanNos);
    }
    /**
     * desc:根据计划编号删除分配方案.<br/>
     * @param planNo planNo
     * @return int
     */
    
    public int deleteByPlanNo(String planNo){
        return planAllocDOMapper.deleteByPlanNo(planNo);
    }
    /**
     * desc:查询已审批未兑付的分配计划.<br/>
     * @param planNo planNo
     * @return List<PlanAllocDO>
     */
    
    public List<PlanAllocDO> queryApprovedNotPayAlloc(String planNo){
        return planAllocDOMapper.queryApprovedNotPayAlloc(planNo);
    }
}
