package com.abssqr.plat.common.dal.sync.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsRepayInfoDO;
import java.util.Date;
import java.util.List;
import java.math.BigDecimal;
import java.util.Map;
import java.util.Set;
import java.util.LinkedHashMap;
import java.util.ArrayList;
import java.util.HashSet;
import com.abssqr.plat.common.dal.sync.auto.mapper.WkMfsRepayInfoDOMapper;

/**
* The Table wk_mfs_repay_info.
* 还款明细
*/
@Repository
public class WkMfsRepayInfoDAO{

    @Autowired
    private WkMfsRepayInfoDOMapper wkMfsRepayInfoDOMapper;

    /**
     * desc:插入表:wk_mfs_repay_info.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(WkMfsRepayInfoDO entity){
        return wkMfsRepayInfoDOMapper.insert(entity);
    }
    /**
     * desc:updateData.<br/>
     * @return int
     */
    
    public int updateData(){
        return wkMfsRepayInfoDOMapper.updateData();
    }
    /**
     * desc:根据会计时间同步还款明细表.<br/>
     * @param syncDate syncDate
     * @param dbLink dbLink
     * @param loanId loanId
     * @param wkxwDbName wkxwDbName
     * @param wkxwSynonymAft wkxwSynonymAft
     * @param wkxwSynonymPre wkxwSynonymPre
     * @return int
     */
    
    public int insertSyncOne(Date syncDate,String dbLink,String loanId,String wkxwDbName,String wkxwSynonymAft,String wkxwSynonymPre){
        return wkMfsRepayInfoDOMapper.insertSyncOne(syncDate, dbLink, loanId, wkxwDbName, wkxwSynonymAft, wkxwSynonymPre);
    }
    /**
     * desc:同步全量还款明细表.<br/>
     * @param syncDate syncDate
     * @param orgNo orgNo
     * @param dbLink dbLink
     * @param wkxwDbName wkxwDbName
     * @param wkxwSynonymAft wkxwSynonymAft
     * @param wkxwSynonymPre wkxwSynonymPre
     * @param lists lists
     * @return int
     */
    
    public int insertSyncAll(Date syncDate,String orgNo,String dbLink,String wkxwDbName,String wkxwSynonymAft,String wkxwSynonymPre,List<String> lists){
        return wkMfsRepayInfoDOMapper.insertSyncAll(syncDate, orgNo, dbLink, wkxwDbName, wkxwSynonymAft, wkxwSynonymPre, lists);
    }
    /**
     * desc:查询待执行的任务.<br/>
     * @param syncDate syncDate
     * @param orgNo orgNo
     * @param dbLink dbLink
     * @param loanId loanId
     * @param wkxwDbName wkxwDbName
     * @param wkxwSynonymAft wkxwSynonymAft
     * @param wkxwSynonymPre wkxwSynonymPre
     * @return List<WkMfsRepayInfoDO>
     */
    
    public List<WkMfsRepayInfoDO> getByLoanId(Date syncDate,String orgNo,String dbLink,String loanId,String wkxwDbName,String wkxwSynonymAft,String wkxwSynonymPre){
        return wkMfsRepayInfoDOMapper.getByLoanId(syncDate, orgNo, dbLink, loanId, wkxwDbName, wkxwSynonymAft, wkxwSynonymPre);
    }
    /**
     * desc:查询待执行的任务.<br/>
     * @param syncDate syncDate
     * @param orgNo orgNo
     * @param loanId loanId
     * @return List<WkMfsRepayInfoDO>
     */
    
    public List<WkMfsRepayInfoDO> getSelfByLoanId(Date syncDate,String orgNo,String loanId){
        return wkMfsRepayInfoDOMapper.getSelfByLoanId(syncDate, orgNo, loanId);
    }
     /**
     * desc:根据机构编号统计项目还款额合计.<br/>
     * @param orgNo orgNo
     * @param dbLink dbLink
     * @param curDate curDate
     * @param wkxwDbName wkxwDbName
     * @param wkxwSynonymAft wkxwSynonymAft
     * @param wkxwSynonymPre wkxwSynonymPre
     * @return Map<String,BigDecimal>
     */
    
    public List<WkMfsRepayInfoDO> queryStatRepayInfoForOrgList(String orgNo,String dbLink,String curDate,String wkxwDbName,String wkxwSynonymAft,String wkxwSynonymPre){
        return wkMfsRepayInfoDOMapper.queryStatRepayInfoForOrg(orgNo, dbLink, curDate, wkxwDbName, wkxwSynonymAft, wkxwSynonymPre);
    }
    /**
     * desc:根据机构编号统计项目还款额合计.<br/>
     * @param orgNo orgNo
     * @param dbLink dbLink
     * @param curDate curDate
     * @param wkxwDbName wkxwDbName
     * @param wkxwSynonymAft wkxwSynonymAft
     * @param wkxwSynonymPre wkxwSynonymPre
     * @return Map<String,BigDecimal>
     */
    
    public Map<String,BigDecimal> queryStatRepayInfoForOrg(String orgNo,String dbLink,String curDate,String wkxwDbName,String wkxwSynonymAft,String wkxwSynonymPre){
        Map<String,BigDecimal> result = new LinkedHashMap();
        List<WkMfsRepayInfoDO> resultList = wkMfsRepayInfoDOMapper.queryStatRepayInfoForOrg(orgNo, dbLink, curDate, wkxwDbName, wkxwSynonymAft, wkxwSynonymPre);
        if(resultList!=null && !resultList.isEmpty()){
            for(WkMfsRepayInfoDO entity:resultList){
                result.put(entity.getProjno(),entity.getIncomeAmt());
            }
        }
        return result;
    }
}
