/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.enums;

import com.general.system.common.model.IEnum;

import java.util.HashMap;
import java.util.Map;

/**
 * 资产流动类型枚举
 *
 * @author zhenxuan.luo
 * @version $Id: AstFlowTypeEnum.java, v 0.1 2017-06-09 下午16:56 zhenxuan.luo Exp $
 */

public enum AstFlowTypeEnum implements IEnum {
    
    FLOW_IN("flowIn", "流入，标的资产增加", 1),
    
    FLOW_OUT("flowOut", "流出，标的资产减少", -1),
    
    NO_CHANGE("noChrg", "不变", 0),
    ;
    
    private static final Map<String, AstFlowTypeEnum> codeMap = new HashMap<>();
    
    static {
        for (AstFlowTypeEnum item : AstFlowTypeEnum.values()) {
            codeMap.put(item.getCode(), item);
        }
    }
    
    private String code;
    private String desc;
    private int sign;
    
    AstFlowTypeEnum(String code, String desc, int sign) {
        this.code = code;
        this.desc = desc;
        this.sign = sign;
    }
    
    public static AstFlowTypeEnum getByCode(String code) {
        return codeMap.get(code);
    }
    
    @Override
    public String getCode() {
        return code;
    }
    
    @Override
    public String getDesc() {
        return desc;
    }
    
    public int getSign() {
        return sign;
    }
    
    /**
     * 取当前流向反向
     * @return
     */
    public AstFlowTypeEnum reverse() {
        if (this.sign == -1){
            return FLOW_IN;
        }else{
            return FLOW_OUT;
        }
    }
}