/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.facade.enums;

import com.general.system.common.model.IEnum;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * 计划监控指标类型枚举
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.facade.model.enums: PlanIndicatorEnum.java, v 0.1 2019-05-28 15:07 zhenxuan.luo Exp $
 */
public enum PlanIndicatorEnum implements IEnum {
    
    NPL_RATE("NPL_RATE", "不良贷款率", "WK026",true,1),
    VACANCY_RATE("VACANCY_RATE","闲置率", "WK002",true,1),
    ;
    
    private static final Map<String, PlanIndicatorEnum> codeMap = new HashMap<>();
    
    static {
        for (PlanIndicatorEnum item : PlanIndicatorEnum.values()) {
            codeMap.put(item.getCode(), item);
        }
    }
    
    private String code;
    private String desc;
    private String reportFieldCode;
    //true 上限 F下限
    private boolean topLimit;
    private long warnDays;
    
    
    PlanIndicatorEnum(String code, String desc, String reportFieldCode,boolean topLimit,long warnDays) {
        this.code = code;
        this.desc = desc;
        this.reportFieldCode = reportFieldCode;
        this.topLimit=topLimit;
        this.warnDays=warnDays;
    }
    
    public static PlanIndicatorEnum getByCode(String code) {
        return codeMap.get(code);
    }
    
    public static PlanIndicatorEnum getByReportFieldCode(String reportFieldCode) {
        for (PlanIndicatorEnum item : PlanIndicatorEnum.values()) {
            if (item.getReportFieldCode().equals(reportFieldCode)){
                return item;
            }
        }
        return null;
    }
    
    @Override
    public String getCode() {
        return code;
    }
    
    @Override
    public String getDesc() {
        return desc;
    }
    
    public String getReportFieldCode() {
        return reportFieldCode;
    }

    public boolean isTopLimit() {
        return topLimit;
    }

    public long getWarnDays() {
        return warnDays;
    }
    
    
}