/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.enums;

import com.general.system.common.model.IEnum;

import java.util.HashMap;
import java.util.Map;

/**
 * 业务场景码
 *
 * @author zhenxuan.luo
 * @version $Id: AstRiskClassEnum.java, v 0.1 2017-06-07 下午21:07 zhenxuan.luo Exp $
 */
public enum TransCodeEnum implements IEnum {
    
    OTHER("ADM9999", "其他", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.NO_CHANGE),
    
    STAT("ADM9998", "统计", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.NO_CHANGE),
    
    ADMIN("ADM9900", "查询", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.NO_CHANGE),
    
    // 资产交易
    SYNC("AST0001", "存量同步", AstFlowTypeEnum.FLOW_IN, AstFlowTypeEnum.NO_CHANGE),
    
    GRANT_REVERSE("AST0002", "发放冲正", AstFlowTypeEnum.FLOW_OUT, AstFlowTypeEnum.FLOW_IN),
    
    REPO("AST0003", "回收", AstFlowTypeEnum.FLOW_OUT, AstFlowTypeEnum.FLOW_IN),
    
    REPO_REVERSE("AST0004", "回收冲正", AstFlowTypeEnum.FLOW_IN, AstFlowTypeEnum.FLOW_OUT),
    
    INTR_FEE_SETTLEMENT("AST0005", "结息费", AstFlowTypeEnum.FLOW_IN, AstFlowTypeEnum.NO_CHANGE),
    
    BUY_AST("AST0006", "资产交易-买入", AstFlowTypeEnum.FLOW_IN, AstFlowTypeEnum.FLOW_OUT),
    
    SELL_AST("AST0007", "资产交易-卖出", AstFlowTypeEnum.FLOW_OUT, AstFlowTypeEnum.FLOW_IN),
    
    ST_CHG("AST0008", "状态变更", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.NO_CHANGE),
    
    EXMPT("AST0009", "减免", AstFlowTypeEnum.FLOW_OUT, AstFlowTypeEnum.NO_CHANGE),

    ADJ_TERM("AST0010", "期次调整", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.NO_CHANGE),

    CAP_AST_MATCH("AST0011", "资金资产匹配", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.NO_CHANGE),

    CAP_AST_UNMATCH("AST0012", "资金资产解绑", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.NO_CHANGE),

    HANG_FEE("AST0013","挂费",AstFlowTypeEnum.FLOW_IN,AstFlowTypeEnum.NO_CHANGE),

    AST_SIMU("AST9999", "模拟交易", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.NO_CHANGE),

//    CLEAR_PAY("AST0009", "清算支出", AstFlowTypeEnum.FLOW_IN, AstFlowTypeEnum.FLOW_OUT),
//
//    CLEAR_REPO("AST0010", "清算收入", AstFlowTypeEnum.FLOW_OUT, AstFlowTypeEnum.FLOW_IN),

    // 计划交易
    ALLOC("PLN1001", "分配", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.FLOW_OUT),
    
    FROZEN("PLN1002", "冻结", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.NO_CHANGE),
    
    PAY_HANG_INTR("PLN1003", "支付应付利息", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.FLOW_OUT),
    
    CAP_ADJ_ADD("PLN1004", "资金调整增加", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.FLOW_IN),
    
    CAP_ADJ_SUB("PLN1005", "资金调整减少", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.FLOW_OUT),
    
    CAP_INVEST("PLN1006", "投入资本", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.FLOW_IN),
    
    // 科目交易
    TITLE_TRANS("TIT2001", "科目交易", AstFlowTypeEnum.NO_CHANGE, AstFlowTypeEnum.NO_CHANGE),
    
    /**
     * 回购交易
     */
    BUY_BACK_IN("AST1006", "回购-买入", AstFlowTypeEnum.FLOW_IN, AstFlowTypeEnum.FLOW_OUT),
    
    BUY_BACK_OUT("AST1007", "回购-卖出", AstFlowTypeEnum.FLOW_OUT, AstFlowTypeEnum.FLOW_IN),
    ;
    
    private static final Map<String, TransCodeEnum> codeMap = new HashMap<String, TransCodeEnum>();
    
    static {
        for (TransCodeEnum item : TransCodeEnum.values()) {
            codeMap.put(item.getCode(), item);
        }
    }
    
    private String          code;
    private String          desc;
    /**
     * 是否资产增加的交易，对于资金相关的交易，代表资产余额的方向
     */
    private AstFlowTypeEnum astFlow;
    /**
     * 是否涉及资金流 资金余额
     */
    private AstFlowTypeEnum fundFlow;
    
    TransCodeEnum(String code, String desc, AstFlowTypeEnum astFlow, AstFlowTypeEnum fundFlow) {
        this.code = code;
        this.desc = desc;
        this.astFlow = astFlow;
        this.fundFlow = fundFlow;
    }
    
    public static TransCodeEnum getByCode(String code) {
        return codeMap.get(code);
    }
    
    @Override
    public String getCode() {
        return code;
    }
    
    @Override
    public String getDesc() {
        return desc;
    }
    
    public AstFlowTypeEnum getAstFlow() {
        return astFlow;
    }
    
    public AstFlowTypeEnum getFundFlow() {
        return fundFlow;
    }
}