/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.model.plan;

import com.abssqr.plat.common.facade.enums.*;
import com.abssqr.plat.common.facade.model.account.Account;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 计划
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.domain.plan: Plan.java, v 0.1 2019-05-29 11:21 bangis.wangdf Exp $
 */
public class PlanBase  extends BaseEntity {
    /**
     * 计划类型
     */
    private PlanTypeEnum planType;
    /**
     * planNo 计划编号.
     */
    private String       planNo;
    /**
     * planName 计划名称.
     */
    private String       planName;
    /**
     * totalAmount 总投入资本 发行总规模.
     */
    private Money        totalAmt;
    /**
     * beginDate 计划设立日（信托设立日）.
     */
    private Date         beginDate;
    /**
     * endDate 计划到期日（信托终止日）计划截止日.
     */
    private Date         endDate;
    /**
     * 募集方式
     * PUBLIC
     * PRIVATE
     * 公募
     * 私募.
     */
    private FundTypeEnum fundType;
    /**
     * sponsorOrgCode    发起机构-都是不带账号.
     */
    private String       sponsorOrgCode;
    /**
     * managerOrgCode    计划代理机构 管理机构-都是不带账号. 改名字  planMgrOrgCode
     */
    private String       planMgrOrgCode;
    /**
     * sponsorOrgName    发起机构名称
     */
    private String       sponsorOrgName;
    /**
     * managerOrgCode 计划代理机构 管理机构-都是不带账号. 改名字  planMgrOrgCode
     */
    private String       planMgrOrgName;
    /**
     * initDate 初始起算日.
     */
    private Date         initCalcDate;
    /**
     * effectDate 计划起息日.
     */
    private Date         effectDate;
    /**
     * firstPayDay 首次结算日.
     */
    private Date         firstClearDate;
    /**
     * firstPayDay 首次偿付日 首次兑付日.
     */
    private Date         firstPayDate;

    /**
     * 计划状态
     * PREPARE
     * CLEARED
     * SUBSIST
     * 准备中
     * 已清算
     * 存续中.
     */
    private PlanStatusEnum    status;
    /**
     * planApproveStatus 计划审批状态
     * OUTOFDATE
     * PASSED
     * FAILED
     * PENDING
     * 已过时
     * 已通过
     * 已失败
     * 审批中.
     */
    private ApproveStatusEnum approveStatus;
    
    /**
     *兑付频率
     * M
     * Q
     * 半年
     * Y
     * Month
     * Season
     * HalfYear
     * Year
     */
    private PlanPayFreqEnum     payFreq;
    /**
     *计息方式
     * ACT/360
     * ACT/365
     */
    private CalcIntrTypeEnum     calcIntrType;
    /**
     *增值税率
     */
    private BigDecimal vatRate;
    /**
     * 最近更新日期
     */
    private Date lastUpdateDate;
    
    /**
     * gmtCreator 创建人.
     */
    private String gmtCreator;
    
    /**
     * gmtModifier 修改人.
     */
    private String gmtModifier;
    
    /**
     * passed      曾经通过过
     */
    private Boolean passed;


    /**
     * 账户信息
     */
    private List<Account>         accounts;
    /**
     * 投资结构
     */
    private List<InvestStructure> investStructureList;
    /**
     * 费用结构 费用清单
     */
    private List<FeeStructure>    feeStructureList;

    /**
     * 定价规则类型
     */
    private PriceTypeEnum priceType;

    /**
     * 资产定价规则
     */
    private List<PlanAstPriceRule> planAstPriceRuleList;
    /**
     * 金融服务费率
     */
    private BigDecimal             finSerFeeRate;

    public String getPlanNo() {
        return planNo;
    }

    public void setPlanNo(String planNo) {
        this.planNo = planNo;
    }

    public String getPlanName() {
        return planName;
    }

    public void setPlanName(String planName) {
        this.planName = planName;
    }

    public Money getTotalAmt() {
        return totalAmt;
    }

    public void setTotalAmt(Money totalAmt) {
        this.totalAmt = totalAmt;
    }

    public Date getBeginDate() {
        return beginDate;
    }

    public void setBeginDate(Date beginDate) {
        this.beginDate = beginDate;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public FundTypeEnum getFundType() {
        return fundType;
    }

    public void setFundType(FundTypeEnum fundType) {
        this.fundType = fundType;
    }

    public String getSponsorOrgCode() {
        return sponsorOrgCode;
    }

    public void setSponsorOrgCode(String sponsorOrgCode) {
        this.sponsorOrgCode = sponsorOrgCode;
    }

    public String getPlanMgrOrgCode() {
        return planMgrOrgCode;
    }

    public void setPlanMgrOrgCode(String planMgrOrgCode) {
        this.planMgrOrgCode = planMgrOrgCode;
    }

    public Date getInitCalcDate() {
        return initCalcDate;
    }

    public void setInitCalcDate(Date initCalcDate) {
        this.initCalcDate = initCalcDate;
    }

    public Date getEffectDate() {
        return effectDate;
    }

    public void setEffectDate(Date effectDate) {
        this.effectDate = effectDate;
    }

    public Date getFirstClearDate() {
        return firstClearDate;
    }

    public void setFirstClearDate(Date firstClearDate) {
        this.firstClearDate = firstClearDate;
    }

    public Date getFirstPayDate() {
        return firstPayDate;
    }

    public void setFirstPayDate(Date firstPayDate) {
        this.firstPayDate = firstPayDate;
    }

    public PlanStatusEnum getStatus() {
        return status;
    }

    public void setStatus(PlanStatusEnum status) {
        this.status = status;
    }

    public ApproveStatusEnum getApproveStatus() {
        return approveStatus;
    }

    public void setApproveStatus(ApproveStatusEnum approveStatus) {
        this.approveStatus = approveStatus;
    }
    
    public BigDecimal getVatRate() { return vatRate; }
    
    public void setVatRate(BigDecimal vatRate) { this.vatRate = vatRate; }
    
    public PlanPayFreqEnum getPayFreq() { return payFreq; }
    
    public void setPayFreq(PlanPayFreqEnum payFreq) { this.payFreq = payFreq; }
    
    public CalcIntrTypeEnum getCalcIntrType() { return calcIntrType; }
    
    public void setCalcIntrType(CalcIntrTypeEnum calcIntrType) { this.calcIntrType = calcIntrType; }
    
    public Date getLastUpdateDate() {
        return lastUpdateDate;
    }
    
    public void setLastUpdateDate(Date lastUpdateDate) {
        this.lastUpdateDate = lastUpdateDate;
    }
    
    public String getGmtCreator() {
        return gmtCreator;
    }
    
    public void setGmtCreator(String gmtCreator) {
        this.gmtCreator = gmtCreator;
    }
    
    public String getGmtModifier() {
        return gmtModifier;
    }
    
    public void setGmtModifier(String gmtModifier) {
        this.gmtModifier = gmtModifier;
    }
    
    public String getSponsorOrgName() {
        return sponsorOrgName;
    }
    
    public void setSponsorOrgName(String sponsorOrgName) {
        this.sponsorOrgName = sponsorOrgName;
    }
    
    public String getPlanMgrOrgName() {
        return planMgrOrgName;
    }
    
    public void setPlanMgrOrgName(String planMgrOrgName) {
        this.planMgrOrgName = planMgrOrgName;
    }
    
    public Boolean isPassed() {
        return passed;
    }
    
    public void setPassed(Boolean passed) {
        this.passed = passed;
    }

    public PlanTypeEnum getPlanType() {
        return planType;
    }

    public void setPlanType(PlanTypeEnum planType) {
        this.planType = planType;
    }

    public List<Account> getAccounts() {
        return accounts;
    }

    public void setAccounts(List<Account> accounts) {
        this.accounts = accounts;
    }

    public List<InvestStructure> getInvestStructureList() {
        return investStructureList;
    }

    public void setInvestStructureList(List<InvestStructure> investStructureList) {
        this.investStructureList = investStructureList;
    }

    public List<FeeStructure> getFeeStructureList() {
        return feeStructureList;
    }

    public void setFeeStructureList(List<FeeStructure> feeStructureList) {
        this.feeStructureList = feeStructureList;
    }

    public PriceTypeEnum getPriceType() {
        return priceType;
    }

    public void setPriceType(PriceTypeEnum priceType) {
        this.priceType = priceType;
    }

    public List<PlanAstPriceRule> getPlanAstPriceRuleList() {
        return planAstPriceRuleList;
    }

    public void setPlanAstPriceRuleList(List<PlanAstPriceRule> planAstPriceRuleList) {
        this.planAstPriceRuleList = planAstPriceRuleList;
    }

    public BigDecimal getFinSerFeeRate() {
        return finSerFeeRate;
    }

    public void setFinSerFeeRate(BigDecimal finSerFeeRate) {
        this.finSerFeeRate = finSerFeeRate;
    }
}
