/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.facade.model.plan;

import com.abssqr.plat.common.facade.enums.PlanCapitalTransTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;
import com.google.common.collect.Maps;

import java.io.Serializable;
import java.util.Date;
import java.util.Map;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.domain.plan: PlanCapitalOrder.java, v 0.1 2019-05-28 17:02 zhenxuan.luo Exp $
 */
public class PlanCapitalOrder extends BaseEntity implements Serializable {
    
    private static final long serialVersionUID = 8255131776400875642L;
    
    /**
     * planNo 计划编号.
     */
    private String                   planNo;
    /**
     * bizNo 业务号.
     */
    private String                   bizNo;
    /**
     * seqNo 流水号.
     */
    private String                   seqNo;
    /**
     * cfrmAmt 确认交易金额.
     */
    private Money                    cfrmAmt    = new Money(0);
    /**
     * transAmt 发起交易金额.
     */
    private Money                    transAmt   = new Money(0);
    /**
     * payableAmt 应付金额.
     */
    private Money                    payableAmt = new Money(0);
    /**
     * accountDate 交易日期（会计日）.
     */
    private Date                     accountDate;
    /**
     * transTime 交易发起时间.
     */
    private Date                     transTime;
    /**
     * transType 交易类型.
     */
    private PlanCapitalTransTypeEnum transType;
    /**
     * capitalType 支付平台约定的交易类型.
     */
    private String                   capitalType;
    /**
     * planAcct 计划资金账号.
     */
    private Account                  planAcct;
    /**
     * 对手方PlanNO
     */
    private String                   cntPtyCode;
    /**
     * cntPtyCode 对手方简称.
     */
    private Account                  cntPtyAcct;
    /**
     * memo 备注.
     */
    private String                   memo;
    /**
     * context 交易上下文.
     */
    private Map<String, String>      context    = Maps.newHashMap();
    /**
     * status 交易状态.
     */
    private TaskStatusEnum           status;
    /**
     * outAcptNo 外部受理流水号.
     */
    private String                   outAcptNo;
    /**
     * outTransNo 外部交易流水号.
     */
    private String                   outTransNo;
    /**
     * finishTime 交易完成时间.
     */
    private Date                     finishTime;
    /**
     * 处理日期
     */
    private Date                     procTime;
    /**
     * lastExecTime 上次执行时间.
     */
    private Date lastExecTime;
    /**
     * nextExecTime 下次执行时间.
     */
    private Date nextExecTime;
    /**
     * execTimes 执行次数.
     */
    private Long execTimes;
    
    public String getPlanNo() {
        return planNo;
    }
    
    public void setPlanNo(String planNo) {
        this.planNo = planNo;
    }
    
    public String getBizNo() {
        return bizNo;
    }
    
    public void setBizNo(String bizNo) {
        this.bizNo = bizNo;
    }
    
    public String getSeqNo() {
        return seqNo;
    }
    
    public void setSeqNo(String seqNo) {
        this.seqNo = seqNo;
    }
    
    public Money getTransAmt() {
        return transAmt;
    }
    
    public void setTransAmt(Money transAmt) {
        this.transAmt = transAmt;
    }
    
    public Date getAccountDate() {
        return accountDate;
    }
    
    public void setAccountDate(Date accountDate) {
        this.accountDate = accountDate;
    }
    
    public Date getTransTime() {
        return transTime;
    }
    
    public void setTransTime(Date transTime) {
        this.transTime = transTime;
    }
    
    public PlanCapitalTransTypeEnum getTransType() {
        return transType;
    }
    
    public void setTransType(PlanCapitalTransTypeEnum transType) {
        this.transType = transType;
    }
    
    public String getCapitalType() {
        return capitalType;
    }
    
    public void setCapitalType(String capitalType) {
        this.capitalType = capitalType;
    }
    
    public String getMemo() {
        return memo;
    }
    
    public void setMemo(String memo) {
        this.memo = memo;
    }
    
    public Map<String, String> getContext() {
        return context;
    }
    
    public void setContext(Map<String, String> context) {
        this.context = context;
    }
    
    public TaskStatusEnum getStatus() {
        return status;
    }
    
    public void setStatus(TaskStatusEnum status) {
        this.status = status;
    }
    
    public String getOutAcptNo() {
        return outAcptNo;
    }
    
    public void setOutAcptNo(String outAcptNo) {
        this.outAcptNo = outAcptNo;
    }
    
    public String getOutTransNo() {
        return outTransNo;
    }
    
    public void setOutTransNo(String outTransNo) {
        this.outTransNo = outTransNo;
    }
    
    public Date getFinishTime() {
        return finishTime;
    }
    
    public void setFinishTime(Date finishTime) {
        this.finishTime = finishTime;
    }
    
    public Money getCfrmAmt() {
        return cfrmAmt;
    }
    
    public void setCfrmAmt(Money cfrmAmt) {
        this.cfrmAmt = cfrmAmt;
    }
    
    public Account getPlanAcct() {
        return planAcct;
    }
    
    public void setPlanAcct(Account planAcct) {
        this.planAcct = planAcct;
    }
    
    public Account getCntPtyAcct() {
        return cntPtyAcct;
    }
    
    public void setCntPtyAcct(Account cntPtyAcct) {
        this.cntPtyAcct = cntPtyAcct;
    }
    
    public Date getProcTime() {
        return procTime;
    }
    
    public void setProcTime(Date procTime) {
        this.procTime = procTime;
    }
    
    public String getCntPtyCode() {
        return cntPtyCode;
    }
    
    public void setCntPtyCode(String cntPtyCode) {
        this.cntPtyCode = cntPtyCode;
    }
    
    public Date getLastExecTime() {
        return lastExecTime;
    }
    
    public void setLastExecTime(Date lastExecTime) {
        this.lastExecTime = lastExecTime;
    }
    
    public Date getNextExecTime() {
        return nextExecTime;
    }
    
    public void setNextExecTime(Date nextExecTime) {
        this.nextExecTime = nextExecTime;
    }
    
    public Long getExecTimes() {
        return execTimes;
    }
    
    public void setExecTimes(Long execTimes) {
        this.execTimes = execTimes;
    }
    
    public Money getPayableAmt() {
        return payableAmt;
    }
    
    public void setPayableAmt(Money payableAmt) {
        this.payableAmt = payableAmt;
    }
}