/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.facade.result.analyse;

import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;

import java.io.Serializable;
import java.math.BigDecimal;

/**
 * 资产情况概览
 * @author hanfei
 * @version $Id: AnalyseDayAstInfoVO.java, v 0.1 2019-09-24 5:23 PM hanfei Exp $ 
 */
public class AnalyseDayAstInfoVO extends BaseEntity implements Serializable {
    
    /**
     * 计划编号
     */
    private String     planNo;
    /**
     * 总资产规模
     */
    private Money      astScale = new Money();
    /**
     * 总资产规模较昨日
     */
    private Money      astScaleChange = new Money();
    /**
     * 转让资产规模
     */
    private Money      astTransfer = new Money();
    /**
     * 累计转让金额
     */
    private Money      astTransferTotal = new Money();
    /**
     * 本月已还金额
     */
    private Money      repaidTotal = new Money();
    /**
     * 还款计划总额
     */
    private Money      repayPlanTotal = new Money();
    /**
     * 整体逾期率
     */
    private BigDecimal ovdRate = BigDecimal.ZERO;
    /**
     * M3+ 逾期率
     */
    private BigDecimal ovdM3Rate = BigDecimal.ZERO;
    /**
     * 上一日逾期率
     */
    private BigDecimal ovdLastRate = BigDecimal.ZERO;
    /**
     * 近30日平均回收率
     */
    private BigDecimal repayAvgRate = BigDecimal.ZERO;
    /**
     * 回收率 上一日
     */
    private BigDecimal repayLastRate = BigDecimal.ZERO;
    /**
     * 早偿率
     */
    private BigDecimal preRepayRate = BigDecimal.ZERO;
    /**
     * 上一日早偿率
     */
    private BigDecimal preRepayLastRate = BigDecimal.ZERO;
    
    /**
     * 本月剩余未还金额
     * @return
     */
    public Money getRepayBal() {
        Money bal = this.getRepayPlanTotal().subtract(this.getRepaidTotal()) ;
        return bal.isNegtive() ? new Money() : bal;
    }
    
    /**
     * 还款计划完成度
     * @return
     */
    public BigDecimal getRepayFinish() {
        BigDecimal finish = CalcUtil.dividePercRate(
                this.getRepaidTotal().getAmount(), this.getRepayPlanTotal().getAmount());
        return finish.compareTo(AbssqrConstants.PERC_SCALER) > 0 ? AbssqrConstants.PERC_SCALER : finish;
    }
    
    public String getPlanNo() {
        return planNo;
    }
    
    public void setPlanNo(String planNo) {
        this.planNo = planNo;
    }
    
    public Money getAstScale() {
        return astScale;
    }
    
    public void setAstScale(Money astScale) {
        this.astScale = astScale;
    }
    
    public Money getAstScaleChange() {
        return astScaleChange;
    }
    
    public void setAstScaleChange(Money astScaleChange) {
        this.astScaleChange = astScaleChange;
    }
    
    public Money getAstTransfer() {
        return astTransfer;
    }
    
    public void setAstTransfer(Money astTransfer) {
        this.astTransfer = astTransfer;
    }
    
    public Money getAstTransferTotal() {
        return astTransferTotal;
    }
    
    public void setAstTransferTotal(Money astTransferTotal) {
        this.astTransferTotal = astTransferTotal;
    }
    
    public Money getRepaidTotal() {
        return repaidTotal;
    }
    
    public void setRepaidTotal(Money repaidTotal) {
        this.repaidTotal = repaidTotal;
    }
    
    public Money getRepayPlanTotal() {
        return repayPlanTotal;
    }
    
    public void setRepayPlanTotal(Money repayPlanTotal) {
        this.repayPlanTotal = repayPlanTotal;
    }
    
    public BigDecimal getOvdRate() {
        return ovdRate;
    }
    
    public void setOvdRate(BigDecimal ovdRate) {
        this.ovdRate = ovdRate;
    }
    
    public BigDecimal getOvdM3Rate() {
        return ovdM3Rate;
    }
    
    public void setOvdM3Rate(BigDecimal ovdM3Rate) {
        this.ovdM3Rate = ovdM3Rate;
    }
    
    public BigDecimal getOvdRateDiff() {
        return ovdRate.subtract(ovdLastRate);
    }
    
    public BigDecimal getRepayAvgRate() {
        return repayAvgRate;
    }
    
    public void setRepayAvgRate(BigDecimal repayAvgRate) {
        this.repayAvgRate = repayAvgRate;
    }
    
    public BigDecimal getRepayAvgRateChange() {
        return this.repayAvgRate.subtract(this.repayLastRate);
    }
    
    public BigDecimal getRepayLastRate() {
        return repayLastRate;
    }
    
    public void setRepayLastRate(BigDecimal repayLastRate) {
        this.repayLastRate = repayLastRate;
    }
    
    public BigDecimal getOvdLastRate() {
        return ovdLastRate;
    }
    
    public void setOvdLastRate(BigDecimal ovdLastRate) {
        this.ovdLastRate = ovdLastRate;
    }
    
    public BigDecimal getPreRepayRate() {
        return preRepayRate;
    }
    
    public void setPreRepayRate(BigDecimal preRepayRate) {
        this.preRepayRate = preRepayRate;
    }
    
    public BigDecimal getPreRepayLastRate() {
        return preRepayLastRate;
    }
    
    public void setPreRepayLastRate(BigDecimal preRepayLastRate) {
        this.preRepayLastRate = preRepayLastRate;
    }
    
    public BigDecimal getPreRepayRateChange() {
        return this.preRepayRate.subtract(this.preRepayLastRate);
    }
}
