/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.facade.result.ast;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.ProfitTypeEnum;
import com.abssqr.plat.common.facade.result.PlanConst;
import com.alibaba.fastjson.annotation.JSONField;
import com.general.system.common.model.Money;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 收益详情
 * @author hanfei
 * @version $Id: LoanAstProfitDetail.java, v 0.1 2019-12-12 6:33 PM hanfei Exp $
 */
public class LoanAstProfitDetail {
    
    /**
     * 收益编号
     */
    private String         astProfitNo;
    /**
     * loanAstNo .
     */
    private String         loanAstNo;
    /**
     * 收益类型
     */
    private ProfitTypeEnum profitType;
    /**
     * astNo .
     */
    private String         astNo;
    /**
     * outAstNo
     */
    private String         outAstNo;
    /**
     * 当前期次号
     */
    private int            termNo;
    /**
     * 收益来源期次号
     */
    private int            srcTermNo;
    /**
     * ownOrg 收益所属机构
     */
    private String         ownOrg;
//    /**
//     * splitOrgCode 交易分润机构.
//     */
//    private String         splitOrgCode;
    /**
     * splitRate 交易分润比例.
     */
    private BigDecimal splitRate = BigDecimal.ZERO;
    /**
     * 分润比例 显示
     */
    private String splitRateDesc;
    /**
     * 余额
     */
    @JSONField(serialize = false)
    private DebtMoney  bal          = new DebtMoney();
    /**
     * 减免
     */
    @JSONField(serialize = false)
    private DebtMoney  rmtAmt       = new DebtMoney();
    /**
     * 实还
     */
    @JSONField(serialize = false)
    private DebtMoney  fundAmt      = new DebtMoney();
    /**
     * 已还
     */
    @JSONField(serialize = false)
    private DebtMoney  paidAmt      = new DebtMoney();
    /**
     * 入账金额 .
     */
    @JSONField(serialize = false)
    private DebtMoney  entryAmt     = new DebtMoney();
    /**
     * 利息入账实数 .
     */
    @JSONField(serialize = false)
    private BigDecimal     intrEntryDec = BigDecimal.ZERO;
    /**
     * 利罚入账实数 .
     */
    @JSONField(serialize = false)
    private BigDecimal     intrPenalEntryDec = BigDecimal.ZERO;
    /**
     * 本罚入账实数 .
     */
    @JSONField(serialize = false)
    private BigDecimal     prinPenalEntryDec = BigDecimal.ZERO;
    /**
     * 费用入账实数 .
     */
    @JSONField(serialize = false)
    private BigDecimal     feeEntryDec       = BigDecimal.ZERO;
    /**
     * 滞纳金入账实数 .
     */
    @JSONField(serialize = false)
    private BigDecimal     feePenalEntryDec  = BigDecimal.ZERO;
    
    /**
     * 总收益
     */
    public Money getProfitAmt() {
        Money profit = new Money();
        // 所有利息 + 所有费用
        profit.addTo(this.bal.getAllIntAndFeeWithoutPenal())
                .addTo(this.paidAmt.getAllIntAndFeeWithoutPenal());
        // 所有罚息
        profit.addTo(this.bal.getAllPenal()).addTo(paidAmt.getAllPenal());
        return profit;
    }
    
    /**
     * 本金余额
     */
    public Money getPrinBal(){
        Money bal = new Money();
        bal.addTo(this.bal.getAllPrin());
        return bal;
    }
    /**
     * 利息
     */
    public Money getIntrBal(){
        Money bal = new Money();
        bal.addTo(this.bal.getAllIntr());
        return bal;
    }
    /**
     * 罚息
     */
    public Money getPenalBal(){
        Money bal = new Money();
        bal.addTo(this.bal.getAllPenal());
        return bal;
    }
    /**
     * 费用
     */
    public Money getFeeBal(){
        Money bal = new Money();
        bal.addTo(this.bal.getAllIntAndFeeWithoutPenal().subtract(this.bal.getAllIntr()));
        return bal;
    }
    
    /**
     * 本金已还
     */
    public Money getPrinPaid(){
        Money bal = new Money();
        bal.addTo(this.paidAmt.getAllPrin());
        return bal;
    }
    /**
     * 利息已还
     */
    public Money getIntrPaid(){
        Money bal = new Money();
        bal.addTo(this.paidAmt.getAllIntr());
        return bal;
    }
    /**
     * 罚息已还
     */
    public Money getPenalPaid(){
        Money bal = new Money();
        bal.addTo(this.paidAmt.getAllPenal());
        return bal;
    }
    /**
     * 费用已还
     */
    public Money getFeePaid(){
        Money bal = new Money();
        bal.addTo(this.paidAmt.getAllIntAndFeeWithoutPenal().subtract(this.paidAmt.getAllIntr()));
        return bal;
    }
    
    /**
     * 减免总额
     * @return
     */
    public Money getRmtAll(){
        Money bal = new Money();
        bal.addTo(this.rmtAmt.getSum());
        return bal;
    }
    
    
    public String getAstProfitNo() {
        return astProfitNo;
    }
    
    public void setAstProfitNo(String astProfitNo) {
        this.astProfitNo = astProfitNo;
    }
    
    public String getLoanAstNo() {
        return loanAstNo;
    }
    
    public void setLoanAstNo(String loanAstNo) {
        this.loanAstNo = loanAstNo;
    }
    
    public ProfitTypeEnum getProfitType() {
        return profitType;
    }
    
    public void setProfitType(ProfitTypeEnum profitType) {
        this.profitType = profitType;
    }
    
    public String getAstNo() {
        return astNo;
    }
    
    public void setAstNo(String astNo) {
        this.astNo = astNo;
    }
    
    public String getOutAstNo() {
        return outAstNo;
    }
    
    public void setOutAstNo(String outAstNo) {
        this.outAstNo = outAstNo;
    }
    
    public int getTermNo() {
        return termNo;
    }
    
    public void setTermNo(int termNo) {
        this.termNo = termNo;
    }
    
    public int getSrcTermNo() {
        return srcTermNo;
    }
    
    public void setSrcTermNo(int srcTermNo) {
        this.srcTermNo = srcTermNo;
    }
    
    public String getOwnOrg() {
        return ownOrg;
    }
    
    public String getOwnOrgName() {
        return PlanConst.getPlanName(ownOrg);
    }
    
    public void setOwnOrg(String ownOrg) {
        this.ownOrg = ownOrg;
    }
    
    public BigDecimal getSplitRate() {
        return splitRate;
    }
    
    public void setSplitRate(BigDecimal splitRate) {
        this.splitRate = splitRate;
    }
    
    public DebtMoney getBal() {
        return bal;
    }
    
    public void setBal(DebtMoney bal) {
        this.bal = bal;
    }
    
    public DebtMoney getRmtAmt() {
        return rmtAmt;
    }
    
    public void setRmtAmt(DebtMoney rmtAmt) {
        this.rmtAmt = rmtAmt;
    }
    
    public DebtMoney getFundAmt() {
        return fundAmt;
    }
    
    public void setFundAmt(DebtMoney fundAmt) {
        this.fundAmt = fundAmt;
    }
    
    public DebtMoney getPaidAmt() {
        return paidAmt;
    }
    
    public void setPaidAmt(DebtMoney paidAmt) {
        this.paidAmt = paidAmt;
    }
    
    public DebtMoney getEntryAmt() {
        return entryAmt;
    }
    
    public void setEntryAmt(DebtMoney entryAmt) {
        this.entryAmt = entryAmt;
    }
    
    public BigDecimal getIntrEntryDec() {
        return intrEntryDec;
    }
    
    public void setIntrEntryDec(BigDecimal intrEntryDec) {
        this.intrEntryDec = intrEntryDec;
    }
    
    public BigDecimal getIntrPenalEntryDec() {
        return intrPenalEntryDec;
    }
    
    public void setIntrPenalEntryDec(BigDecimal intrPenalEntryDec) {
        this.intrPenalEntryDec = intrPenalEntryDec;
    }
    
    public BigDecimal getPrinPenalEntryDec() {
        return prinPenalEntryDec;
    }
    
    public void setPrinPenalEntryDec(BigDecimal prinPenalEntryDec) {
        this.prinPenalEntryDec = prinPenalEntryDec;
    }
    
    public BigDecimal getFeeEntryDec() {
        return feeEntryDec;
    }
    
    public void setFeeEntryDec(BigDecimal feeEntryDec) {
        this.feeEntryDec = feeEntryDec;
    }
    
    public BigDecimal getFeePenalEntryDec() {
        return feePenalEntryDec;
    }
    
    public void setFeePenalEntryDec(BigDecimal feePenalEntryDec) {
        this.feePenalEntryDec = feePenalEntryDec;
    }
    
    public String getSplitRateDesc() {
        return splitRateDesc;
    }
    
    public void setSplitRateDesc(String splitRateDesc) {
        this.splitRateDesc = splitRateDesc;
    }
}
