/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.integration.notice.impl;

import com.abssqr.plat.common.integration.notice.NoticeIntegration;
import com.abssqr.plat.common.integration.notice.RobotClient;
import com.abssqr.plat.common.integration.notice.RobotConfig;
import com.abssqr.plat.common.integration.notice.model.RobotRequest;
import com.abssqr.plat.common.integration.notice.model.RobotResponse;
import com.abssqr.plat.common.integration.notice.model.impl.DefaultRobotClient;
import com.abssqr.plat.common.util.loggers.CommonLoggers;
import com.general.system.common.util.LogUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.List;

/**
 * 钉钉消息通知
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.integration.notice.impl: DingTalkNoticeIntegration.java, v 0.1 2019-08-10 14:12
 * bangis.wangdf Exp $
 */
@Component

public class RobotNoticeIntegration implements NoticeIntegration {
    private final static Logger LOGGER = CommonLoggers.DING_TALK_INTEGRATION;

    @Autowired
    private RobotConfig robotConfig;

    private RobotClient client;

    private String hostName;

    @PostConstruct
    public void init() {
        client = new DefaultRobotClient();

        try {
            InetAddress localHost = InetAddress.getLocalHost();
            hostName = localHost.getHostName();
        } catch (UnknownHostException e) {
            LogUtil.error(LOGGER, e, "UnknownHostException");
        }
    }

    @Override
    public boolean sendMsg(String title, String msg) {
        return this.sendMsg(title, msg, null);
    }

    /**
     * 发送消息
     *
     * @param title
     * @param msg
     * @param type 通知群组
     * @return
     */
    @Override
    public boolean sendMsg(String title, String msg, String type) {
        title = StringUtils.isBlank(title) ? "告警" : title;
        List<RobotRequest> robotRequestList = robotConfig.getRobotRequestListByType(type);
        if (CollectionUtils.isNotEmpty(robotRequestList)) {
            Boolean succ = true;
            for (RobotRequest req : robotRequestList) {
                req.setTitle(title);
                req.setContent("hostName[" + hostName + "]\n > " + msg);
                try {
                    RobotResponse response = client.execute(req);
                    succ = succ && response.isSuccess();
                    LogUtil.info(LOGGER, "[sendMsg]status={0},title={1},msg={2}", response.getErrcode(), title, msg);
                } catch (Exception e) {
                    succ = false;
                    LogUtil.error(LOGGER, e, "[sendMsg-error]title={0},msg={1}", title, msg);
                }
            }
            return succ;
        }
        return false;
    }


}
