/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */

package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstPackDO;
import com.abssqr.plat.common.facade.enums.AstPackStatusEnum;
import com.abssqr.plat.common.facade.enums.CalcIntrTypeEnum;
import com.abssqr.plat.common.facade.model.prod.LoanProduct;
import com.abssqr.plat.common.model.domain.ast.LoanAstBuyBackPack;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.Money;
import com.google.common.collect.Lists;

import java.util.List;
import java.util.Optional;

/**
 * 资产回购筛选包
 *
 * @author xiachenxiang
 * @version com.abssqr.plat.common.model.convertor: LoanAstBuyBackPackConvert.java, v 0.1 2020-02-11 3:09 PM xiachenxiang Exp $
 */
public class LoanAstBuyBackPackConvert {


    public static final AstPackDO domain2DO(LoanAstBuyBackPack param) {
        if (param == null) {
            return null;
        }
        AstPackDO result = new AstPackDO();
        result.setPlanNo(param.getPlanNo());
        //packCode 资产包编码
        result.setPackCode(param.getPackCode());
        //packName 资产包名称
        result.setPackName(param.getPackName());
        //avalAmt 预期规模
        result.setAvalAmt(param.getAvalAmt().getCent());
        //dymRule 动态规则列表
        result.setDymRule(param.getDymRule());
        //packStatus 资产包状态
        result.setPackStatus(param.getPackStatus().getCode());
        //scenRuleNos 筛选规则列表
        result.setTransRule(param.getTransferRule().getTransferNo());
        //scenDate 刷选日期
        result.setScenDate(param.getScenDate());
        //maxEndDate 最大到期日
        result.setMaxEndDate(param.getMaxEndDate());
        //minEndDate 最小到期日
        result.setMinEndDate(param.getMinEndDate());
        result.setCalcIntrType(param.getCalcIntrType().getCode());
        result.setManagerOrgCode(param.getManagerOrgCode());
        result.setPackType(param.getPackType().getCode());

        result.setAbssqrProdCode(Optional.ofNullable(param.getLoanProduct()).map(LoanProduct::getAbssqrProdCode).orElse(null));
        result.setApproveStatus(Optional.ofNullable(param.getApproveStatus()).map(ApproveStatusEnum::getCode).orElse(null));
        result.setApproveDate(param.getApproveDate());
        result.setBackOutCode(param.getBackOutCode());

        return result;
    }
    
    
    public static final LoanAstBuyBackPack do2Domain(AstPackDO param) {
        if (param == null) {
            return null;
        }
        LoanAstBuyBackPack result = new LoanAstBuyBackPack();
        result.setPlanNo(param.getPlanNo());
        //avalAmt 预期规模
        result.setAvalAmt(Money.createWithCentNull(param.getAvalAmt()));
        //dymRule 动态规则列表
        result.setDymRule(param.getDymRule());
        //packCode 资产包编码
        result.setPackCode(param.getPackCode());
        //packName 资产包名称
        result.setPackName(param.getPackName());
        //packStatus 资产包状态
        result.setPackStatus(AstPackStatusEnum.getByCode(param.getPackStatus()));
        //scenDate 刷选日期
        result.setScenDate(param.getScenDate());
        //maxEndDate 最大到期日
        result.setMaxEndDate(param.getMaxEndDate());
        //minEndDate 最小到期日
        result.setMinEndDate(param.getMinEndDate());
        result.setCalcIntrType(CalcIntrTypeEnum.getByCode(param.getCalcIntrType()));
        result.setManagerOrgCode(param.getManagerOrgCode());

        result.setMemo(param.getMemo());
        result.setApproveDate(param.getApproveDate());
        result.setApproveStatus(ApproveStatusEnum.getEnum(param.getApproveStatus()));
        result.setBackOutCode(param.getBackOutCode());
        result.getLoanProduct().setAbssqrProdCode(param.getAbssqrProdCode());

        return result;
    }


    public static List<LoanAstBuyBackPack> convert2DoList(List<AstPackDO> param) {
        List<LoanAstBuyBackPack> res = Lists.newArrayList();
        for (AstPackDO astPackDO : param) {
            res.add(LoanAstBuyBackPackConvert.do2Domain(astPackDO));
        }
        return res;
    }
}
