/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.domain.ast;

import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.model.utils.DecimalCalcUtil;
import com.abssqr.plat.common.util.utils.BigDecimalUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.MoneyUtil;
import com.google.common.collect.Maps;

import java.math.BigDecimal;
import java.util.Map;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.profit.entity: LoanAstMeasure.java, v 0.1 2019-11-18 上午01:11 zhenxuan.luo Exp $ 
 */
public class LoanAstProfitMeasure extends ProfitMeasure {
    private LoanAstProfit loanAstProfit;
    
    /**
     * 通过机构资产收益构造
     *
     * @param loanAstProfit
     */
    public static LoanAstProfitMeasure buildByLoanAstProfit(LoanAstProfit loanAstProfit) {
        LoanAstProfitMeasure astProfitMeasure = new LoanAstProfitMeasure();
        astProfitMeasure.loanAstProfit = loanAstProfit;
        
        astProfitMeasure.intrEntry = new AstProfitUnit(DebtMoneyFieldEnum.INTR, new Money(),
                loanAstProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.INTR), loanAstProfit.getIntrEntryDec());
        
        // 本罚全归持有方，不分润
        
        astProfitMeasure.intrPenalEntry = new AstProfitUnit(DebtMoneyFieldEnum.INTR_ODUE_PNY,
                loanAstProfit.getBal().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY),
                loanAstProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY),
                loanAstProfit.getIntrPenalEntryDec());
        
        astProfitMeasure.feeEntry = new AstProfitUnit(DebtMoneyFieldEnum.FEE, new Money(),
                loanAstProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.FEE), loanAstProfit.getFeeEntryDec());
        
        astProfitMeasure.feePenalEntry = new AstProfitUnit(DebtMoneyFieldEnum.FEE_PNY,
                loanAstProfit.getBal().getByFieldType(DebtMoneyFieldEnum.FEE_PNY),
                loanAstProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY),
                loanAstProfit.getFeePenalEntryDec());
        
        return astProfitMeasure;
    }
    
    public void calcSpit(BigDecimal splitRate, LoanAstMeasure astMeasure, LoanAstProfitMeasure ownProfitMeasure) {
        entrySettleField(astMeasure, ownProfitMeasure, DebtMoneyFieldEnum.INTR_ODUE_PNY);
        entrySettleField(astMeasure, ownProfitMeasure, DebtMoneyFieldEnum.FEE_PNY);
        
        entrySplitField(astMeasure, DebtMoneyFieldEnum.INTR, splitRate);
        entrySplitField(astMeasure, DebtMoneyFieldEnum.FEE, splitRate);
    }
    
    private void entrySplitField(LoanAstMeasure astMeasure, DebtMoneyFieldEnum fieldType, BigDecimal splitRate) {
        AstProfitUnit astUnit = astMeasure.getUnit(fieldType);
        AstProfitUnit profitUnit = this.getUnit(fieldType);
        if (BigDecimalUtil.isZero(astUnit.getTransEntryDec())) {
            return;
        }
        
        BigDecimal totalEntryDec = astUnit.getEntryDec().multiply(splitRate);
        BigDecimal transEntryDec = totalEntryDec.subtract(profitUnit.getEntryDec());
        if (BigDecimalUtil.isZero(transEntryDec)) {
            return;
        }
        
        Money transEntryAmt = new Money(totalEntryDec, AbssqrConstants.DECIMAL_ROUNDING_MODE)
                                 .subtract(profitUnit.getEntryAmt());
        transEntryAmt = MoneyUtil.max(transEntryAmt, new Money());
        transEntryAmt = MoneyUtil.min(transEntryAmt, astUnit.getTransEntryAmt());
        
        profitUnit.trans(transEntryAmt, transEntryDec);
    }
    
    private void entrySettleField(LoanAstMeasure astMeasure, LoanAstProfitMeasure ownProfitMeasure,
                                  DebtMoneyFieldEnum fieldType) {
        
        Map<String, BigDecimal> splitBase = buildSplitBase(ownProfitMeasure, fieldType);
        Map<String, BigDecimal> ratios = Maps.newHashMap();
        DecimalCalcUtil.calcSplitFactor(ratios, splitBase, true);
        
        BigDecimal ratio = ratios.get(this.getLoanAstProfit().getAstProfitNo());
        entrySplitField(astMeasure, fieldType, ratio);
    }
    
    private Map<String, BigDecimal> buildSplitBase(LoanAstProfitMeasure ownProfitMeasure,
                                                   DebtMoneyFieldEnum fieldType) {
        Map<String, BigDecimal> ret = Maps.newHashMap();
        
        ret.put(this.loanAstProfit.getAstProfitNo(), this.getUnit(fieldType).getBase().getAmount());
        ret.put(ownProfitMeasure.loanAstProfit.getLoanAstNo(),
                ownProfitMeasure.getUnit(fieldType).getBase().getAmount());
        return ret;
    }
    
    public LoanAstProfit getLoanAstProfit() {
        return loanAstProfit;
    }
}