/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.model.domain.loanTrans;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.base.OrgDistPftMoney;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 资产分期变更流水
 *
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.domain.loanTrans: LoanInstalLog.java, v 0.1 2017-12-12 下午2:49 guozhenhua Exp $
 */
public class LoanInstalLog extends BaseEntity {

    /**
     * execTms 执行时间戳.
     */
    private String  execTms;
    /**
     * id 自增ID.
     */
    private Long    id;
    /**
     * astNo 资产编号.
     */
    private String  astNo;
    /**
     * 用户ID
     */
    private String  userId;
    /**
     * 期次编号
     */
    private Integer termNo;
    /**
     * outAstNo 外部资产编号.
     */
    private String  outAstNo;
    /**
     * outBizNo 外部业务流水号.
     */
    private String  outBizNo;
    /**
     * bizNo 内部业务流水号.
     */
    private String  bizNo;
    /**
     * seqNo 日志流水号.
     */
    private String  seqNo;
    /**
     * lenderOrgCode 原始权益机构（计划）.
     */
    private String  lenderOrgCode;
    /**
     * managerOrgCode 资产管理服务机构.
     */
    private String  managerOrgCode;
    /**
     * sponserOrgCode 业务牵头机构.
     */
    private String  sponserOrgCode;
    /**
     * ownOrg 所属机构（计划）.
     */
    private String  ownOrg;
    /**
     * ownPack 所属资产包.
     */
    private String  ownPack;
    /**
     * prodNo 产品码.
     */
    private String  prodNo;
    /**
     * outProdNo 外部产品码.
     */
    private String  outProdNo;
    /**
     * contractNo 合同编号.
     */
    private String  contractNo;

    /**
     * bfStatus 原状态.
     */
    private LoanStatusEnum bfStatus;
    /**
     * afStatus 交易后状态.
     */
    private LoanStatusEnum afStatus;
    /**
     * eventCode 交易码.
     */
    private EventCodeEnum  eventCode;
    /**
     * bfOvdDays 原逾期天数.
     */
    private Integer        bfOvdDays;
    /**
     * afOvdDays 交易后逾期天数.
     */
    private Integer        afOvdDays;
    /**
     * transTime 交易时间.
     */
    private Date           transTime;
    /**
     * accountDate 会计日期.
     */
    private Date           accountDate;
    /**
     * clrgAmt 清算金额.
     */
    private Money          clrgAmt         = new Money();
    /**
     * hangIntrAmt 清算金额.
     */
    private Money               hangIntrAmt     = new Money();
    /**
     * hangIntrAmt 实付清算金额.
     */
    private Money               paidHangIntrFundAmt = new Money();
    /**
     * 发生额
     */
    private DebtMoney           amt             = new DebtMoney();
    /**
     * 资金发生额
     */
    private DebtMoney           paidFundAmt     = new DebtMoney();
    /**
     * 交易后余额
     */
    private DebtMoney           afBal           = new DebtMoney();
    /**
     * 机构分润
     */
    private OrgDistPftMoney     orgDistPftAmt   = new OrgDistPftMoney();
    /**
     * extInf 扩展上下文.
     */
    private Map<String, String> extInf          = new HashMap<String, String>();

    public String getExecTms() {
        return execTms;
    }

    public void setExecTms(String execTms) {
        this.execTms = execTms;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getUserId() {
        return userId;
    }

    public void setUserId(String userId) {
        this.userId = userId;
    }

    public String getAstNo() {
        return astNo;
    }

    public void setAstNo(String astNo) {
        this.astNo = astNo;
    }

    public Integer getTermNo() {
        return termNo;
    }

    public void setTermNo(Integer termNo) {
        this.termNo = termNo;
    }

    public String getOutAstNo() {
        return outAstNo;
    }

    public void setOutAstNo(String outAstNo) {
        this.outAstNo = outAstNo;
    }

    public String getOutBizNo() {
        return outBizNo;
    }

    public void setOutBizNo(String outBizNo) {
        this.outBizNo = outBizNo;
    }

    public String getBizNo() {
        return bizNo;
    }

    public void setBizNo(String bizNo) {
        this.bizNo = bizNo;
    }

    public String getSeqNo() {
        return seqNo;
    }

    public void setSeqNo(String seqNo) {
        this.seqNo = seqNo;
    }

    public String getLenderOrgCode() {
        return lenderOrgCode;
    }

    public void setLenderOrgCode(String lenderOrgCode) {
        this.lenderOrgCode = lenderOrgCode;
    }

    public String getManagerOrgCode() {
        return managerOrgCode;
    }

    public void setManagerOrgCode(String managerOrgCode) {
        this.managerOrgCode = managerOrgCode;
    }

    public String getSponserOrgCode() {
        return sponserOrgCode;
    }

    public void setSponserOrgCode(String sponserOrgCode) {
        this.sponserOrgCode = sponserOrgCode;
    }

    public String getOwnOrg() {
        return ownOrg;
    }

    public void setOwnOrg(String ownOrg) {
        this.ownOrg = ownOrg;
    }

    public String getOwnPack() {
        return ownPack;
    }

    public void setOwnPack(String ownPack) {
        this.ownPack = ownPack;
    }

    public String getProdNo() {
        return prodNo;
    }

    public void setProdNo(String prodNo) {
        this.prodNo = prodNo;
    }

    public String getOutProdNo() {
        return outProdNo;
    }

    public void setOutProdNo(String outProdNo) {
        this.outProdNo = outProdNo;
    }

    public String getContractNo() {
        return contractNo;
    }

    public void setContractNo(String contractNo) {
        this.contractNo = contractNo;
    }

    public LoanStatusEnum getBfStatus() {
        return bfStatus;
    }

    public void setBfStatus(LoanStatusEnum bfStatus) {
        this.bfStatus = bfStatus;
    }

    public LoanStatusEnum getAfStatus() {
        return afStatus;
    }

    public void setAfStatus(LoanStatusEnum afStatus) {
        this.afStatus = afStatus;
    }

    public EventCodeEnum getEventCode() {
        return eventCode;
    }

    public void setEventCode(EventCodeEnum eventCode) {
        this.eventCode = eventCode;
    }

    public Integer getBfOvdDays() {
        return bfOvdDays;
    }

    public void setBfOvdDays(Integer bfOvdDays) {
        this.bfOvdDays = bfOvdDays;
    }

    public Integer getAfOvdDays() {
        return afOvdDays;
    }

    public void setAfOvdDays(Integer afOvdDays) {
        this.afOvdDays = afOvdDays;
    }

    public Date getTransTime() {
        return transTime;
    }

    public void setTransTime(Date transTime) {
        this.transTime = transTime;
    }

    public Date getAccountDate() {
        return accountDate;
    }

    public void setAccountDate(Date accountDate) {
        this.accountDate = accountDate;
    }

    public Money getClrgAmt() {
        return clrgAmt;
    }

    public void setClrgAmt(Money clrgAmt) {
        this.clrgAmt = clrgAmt;
    }


    public DebtMoney getAmt() {
        return amt;
    }

    public void setAmt(DebtMoney amt) {
        this.amt = amt;
    }

    public DebtMoney getPaidFundAmt() {
        return paidFundAmt;
    }

    public void setPaidFundAmt(DebtMoney paidFundAmt) {
        this.paidFundAmt = paidFundAmt;
    }

    public DebtMoney getAfBal() {
        return afBal;
    }

    public void setAfBal(DebtMoney afBal) {
        this.afBal = afBal;
    }

    public Map<String, String> getExtInf() {
        return extInf;
    }

    public void setExtInf(Map<String, String> extInf) {
        this.extInf = extInf;
    }

    public Money getHangIntrAmt() {
        return hangIntrAmt;
    }

    public void setHangIntrAmt(Money hangIntrAmt) {
        this.hangIntrAmt = hangIntrAmt;
    }

    public OrgDistPftMoney getOrgDistPftAmt() {
        return orgDistPftAmt == null ? new OrgDistPftMoney() : orgDistPftAmt;
    }

    public void setOrgDistPftAmt(OrgDistPftMoney orgDistPftAmt) {
        this.orgDistPftAmt = orgDistPftAmt;
    }

    public Money getPaidHangIntrFundAmt() {
        return paidHangIntrFundAmt;
    }

    public void setPaidHangIntrFundAmt(Money paidHangIntrFundAmt) {
        this.paidHangIntrFundAmt = paidHangIntrFundAmt;
    }



    public String forLog() {
        return "LoanInstalLog{" +
            ", astNo='" + astNo + '\'' +
            ", termNo=" + termNo +
            ", outAstNo='" + outAstNo + '\'' +
            ", bizNo='" + bizNo + '\'' +
            ", seqNo='" + seqNo + '\'' +
            ", bfStatus=" + bfStatus +
            ", afStatus=" + afStatus +
            ", eventCode=" + eventCode +
            ", bfOvdDays=" + bfOvdDays +
            ", afOvdDays=" + afOvdDays +
            ", transTime=" + transTime +
            ", accountDate=" + accountDate +
            ", clrgAmt=" + clrgAmt +
            ", amt=" + amt +
            ", orgDistPftAmt=" + orgDistPftAmt +
            ", extInf=" + extInf +
            '}';
    }
}
