/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2020 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.repo.acct.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.AcctDateDAO;
import com.abssqr.plat.common.model.convertor.AcctDateConvert;
import com.abssqr.plat.common.model.domain.accounting.AcctDate;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.acct.AcctDateRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 *
 * @author hanfei
 * @version $Id: AcctDateRepoImpl.java, v 0.1 2020-03-06 10:58 AM hanfei Exp $ 
 */
@Repository
public class AcctDateRepoImpl implements AcctDateRepo {
    
    
    @Autowired
    private AcctDateDAO acctDateDAO;
    
    /**
     * desc:插入表:abs_acct_date.<br/>
     * @param acctDate entity
     * @return int
     */
    @Override
    public int addAcctDate(AcctDate acctDate){
        return acctDateDAO.insert(AcctDateConvert.convert2DO(acctDate));
    }
    /**
     * desc:更新表:abs_acct_date.<br/>
     * @param acctDate
     * @return int
     */
    @Override
    public int modifyAcctDate(AcctDate acctDate){
        return acctDateDAO.update(acctDate.getAcctDate(), acctDate.getOrgCode().getCode());
    }
    /**
     * desc:根据主键获取数据:abs_acct_date.<br/>
     * @param orgCode
     * @return AcctDateDO
     */
    @Override
    public AcctDate getAcctDate(OrgCodeEnum orgCode){
        return AcctDateConvert.convert2Domain(acctDateDAO.getDate(orgCode.getCode()));
    }
    /**
     * desc:查询所有机构的会计日.<br/>
     * @return List<AcctDateDO>
     */
    @Override
    public List<AcctDate> queryAcctDate(){
        return AcctDateConvert.convert2DomainList(acctDateDAO.queryAcctDate());
    }
    /**
     * desc:根据机构编码锁会计日.<br/>
     * @param orgCode orgCode
     * @return AcctDateDO
     */
    @Override
    public AcctDate getForLock(OrgCodeEnum orgCode){
        return AcctDateConvert.convert2Domain(acctDateDAO.getForLock(orgCode.getCode()));
    }
}

