/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.repo.ast.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.LoanBakDAO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstScale;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanAstReportStat;
import com.abssqr.plat.common.model.convertor.LoanBakConvert;
import com.abssqr.plat.common.model.repo.ast.LoanBakRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 *
 * @author hanfei
 * @version $Id: LoanBakRepoImpl.java, v 0.1 2019-12-05 11:15 AM hanfei Exp $ 
 */
@Component
public class LoanBakRepoImpl implements LoanBakRepo {
    
    @Autowired
    private LoanBakDAO loanBakDAO;
    
    /**
     * desc:资产余额状态统计代替statBal.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @return List<LoanBakReportStat>
     */
    @Override
    public List<PlanAstReportStat> statLoanBakStatus(Date accountDate,String planNo){
        return LoanBakConvert.convert2Domains(loanBakDAO.statLoanBakStatus(accountDate, planNo));
    }
    /**
     * desc:五级分类统计代替statBal.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @return List<LoanBakReportStat>
     */
    @Override
    public List<PlanAstReportStat> statLoanBakClz(Date accountDate,String planNo){
        return LoanBakConvert.convert2Domains(loanBakDAO.statLoanBakClz(accountDate, planNo));
    }
    /**
     * desc:91逾期未核销统计代替statBal.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param statVal statVal
     * @param statType statType
     * @param wtfStatus wtfStatus
     * @param ovdDays ovdDays
     * @return List<LoanBakReportStat>
     */
    @Override
    public List<PlanAstReportStat> statLoanBakOvdWtf(Date accountDate,String planNo,String statVal,String statType,String wtfStatus,Integer ovdDays){
        return LoanBakConvert.convert2Domains(loanBakDAO.statLoanBakOvdWtf(accountDate, planNo, statVal, statType, wtfStatus, ovdDays));
    }
    /**
     * desc:ovd_day天数统计代替statBal.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @return List<LoanBakReportStat>
     */
    @Override
    public List<PlanAstReportStat> statLoanBakOvdDays(Date accountDate,String planNo){
        return LoanBakConvert.convert2Domains(loanBakDAO.statLoanBakOvdDays(accountDate, planNo));
    }
    /**
     * desc:统计当日新增资产年化收益.<br/>
     * @param disbDate disbDate
     * @param accountDate accountDate
     * @param planNo planNo
     * @return List<LoanBakReportStat>
     */
    @Override
    public List<PlanAstReportStat> statSpvProfit(Date disbDate,Date accountDate,String planNo){
        return LoanBakConvert.convert2Domains(loanBakDAO.statLoanBakProfit(null, disbDate, accountDate, planNo));
    }
    
    /**
     * desc:统计当日新增资产年化收益.<br/>
     * @param inDate inDate
     * @param accountDate accountDate
     * @param planNo planNo
     * @return List<LoanBakReportStat>
     */
    @Override
    public List<PlanAstReportStat> statPlanProfit(Date inDate, Date accountDate,String planNo){
        return LoanBakConvert.convert2Domains(loanBakDAO.statLoanBakProfit(inDate, null, accountDate, planNo));
    }

    @Override
    public List<AstScale> getAstScale(Date accountDate, String planNo){
        return loanBakDAO.getAstScale(accountDate,planNo);
    }
}
