/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.ast.impl;

import cn.hutool.core.util.StrUtil;
import com.abssqr.plat.common.dal.mysql.auto.dao.LoanBakDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.LoanContractDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.LoanDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.LoanBakDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.CapitalAstMatchPage;
import com.abssqr.plat.common.dal.mysql.auto.paging.CapitalAstUnMatchPage;
import com.abssqr.plat.common.dal.mysql.auto.paging.LoanBakListPage;
import com.abssqr.plat.common.dal.mysql.auto.paging.LoanListPage;
import com.abssqr.plat.common.facade.param.loan.LoanPageQryParam;
import com.abssqr.plat.common.model.convertor.LoanConvert;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanBak;
import com.abssqr.plat.common.model.repo.ast.LoanRepository;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * 贷款资产仓储
 *
 * @author guozhenhua
 * @version com.abssqr.plat.processor.core.service.loanTrans.impl: AstBillRepositoryImpl.java, v 0.1 2017-12-13 下午1:51 guozhenhua Exp $
 */
@Component
public class LoanRepositoryImpl implements LoanRepository {

    @Autowired
    private LoanDAO         loanDAO;
    @Autowired
    private LoanContractDAO loanContractDAO;

    @Autowired
    private LoanBakDAO loanBakDAO;

    @Override
    public Loan lockByAstNo(String astNo) {
        if (StringUtils.isBlank(astNo)) {
            return null;
        }
        return LoanConvert.convert2Domain(loanDAO.lockByAstNo(astNo));
    }

    @Override
    public Loan getByAstNo(String astNo) {
        if (StringUtils.isBlank(astNo)) {
            return null;
        }
        return LoanConvert.convert2Domain(loanDAO.getByAstNo(astNo));
    }

    @Override
    public Loan lockByOutAstNo(String outAstNo, String managerOrgCode) {
        if (StringUtils.isBlank(outAstNo) || StringUtils.isBlank(managerOrgCode)) {
            return null;
        }
        return LoanConvert.convert2Domain(loanDAO.lockByOutAstNo(outAstNo, managerOrgCode));
    }

    @Override
    public Loan getByOutAstNo(String outAstNo, String managerOrgCode) {
        if (StringUtils.isBlank(outAstNo) || StringUtils.isBlank(managerOrgCode)) {
            return null;
        }
        return LoanConvert.convert2Domain(loanDAO.getByOutAstNo(outAstNo, managerOrgCode));
    }

    @Override
    public int createOrUpdate(Loan loan) {
        if (loan == null) {
            return 0;
        }

        if (loan.isNew()) {
            return loanDAO.insert(LoanConvert.convert2DO(loan));
        } else {
            return loanDAO.updateByAstNo(LoanConvert.convert2DO(loan));
        }
    }

    @Override
    public int backup(List<LoanBak> bakList) {
        if (CollectionUtils.isEmpty(bakList)) {
            return 0;
        }
        List<LoanBakDO> bakDOS = Lists.newArrayList();
        for (LoanBak loanBak : bakList) {
            bakDOS.add(LoanConvert.convert2BakDO(loanBak));
        }
        return loanBakDAO.insertBatch(bakDOS);
    }

    @Override
    public LoanBakListPage getPageByOwnOrg(LoanPageQryParam param, Date accountDate) {
        LoanBakListPage page = new LoanBakListPage();
        page.setAccountDate(accountDate);
        page.setOvdDays(param.getOvdDays());
        page.setOwnOrg(param.getOwnOrg());
        page.setOutAstNo(StringUtils.trim(param.getOutAstNo()));
        page.setLimit(param.getPageSize());
        page.setCurrPageNo(param.getPageNum());
        return loanBakDAO.getPageByOwnOrg(page);
    }

    @Override
    @PerformTrace
    public LoanBakListPage getPageByOwnOrg(LoanBakListPage page) {
        return loanBakDAO.getPageByOwnOrg(page);
    }

    /**
     * 根据原始放款机构查询资产列表
     *
     * @param param
     * @return
     */
    @Override
    public LoanListPage queryLoanPage(LoanPageQryParam param) {
        LoanListPage page = new LoanListPage();
        page.setOvdDays(param.getOvdDays());
        page.setLenderOrgCode(StrUtil.trim(param.getOwnOrg()));
        page.setOutAstNo(StrUtil.trim(param.getOutAstNo()));
        page.setAstNo(StrUtil.trim(param.getAstNo()));
        page.setRiskCl(StrUtil.trim(param.getRiskCl()));
        page.setLimit(param.getPageSize());
        page.setCurrPageNo(param.getPageNum());
        page.setCapitalNo(param.getCapitalNo());
        return loanDAO.getPageByOwnOrg(page);
    }

    @Override
    public CapitalAstMatchPage queryForCapitalAstMatch(CapitalAstMatchPage capitalAstMatch) {
        return loanDAO.queryForCapitalAstMatch(capitalAstMatch);
    }

    @Override
    public CapitalAstUnMatchPage queryForCapitalAstUnMatch(CapitalAstUnMatchPage capitalAstUnBind) {
        return loanDAO.queryForCapitalAstUnMatch(capitalAstUnBind);
    }

}
