/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.ast.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.PlanAstStatTaskDAO;
import com.abssqr.plat.common.model.convertor.PlanAstStatTaskConvert;
import com.abssqr.plat.common.model.domain.plan.PlanAstStatTask;
import com.abssqr.plat.common.model.repo.ast.PlanAstStatTaskRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;

/**
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.repo.ast.impl: PlanAstStatTaskRepoImpl.java, v 0.1 2019-06-12 10:34 bangis.wangdf Exp $ 
 */
@Repository
public class PlanAstStatTaskRepoImpl implements PlanAstStatTaskRepo {
    
    @Autowired
    private PlanAstStatTaskDAO planAstStatTaskDAO;
    
    /**
     * 根据计划编号获取任务列表
     *
     * @param planNo
     * @return
     */
    @Override
    public PlanAstStatTask getPlanNo(String planNo) {
        return PlanAstStatTaskConvert.convert2Domain(planAstStatTaskDAO.getByPlanNo(planNo));
    }
    
    /**
     * 更新任务
     *
     * @param task
     * @return
     */
    @Override
    public int updateTask(PlanAstStatTask task) {
        return planAstStatTaskDAO.updateTask(PlanAstStatTaskConvert.convert2DO(task));
    }
    
    /**
     * @param ip
     * @return
     */
    @Override
    public int updateHangTasksByIp(String ip) {
        return planAstStatTaskDAO.updateHangTasksByIp(ip);
    }
    
    /**
     * 创建任务
     *
     * @param task
     * @return
     */
    @Override
    public int createTask(PlanAstStatTask task) {
        if (task == null) {
            return 0;
        }
        return planAstStatTaskDAO.insert(PlanAstStatTaskConvert.convert2DO(task));
    }
    
    /**
     * @param planNo
     * @return
     */
    @Override
    public PlanAstStatTask lock(String planNo) {
        return PlanAstStatTaskConvert.convert2Domain(planAstStatTaskDAO.lockByPlanNo(planNo));
    }
    
    @Override
    public List<PlanAstStatTask> getToDo(Date curTime, Integer limit) {
        return PlanAstStatTaskConvert.convert2Domains(planAstStatTaskDAO.getToDo(curTime, limit));
    }
    
    @Override
    public int updateTaskExec(Date lastExecTime, String lastHost, List<String> planNos) {
        return planAstStatTaskDAO.updateTaskExec(lastExecTime, lastHost, planNos);
    }
    
    /**
     * desc:查询执行次数超次数的统计任务.<br/>
     * @param execTimes execTimes
     * @return List<PlanAstStatTaskDO>
     */
    @Override
    public List<PlanAstStatTask> queryFailStatTask(Long execTimes){
        return PlanAstStatTaskConvert.convert2Domains(planAstStatTaskDAO.queryFailStatTask(execTimes));
    }

    /**
     * 根据状态计数
     * @param deadLine
     * @param status
     * @return
     */
    @Override
    public Long countByStatusAndTime(Date deadLine,String status){
        return planAstStatTaskDAO.countByStatusAndTime(deadLine,status);
    }
}
