/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.repo.plan.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.PlanAcctTitleLogDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanAcctTitleLogDO;
import com.abssqr.plat.common.facade.enums.CalcTypeEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.TitleTypeEnum;
import com.abssqr.plat.common.model.convertor.PlanAcctTitleLogConvertor;
import com.abssqr.plat.common.model.domain.plan.PlanAcctTitleLog;
import com.abssqr.plat.common.model.repo.plan.PlanAcctTitleLogRepo;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.repo.plan.impl: PlanAcctTitleLogRepoImpl.java, v 0.1 2019-07-01 00:10 zhenxuan.luo Exp $ 
 */
@Component
public class PlanAcctTitleLogRepoImpl implements PlanAcctTitleLogRepo {
    
    @Autowired
    private PlanAcctTitleLogDAO planAcctTitleLogDAO;
    
    @Override
    public int insertBatch(List<PlanAcctTitleLog> logList) {
        
        if (CollectionUtils.isEmpty(logList)) {
            return 0;
        }
        List<PlanAcctTitleLogDO> dos = Lists.newArrayList();
        for (PlanAcctTitleLog planAcctTitleLog : logList) {
            dos.add(PlanAcctTitleLogConvertor.convert2DO(planAcctTitleLog));
        }
        return planAcctTitleLogDAO.insertBatch(dos);
    }
    
    @Override
    public List<PlanAcctTitleLog> getByDate(String planNo, Date startDate, Date endDate) {
        
        List<PlanAcctTitleLogDO> dos = planAcctTitleLogDAO.getByDate(endDate, startDate, planNo);
        List<PlanAcctTitleLog> ret = Lists.newArrayList();
        for (PlanAcctTitleLogDO logDO : dos) {
            ret.add(PlanAcctTitleLogConvertor.convert2Domain(logDO));
        }
        return ret;
    }
    
    @Override
    public PlanAcctTitleLog getSettleFinFee(Date accountDate, String planNo) {
        PlanAcctTitleLogDO dos = planAcctTitleLogDAO.getTitleLogByCondition(
                accountDate, planNo, CalcTypeEnum.FIN_SER_FEE.getCode(), EventCodeEnum.TITLE_SETTLE.getCode());
        PlanAcctTitleLog result = PlanAcctTitleLogConvertor.convert2Domain(dos);
        return result;
    }
    
    /**
     * 查询对应科目类型的流水
     * @param accountDate
     * @param planNo
     * @param titleTypes
     * @return
     */
    @Override
    public PlanAcctTitleLog getAcctTitleByType(Date accountDate, String planNo, EventCodeEnum eventCode, List<TitleTypeEnum> titleTypes) {
        List<String> codes = titleTypes.stream().map(TitleTypeEnum::getCode).collect(Collectors.toList());
        PlanAcctTitleLogDO titleLogDO = planAcctTitleLogDAO.getSummaryAcctTitleByTypes(accountDate, planNo, eventCode.getCode(), codes);
        return PlanAcctTitleLogConvertor.convert2Domain(titleLogDO);
    }
    
    @Override
    public List<PlanAcctTitleLog> getByEventAndTitleNo(String planNo, String titleNo, Date accountDate, EventCodeEnum eventCode) {
        List<PlanAcctTitleLogDO> dos = planAcctTitleLogDAO.getByEventAndTitleNo(accountDate, planNo, titleNo, eventCode.getCode());
        return PlanAcctTitleLogConvertor.convert2Domains(dos);
    }
    
    /**
     * desc:根据计划与事件码获取科目流水.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param eventCode eventCode
     * @return List<PlanAcctTitleLogDO>
     */
    @Override
    public List<PlanAcctTitleLog> getByPlanAndEvent(Date accountDate, String planNo, EventCodeEnum eventCode, List<TitleTypeEnum> titleTypes){
        List<String> codes = titleTypes.stream().map(TitleTypeEnum::getCode).collect(Collectors.toList());
        List<PlanAcctTitleLogDO> dos = planAcctTitleLogDAO.getByPlanAndEvent(accountDate, planNo, eventCode.getCode(), codes);
        return PlanAcctTitleLogConvertor.convert2Domains(dos);
    }
}