/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.plan.impl;

import java.util.Date;
import java.util.List;
import java.util.Map;

import com.abssqr.plat.common.dal.mysql.auto.dao.PlanTaskLogDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanTaskLogDO;
import com.abssqr.plat.common.facade.enums.PlanTaskTypeEnum;
import com.abssqr.plat.common.model.convertor.PlanTaskLogConvert;
import com.abssqr.plat.common.model.domain.plan.PlanTaskLog;
import com.abssqr.plat.common.model.repo.plan.PlanTaskLogRepo;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.repo.plan.impl: PlanTaskLogRepoImpl.java, v 0.1 2019-09-23 11:28 bangis
 * .wangdf Exp $
 */
@Component
public class PlanTaskLogRepoImpl implements PlanTaskLogRepo {
    @Autowired
    PlanTaskLogDAO planTaskLogDAO;

    @Override
    public int createTask(PlanTaskLog task) {
        return planTaskLogDAO.insert(PlanTaskLogConvert.model2Do(task));
    }

    @Override
    public int createTasks(List<PlanTaskLog> tasks) {
        if(CollectionUtils.isNotEmpty(tasks)){
            List<PlanTaskLogDO> taskLogDOS=Lists.newArrayList();
            tasks.forEach(task->{
                taskLogDOS.add(PlanTaskLogConvert.model2Do(task));
            });
            planTaskLogDAO.insertBatch(taskLogDOS);
        }
        return 0;
    }

    /**
     * 获取任务
     *
     * @param planNo
     * @param bizDate
     * @return
     */
    @Override
    public PlanTaskLog getTask(String planNo, Date bizDate, PlanTaskTypeEnum taskType) {
        PlanTaskLogDO taskLogDO = planTaskLogDAO.getByPlanNoAndDate(bizDate, planNo, taskType.getCode());
        return PlanTaskLogConvert.do2Model(taskLogDO);
    }

    @Override
    public List<PlanTaskLog> getTasks(Date bizDate, PlanTaskTypeEnum taskType) {
        List<PlanTaskLogDO> tasks = planTaskLogDAO.getTasks(bizDate, taskType.getCode());
        List<PlanTaskLog> result = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(tasks)) {
            tasks.forEach(planTaskLogDO -> {
                result.add(PlanTaskLogConvert.do2Model(planTaskLogDO));
            });
        }
        return result;
    }

    @Override
    public Map<String, PlanTaskLog> getTaskMap(Date bizDate, PlanTaskTypeEnum taskType) {
        Map<String, PlanTaskLog> result = Maps.newHashMap();
        List<PlanTaskLog> tasks = getTasks(bizDate, taskType);
        if(CollectionUtils.isNotEmpty(tasks)){
            tasks.forEach(planTaskLog -> {
                result.put(planTaskLog.getPlanNo(),planTaskLog);
            });
        }
        return result;
    }
}
