/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.common.model.utils;

import cn.hutool.core.collection.CollectionUtil;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.enums.CommonErrorCodeEnum;
import com.general.system.common.model.exception.BaseCommonException;
import com.general.system.common.util.MessageUtil;
import com.general.system.tool.util.ToolUtil;

import java.util.List;

/**
 *
 * @author yangcheng
 * @version ApproveChangeUtil.java, v0.12019-06-2717:31 yangchengExp$
 */
public class ApproveChangeUtil {
    
    
    /**
     * 如果obj1=obj2，则抛异常
     * @param statusList
     * @param checkStatus
     * @param msgTemplate
     * @param vars
     */
    public static void checkEqualTrueError(List<ApproveStatusEnum> statusList, ApproveStatusEnum checkStatus, String msgTemplate, Object... vars) {
        if (ToolUtil.isNotEmpty(statusList) && checkStatus != null) {
            if (statusList.contains(checkStatus)) {
                throw new BaseCommonException(MessageUtil.formatMsg(msgTemplate, vars), CommonErrorCodeEnum.INVALID_PARAM);
            }
        } else {
            throw new BaseCommonException(MessageUtil.formatMsg(msgTemplate, vars), CommonErrorCodeEnum.INVALID_PARAM);
        }
    }
    
    /**
     * 回调放行
     */
    public static boolean callBackPass(ApproveStatusEnum approveStatusEnum){
        if (approveStatusEnum == ApproveStatusEnum.PASSED ||
            approveStatusEnum == ApproveStatusEnum.FAILED ||
            approveStatusEnum == ApproveStatusEnum.UNWANTED ||
            approveStatusEnum == ApproveStatusEnum.OUTOFDATE ||
            approveStatusEnum == ApproveStatusEnum.CANCEL){
            return  true;
        }
        return false;
    }
    
    /**
     * 不允许修改情况:审批中--针对计划
     */
    public static void bizPassedModification(ApproveStatusEnum approveStatusEnum,String bizNo,String bizName){
        checkEqualTrueError(CollectionUtil.newArrayList(ApproveStatusEnum.PENDING),
                approveStatusEnum,
                "[{0}][{1}]不允许修改,审批状态为[{2}]",
                bizNo,bizName,approveStatusEnum.getDesc());
    }
    /**
     * 不允许修改情况:审批中和审批完成
     */
    public static void bizModification(ApproveStatusEnum approveStatusEnum,String bizNo,String bizName){
        checkEqualTrueError(CollectionUtil.newArrayList(ApproveStatusEnum.PASSED, ApproveStatusEnum.PENDING),
                approveStatusEnum,
                "[{0}][{1}]不允许修改,审批状态为[{2}]",
                        bizNo,bizName,approveStatusEnum.getDesc());
    }
    /**
     * 回调成功执行后续业务逻辑判断
     */
    public static boolean callBackSucceed(ApproveStatusEnum approveStatusEnum){
        if (approveStatusEnum == ApproveStatusEnum.PASSED
            || approveStatusEnum == ApproveStatusEnum.UNWANTED){
                return true;
         }
        return false;
    }
    
    /**
     * 常规允许编辑判断方法
     * 不可编辑：
     * 1.项目状态为已清算
     * 2.审批状态为审批中
     *
     * 可编辑特殊情况
     * 放款池：存续中只能追加
     * 承接池： 存续中编辑转让规则和追加
     * @param approveStatusEnum
     * @return
     *
     */
    public static int allowEdit(ApproveStatusEnum approveStatusEnum){
        if (ApproveStatusEnum.UNWANTED.equals(approveStatusEnum)
            ||ApproveStatusEnum.PASSED.equals(approveStatusEnum)
            ||ApproveStatusEnum.PENDING.equals(approveStatusEnum)){
            return 0;
        } else {
            return 1;
        }
    }
    
    /**
     * 计划判断允许编辑逻辑
     * 不可编辑：
     * 1.项目状态为已清算
     * 2.审批状态为审批中
     * @param approveStatusEnum
     * @return
     */
    public static int allowEditPlan(ApproveStatusEnum approveStatusEnum, PlanStatusEnum planStatusEnum,PlanTypeEnum planTypeEnum){
        if (approveStatusEnum == ApproveStatusEnum.PENDING) {
            return 0;
        }
        if (planStatusEnum == PlanStatusEnum.COMPLETED) {
            return 0;
        }
        return 1;
    }
    
    /**
     * 未通过情况 可进行编辑审批状态判断
     */
    public static boolean makeEditNotSucc(ApproveStatusEnum approveStatusEnum){
        if (approveStatusEnum == ApproveStatusEnum.OUTOFDATE ||
            approveStatusEnum == ApproveStatusEnum.UNSUBMIT ||
            approveStatusEnum == ApproveStatusEnum.FAILED ||
            approveStatusEnum == ApproveStatusEnum.CANCEL){
            return  true;
        }
        return false;
    }
}
