/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.model.utils;

import com.abssqr.plat.common.facade.enums.AstRiskClassEnum;
import com.abssqr.plat.common.facade.model.prod.ProductRiskCl;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * 资产操作相关工具类
 * @author guozhenhua
 * @version com.abssqr.plat.processor.core.service.util: AstUtil.java, v 0.1 2017-12-13 下午2:58 guozhenhua Exp $
 */
public class AstUtil {
    
    /**
     * 根据逾期天数计算五级分类
     *
     * @param ovdDays
     * @param productRiskCls
     * @return
     */
    public static AstRiskClassEnum calcRiskCl(int ovdDays, List<ProductRiskCl> productRiskCls) {
        
        // 将五级分类设置按逾期天数顺序排序
        Collections.sort(productRiskCls, Comparator.comparing(ProductRiskCl::getOvdDays));
    
        ProductRiskCl referRiskCl = null;
        for (ProductRiskCl productRiskCl : productRiskCls) {
            referRiskCl = productRiskCl;
            if (null != productRiskCl.getOvdDays() && ovdDays <= productRiskCl.getOvdDays()) {
                return productRiskCl.getRiskCl();
            }
        }
        
        return referRiskCl.getRiskCl();
    }


//    public static void main(String[] args) {
//        List<ProductRiskCl> productRiskCls = Lists.newArrayList();
//        ProductRiskCl productRiskCl = new ProductRiskCl();
//        productRiskCl.setOvdDays(1);
//        productRiskCl.setRiskCl(AstRiskClassEnum.NORMAL);
//
//        ProductRiskCl productRiskCl2 = new ProductRiskCl();
//        productRiskCl2.setOvdDays(3);
//        productRiskCl2.setRiskCl(AstRiskClassEnum.ATTENTION);
//
//
//        ProductRiskCl productRiskCl3 = new ProductRiskCl();
//        productRiskCl3.setOvdDays(5);
//        productRiskCl3.setRiskCl(AstRiskClassEnum.SUSPICIOUS);
//
//
//        ProductRiskCl productRiskCl4 = new ProductRiskCl();
//        productRiskCl4.setOvdDays(5);
//        productRiskCl4.setRiskCl(AstRiskClassEnum.SUSPICIOUS);
//
//
//
//        productRiskCls.add(productRiskCl);
//        productRiskCls.add(productRiskCl2);
//        productRiskCls.add(productRiskCl3);
//
//        System.out.println(AstUtil.calcRiskCl(4,productRiskCls));
//    }
}
