/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.utils;

import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanAstStatusEnum;
import com.abssqr.plat.common.facade.enums.PriceTypeEnum;
import com.abssqr.plat.common.facade.enums.ProfitTypeEnum;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanAst;
import com.abssqr.plat.common.model.domain.ast.LoanAstProfit;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.ast.Pack;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstProfitTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalTransDetail;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.MoneyUtil;
import com.general.system.common.util.VarChecker;

import java.math.BigDecimal;
import java.util.Date;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.util: LoanAstUtil.java, v 0.1 2019-11-14 上午03:08 zhenxuan.luo Exp $ 
 */
public class LoanAstUtil {
    
    /**
     * 构建转让交易 的机构资产
     * @param srcInstal 原分期
     * @param loanAst  原机构资产
     * @param pack  资产包
     * @param accountDate   交易日
     * @param astPrice  分润比例
     * @param priceType  分润规则
     * @param inRule  转入规则编号
     * @return
     */
    public static LoanAst buildForTransfer(LoanInstalDomain srcInstal, LoanAst loanAst, Pack pack, Date accountDate,
                                           BigDecimal astPrice,
                                           PriceTypeEnum priceType, String inRule) {
        LoanAst ret = new LoanAst();
        ret.setLoanAstNo(SequenceUtil.genLoanId(srcInstal.getRefLoan()));
        ret.setAstNo(loanAst.getAstNo());
        ret.setOutAstNo(loanAst.getOutAstNo());
        ret.setLenderOrgCode(loanAst.getLenderOrgCode());
        ret.setProdNo(loanAst.getProdNo());
        ret.setOutProdNo(loanAst.getOutProdNo());
        ret.setUserId(loanAst.getUserId());
        ret.setTermNo(loanAst.getTermNo());
        ret.setInRule(inRule);
        
        ret.setOwnPack(pack.getPackCode());
        ret.setOwnOrg(pack.getPlanNo());
        ret.setInAstNo(loanAst.getLoanAstNo());
        ret.setAstStatus(LoanAstStatusEnum.NORMAL);
        ret.setRefInstal(loanAst.getRefInstal());
        ret.getRefInstal().addLoanAst(ret);
        ret.setAstPriceType(priceType);
        ret.setAstPrice(astPrice);
        
        LoanAstProfit loanAstProfit = buildDefaultProfit(srcInstal, ret, accountDate);
    
        // 暂时只支持利息分润的模式
        if (priceType == PriceTypeEnum.SPLIT_PROFIT) {
            // 默认将超额收益留在放款机构，固定逻辑，后面需要变更再说
            loanAstProfit.setSplitOrgCode(loanAst.getLenderOrgCode());
            loanAstProfit.setSplitRate(astPrice);
        }
        ret.setLastUpdateDate(accountDate);
        return ret;
    }
    
    /**
     * 数据迁移用，上线后可删掉
     * @param instalTransDetail
     * @param loanAst
     * @param accountDate
     * @return
     */
    public static LoanAst buildForLenderOrg(Loan loan, LoanInstalTransDetail instalTransDetail, LoanAst loanAst,
                                            Date accountDate) {
        LoanInstalDomain instal = instalTransDetail.getLoanInstal();
        LoanAst ret = new LoanAst();
        ret.setLoanAstNo(SequenceUtil.genLoanId(loan));
        ret.setAstNo(loanAst.getAstNo());
        ret.setOutAstNo(loanAst.getOutAstNo());
        ret.setLenderOrgCode(loanAst.getLenderOrgCode());
        ret.setProdNo(loanAst.getProdNo());
        ret.setOutProdNo(loanAst.getOutProdNo());
        ret.setUserId(loanAst.getUserId());
        ret.setTermNo(loanAst.getTermNo());
        
        Money disbAmt = instal.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN)
                                .add(instal.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.PRIN));
        ret.setCostAmt(disbAmt);
        
        Money paidOnLender = disbAmt.subtract(instal.getCost());
        ret.setPaidPrinAmt(paidOnLender);
        
        ret.setOwnOrg(loanAst.getLenderOrgCode());
        ret.setAstStatus(LoanAstStatusEnum.OUT);
        ret.setRefInstal(loanAst.getRefInstal());
        ret.getRefInstal().addLoanAst(ret);
        ret.setInDate(loan.getAstPeriod().getInDate());
        ret.setClearDate(loanAst.getInDate());
        ret.setClearEvnt(EventCodeEnum.TRANS_OUT);
        
        // 修正受让Ast上的信息
        loanAst.setInAstNo(ret.getLoanAstNo());
        loanAst.setAstPriceType(PriceTypeEnum.EQUAL);
        loanAst.setAstPrice(BigDecimal.ZERO);
        loanAst.getPaidPrinAmt().subtractFrom(paidOnLender);
        
        LoanAstProfit astProfit = buildDefaultProfit(instal, ret, instal.getAstPeriod().getInDate());
        
        LoanAstProfit ownProfit = loanAst.getLoanAstProfit(loanAst.getTermNo(), ProfitTypeEnum.OWN);
        Money hangIntrAmt = instal.getHangIntrBal();
        if (hangIntrAmt.isGreaterThanZero()) {
            hangIntrAmt = MoneyUtil.min(hangIntrAmt, instal.getBal().getByFieldType(DebtMoneyFieldEnum.INTR));
            instalTransDetail.dealHangIntrAmt(hangIntrAmt);
            
            Money paidHangIntrAmt = instal.getPaidHangIntrAmt();
            paidHangIntrAmt = MoneyUtil.min(paidHangIntrAmt,
                    ownProfit.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.INTR));
            instalTransDetail.movePaidHangIntrAmt(paidHangIntrAmt);
            Money paidHangIntrFundAmt = CalcUtil.calcMoneyFactor(paidHangIntrAmt,
                    ownProfit.getFundAmt().getSum().getAmount(),
                    ownProfit.getPaidAmt().getSum().getAmount());
            
            paidHangIntrFundAmt = MoneyUtil.min(paidHangIntrFundAmt,
                    ownProfit.getFundAmt().getByFieldType(DebtMoneyFieldEnum.INTR));
            
            astProfit.getBal().setByFieldType(DebtMoneyFieldEnum.INTR, hangIntrAmt);
            astProfit.getPaidAmt().setByFieldType(DebtMoneyFieldEnum.INTR, paidHangIntrAmt);
            astProfit.getFundAmt().setByFieldType(DebtMoneyFieldEnum.INTR, paidHangIntrFundAmt);
            
            Money afterAstIntr = instal.getHangIntrBal()
                                         .add(astProfit.getBal().getByFieldType(DebtMoneyFieldEnum.INTR));
            VarChecker.checkEquals(instal.getHangIntrBak(), afterAstIntr,
                    "分期[{0}-{1}]挂账应收利息转机构收益，前后金额不等[{2}-{3}]",
                    instal.getHangIntrBak(), afterAstIntr);
            Money intrBalNew = ownProfit.getBal().getByFieldType(DebtMoneyFieldEnum.INTR).subtract(hangIntrAmt);
            Money intrPaidNew = ownProfit.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.INTR)
                                        .subtract(paidHangIntrAmt);
            Money intrFundNew = ownProfit.getFundAmt().getByFieldType(DebtMoneyFieldEnum.INTR)
                                        .subtract(paidHangIntrFundAmt);
            
            ownProfit.getBal().setByFieldType(DebtMoneyFieldEnum.INTR, intrBalNew);
            ownProfit.getPaidAmt().setByFieldType(DebtMoneyFieldEnum.INTR, intrPaidNew);
            ownProfit.getFundAmt().setByFieldType(DebtMoneyFieldEnum.INTR, intrFundNew);
            
            astProfit.setEntryAmt(
                    astProfit.getBal().cloneAll().addAll(astProfit.getPaidAmt()).addAll(astProfit.getRmtAmt()));
            astProfit.setIntrEntryDec(astProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.INTR).getAmount());
            astProfit.setPrinPenalEntryDec(
                    astProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY).getAmount());
            astProfit.setIntrPenalEntryDec(
                    astProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY).getAmount());
            astProfit.setFeeEntryDec(astProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.FEE).getAmount());
            astProfit.setFeePenalEntryDec(
                    astProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY).getAmount());
            
            ownProfit.setEntryAmt(
                    ownProfit.getBal().cloneAll().addAll(ownProfit.getPaidAmt()).addAll(ownProfit.getRmtAmt()));
            ownProfit.setIntrEntryDec(ownProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.INTR).getAmount());
            ownProfit.setPrinPenalEntryDec(
                    ownProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY).getAmount());
            ownProfit.setIntrPenalEntryDec(
                    ownProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY).getAmount());
            ownProfit.setFeeEntryDec(ownProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.FEE).getAmount());
            ownProfit.setFeePenalEntryDec(
                    ownProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY).getAmount());
        }
        
        ret.setLastUpdateDate(accountDate);
        return ret;
    }
    
    public static LoanAst buildFromInstal(LoanInstalDomain instal, Date accountDate) {
        LoanAst ret = buildDummyFromInstal(instal, accountDate);
        
        ret.setCostAmt(instal.getCost().clone());
        ret.setPrinBal(instal.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN));
        ret.setPaidPrinAmt(instal.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.PRIN));
        
        LoanAstProfit astProfit = ret.getLoanAstProfit(ret.getTermNo(), ProfitTypeEnum.OWN);
        // 利息留在当期
        astProfit.getBal().addAll(instal.getBal(), DebtMoneyFieldEnum.PRIN);
        astProfit.getPaidAmt().addAll(instal.getPaidAmt(), DebtMoneyFieldEnum.PRIN);
        astProfit.getRmtAmt().addAll(instal.getRmtAmt(), DebtMoneyFieldEnum.PRIN);
        astProfit.getFundAmt().addAll(instal.getPaidFundAmt(), DebtMoneyFieldEnum.PRIN);
        astProfit.setEntryAmt(
                astProfit.getBal().cloneAll().addAll(astProfit.getPaidAmt()).addAll(astProfit.getRmtAmt()));
        astProfit.setIntrEntryDec(astProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.INTR).getAmount());
        astProfit.setPrinPenalEntryDec(
                astProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY).getAmount());
        astProfit.setIntrPenalEntryDec(
                astProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY).getAmount());
        astProfit.setFeeEntryDec(astProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.FEE).getAmount());
        astProfit.setFeePenalEntryDec(astProfit.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY).getAmount());
        return ret;
    }
    
    public static LoanAst initFromInstal(LoanInstalTransDetail instalTransDetail) {
        LoanInstalDomain loanInstal = instalTransDetail.getLoanInstal();
        LoanInstalLog loanInstalLog = instalTransDetail.getActiveLog();
        LoanAst loanAst = buildFromInstal(loanInstal, loanInstalLog.getAccountDate());
        LoanAstTransDetail astTransDetail = LoanAstTransDetail.createTransDtl(loanAst, instalTransDetail,
                loanInstalLog.getEventCode());
        astTransDetail.processTransAmtOnly(loanInstal.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN));
        // 初始化，强制插入
        astTransDetail.setUpdate(true);
        
        LoanAstProfit astProfit = loanAst.getLoanAstProfit(loanAst.getTermNo(), ProfitTypeEnum.OWN);
        LoanAstProfitTransDetail astProfitTransDetail = LoanAstProfitTransDetail
                                                                .createTransDtl(astProfit, instalTransDetail,
                                                                        loanInstalLog.getEventCode());
        astProfitTransDetail.processTransAmtOnly(loanInstal.getBal().cloneAll(DebtMoneyFieldEnum.PRIN));
        // 初始化，强制插入
        astProfitTransDetail.setUpdate(true);
        
        return loanAst;
    }
    
    /**
     * 根据分期创建初始默认机构资产
     *
     * @param instal
     * @param accountDate
     * @return
     */
    public static LoanAst buildDummyFromInstal(LoanInstalDomain instal, Date accountDate) {
        LoanAst ret = new LoanAst();
        ret.setLoanAstNo(SequenceUtil.genLoanId(instal.getRefLoan()));
        ret.setAstNo(instal.getAstNo());
        ret.setOutAstNo(instal.getOutAstNo());
        ret.setLenderOrgCode(instal.getAstOrg().getAstOwnership().getLenderOrgCode());
        ret.setProdNo(instal.getProdNo());
        ret.setOutProdNo(instal.getOutProdNo());
        ret.setUserId(instal.getUserId());
        ret.setTermNo(instal.getTermNo());
        
        ret.setOwnPack(instal.getOwnPack());
        ret.setOwnOrg(instal.getAstOrg().getAstOwnership().getOwnOrg());
        ret.setAstStatus(LoanAstStatusEnum.NORMAL);
        ret.setRefInstal(instal);
        ret.getRefInstal().addLoanAst(ret);
        ret.setInDate(instal.getAstPeriod().getInDate());
        
        buildDefaultProfit(instal, ret, accountDate);
        
        ret.setLastUpdateDate(accountDate);
        return ret;
    }
    
    public static LoanAstProfit buildDefaultProfit(LoanInstalDomain srcInstal, LoanAst loanAst, Date accountDate) {
        return buildAstProfit(srcInstal, loanAst, accountDate, ProfitTypeEnum.OWN);
    }
    
    public static LoanAstProfit buildSplitProfit(LoanInstalDomain srcInstal, LoanAstProfit ownSplitProfit,
                                                 Date accountDate) {
        LoanAst loanAst = ownSplitProfit.getRefLoanAst();
        
        LoanAstProfit profit = new LoanAstProfit();
        profit.setAstProfitNo(SequenceUtil.genLoanId(srcInstal.getRefLoan()));
        profit.setLoanAstNo(loanAst.getLoanAstNo());
        profit.setProfitType(ProfitTypeEnum.SPLIT);
        profit.setAstNo(loanAst.getAstNo());
        profit.setOutAstNo(loanAst.getOutAstNo());
        profit.setTermNo(loanAst.getTermNo());
        profit.setSrcTermNo(srcInstal.getTermNo());
        profit.setOwnOrg(ownSplitProfit.getSplitOrgCode());
        profit.setLastUpdateDate(accountDate);
        
        loanAst.addLoanAstProfit(profit);
        srcInstal.getSourcedProfits().add(profit);
        
        return profit;
    }
    
    private static LoanAstProfit buildAstProfit(LoanInstalDomain srcInstal, LoanAst loanAst, Date accountDate,
                                                ProfitTypeEnum profitType) {
        LoanAstProfit profit = new LoanAstProfit();
        profit.setAstProfitNo(SequenceUtil.genLoanId(srcInstal.getRefLoan()));
        profit.setLoanAstNo(loanAst.getLoanAstNo());
        profit.setProfitType(profitType);
        profit.setAstNo(loanAst.getAstNo());
        profit.setOutAstNo(loanAst.getOutAstNo());
        profit.setTermNo(loanAst.getTermNo());
        profit.setSrcTermNo(srcInstal.getTermNo());
        profit.setOwnOrg(loanAst.getOwnOrg());
        //        profit.setSplitOrgCode();
        //        profit.setSplitRate();
        profit.setLastUpdateDate(accountDate);
        // 增加机构资产收益
        loanAst.addLoanAstProfit(profit);
        srcInstal.getSourcedProfits().add(profit);
        
        return profit;
    }
}