/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.wk.repo.impl;

import com.abssqr.plat.common.dal.sync.auto.dao.WkDhccBatchDAO;
import com.abssqr.plat.common.dal.sync.auto.resultmap.DhccBatch;
import com.abssqr.plat.common.model.wk.repo.WkDhccBatchRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.general.system.common.util.LogUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.wk.repo.impl: WkDhccBatchRepoImpl.java, v 0.1 2019-06-12 21:50 bangis
 * .wangdf Exp $
 */
@Repository
public class WkDhccBatchRepoImpl implements WkDhccBatchRepo {
    protected final Logger         LOGGER = LoggerFactory.getLogger(getClass());
    @Autowired
    private         WkDhccBatchDAO wkDhccBatchDAO;
    @Autowired
    private         AbsConfig absConfig;
    /**
     * 数据是否准备完成
     *
     * @param brType
     * @param orgDay 机构业务会计日期
     * @return
     */
    @Override
    public boolean checkDataReady(String brType, Day orgDay) {
        //readyStatus 【111】第一个1表示 源始数据加载完成，第二个1 表示 中间层入库完成，第三个1表对账完成
        DhccBatch dhccBatch = getDataReadyStatus(brType, orgDay);
        if (dhccBatch != null) {
            return StringUtils.startsWith(dhccBatch.getStepFlag(), "11");
        }
        return false;
    }
    
    /**
     * 对账是否准备完成
     *
     * @param brType
     * @param orgDay 机构业务会计日期
     * @return
     */
    @Override
    public boolean checkFundReady(String brType, Day orgDay) {
        //readyStatus 【111】第一个1表示 源始数据加载完成，第二个1 表示 中间层入库完成，第三个1表对账完成
        DhccBatch dhccBatch = getDataReadyStatus(brType, orgDay);
        if (dhccBatch != null) {
            return StringUtils.startsWith(dhccBatch.getStepFlag(), "111");
        }
        return false;
    }
    
    /**
     *
     * @param brType
     * @param orgDay 机构会计账务日期，例如 今天跑11号的数据，则存20190611
     * @return
     */
    @Override
    public DhccBatch getDataReadyStatus(String brType, Day orgDay) {
        String dbLink = absConfig.getDbLink(), wkxwDbName = absConfig.getWkxwDbName();
        //readyStatus 【111】第一个1表示 源始数据加载完成，第二个1 表示 中间层入库完成，第三个1表对账完成
        try {
            return wkDhccBatchDAO.getDataReadyStatus(brType, dbLink, orgDay.getDayString(), wkxwDbName);
        }catch (Throwable e){
            LogUtil.error(LOGGER,e,"检查五矿数据是否完成出错，e={0}",e.getMessage());
        }
        return null;
    }
}
