/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.util.profiler;

import com.google.common.collect.Lists;

import java.text.NumberFormat;
import java.util.List;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.util.profiler: PerformRecord.java, v 0.1 2019-11-28 上午03:05 zhenxuan.luo Exp $ 
 */
public class PerformRecord {
    
    private final static String       SEP    = ": ";
    private final static String       START  = "- ";
    private final static NumberFormat FORMAT = NumberFormat.getNumberInstance();
    
    static {
        FORMAT.setGroupingUsed(false);
    }
    
    /**
     * name
     */
    private String              name;
    /**
     *
     */
    private boolean             isRoot = false;
    /**
     *
     */
    private long                start  = 0;
    /**
     *
     */
    private long                elapse = 0;
    /**
     *
     */
    private long                relElapse = 0;
    /**
     *
     */
    private PerformRecord       parent;
    /**
     *
     */
    private List<PerformRecord> child  = Lists.newArrayList();
    
    public PerformRecord(String name) {
        this.name = name;
        this.start = System.currentTimeMillis();
    }
    
    public PerformRecord() {
        isRoot = true;
    }
    
    public String getPerformDetail(int level) {
        
        StringBuilder builder = new StringBuilder();
        if (level > 0) {
            builder.append("\n");
        }
        for (int i = 0; i < level; i++) {
            builder.append("  ");
        }
        builder.append(START);
        if (!parent.isRoot) {
            builder.append(FORMAT.format(relElapse)).append(SEP);
        }
        builder.append(name).append(SEP).append(FORMAT.format(elapse)).append("ms");
        for (PerformRecord record : this.getChild()) {
            builder.append(record.getPerformDetail(level + 1));
        }
        return builder.toString();
    }
    
    public void end() {
        long now = System.currentTimeMillis();
        elapse = now - start;
        if (!parent.isRoot) {
            relElapse = now - parent.start;
        }
    }
    
    public long getElapse() {
        return elapse;
    }
    
    public boolean isRoot() {
        return isRoot;
    }
    
    public String getName() {
        return name;
    }
    
    public PerformRecord getParent() {
        return parent;
    }
    
    public void setParent(PerformRecord parent) {
        this.parent = parent;
    }
    
    public List<PerformRecord> getChild() {
        return child;
    }
    
    public void setChild(List<PerformRecord> child) {
        this.child = child;
    }
}