/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.util.utils;

import com.general.system.common.util.VarChecker;

import java.math.BigDecimal;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.util.utils: BigDecimalUtil.java, v 0.1 2019-11-17 上午01:38 zhenxuan.luo Exp $ 
 */
public class BigDecimalUtil {
    
    public static boolean isZero(BigDecimal val) {
        checkNull(val);
        return isEQ(val, BigDecimal.ZERO);
    }
    
    public static boolean isNegative(BigDecimal val) {
        checkNull(val);
        return isLT(val, BigDecimal.ZERO);
    }
    
    public static boolean isNotNegative(BigDecimal val) {
        checkNull(val);
        return isGTE(val, BigDecimal.ZERO);
    }
    
    public static BigDecimal min(BigDecimal... vals) {
        VarChecker.checkNotEmpty(vals, "min处理，数值数组不能为空");
        
        BigDecimal min = null;
        for (BigDecimal val : vals) {
            checkNull(val);
            
            if (null == min) {
                min = val;
            } else if (min.compareTo(val) > 0) {
                min = val;
            }
        }
        return clone(min);
    }
    
    public static BigDecimal max(BigDecimal... vals) {
        VarChecker.checkNotEmpty(vals, "max处理，数值数组不能为空");
        
        BigDecimal min = null;
        for (BigDecimal val : vals) {
            checkNull(val);
            
            if (null == min) {
                min = val;
            } else if (min.compareTo(val) < 0) {
                min = val;
            }
        }
        return clone(min);
    }
    
    public static boolean isGT(BigDecimal v1, BigDecimal v2) {
        checkNull(v1);
        checkNull(v2);
        return v1.compareTo(v2) > 0;
    }
    
    public static boolean isLT(BigDecimal v1, BigDecimal v2) {
        checkNull(v1);
        checkNull(v2);
        return v1.compareTo(v2) < 0;
    }
    
    public static boolean isGTE(BigDecimal v1, BigDecimal v2) {
        checkNull(v1);
        checkNull(v2);
        return v1.compareTo(v2) >= 0;
    }
    
    public static boolean isLTE(BigDecimal v1, BigDecimal v2) {
        checkNull(v1);
        checkNull(v2);
        return v1.compareTo(v2) <= 0;
    }
    
    public static boolean isEQ(BigDecimal v1, BigDecimal v2) {
        checkNull(v1);
        checkNull(v2);
        return v1.compareTo(v2) == 0;
    }
    
    public static BigDecimal clone(BigDecimal val) {
        checkNull(val);
        return BigDecimal.ZERO.add(val);
    }
    
    private static void checkNull(BigDecimal val) {
        VarChecker.checkNotNull(val, "数值处理不能为空");
    }
}