/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ZipUtil;
import com.abssqr.plat.common.dal.AbsSynDataConstants.WK_ABS_MFS_BILL_CTR;
import com.abssqr.plat.common.dal.AbsSynDataConstants.WK_MFS_DUE_BILL;
import com.abssqr.plat.common.dal.AbsSynDataConstants.WK_MFS_LN_PACT;
import com.abssqr.plat.common.dal.AbsSynDataConstants.WK_MFS_OPEN_REPAY;
import com.abssqr.plat.common.dal.AbsSynDataConstants.WK_MFS_REPAY_INFO;
import com.abssqr.plat.common.dal.AbsSynDataConstants.WK_MFS_REPAY_PLAN;
import com.abssqr.plat.common.dal.AbssqrCoreConstants.ABS_ACCT_DATE;
import com.abssqr.plat.common.dal.AbssqrCoreConstants.ABS_LOAN;
import com.abssqr.plat.common.dal.AbssqrCoreConstants.ABS_LOAN_AST;
import com.abssqr.plat.common.dal.AbssqrCoreConstants.ABS_LOAN_AST_PROFIT;
import com.abssqr.plat.common.dal.AbssqrCoreConstants.ABS_LOAN_CONTRACT;
import com.abssqr.plat.common.dal.AbssqrCoreConstants.ABS_LOAN_INSTAL;
import com.abssqr.plat.common.dal.AbssqrCoreConstants.ABS_ORGANIZATION_ENTITY;
import com.abssqr.plat.common.dal.AbssqrCoreConstants.ABS_PLAN;
import com.abssqr.plat.common.dal.AbssqrCoreConstants.ABS_PRODUCT;
import com.abssqr.plat.common.dal.AbssqrCoreConstants.ABS_PRODUCT_RISK_CL;
import com.abssqr.plat.core.service.debugdata.manager.QueryManager;
import com.abssqr.plat.core.service.debugdata.model.QueryModel;
import com.abssqr.plat.core.service.util.FreeMarkerUtil;
import com.abssqr.plat.core.service.wk.ast.sync.WkMfsAstManager;
import com.general.system.common.util.DateTimeUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.core.service: OfflineDebugService.java, v 0.1 2019-11-28 21:34 bangis.wangdf Exp $
 */
@Service
public class OfflineDebugService {
    private static final String outPath = "/tmp/logs/wkplat/";
    @Autowired
    QueryManager      queryManager;
    @Autowired
    WkMfsAstManager  wkMfsAstManager;
    
    List<QueryModel> initQueryModels      = Lists.newArrayList();
    List<QueryModel> dataCheckQueryModels = Lists.newArrayList();

    {
        //五矿数据
        initQueryModels.add(
            new QueryModel(WK_ABS_MFS_BILL_CTR.tableName, "*",
                Lists.newArrayList("LOAN_ID")).setParamDelService((res, ftlParams) -> {
                ftlParams.put("PROD_CODE", (String)res.get("PRODUCT_ID"));
                ftlParams.put("ORG_NO", (String)res.get("ORG_NO"));
                ftlParams.put("ABS_SYNC_DATE", DateTimeUtil.formatY_M_D((Date)res.get("ABS_SYNC_DATE")));
            }));
        initQueryModels.add(
            new QueryModel(WK_MFS_DUE_BILL.tableName, "*", Lists.newArrayList("LOAN_ID"))
                .setParamDelService((res, ftlParams) -> {
                    ftlParams.put("OWN_ORG", (String)res.get("PROJ_NO"));
                }));
        initQueryModels.add(new QueryModel(WK_MFS_OPEN_REPAY.tableName, "*",
            Lists.newArrayList("LOAN_ID")));
        initQueryModels.add(
            new QueryModel(WK_MFS_LN_PACT.tableName, "*", Lists.newArrayList("APP_ID")));
        initQueryModels.add(new QueryModel(WK_MFS_REPAY_INFO.tableName, "*",
            Lists.newArrayList("LOAN_ID")));
        initQueryModels.add(new QueryModel(WK_MFS_REPAY_PLAN.tableName, "*",
            Lists.newArrayList("LOAN_ID")));

        //ABS数据
        initQueryModels.add(new QueryModel(ABS_ACCT_DATE.tableName, "*", null));
        initQueryModels.add(new QueryModel(ABS_PRODUCT.tableName,"*",
            Lists.newArrayList("PROD_CODE")).setParamDelService((res, ftlParams) -> {
            ftlParams.put("ABSSQR_PROD_CODE", (String)res.get("ABSSQR_PROD_CODE"));
        }));
        initQueryModels.add(new QueryModel(ABS_PRODUCT_RISK_CL.tableName, "*",
            Lists.newArrayList("ABSSQR_PROD_CODE")));


        initQueryModels.add(new QueryModel(ABS_PLAN.tableName, "*",null));
        // 字段 ID,ENTITY_CODE,PROD_LST,NEED_ALLOC,STOP_PAY_DATE,MANAGER_ORG_CODE
        initQueryModels.add(new QueryModel(ABS_ORGANIZATION_ENTITY.tableName, "*",null));

        initQueryModels.add(new QueryModel(ABS_LOAN.tableName, "*",
            Lists.newArrayList("OUT_AST_NO")).setParamDelService((res, ftlParams) -> {
            ftlParams.put("AST_NO", (String)res.get("AST_NO"));
        }));
        initQueryModels.add(new QueryModel(ABS_LOAN_CONTRACT.tableName, "*",
            Lists.newArrayList("AST_NO")));
        initQueryModels.add(new QueryModel(ABS_LOAN_INSTAL.tableName, "*",
            Lists.newArrayList("AST_NO"), Lists.newArrayList("TERM_NO")));

        initQueryModels.add(new QueryModel(ABS_LOAN_AST.tableName, "*",
            Lists.newArrayList("AST_NO"), Lists.newArrayList("TERM_NO")));
        initQueryModels.add(new QueryModel(ABS_LOAN_AST_PROFIT.tableName, "*",
            Lists.newArrayList("AST_NO"), Lists.newArrayList("TERM_NO")));
    }
    
    /**
     * 拉取数据
     * @param loanId
     * @return
     */
    public File generateAstSyncData(String loanId) {
//        WkMfsSyncCtr ctrl = new WkMfsSyncCtr();
//        ctrl.setAbsSyncDate(orgAcctDayComponent.getTrfAcctDay().getStandardDate());
//        ctrl.setOrgNo(ORG_CODE);
//        ctrl.setLoanId(loanId);
//        wkMfsAstManager.syncData2Local(ctrl);

        String baseOutPaht = outPath + "/util/" + loanId;
        Map<String, String> qyeryParams = Maps.newHashMap();
        qyeryParams.put("LOAN_ID", loanId);
        qyeryParams.put("OUT_AST_NO", loanId);
        qyeryParams.put("APP_ID", loanId);
        queryManager.exec(initQueryModels, qyeryParams);
        queryManager.exec(dataCheckQueryModels, qyeryParams);

        Map<String, Object> ftlData = Maps.newHashMap();
        ftlData.put("param", qyeryParams);
        ftlData.put("initDatas", initQueryModels);
        ftlData.put("checkDatas", dataCheckQueryModels);
        FreeMarkerUtil.genFile(ftlData, "junit/sync/case0004.ftl",
            baseOutPaht + "/case0004_" + loanId + ".yml");
        FreeMarkerUtil.genFile(ftlData, "junit/sync/case0004/clearDBFiles.ftl",
            baseOutPaht + "/case0004_" + loanId + "/clearDBFiles.yaml");
        for (QueryModel initQueryModel : initQueryModels) {
            if (CollectionUtil.isNotEmpty(initQueryModel.getResult())) {
                FreeMarkerUtil.genFile(initQueryModel, "junit/sync/case0004/initDB/initDBCsv.csv.ftl",
                    baseOutPaht + "/case0004_" + loanId + "/initDB/" + initQueryModel.getTableName()
                        + ".csv");
            }
        }
        return ZipUtil.zip(baseOutPaht, baseOutPaht + ".zip");
    }

}
