/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2020 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.accounting;

import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtrLog;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.model.wk.repo.WkDhccBatchRepo;
import com.abssqr.plat.common.util.base.Day;
import com.general.system.common.util.DateTimeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 *
 * @author hanfei
 * @version $Id: AstSyncTaskComponent.java, v 0.1 2020-03-06 3:37 PM hanfei Exp $ 
 */
@Component
public class AstSyncTaskComponent {
    
    @Autowired
    private AstSyncTaskCtrRepo astSyncTaskCtrRepo;
    @Autowired
    private WkDhccBatchRepo    wkDhccBatchRepo;
    /**
     * 判断对应日期的任务完成情况
     * @param orgDate 执行日
     * @param orgCode 机构
     * @param taskType 任务类型
     * @return
     */
    public boolean isTaskFinishByOrgCode(Date orgDate, OrgCodeEnum orgCode, SyncTaskTypeEnum taskType){
        // 判断+1后日期对应机构编码的模型转换是否完成
        AstSyncTaskCtrLog lastTaskLog = astSyncTaskCtrRepo.getTaskLogByOrgDate(orgDate, taskType, orgCode.getCode());
        if (lastTaskLog != null && lastTaskLog.getStatus() == TaskStatusEnum.SUCCESS){
            return true;
        }
        return false;
    }
    
    /**
     * 判断对应日期五矿数据准备情况
     * @param exeDate 执行日
     * @param orgCode 机构
     * @return
     */
    public boolean isWKDataReady(Date exeDate, OrgCodeEnum orgCode){
        // 获取机构对应控制任务
        AstSyncTaskCtr taskCtr = astSyncTaskCtrRepo.getAstSyncTaskCtr(orgCode.getCode());
        // 判断机构数据五矿准备完成标记
        if (wkDhccBatchRepo.checkDataReady(taskCtr.getExtData().getCtrFlg(), new Day(exeDate))) {
            // 未完成直接跳出等待下一次调度 判断知道五矿数据准备完毕
            return true;
        }
        return false;
    }
    
    /**
     * 判断 招联机构
     * 放款池校验：
     *  资产是否同步完成
     *  账务是否对账完毕
     * @return  false 可以查询余额
     */
    public boolean cantBankBalance(PlanBase entity, Day execDate){
        if (entity.getPlanType() == PlanTypeEnum.SPV
            && entity instanceof OrganizationEntity){
            // 获取 计划的服务机构
            OrganizationEntity orgEntity = (OrganizationEntity)  entity;
            
            // 判断今天是否同步完成 取最后的机构日
            AstSyncTaskCtrLog syncLog = astSyncTaskCtrRepo.getTaskLogByExecDate(
                    execDate.getStandardDate(), SyncTaskTypeEnum.AST_MOD, orgEntity.getManagerOrgCode());
            // 正常情况 模型转换最后完成的日期应该 等于 转让会计日-1
            if (syncLog != null
                && syncLog.getStatus() == TaskStatusEnum.SUCCESS
                && DateTimeUtil.isSameDay(execDate.getYesterday().getStandardDate(), syncLog.getOrgDate())) {
                return false;
            }
            // 不能查询余额
            return true;
        }
        return false;
    }
}
