/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.astSync.BaseTsfRes;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlanTrans;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncTrans;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.core.service.period.PeriodAdjComponent;
import com.general.system.common.model.Money;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * 分期调整处理
 *
 * @author zhenxuan.luo
 * @version LoanAstAdjTermManager.java, v 0.1 2019-06-17 20:07 zhenxuan.luo Exp $
 */
@Component
public class LoanAstAdjTermManager extends LoanAstBaseManager {
    
    private final static Logger LOGGER = LoggerFactory.getLogger(LoanAstAdjTermManager.class);
    
    @Autowired
    private PeriodAdjComponent periodAdjComponent;
    
    @Override
    protected Loan doProcess(BaseTsfRes baseTsfRes, LoanAstSyncTrans loanAstSyncTrans,
                             LoanTransDetail loanTransDetail) {
        
        Loan loan = loanTransDetail.getLoan();
        Map<Integer, LoanInstalDomain> loanInstalMap = loan.getLoanInstalMap();
        List<LoanInstalTransDetail> instalTransDetails = Lists.newArrayList();
        
        Money allInAmt = new Money();
        Money allOutAmt = new Money();
        for (LoanAstSyncRepayPlanTrans repayPlanTrans : loanAstSyncTrans.getRepayPlanTransList()) {
            LogUtil.info(LOGGER, "分期调整[{0}]", repayPlanTrans);
            LoanInstalDomain refInstal = loanInstalMap.get(repayPlanTrans.getTermNo());
            
            LoanInstalTransDetail loanInstalTransDetail = loanTransDetail.addLoanInstalTransDtl(refInstal,
                    repayPlanTrans.getEventCode());
            
            // 1. 处理金额
            loanInstalTransDetail.processTransAmt(repayPlanTrans.getAmt());
            if (repayPlanTrans.getEventCode() == EventCodeEnum.ADJ_TERM_IN) {
                allInAmt.addTo(repayPlanTrans.getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN));
            } else if (repayPlanTrans.getEventCode() == EventCodeEnum.ADJ_TERM_OUT) {
                allOutAmt.addTo(repayPlanTrans.getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN));
            }
            instalTransDetails.add(loanInstalTransDetail);
        }
        
        VarChecker.checkEquals(allInAmt, allOutAmt, "[{0}-{1}]分期调整：调入金额[{2}]不等于调出金额[{3}]", loan.getAstNo(),
                loan.getOutAstNo(), allInAmt, allOutAmt);
        
        periodAdjComponent.process(instalTransDetails);
        
        return loanTransDetail.getLoan();
    }
}