/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanAstStatusEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.facade.model.plan.Plan;
import com.abssqr.plat.common.facade.model.report.ReportEntity;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanAst;
import com.abssqr.plat.common.model.domain.ast.LoanAstProfit;
import com.abssqr.plat.common.model.domain.ast.LoanAstProfitGroup;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.contract.LoanContract;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstProfitLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstProfitTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.TransInfoTemplate;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.ast.LoanAstProfitRepo;
import com.abssqr.plat.common.model.repo.ast.LoanAstRepo;
import com.abssqr.plat.common.model.repo.ast.LoanInstalRepository;
import com.abssqr.plat.common.model.repo.ast.LoanRepository;
import com.abssqr.plat.common.model.repo.contract.LoanContractRepository;
import com.abssqr.plat.common.model.repo.loanTrans.LoanAstLogRepo;
import com.abssqr.plat.common.model.repo.loanTrans.LoanAstProfitLogRepo;
import com.abssqr.plat.common.model.repo.loanTrans.LoanInstalLogRepository;
import com.abssqr.plat.common.model.repo.loanTrans.LoanLogRepository;
import com.abssqr.plat.common.model.repo.report.PlanReportEntityRepo;
import com.abssqr.plat.common.model.utils.LoanAstUtil;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.abssqr.plat.core.service.profit.ProfitSplitComponent;
import com.abssqr.plat.core.service.report.PlanReportEntityManager;
import com.abssqr.plat.core.service.report.PlanReportHolder;
import com.abssqr.plat.core.service.system.TimeStampUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 资产操作manager
 *
 * @author guozhenhua
 * @version LoanAstAssembler.java, v 0.1 2017-12-12 下午6:56
 * guozhenhua Exp $
 */
@Component
public class LoanAstAssembler {
    
    private final static Logger LOGGER = LoggerFactory.getLogger(LoanAstAssembler.class);
    
    @Autowired
    private LoanInstalRepository loanInstalRepository;
    
    @Autowired
    private LoanInstalLogRepository loanInstalLogRepository;
    
    @Autowired
    private LoanRepository loanRepository;
    
    @Autowired
    private LoanLogRepository loanLogRepository;
    
    @Autowired
    private AstPoolManager astPoolManager;
    
    @Autowired
    private PlanReportEntityManager planReportEntityManager;
    
    @Autowired
    private PlanReportEntityRepo planReportEntityRepo;
    
    @Autowired
    private LoanAstRepo loanAstRepo;
    
    @Autowired
    private LoanAstProfitRepo loanAstProfitRepo;
    
    @Autowired
    private LoanAstLogRepo loanAstLogRepo;
    
    @Autowired
    private LoanAstProfitLogRepo loanAstProfitLogRepo;
    
    @Autowired
    private LoanContractRepository loanContractRepository;
    
    @Autowired
    private ProfitSplitComponent profitSplitComponent;
    
    @Autowired
    private PlanManager planManager;
    
    @Autowired
    private AbsConfig absConfig;
    
    @PerformTrace
    public Loan lockByAstNo(String astNo, Date accountDate) {
        Loan loan = loanRepository.lockByAstNo(astNo);
        return assembleLoan(loan, accountDate);
    }
    
    @PerformTrace
    public Loan lockByOutAstNo(String outAstNo, String managerCode, Date accountDate) {
        Loan loan = loanRepository.lockByOutAstNo(outAstNo, managerCode);
        return assembleLoan(loan, accountDate);
    }
    
    public void initForTrans(Loan loan, Date accountDate) {
        if (null != accountDate) {
            //累减bal统计
            PlanReportHolder.clear();
            //if (StringUtils.equals(absConfig.getRecordBalReport(), "true")) {
            //    List<ReportEntity> entities = planReportManager
            //                                          .getReportItems(loan, EventCodeEnum.STAT_SUB, accountDate);
            //    PlanReportHolder.add(entities);
            //}
        }
    }
    
    
    @PerformTrace
    public void store(Loan loan, Date accountDate, boolean isFake) {
        doStore(loan, accountDate, isFake);
    }
    
    
    private void doStore(Loan loan, Date accountDate, boolean isFake) {
        loan.checkSelf(accountDate);
        if (null != accountDate) {
            //累加bal统计
            //if (StringUtils.equals(absConfig.getRecordBalReport(), "true")) {
            //    List<ReportEntity> entities = planReportManager
            //                                          .getReportItems(loan, EventCodeEnum.STAT_ADD, accountDate);
            //    PlanReportHolder.add(entities);
            //}
        }
        
        if (isFake) {
            return;
        }
        
        if (CollectionUtils.isNotEmpty(loan.getLoanTransDetailList())) {
            List<LoanLog> logs = Lists.newArrayList();
            List<LoanInstalLog> instalLogs = Lists.newArrayList();
            
            List<LoanAstLog> loanAstLogs = Lists.newArrayList();
            List<LoanAstProfitLog> loanAstProfitLogs = Lists.newArrayList();
            
            Map<String, LoanAst> loanAstMap = Maps.newHashMap();
            Map<String, LoanAstProfit> loanAstProfitMap = Maps.newHashMap();
            
            boolean isChange = false;
            for (LoanTransDetail loanTransDetail : loan.getLoanTransDetailList()) {
                if (loanTransDetail.isUpdate()) {
                    isChange = true;
                    LoanLog loanLog = loanTransDetail.getActiveLog();
                    LOGGER.info("[storeLoanLog] " + loanLog.forLog());
                    logs.add(loanLog);
                    
                    for (LoanInstalTransDetail loanInstalTransDetail : loanTransDetail.getLoanInstalTransDetails()) {
                        if (loanInstalTransDetail.isUpdate()) {
                            LoanInstalLog loanInstalLog = loanInstalTransDetail.getActiveLog();
                            loanInstalLog.getAmt().check(MessageUtil.formatMsg("分期交易流水发生额不能有负数-{0}", loanInstalLog.forLog()));
                           // LOGGER.info("[storeLoanInstalLog] " + loanInstalLog);
                            instalLogs.add(loanInstalLog);
                        }
                        
                        for (LoanAstTransDetail transDetail : loanInstalTransDetail.getLoanAstTransDetails()) {
                            if (transDetail.isUpdate()) {
                                LoanAstLog loanAstLog = transDetail.getActiveLog();
                                VarChecker
                                        .checkNonnegative(loanAstLog.getPrinAmt(), "机构资产交易流水发生额不能有负数-{0}", loanAstLog.forLog());
                                //LOGGER.info("[storeLoanAstLog] " + loanAstLog.getLoanAstNo(),loanAstLog.getTermNo());
                                
                                if (loanAstLog.getEventCode().getSceneCode() != TransCodeEnum.AST_SIMU) {
                                    loanAstLogs.add(loanAstLog);
                                }
                                
                                String key = transDetail.getLoanAst().getLoanAstNo();
                                if (!loanAstMap.containsKey(key)) {
                                    loanAstMap.put(key, transDetail.getLoanAst());
                                }
                            }
                        }
                        
                        for (LoanAstProfitTransDetail transDetail : loanInstalTransDetail
                                                                            .getLoanAstProfitTransDetails()) {
                            if (transDetail.isUpdate()) {
                                LoanAstProfitLog astProfitLog = transDetail.getActiveLog();
                                astProfitLog.getAmt()
                                        .check(MessageUtil.formatMsg("机构资产收益交易流水发生额不能有负数-{0}", astProfitLog.forLog()));
                                //LOGGER.info("[storeLoanAstProfitLog]{1}.{2}.{3}-{0}" , astProfitLog);
                                
                                if (astProfitLog.getEventCode().getSceneCode() != TransCodeEnum.AST_SIMU) {
                                    loanAstProfitLogs.add(astProfitLog);
                                }
                                
                                String key = transDetail.getLoanAstProfit().getAstProfitNo();
                                if (!loanAstProfitMap.containsKey(key)) {
                                    loanAstProfitMap.put(key, transDetail.getLoanAstProfit());
                                }
                            }
                        }
                    }
                    
                    //发生额统计
                    List<ReportEntity> logEntities = planReportEntityManager.getReportItemsByLog(loanTransDetail);
                    
                    //发生结息生成日初信息
                    List<ReportEntity> logSettOrTrfEntities =
                            planReportEntityManager.getRptItemsBySettOrTrf(loanTransDetail, accountDate);
                    if (CollectionUtils.isNotEmpty(logSettOrTrfEntities)) {
                        logEntities.addAll(logSettOrTrfEntities);
                    }
                    
                    PlanReportHolder.add(logEntities);
                }
                EventCodeEnum eventCode = loanTransDetail.getActiveLog().getEventCode();
                
                switch (eventCode) {
                    case TRANS_IN:
                    case TRANS_OUT:
                        astPoolManager.astPoolItemTrans(loan);
                        break;
                    case SYNC:
                        //同步
                        // 3. 记录待转让资产
                        astPoolManager.buildAstPoolEntity(loan, false, accountDate);
                        //当天放款当天还款情况需要删除
                    default:
                        //astPool是否删除
                        if (!loan.isNew() && (loan.getStatus() == LoanStatusEnum.CLEAR
                                              || loan.getStatus() == LoanStatusEnum.REVERSE)) {
                            // 结清，清理资产池
                            astPoolManager.removeAstPoolItem(loan.getAstNo(), loan.getTermCnt());
                        }
                }
            }
            
            if (isChange) {
                loanRepository.createOrUpdate(loan);
                loanLogRepository.insertBatch(logs);
                loanInstalLogRepository.insertBatch(instalLogs);
                loanAstLogRepo.insertBatch(loanAstLogs);
                loanAstProfitLogRepo.insertBatch(loanAstProfitLogs);
                
                for (LoanInstal loanInstal : loan.getLoanInstalList()) {
                    loanInstalRepository.createOrUpdate(loanInstal);
                    if (!loanInstal.isNew() && (loanInstal.getStatus() == LoanStatusEnum.CLEAR
                                                || loanInstal.getStatus() == LoanStatusEnum.REVERSE)) {
                        // 结清，清理资产池
                        astPoolManager.removeAstPoolItem(loanInstal.getAstNo(), loanInstal.getTermNo());
                    }
                }
                
                for (LoanAst loanAst : loanAstMap.values()) {
                    loanAstRepo.createOrUpdate(loanAst);
                }
                
                for (LoanAstProfit loanAstProfit : loanAstProfitMap.values()) {
                    loanAstProfitRepo.createOrUpdate(loanAstProfit);
                }
            }
        }
        //批量插入reportEntity
        planReportEntityRepo.insertBatch(PlanReportHolder.get());
        PlanReportHolder.clear();
    }
    
    
    
    public Loan assembleLoan(Loan loan, Date accountDate) {
        if (null == loan) {
            return null;
        }
        // 贷款合同
        LoanContract loanContract = loanContractRepository.getByAstNo(loan.getAstNo());
        loan.setContract(loanContract);
        
        List<LoanInstalDomain> loanInstalList = loanInstalRepository.getByAstNo(loan);
        // 分期处理完毕进行组装
        loan.setLoanInstalList(loanInstalList);
        
        List<LoanAst> loanAstList = loanAstRepo.getByAstNo(loan.getAstNo());
        if (CollectionUtils.isNotEmpty(loanAstList)) {
            // 如果机构资产模型已经存在：直接构造
            Map<Integer, LoanInstalDomain> loanInstalDomainMap = loanInstalList.stream().collect(
                    Collectors.toMap(LoanInstalDomain::getTermNo, item -> item));
            Map<String, LoanAst> loanAstMap = loanAstList.stream()
                                                      .collect(Collectors.toMap(LoanAst::getLoanAstNo, item -> item));
            List<LoanAstProfit> loanAstProfitList = loanAstProfitRepo.getByAstNo(loan.getAstNo());
            
            // 构造机构资产关系
            for (LoanAst loanAst : loanAstList) {
                LoanInstalDomain instalDomain = loanInstalDomainMap.get(loanAst.getTermNo());
                instalDomain.addLoanAst(loanAst);
                loanAst.setRefInstal(instalDomain);
            }
            
            // 构造机构收益关系
            for (LoanAstProfit loanAstProfit : loanAstProfitList) {
                LoanInstalDomain sourceDomain = loanInstalDomainMap.get(loanAstProfit.getSrcTermNo());
                sourceDomain.getSourcedProfits().add(loanAstProfit);
                LoanAst loanAst = loanAstMap.get(loanAstProfit.getLoanAstNo());
                
                if (null == loanAst) {
                    throw new AbssqrBizException(MessageUtil.formatMsg("资产收益[{0}-{1}]找不到对应的机构资产", loanAstProfit.getAstProfitNo(),
                            loanAstProfit.getLoanAstNo()), CommonErrorCodeEnum.INVALID_INTERNAL_MODEL, false);
                }
                loanAst.addLoanAstProfit(loanAstProfit);
            }
        } else if (loan.getStatus().isActive()) {
            //if (null != accountDate) {
                //累减bal统计
                //PlanReportHolder.clear();
                //if (StringUtils.equals(absConfig.getRecordBalReport(), "true")) {
                //    List<ReportEntity> entities = planReportManager
                //                                          .getReportItemsForInit(loan, EventCodeEnum.STAT_SUB,
                //                                                  accountDate);
                //    PlanReportHolder.add(entities);
                //}
            //}
            /**
             * 兼容旧模型数据初始化机构资产
             */
            initNewLoanDomain(loan, loanInstalList, accountDate);
            
            //if (null != accountDate) {
                //累减bal统计
                //PlanReportHolder.clear();
                //if (StringUtils.equals(absConfig.getRecordBalReport(), "true")) {
                //    List<ReportEntity> entities = planReportManager
                //                                          .getReportItems(loan, EventCodeEnum.STAT_ADD, accountDate);
                //    PlanReportHolder.add(entities);
                //}
            //}
        }
        
        initForTrans(loan, accountDate);
        return loan;
    }
    
    /**
     * 兼容旧模型数据初始化机构资产
     * @param loan
     * @param loanInstalList
     * @param accountDate
     */
    private void initNewLoanDomain(Loan loan, List<LoanInstalDomain> loanInstalList, Date accountDate) {
        TransInfoTemplate transInfoTemplate = new TransInfoTemplate();
        transInfoTemplate.setTransTime(SystemDateUtil.getSystemDate());
        transInfoTemplate.setAccountDate(accountDate);
        transInfoTemplate.setEventCode(EventCodeEnum.GRANT_SIMU);
        transInfoTemplate.setOutBizNo(loan.getAstNo());
        transInfoTemplate.setBizNo(loan.getAstNo() + "-" + transInfoTemplate.getEventCode().getCode());
        transInfoTemplate.setTimestamp(TimeStampUtil.getDefaultTimeStamp());
        
        LoanTransDetail loanTransDetail = LoanTransDetail.createLoanAstTxnDtl(loan, transInfoTemplate);
        // 如果机构资产模型为空：说明是历史数据，构造机构资产模型
        // 1. 构造机构资产的基础结构和本金
        Plan plan = null;
        for (LoanInstalDomain instalDomain : loanInstalList) {
            
            LoanInstalTransDetail instalTransDetail = loanTransDetail.addLoanInstalTransDtl(instalDomain);
            instalTransDetail.setUpdate();
            LoanAst newAst = LoanAstUtil.buildFromInstal(instalDomain, accountDate);
            
            if (!StringUtils.equals(newAst.getOwnOrg(), newAst.getLenderOrgCode())) {
                // 已转让：构造转让前的机构资产
                LoanAst lenderAst = LoanAstUtil.buildForLenderOrg(loan, instalTransDetail, newAst, accountDate);
                buildInitTransDetail(lenderAst, instalTransDetail);
                if (plan == null) {
                    plan = planManager.getPlanFullByNo(newAst.getOwnOrg());
                }
                newAst.setInRule(plan.getPlanTransferRule().getTransferNo());
            }
            
            LoanAstTransDetail astTransDetail = buildInitTransDetail(newAst, instalTransDetail);
            if (!instalTransDetail.getLoanInstal().getStatus().isActive()) {
                astTransDetail.clearLoanAst(instalDomain.getAstPeriod().getClearDate(), EventCodeEnum.REPO_SIMU,
                        LoanAstStatusEnum.CLEAR);
            }
        }
        loanTransDetail.processBeforeUpdate();
    }
    
    private LoanAstTransDetail buildInitTransDetail(LoanAst newAst, LoanInstalTransDetail instalTransDetail) {
        EventCodeEnum eventCode = instalTransDetail.getActiveLog().getEventCode();
        newAst.setExtInf("EXT:SIMU");
        LoanAstTransDetail astTransDetail = LoanAstTransDetail.createTransDtl(newAst, instalTransDetail, eventCode);
        astTransDetail.processTransAmtOnly(newAst.getPrinBal());
        
        for (LoanAstProfitGroup astProfitGroup : newAst.getLoanAstProfits().values()) {
            LoanAstProfit astProfit = astProfitGroup.getOwnAstProfit();
            astProfit.setExtInfo("EXT:SIMU");
            LoanAstProfitTransDetail astProfitTransDetail = LoanAstProfitTransDetail
                                                                    .createTransDtl(astProfit, instalTransDetail,
                                                                            eventCode);
            astProfitTransDetail.processTransAmtOnly(astProfit.getBal());
        }
        
        return astTransDetail;
    }
    
}