/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanAst;
import com.abssqr.plat.common.model.domain.ast.LoanAstGroup;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.astSync.BaseTsfRes;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlanTrans;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncTrans;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.core.service.profit.ProfitSplitCalculator;
import com.abssqr.plat.core.service.profit.ProfitSplitComponent;
import com.abssqr.plat.core.service.repo.AstFlowOutComponent;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 挂费处理
 *
 * @author zhenxuan.luo
 * @version LoanAstHangFeeManager.java, v 0.1 2019-06-17 20:07 zhenxuan.luo Exp $
 */
@Component
public class LoanAstHangFeeManager extends LoanAstBaseManager {

    final static String TRANS_MEMO = "提前还款费";

    @Autowired
    ProfitSplitCalculator profitSplitCalculator;
    @Autowired
    private AstFlowOutComponent  astFlowOutComponent;
    @Autowired
    private ProfitSplitComponent profitSplitComponent;

    @Override
    protected Loan doProcess(BaseTsfRes baseTsfRes, LoanAstSyncTrans loanAstSyncTrans,
                             LoanTransDetail loanTransDetail) {

        Loan loan = loanTransDetail.getLoan();
        Map<Integer, LoanInstalDomain> loanInstalMap = loan.getLoanInstalMap();

        for (LoanAstSyncRepayPlanTrans repayPlanTrans : loanAstSyncTrans.getRepayPlanTransList()) {

            VarChecker.checkArgument(repayPlanTrans.getEventCode().getSceneCode() == TransCodeEnum.HANG_FEE,
                    "LoanAstHangFeeManager不处理交易码为[{0}]的交易", repayPlanTrans.getEventCode());

            LoanInstalDomain refInstal = loanInstalMap.get(repayPlanTrans.getTermNo());

            LoanInstalTransDetail loanInstalTransDetail = loanTransDetail.addLoanInstalTransDtl(refInstal,
                    repayPlanTrans.getEventCode());

            loanInstalTransDetail.getActiveLog().setTransTime(repayPlanTrans.getTransTime());

            loanInstalTransDetail.getActiveLog().setOutBizNo(repayPlanTrans.getOutBizNo());

            Money hangFeeAmt = repayPlanTrans.getOtherFeeAmt();

            //获取提前还款本金
            Money prinAmt = repayPlanTrans.getPrinAmt();

            LoanInstalDomain loanInstalDomain = loanInstalTransDetail.getLoanInstal();

            //计算提前还款本金比例。
            Map<String, Money> prinTransAmtMap = astFlowOutComponent.calcRepoPrin(prinAmt, loanInstalDomain);

            LoanAstGroup astGroup = new LoanAstGroup();

            List<LoanAst> filterAstGroup = loanInstalDomain.getLoanAsts().stream()
                                                   .filter(loanAst -> prinTransAmtMap.get(loanAst.getLoanAstNo())
                                                                      != null).collect(Collectors.toList());

            astGroup.getLoanAsts().addAll(filterAstGroup);

            //计算机构资产分摊费用

            DebtMoney transAmt = new DebtMoney();

            transAmt.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE, hangFeeAmt);

            profitSplitComponent.splitOtherFeeInstal(loanTransDetail, loanInstalTransDetail, transAmt, prinTransAmtMap, TRANS_MEMO);



            loanInstalTransDetail.processTransAmt(transAmt, true);

            loanInstalTransDetail.getLoanTransDetail().getActiveLog().setMemo(TRANS_MEMO);
        }
        return loan;
    }

}