/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.astSync.BaseTsfRes;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlan;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlanTrans;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncTrans;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.core.service.profit.ProfitSplitComponent;
import com.abssqr.plat.core.share.context.PlanContextHolder;
import com.general.system.common.util.DateTimeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.stream.Collectors;

/**
 * 结息处理
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.ast: LoanAstSettleManager.java, v 0.1 2019-06-17 20:07 zhenxuan.luo Exp $
 */
@Component
public class LoanAstSettleManager extends LoanAstBaseManager {
    
    @Autowired
    private LoanStatusManager loanStatusManager;
    
    @Autowired
    private ProfitSplitComponent profitSplitComponent;
    
    @Override
    protected Loan doProcess(BaseTsfRes baseTsfRes, LoanAstSyncTrans loanAstSyncTrans,
                             LoanTransDetail loanTransDetail) {
        // 贷款
        Loan loan = loanTransDetail.getLoan();
        // 分期账单
        Map<Integer, LoanInstalDomain> loanInstalMap = loan.getLoanInstalMap();
        // 同步分期账单
        Map<Integer, LoanAstSyncRepayPlan> syncRepayPlanMap = baseTsfRes.getRepayPlans().stream().collect(
                Collectors.toMap(LoanAstSyncRepayPlan::getTermNo, item -> item));
        
        try {
            // 通过分期交易数据进行结息处理
            for (LoanAstSyncRepayPlanTrans repayPlanTrans : loanAstSyncTrans.getRepayPlanTransList()) {
                // 获取当前分期
                LoanInstalDomain refInstal = loanInstalMap.get(repayPlanTrans.getTermNo());
                // 结清不再处理
                if (refInstal.getStatus() == LoanStatusEnum.CLEAR) {
                    continue;
                }
                // 根据分期及交易明细 创建分期交易明细
                LoanInstalTransDetail loanInstalTransDetail = loanTransDetail.addLoanInstalTransDtl(refInstal);
        
                // 1. 处理金额
                DebtMoney transAmt = repayPlanTrans.getAmt();
                profitSplitComponent.splitInstal(loanTransDetail, loanInstalTransDetail, transAmt);
                loanInstalTransDetail.processTransAmt(transAmt, true);
        
                // 2. 处理逾期
                LoanAstSyncRepayPlan syncRepayPlan = syncRepayPlanMap.get(refInstal.getTermNo());
                if (refInstal.getStatus() == LoanStatusEnum.NORMAL && syncRepayPlan.getStatus() == LoanStatusEnum.OVERDUE) {
                    loanInstalTransDetail.changeStatus(syncRepayPlan.getStatus());
                }
                int ovdDays = (int)DateTimeUtil.calcDayDiff(loanAstSyncTrans.getAccountDate(),
                        refInstal.getAstPeriod().getEndDate());
                if (ovdDays > 0) {
                    loanInstalTransDetail.setOvdDays(ovdDays);
                }
            }
            
        } finally {
            PlanContextHolder.clear();
        }
        
        // 计算五级分类
        loanStatusManager.calcLoanRiskStatus(loanTransDetail, true);
        return loanTransDetail.getLoan();
    }
}