/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.astSync.BaseTsfRes;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncTrans;
import com.abssqr.plat.common.model.domain.clearing.PlanClearingLogKey;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.core.service.plan.PlanClearingManager;
import com.abssqr.plat.core.service.repo.AstFlowOutComponent;
import com.general.system.common.model.Money;
import com.general.system.common.util.LogUtil;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * 放款冲正处理
 *
 * @author zhenxuan.luo
 * @version LoanGrantReverseManager.java, v 0.1 2019-06-17 20:07 zhenxuan.luo Exp $
 */
@Component
public class LoanGrantReverseManager extends LoanAstBaseManager {
    
    @Autowired
    private PlanClearingManager planClearingManager;
    
    @Autowired
    private LoanStatusManager loanStatusManager;
    
    @Autowired
    private AstFlowOutComponent astFlowOutComponent;
    
    @Override
    protected Loan doProcess(BaseTsfRes baseTsfRes, LoanAstSyncTrans loanAstSyncTrans,
                             LoanTransDetail loanTransDetail) {
        
        Loan loan = loanTransDetail.getLoan();
        
        if (loan.getStatus() == LoanStatusEnum.REVERSE) {
            LogUtil.warn(LOGGER, "贷款[{0}]状态已经为冲正，不做处理", loan);
            return loan;
        }
        
        for (LoanInstalDomain loanInstal : loan.getLoanInstalList()) {
            LoanInstalTransDetail loanInstalTransDetail = loanTransDetail.addLoanInstalTransDtl(loanInstal);
            
            astFlowOutComponent.flowOutInstal(loanInstalTransDetail, loanInstal.getBal(), new DebtMoney(), true);
            loanInstalTransDetail.processTransAmt(loanInstal.getBal());
            // 已转让，需要从放款计划打款到受让计划
            if (loanInstal.getAstOrg().getAstOwnership().isTransfered()) {
                loanInstalTransDetail.setClrgAmt(loanInstal.getCost());
            }
            
            loanInstalTransDetail.changeStatus(LoanStatusEnum.REVERSE);
            loanInstalTransDetail.clearLoanAst();
        }
        
        planClearingManager.processClearingForLoanAst(loanTransDetail.getLoanInstalTransDetails(),
                TransCodeEnum.GRANT_REVERSE,
                loanAstSyncTrans.getAccountDate(),
                instalTransDetail -> {
                    Map<PlanClearingLogKey, Money> clearingMap = Maps.newHashMap();
                    for (LoanAstTransDetail astTransDetail : instalTransDetail.getLoanAstTransDetails()) {
                        LoanAstLog loanAstLog = astTransDetail.getActiveLog();
                        PlanClearingLogKey key = new PlanClearingLogKey(loanAstLog.getOwnOrg(),
                                loanAstLog.getLenderOrgCode(),
                                loanAstLog.getEventCode().getSceneCode());
                        
                        if (!clearingMap.containsKey(key)) {
                            clearingMap.put(key, new Money());
                        }
                        clearingMap.get(key).addTo(loanAstLog.getClrgAmt());
                    }
                    return clearingMap;
                });
        
        loanTransDetail.setStatus(LoanStatusEnum.REVERSE);
        return loanTransDetail.getLoan();
    }
}