/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.plan;

import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.enums.FeeTimeTypeEnum;
import com.abssqr.plat.common.facade.enums.IntrTypeEnum;
import com.abssqr.plat.common.facade.enums.LayerTypeEnu;
import com.abssqr.plat.common.facade.enums.PlanAllocStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanPayFreqEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.TitleTypeEnum;
import com.abssqr.plat.common.facade.model.BizDataChange;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.FeeStructure;
import com.abssqr.plat.common.facade.model.plan.InvestStructure;
import com.abssqr.plat.common.facade.model.plan.Plan;
import com.abssqr.plat.common.facade.model.plan.PlanAcctTitle;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanAllocItem;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocAssignedOprParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocEditParam;
import com.abssqr.plat.common.facade.result.workbench.ProdPayItem;
import com.abssqr.plat.common.model.convertor.PlanAllocConvert;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.plan.PlanAcctTitleRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAllocRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.model.utils.ApproveChangeUtil;
import com.abssqr.plat.core.service.change.ApproveCallBack;
import com.abssqr.plat.core.service.change.ChangeManager;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.DateTimeUtil.BetweenType;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 计划分配方案
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.core.service.plan: PlanAllocManager.java, v 0.1 2019-06-03 14:23 bangis.wangdf Exp $
 */
@Component
public class PlanAllocManager implements ApproveCallBack<PlanAlloc> {
    @Autowired
    PlanAllocRepo planAllocRepo;
    @Autowired
    ChangeManager changeManager;
    @Autowired
    PlanManager planManager;
    @Autowired
    PlanRepo planRepo;
    @Autowired
    PlanAcctTitleRepo planAcctTitleRepo;
    @Autowired
    PlanAllocAmtCalcManager planAllocAmtCalcManager;

    List<TitleTypeEnum> taxTitleTypes = Lists.newArrayList(TitleTypeEnum.INVEST_PROFIT, TitleTypeEnum.FEE);

    /**
     * 获取分配计划
     *
     * @param planNo
     * @return
     */
    public List<PlanAlloc> queryAllocByEntityNo(String planNo) {
        List<PlanAlloc> planAllocs = planAllocRepo.queryByPlanNo(planNo);
        return planAllocs;
    }

    /**
     * 初始化分配计划
     *
     * @param planBase
     * @param status
     */
    public void initOrAppendAlloc(PlanBase planBase, PlanStatusEnum status, Date accountDate) {
        if (status == PlanStatusEnum.PREPARING) {
            planAllocRepo.cleanByPlanNo(planBase.getPlanNo());
        }
        List<PlanAlloc> planAlloc = this.queryAllocByEntityNo(planBase.getPlanNo());
        if (ToolUtil.isEmpty(planAlloc)) {
            this.initPlanAlloc(planBase, accountDate);
        } else {
            this.appendPlanAlloc(planBase, planAlloc, accountDate);
        }
    }


    /**
     * 未分配情况 获取分配计划详情
     * 当期进行试算
     *
     * @return
     */
    public PlanAlloc getPlanAllocDetail(String allocNo) {
        PlanAlloc planAlloc = planAllocRepo.getPlanAllocDetail(allocNo);
        // 调用试算
        // 判断是否档期，不是档期不调用试算
        if (planAlloc != null) {
            if (planAlloc.getStatus() == PlanAllocStatusEnum.UNASSIGNE) {
                List<PlanAlloc> planAllocs = planAllocRepo.queryByPlanNo(planAlloc.getPlanNo());
                if (ToolUtil.isNotEmpty(planAllocs)) {
                    planAllocs.sort(Comparator.comparing(PlanAlloc::getClearDate));

                    PlanBase planBase = planManager.getPlanFullByNo(planAlloc.getPlanNo());
                    List<PlanAcctTitle> planAcctTitles = planAcctTitleRepo.getByEntityNo(planAlloc.getPlanNo());
                    //调用试算
                    planAllocAmtCalcManager.calcTriAmtByScheduler(planBase, planAllocs, planAcctTitles, false);
                    // 重新过滤取出对应分配计划
                    return planAllocs.stream().filter(alloc ->
                            StringUtils.equals(alloc.getAllocNo(), allocNo))
                            .collect(Collectors.toList()).stream()
                            .findFirst().get();
                }
            }
        }
        return planAlloc;
    }

    /**
     * 获取 审批的变更数据
     *
     * @param changeNo
     * @return
     */
    public PlanAlloc getDataChangePlanAllocBy(String changeNo) {
        BizDataChange<PlanAlloc> dataChange = changeManager.getDataChange(changeNo);
        if (dataChange == null) {
            throw new AbssqrBizException("未找到对应待审批的产品准入规则数据");
        }
        dataChange.getDatas().setApproveStatus(ApproveStatusEnum.getEnum(dataChange.getApproveStatus()));
        return dataChange.getDatas();
    }

    /**
     * 初始化分配计划
     *
     * @param planBase
     */
    private List<PlanAlloc> initPlanAlloc(PlanBase planBase, Date accountDate) {
        VarChecker.checkNotNull(planBase, "[{0}]找不到对应的计划", planBase.getPlanNo());
        //计划起息日
        Date effectDate = planBase.getEffectDate();

        //计划到期日
        Date endDate = planBase.getEndDate();
        Date firstClearDate = planBase.getFirstClearDate();
        Date firstPayDate = planBase.getFirstPayDate();
        PlanPayFreqEnum payFreq = planBase.getPayFreq();

        Map<String, PlanAcctTitle> planAcctTitleMap = getStringPlanAcctTitleMap(planBase.getPlanNo());

        //根据时间周期计算分配计划
        List<PlanAlloc> allocs = this.calcPlanAlloc(planBase.getPlanNo(), firstClearDate, firstPayDate, endDate,
                payFreq);
        VarChecker
                .checkNotEmpty(allocs, "[{0}]按时间周期计算分配计划异常,清算日{1}-支付日{2}-结束日{3}", planBase.getPlanNo(), firstClearDate,
                        firstPayDate,
                        endDate);

        //根据费用周期计算计划
        List<PlanAlloc> newAddFeeAlloc = calcFeeStructure(planBase, effectDate, firstClearDate, planAcctTitleMap,
                allocs);

        //投资结构
        List<PlanAlloc> newAddInvestAlloc = calcInvestStructure(planBase, planAcctTitleMap, allocs, false);

        //合并分配计划
        margeFeeInvestAlloc(allocs, newAddFeeAlloc, newAddInvestAlloc);

        // 过滤没有分配方案的分配计划
        allocs = allocs.stream().filter(
                planAlloc -> CollectionUtils.isNotEmpty(planAlloc.getPlanAllocItemList())).collect(
                Collectors.toList());

        allocs.sort(Comparator.comparing(PlanAlloc::getClearDate));

        // 过滤失效分配计划
        filterInvalidAlloc(accountDate, allocs);

        // 调整结算日和兑付日，保证相同的天数差
        long diffDays = DateTimeUtil.calcDayDiff(firstClearDate, firstPayDate);
        for (PlanAlloc alloc : allocs) {
            //设置计划类型
            alloc.setPlanType(planBase.getPlanType());
            if (alloc.getTimeType() != FeeTimeTypeEnum.PETD) {
                Date clearDate = DateTimeUtil.addDays(alloc.getPayDate(), (int) diffDays);
                alloc.setClearDate(clearDate);
            }
            alloc.getPlanAllocItemList().sort(Comparator.comparingInt(PlanAllocItem::getLayerOrder)
                    .thenComparing(PlanAllocItem::getRelNo)
                    .thenComparingInt(PlanAllocItem::getOrder));
            alloc.sortOrders();
        }

        // 去重
        distinctAlloc(allocs, planAcctTitleMap.values());

        //排序投资结构排序
        allocs.sort(Comparator.comparing(PlanAlloc::getClearDate));

        planAllocRepo.insertPlanAllocs(allocs);
        return allocs;
    }

    /**
     * 去重
     * 1、相同结算日的分配计划去重
     * 2、起息日等于到当期结算日的分配事项删除
     *
     * @param allocs
     * @param titleList
     */
    private void distinctAlloc(final List<PlanAlloc> allocs, Collection<PlanAcctTitle> titleList) {
        Map<Date, List<PlanAlloc>> allocListMap = allocs.stream().collect(Collectors.groupingBy(PlanAlloc::getClearDate));
        allocListMap.forEach((key, planAllocList) -> {
            if ((planAllocList.size() > 1)) {
                Map<String, PlanAlloc> allocMap = Maps.newHashMap();
                for (PlanAlloc planAlloc : planAllocList) {
                    PlanAlloc _alloc = allocMap.get(DateTimeUtil.formatYMD(planAlloc.getClearDate()));
                    if (_alloc == null) {
                        allocMap.put(DateTimeUtil.formatYMD(planAlloc.getClearDate()), planAlloc);
                    } else {
                        for (PlanAllocItem planAllocItem : planAlloc.getPlanAllocItemList()) {
                            List<PlanAllocItem> collect = _alloc.getPlanAllocItemList().stream().filter(item -> item.getTitleNo().equals(planAllocItem.getTitleNo())).collect(Collectors.toList());
                            if (CollectionUtils.isEmpty(collect)) {
                                _alloc.getPlanAllocItemList().add(planAllocItem);
                            }
                        }
                        allocs.remove(planAlloc);
                    }
                }
            }
        });

        Map<String, PlanAcctTitle> titleMap = titleList.stream().collect(Collectors.toMap(PlanAcctTitle::getTitleNo, value -> value, (k1, k2) -> k1));
        allocs.forEach(planAlloc -> {
            List<PlanAllocItem> _planAllocItemList = planAlloc.getPlanAllocItemList().stream().filter(item -> {
                PlanAcctTitle planAcctTitle = titleMap.get(item.getTitleNo());
                // 排除 结算日小于等于起息日的科目
                return planAcctTitle.getEffectDate().compareTo(planAlloc.getClearDate())<=0;
            }).collect(Collectors.toList());
            planAlloc.getPlanAllocItemList().clear();
            if (CollectionUtils.isNotEmpty(_planAllocItemList)) {
                planAlloc.getPlanAllocItemList().addAll(_planAllocItemList);
            }
        });
    }

    /**
     * 过滤失效分配计划
     *
     * @param curDate
     * @param allocs
     */
    private void filterInvalidAlloc(Date curDate, List<PlanAlloc> allocs) {
        // 调整已经过期的分配计划
        Map<String, PlanAllocItem> combineItems = Maps.newHashMap();
        Iterator<PlanAlloc> iterator = allocs.iterator();
        PlanAlloc validPlanAlloc = null;
        while (iterator.hasNext()) {
            validPlanAlloc = iterator.next();

            // 如果约定是生效日兑付的，那就不管了，延迟就延迟了吧，运营人员立马发起兑付就行了
            if (validPlanAlloc.getTimeType() == FeeTimeTypeEnum.PETD) {
                continue;
            }

            validPlanAlloc.getPlanAllocItemList().forEach(item -> combineItems.put(item.getTitleNo(), item));

            // 当前日期大于分配计划的结算日，分配计划已经没有意义了
            if (DateTimeUtil.calcDayDiff(curDate, validPlanAlloc.getClearDate()) > 0) {
                iterator.remove();
            } else {
                break;
            }
        }

        if (null != validPlanAlloc) {
            validPlanAlloc.setPlanAllocItemList(Lists.newArrayList(combineItems.values()));
        }
    }

    /**
     * 追加投资结构->更新分配计划
     *
     * @param planBase
     * @param existPlanAlloc
     * @param accountDate 转让会计日
     */
    private void appendPlanAlloc(PlanBase planBase, List<PlanAlloc> existPlanAlloc, Date accountDate) {
        VarChecker.checkNotNull(planBase, "[{0}]找不到对应的计划", planBase.getPlanNo());
        //计划到期日
        Date endDate = planBase.getEndDate();
        Date firstClearDate = planBase.getFirstClearDate();
        Date firstPayDate = planBase.getFirstPayDate();
        PlanPayFreqEnum payFreq = planBase.getPayFreq();

        Map<String, PlanAcctTitle> planAcctTitleMap = getStringPlanAcctTitleMap(planBase.getPlanNo());

        //根据时间周期计算分配计划
        List<PlanAlloc> allocs = this.calcPlanAlloc(planBase.getPlanNo(), firstClearDate, firstPayDate, endDate,
                payFreq);
        VarChecker
                .checkNotEmpty(allocs, "[{0}]按时间周期计算分配计划异常,清算日{1}-支付日{2}-结束日{3}", planBase.getPlanNo(), firstClearDate,
                        firstPayDate,
                        endDate);
        //投资结构
        List<PlanAlloc> newAddInvestAlloc = calcInvestStructure(planBase, planAcctTitleMap, allocs, true);
        //合并分配计划
        margeFeeInvestAlloc(allocs, Lists.newArrayList(), newAddInvestAlloc);
        // 过滤没有分配方案的分配计划
        allocs = allocs.stream().filter(
                planAlloc -> CollectionUtils.isNotEmpty(planAlloc.getPlanAllocItemList())).collect(
                Collectors.toList());

        existPlanAlloc.sort(Comparator.comparing(PlanAlloc::getClearDate));
        allocs.sort(Comparator.comparing(PlanAlloc::getClearDate));

        // 过滤失效的分配计划
        filterInvalidAlloc(accountDate, allocs);
        // 调整结算日和兑付日，保证相同的天数差
        long diffDays = DateTimeUtil.calcDayDiff(firstClearDate, firstPayDate);
        for (PlanAlloc alloc : allocs) {
            //设置计划类型
            alloc.setPlanType(planBase.getPlanType());
            if (alloc.getTimeType() != FeeTimeTypeEnum.PETD) {
                Date clearDate = DateTimeUtil.addDays(alloc.getPayDate(), (int) diffDays);
                alloc.setClearDate(clearDate);
            }
            alloc.getPlanAllocItemList().sort(Comparator.comparingInt(PlanAllocItem::getLayerOrder)
                    .thenComparing(PlanAllocItem::getRelNo)
                    .thenComparingInt(PlanAllocItem::getOrder));

        }
        // 去重
        distinctAlloc(allocs, planAcctTitleMap.values());
        // 追加
        appendInvestAlloc(existPlanAlloc, allocs);
    }

    /**
     * 追加
     *
     * @param existPlanAlloc
     * @param allocs
     */
    private void appendInvestAlloc(List<PlanAlloc> existPlanAlloc, List<PlanAlloc> allocs) {
        Map<String, PlanAlloc> allocMap = Maps.newHashMap();
        Date clearDay = existPlanAlloc.get(existPlanAlloc.size() - 1).getClearDate();
        // 转map
        existPlanAlloc.forEach(item -> {
            PlanAlloc _alloc = allocMap.get(DateTimeUtil.formatYMD(item.getClearDate()));
            if (_alloc == null) {
                allocMap.put(DateTimeUtil.formatYMD(item.getClearDate()), item);
            } else {
                _alloc.getPlanAllocItemList().addAll(item.getPlanAllocItemList());
            }
        });

        // 是否存在对应的数据库分配计划
        for (int i = 0; i < allocs.size(); i++) {
            PlanAlloc item = allocs.get(i);
            String key = DateTimeUtil.formatYMD(item.getClearDate());
            PlanAlloc _alloc = allocMap.get(key);
            if (_alloc != null) {
                _alloc.getPlanAllocItemList().addAll(item.getPlanAllocItemList());
                // 设置序号
                _alloc.sortOrders();
                planAllocRepo.addOrUpdate(_alloc, true);
            } else {
                // 如果当期结算日大于数据库分配计划最后一期结算日则直接插入 或是追加资金的最后一期 则直接插入
                if (DateTimeUtil.isBeforeOnDate(clearDay, item.getClearDate()) || i == allocs.size() - 1) {
                    // 设置序号
                    item.sortOrders();
                    planAllocRepo.addOrUpdate(item, false);
                }
            }

        }

    }

    /**
     * 合并分配计划
     *
     * @param allocs
     * @param newAddFeeAlloc
     * @param newAddInvestAlloc
     * @return
     */
    private void margeFeeInvestAlloc(List<PlanAlloc> allocs, List<PlanAlloc> newAddFeeAlloc,
                                     List<PlanAlloc> newAddInvestAlloc) {
        Map<String, PlanAlloc> allocMap = Maps.newHashMap();
        List<PlanAlloc> _allocs = Lists.newArrayList();
        _allocs.addAll(allocs);
        _allocs.addAll(newAddFeeAlloc);
        _allocs.addAll(newAddInvestAlloc);
        for (PlanAlloc planAlloc : _allocs) {
            PlanAlloc _alloc = allocMap.get(DateTimeUtil.formatYMD(planAlloc.getClearDate()));
            if (_alloc == null) {
                allocMap.put(DateTimeUtil.formatYMD(planAlloc.getClearDate()), planAlloc);
            } else {
                _alloc.getPlanAllocItemList().addAll(planAlloc.getPlanAllocItemList());
            }
        }
        allocs.clear();
        if (MapUtils.isNotEmpty(allocMap)) {
            allocs.addAll(allocMap.values());
        }
    }

    private Map<String, PlanAcctTitle> getStringPlanAcctTitleMap(String planNo) {
        //获取所有科目
        List<PlanAcctTitle> planAcctTitles = planAcctTitleRepo.getByEntityNo(planNo);
        VarChecker.checkNotNull(planAcctTitles, "[{0}]找不到对应的科目信息", planNo);
        Map<String, PlanAcctTitle> planAcctTitleMap = Maps.newHashMap();
        planAcctTitles.forEach(planAcctTitle -> {
            planAcctTitleMap.put(planAcctTitle.getRefNo() + planAcctTitle.getTitleType().getCode(), planAcctTitle);
        });
        return planAcctTitleMap;
    }

    /**
     * 计算费用结构
     *
     * @param planBase
     * @param effectDate
     * @param firstClearDate
     * @param planAcctTitleMap
     * @param allocs
     */
    private List<PlanAlloc> calcFeeStructure(PlanBase planBase, Date effectDate, Date firstClearDate,
                                             Map<String, PlanAcctTitle> planAcctTitleMap, List<PlanAlloc> allocs) {

        List<PlanAlloc> newAddFeeAlloc = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(planBase.getFeeStructureList())) {
            Map<String, PlanAlloc> allocMap = Maps.newHashMap();
            allocs.forEach(planAlloc -> {
                allocMap.put(DateTimeUtil.formatYMD(planAlloc.getClearDate()), planAlloc);
            });
            String effectDateStr = DateTimeUtil.formatYMD(effectDate);
            for (FeeStructure feeStructure : planBase.getFeeStructureList()) {
                //计划起息日
                if (FeeTimeTypeEnum.PETD == feeStructure.getFeeTimeType()) {
                    PlanAlloc planAlloc = allocMap.get(effectDateStr);
                    if (planAlloc == null) {
                        planAlloc = calcPlanAlloc(planBase.getPlanNo(), effectDate, effectDate, effectDate,
                                FeeTimeTypeEnum.PETD);
                        newAddFeeAlloc.add(planAlloc);
                        allocMap.put(effectDateStr, planAlloc);
                    }
                    putPlanAllocItem(planBase, planAcctTitleMap, feeStructure.getFeeNo(), planAlloc,
                            TitleTypeEnum.FEE, 0);
                } else if (FeeTimeTypeEnum.PEDD == feeStructure.getFeeTimeType()) {
                    PlanAlloc planAlloc = allocs.get(allocs.size() - 1);
                    putPlanAllocItem(planBase, planAcctTitleMap, feeStructure.getFeeNo(), planAlloc,
                            TitleTypeEnum.FEE, 0);
                } else if (FeeTimeTypeEnum.PFCD == feeStructure.getFeeTimeType()) {
                    PlanAlloc planAlloc = allocMap.get(DateTimeUtil.formatYMD(firstClearDate));
                    putPlanAllocItem(planBase, planAcctTitleMap, feeStructure.getFeeNo(), planAlloc,
                            TitleTypeEnum.FEE, 0);
                } else if (FeeTimeTypeEnum.TERM == feeStructure.getFeeTimeType()) {
                    allocs.forEach(planAlloc -> {
                        putPlanAllocItem(planBase, planAcctTitleMap, feeStructure.getFeeNo(), planAlloc,
                                TitleTypeEnum.FEE, 0);
                    });
                } else {
                    VarChecker.checkArgument(false, "[{0}]-[{1}]不支持的费用结构", planBase.getPlanNo(),
                            feeStructure.getFeeNo());
                }
            }
        }
        return newAddFeeAlloc;
    }

    /**
     * 计算投资结构
     *
     * @param planBase
     * @param planAcctTitleMap
     * @param allocs
     * @param append           是否只计算追加资金结构
     */
    private List<PlanAlloc> calcInvestStructure(PlanBase planBase,
                                                Map<String, PlanAcctTitle> planAcctTitleMap,
                                                List<PlanAlloc> allocs, Boolean append) {
        List<PlanAlloc> newAddAlloc = Lists.newArrayList();

        if (CollectionUtils.isNotEmpty(planBase.getInvestStructureList())) {
            //停止转让日期
            Date stopTrPayDate = getStopTrPayDate(planBase);
            for (InvestStructure investStructure : planBase.getInvestStructureList()) {
                // 过滤掉非追加的资金结构
                if (append == true && investStructure.getAppend() == false) {
                    continue;
                }
                //本金科目
                TitleTypeEnum prinTitleType = investStructure.getLayerType() == LayerTypeEnu.PRIOR
                        ? TitleTypeEnum.PRIOR_INVESTED
                        : TitleTypeEnum.SECND_INVESTED;
                for (PlanAlloc planAlloc : allocs) {
                    // 判断结算日是否大于投资结构起息日
                    if (DateTimeUtil.isBeforeOnDate(planAlloc.getClearDate(), investStructure.getEffectDate())) {
                        continue;
                    }
                    if (DateTimeUtil.between(investStructure.getLayerEndDate(), planAlloc.getClearDate(),
                            planAlloc.getPayDate(), BetweenType.se)) {
                        // 结算日和支付日不会差太久，变更支付日为该层级的兑付日
                        // 本金科目
                        putInvestPlanAllocItem(planBase, planAcctTitleMap, investStructure, planAlloc, prinTitleType);
                        planAlloc.setPayDate(investStructure.getLayerEndDate());
                        // 后面的分配计划都已经超过层级的兑付日了，直接退出
                        break;
                    } else if (DateTimeUtil.isBeforeOnDate(investStructure.getLayerEndDate(),
                            planAlloc.getClearDate())) {
                        // 不在结算日和兑付日的区间范围，说明在两个分配计划之间
                        // 找到第一个结算日大于层级兑付日的分配计划，在它之前插入新的分配计划
                        // 插入新分配计划
                        PlanAlloc _planAlloc = this.calcPlanAlloc(planBase.getPlanNo(),
                                investStructure.getLayerEndDate(),
                                investStructure.getLayerEndDate(), investStructure.getLayerEndDate(),
                                FeeTimeTypeEnum.PEDD);
                        // 本金科目
                        putInvestPlanAllocItem(planBase, planAcctTitleMap, investStructure, _planAlloc,
                                prinTitleType);
                        newAddAlloc.add(_planAlloc);
                        // 后面的分配计划都已经超过层级的兑付日了，直接退出
                        break;
                    } else if (DateTimeUtil.isBeforeOnDate(stopTrPayDate, planAlloc.getClearDate())) {
                        putInvestPlanAllocItem(planBase, planAcctTitleMap, investStructure, planAlloc, prinTitleType);
                        continue;
                    }
                    //利息科目 - 层级兑付日之前，默认每期都插入收益的分配
                    putPlanAllocItem(planBase, planAcctTitleMap, investStructure.getLayerNo(), planAlloc,
                            TitleTypeEnum.INVEST_PROFIT, investStructure.getLayerType().getOrder());

                }
            }
        }
        return newAddAlloc;
    }

    /**
     * 投资人收益 事项
     *
     * @param planBase
     * @param planAcctTitleMap
     * @param investStructure
     * @param planAlloc
     * @param titleTypeEnum
     */
    private void putInvestPlanAllocItem(PlanBase planBase, Map<String, PlanAcctTitle> planAcctTitleMap,
                                        InvestStructure investStructure,
                                        PlanAlloc planAlloc, TitleTypeEnum titleTypeEnum) {
        putPlanAllocItem(planBase, planAcctTitleMap, investStructure.getLayerNo(), planAlloc,
                titleTypeEnum, investStructure.getLayerType().getOrder());

        if (investStructure.getIntrType() == IntrTypeEnum.FIXED
                || investStructure.getIntrType() == IntrTypeEnum.NOTICKET) {
            putPlanAllocItem(planBase, planAcctTitleMap, investStructure.getLayerNo(), planAlloc,
                    TitleTypeEnum.INVEST_PROFIT, investStructure.getLayerType().getOrder());
        }

    }

    /**
     * 对应科目 事项
     *
     * @param planBase
     * @param planAcctTitleMap
     * @param relNo
     * @param planAlloc
     * @param titleTypeEnum
     */
    private void putPlanAllocItem(PlanBase planBase, Map<String, PlanAcctTitle> planAcctTitleMap,
                                  String relNo,
                                  PlanAlloc planAlloc, TitleTypeEnum titleTypeEnum, int layerOrder) {
        PlanAllocItem allocItem = buildPlanAllocItem(planBase, planAcctTitleMap, relNo, titleTypeEnum, layerOrder);
        planAlloc.addPlanAllocItem(allocItem);
    }

    private PlanAllocItem buildPlanAllocItem(PlanBase planBase, Map<String, PlanAcctTitle> planAcctTitleMap,
                                             String relNo,
                                             TitleTypeEnum titleTypeEnum, int layerOrder) {
        PlanAcctTitle planAcctTitle = planAcctTitleMap.get(relNo + titleTypeEnum.getCode());
        VarChecker.checkNotNull(planAcctTitle, "[{0}]-[{2}]-[{1}]没有找到对应科目", planBase.getPlanNo(), relNo,
                titleTypeEnum.getCode());
        PlanAllocItem allocItem = new PlanAllocItem();
        allocItem.setItemNo(SequenceUtil.genId(IdTypeEnum.ALLOCITM));
        allocItem.setTax(taxTitleTypes.contains(titleTypeEnum));
        allocItem.setTitleNo(planAcctTitle.getTitleNo());
        allocItem.setTitleName(planAcctTitle.getTitleName());
        allocItem.setRelNo(relNo);
        allocItem.setLayerOrder(layerOrder);
        allocItem.setOrder(titleTypeEnum.getOrder());
        return allocItem;
    }

    /**
     * 计算分配计划
     *
     * @param firstClearDate
     * @param firstPayDate
     * @param endDate
     * @param payFreq
     * @return
     */
    private List<PlanAlloc> calcPlanAlloc(String planNo, Date firstClearDate, Date firstPayDate, Date endDate,
                                          PlanPayFreqEnum payFreq) {
        Date _clearDate = firstClearDate;
        Date _payDate = firstPayDate;
        List<PlanAlloc> result = Lists.newArrayList();
        PlanAlloc lastPlanAlloc = null;

        long dayDiff = DateTimeUtil.calcDayDiff(firstPayDate, firstClearDate);
        int cnt = 1;
        while (DateTimeUtil.calcDayDiff(_clearDate, endDate) <= 0) {
            PlanAlloc planAlloc = this.calcPlanAlloc(planNo, _clearDate, _payDate, endDate, FeeTimeTypeEnum.TERM);
            result.add(planAlloc);
            lastPlanAlloc = planAlloc;
            _clearDate = DateTimeUtil.addMonths(firstClearDate, payFreq.getCnt() * cnt);
            _payDate = DateTimeUtil.addDays(_clearDate, (int) dayDiff);
            cnt++;
        }
        if (lastPlanAlloc != null && DateTimeUtil.isBeforeOnDate(lastPlanAlloc.getPayDate(), endDate)) {
            //最后一次兑付日与计划结束日差小于15天，则修改最后一次分配时间
            if (DateTimeUtil.calcDayDiff(endDate, lastPlanAlloc.getPayDate()) <= 15) {
                lastPlanAlloc.setPayDate(endDate);
                lastPlanAlloc.setClearDate(DateTimeUtil.addDays(lastPlanAlloc.getPayDate(), -1 * (int) dayDiff));
            } else {
                result.add(this.calcPlanAlloc(planNo, endDate, endDate, endDate, FeeTimeTypeEnum.TERM));
            }
        } else if (lastPlanAlloc == null) {
            result.add(this.calcPlanAlloc(planNo, endDate, endDate, endDate, FeeTimeTypeEnum.TERM));
        }
        return result;
    }

    private PlanAlloc calcPlanAlloc(String planNo, Date clearDate, Date payDate, Date endDate,
                                    FeeTimeTypeEnum timeType) {
        PlanAlloc planAlloc = new PlanAlloc();
        planAlloc.setPlanNo(planNo);
        planAlloc.setAllocNo(SequenceUtil.genId(IdTypeEnum.ALLOCPRG));
        planAlloc.setClearDate(clearDate);
        planAlloc.setPayDate(DateTimeUtil.getMin(payDate, endDate));
        planAlloc.setStatus(PlanAllocStatusEnum.UNASSIGNE);
        //默认批量新增的时候为未提交状态
        planAlloc.setApproveStatus(ApproveStatusEnum.UNSUBMIT);
        planAlloc.setTimeType(timeType);
        return planAlloc;
    }

    /**
     * 保存修改分配计划
     *
     * @param param
     * @return
     */
    public PlanAlloc addOrUpdate(PlanBase planEntity, PlanAllocEditParam param, String userCode, String orgCode,
                                 boolean submit) {
        boolean upd = false;
        PlanAlloc alloc = null;
        if (StringUtils.isBlank(param.getAllocNo())) {
            alloc = new PlanAlloc();
            //入参注入分配编号后续设置模型注入alloc
            param.setAllocNo(SequenceUtil.genId(IdTypeEnum.ALLOCPRG));
            alloc.setApproveStatus(ApproveStatusEnum.UNSUBMIT);
            alloc.setPlanType(planEntity.getPlanType());
        } else {
            upd = true;
            // 查询最新分配计划
            alloc = planAllocRepo.getPlanAllocDetail(param.getAllocNo());
            VarChecker.checkEquals(PlanAllocStatusEnum.UNASSIGNE, alloc.getStatus(),
                    "[{0}]-[{1}]已分配", alloc.getPlanNo(), alloc.getAllocNo());
            // 验证审批状态
            ApproveChangeUtil
                    .bizModification(alloc.getApproveStatus(), alloc.getAllocNo(), "分配计划");
        }
        // 排除修改的分配计划编号 查询当前相同结算日的数量
        VarChecker.checkArgument(
                planAllocRepo.querySameClearDate(param.getClearDate(), planEntity.getPlanNo(), param.getAllocNo()) == 0,
                "不能添加相同结算日期的分配计划");

        // 查询对应分配科目信息
        List<PlanAcctTitle> planAcctTitles = planAcctTitleRepo.getByEntityNo(planEntity.getPlanNo());
        // 设置模型模型
        PlanAllocConvert.convertParam2Domain(alloc, param, planAcctTitles);
        if (submit) {
            // 提交状态为审批中
            alloc.setApproveStatus(ApproveStatusEnum.PENDING);
            // 调用数据变更 提交审批
            DataChangeBizTypeEnum dataChangeBizType =
                    planEntity.getPlanType() == PlanTypeEnum.SPV ?
                            DataChangeBizTypeEnum.PlanAllocItemEntity : DataChangeBizTypeEnum.PlanAllocItem;
            changeManager.addApprove(dataChangeBizType, alloc.getAllocNo(),
                    planEntity.getPlanName() + "：" + planEntity.getPlanNo(), alloc,
                    userCode, orgCode);
        }
        // 先审批 后 保存数据
        planAllocRepo.addOrUpdate(alloc, upd);
        return alloc;
    }

    public Boolean delete(String allocNo) {
        VarChecker.checkNotEmpty(allocNo, "分配方案编号不能为空");
        int cnt = planAllocRepo.delete(allocNo);
        return cnt > 0;
    }

    /**
     * 更新为已分配计划
     *
     * @param param
     * @return
     */
    public Boolean planAllocDetailAssigned(PlanAllocAssignedOprParam param) {
        return planAllocRepo.planAllocDetailAssigned(param.getAllocNo(), PlanAllocStatusEnum.ASSIGNED);
    }

    /**
     * 查询计划分配支付日期
     *
     * @return
     */
    public ProdPayItem queryRecentlyPlans(PlanTypeEnum planType) {
        ProdPayItem payItem = new ProdPayItem();
        PlanAlloc alloc = planAllocRepo.queryRecentlyPlans(planType);
        if (alloc == null) {
            return null;
        }
        payItem.setPayDay(alloc.getPayDate());
        payItem.setPlanNo(alloc.getPlanNo());
        PlanBase planBase = planRepo.getPlanByNo(alloc.getPlanNo());
        if (planBase != null) {
            payItem.setPlanName(planBase.getPlanName());
        }
        return payItem;
    }

    /**
     * 审批完成后回调
     *
     * @param approveStatus
     * @param bizDataChange
     */
    @Override
    public void approveCallBack(BizDataChange<PlanAlloc> bizDataChange, ApproveStatusEnum approveStatus) {
        //审批通过后 创建
        PlanAlloc alloc = bizDataChange.getDatas();
        alloc.setApproveStatus(approveStatus);
        planAllocRepo.addOrUpdate(alloc, true);
    }

    @Override
    public DataChangeBizTypeEnum getChangeType() {
        return DataChangeBizTypeEnum.PlanAllocItem;
    }

    @Override
    public List<DataChangeBizTypeEnum> getChangeTypes() {
        return Lists.newArrayList(DataChangeBizTypeEnum.PlanAllocItem, DataChangeBizTypeEnum.PlanAllocItemEntity);
    }

    /**
     * 变更分配计划状态
     *
     * @param alloc
     */
    public void updateAllocStatus(PlanAlloc alloc) {
        planAllocRepo.updateAllocStatus(alloc);
    }

    /**
     * 计划终止转让日期修改时分配计划调整本金收益
     *
     * @param planBase
     */
    public void modifyPlanAlloc(PlanBase planBase) {
        List<PlanAlloc> planAllocs = planAllocRepo.queryByPlanNo(planBase.getPlanNo());

        if (CollectionUtils.isNotEmpty(planAllocs)) {
            Map<String, PlanAcctTitle> planAcctTitleMap = getStringPlanAcctTitleMap(planBase.getPlanNo());
            //停止转让日期
            Date stopTrPayDate = getStopTrPayDate(planBase);
            for (PlanAlloc planAlloc : planAllocs) {
                //未分配的计划
                if (planAlloc.getStatus() == PlanAllocStatusEnum.UNASSIGNE) {

                    Map<String, PlanAllocItem> titleItemMap = planAlloc.getPlanAllocItemList().stream().collect(
                            Collectors.toMap(PlanAllocItem::getTitleNo, item -> {
                                item.setOrder(item.getOrder() * 10);
                                return item;
                            }));

                    for (InvestStructure investStructure : planBase.getInvestStructureList()) {
                        //本金科目类型
                        TitleTypeEnum prinTitleType = investStructure.getLayerType() == LayerTypeEnu.PRIOR
                                ? TitleTypeEnum.PRIOR_INVESTED
                                : TitleTypeEnum.SECND_INVESTED;
                        //获取本金科目
                        PlanAcctTitle planPrinAcctTitle = planAcctTitleMap.get(
                                investStructure.getLayerNo() + prinTitleType.getCode());
                        // 获取层级结构收益
                        PlanAcctTitle planInvestAcctTitle = planAcctTitleMap.get(
                                investStructure.getLayerNo() + TitleTypeEnum.INVEST_PROFIT.getCode());

                        //投资结构到期日在分配清算日之后 需要处理，
                        if (!DateTimeUtil.isBeforeOnDate(investStructure.getLayerEndDate(), planAlloc.getClearDate())) {
                            PlanAllocItem planPrinAllocItem = titleItemMap.get(planPrinAcctTitle.getTitleNo());
                            PlanAllocItem planInvestAllocItem = titleItemMap.get(planInvestAcctTitle.getTitleNo());

                            ////停止转让日 在 分配清算日之后,不分配本金
                            if (!DateTimeUtil.isBeforeOnDate(stopTrPayDate, planAlloc.getClearDate())) {
                                if (planPrinAllocItem != null) {
                                    planAlloc.getPlanAllocItemList().remove(planPrinAllocItem);
                                    // planAllocRepo.deleteAllocItem(planPrinAllocItem.getItemNo());
                                }
                            } else {
                                // 停止转让日 在 分配清算日之前 or 同一天,分配本金
                                if (planPrinAllocItem == null) {
                                    PlanAllocItem newPlanAllocItem = buildPlanAllocItem(planBase, planAcctTitleMap,
                                            investStructure.getLayerNo(),
                                            prinTitleType, 0);
                                    newPlanAllocItem.setOrder(planInvestAllocItem.getOrder() + 1);
                                    planAlloc.getPlanAllocItemList().add(newPlanAllocItem);
                                }
                            }
                        }
                    }
                    planAllocRepo.delete(planAlloc.getAllocNo());
                    planAllocRepo.insertPlanAllocs(Lists.newArrayList(planAlloc));
                }
            }
        }

    }

    /**
     * 对分配计划加锁
     *
     * @param allocNo
     * @return
     */
    public PlanAlloc lockByAllocNo(String allocNo) {
        return planAllocRepo.lockByAllocNo(allocNo);
    }

    /**
     * 后去停止放款日或停止转让日
     *
     * @param planBase
     * @return
     */
    private Date getStopTrPayDate(PlanBase planBase) {
        if (planBase.getPlanType() == PlanTypeEnum.SPV && planBase instanceof OrganizationEntity) {
            return ((OrganizationEntity) planBase).getStopPayDate();
        } else {
            return ((Plan) planBase).getPlanTransferRule().getStopTrDate();
        }
    }
}
