/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.core.service.prod;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.ProductRiskClDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.ProductListPage;
import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.enums.HandleTypeEnum;
import com.abssqr.plat.common.facade.model.BizDataChange;
import com.abssqr.plat.common.facade.model.prod.LoanProduct;
import com.abssqr.plat.common.facade.model.prod.ProductRiskCl;
import com.abssqr.plat.common.facade.param.prod.ProdEditParam;
import com.abssqr.plat.common.facade.param.prod.ProdPageQryParam;
import com.abssqr.plat.common.model.convertor.ProdRiskClConvert;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.org.OrgRepository;
import com.abssqr.plat.common.model.repo.prod.LoanProdRepository;
import com.abssqr.plat.common.model.repo.prod.LoanProdRiskClRepository;
import com.abssqr.plat.common.model.utils.ApproveChangeUtil;
import com.abssqr.plat.common.model.convertor.ProdParamConvert;
import com.abssqr.plat.core.service.change.ApproveCallBack;
import com.abssqr.plat.core.service.change.ChangeManager;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Sets;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 产品准入规则
 * @author yangcheng
 * @version ProdManager.java, v0.12019-06-2214:48 yangchengExp$
 */
@Service
public class ProdManager implements ApproveCallBack<LoanProduct> {
    @Autowired
    private LoanProdRepository       loanProdRepository;
    @Autowired
    private ChangeManager            changeManager;
    @Autowired
    private LoanProdRiskClRepository loanProdRiskClRepository;
    @Autowired
    private OrgRepository            orgRepository;
    
    /**
     * 查询产品分页
     *
     * @param param
     * @return
     */
    public ProductListPage getProdList(ProdPageQryParam param, String userCode) {
        ProductListPage page = new ProductListPage();
        // 如果是查询接口则加入身份作为条件
        if (HandleTypeEnum.LIST.getCode().equals(param.getHandleType())){
            page.setUserCode(userCode);
        }
        // 查询审批通过
        page.setApproveStatus(ApproveStatusEnum.PASSED.getCode());
        page.setProdName(param.getProdName());
        page.setCurrPageNo(param.getPageNum());
        page.setLimit(param.getPageSize());
        return loanProdRepository.getProdList(page);
    }
    
    /**
     * 获取产品准入规则
     * @param productNo
     * @return
     */
    public LoanProduct getProductBy(String productNo) {
        return loanProdRepository.getBybssqrProdCode(productNo);
    }
    /**
     * 根据 changeNo 查询数据
     * @param changeNo
     * @return
     */
    public LoanProduct getDataChangeProductBy(String changeNo) {
        BizDataChange<LoanProduct> dataChange = changeManager.getDataChange(changeNo);
        if (dataChange == null) {
            throw new AbssqrBizException("未找到对应待审批的产品准入规则数据");
        }
        dataChange.getDatas().setApproveStatus(ApproveStatusEnum.getEnum(dataChange.getApproveStatus()));
        return dataChange.getDatas();
    }
    
    /**
     * 新增Or删除产品
     * 修改了abssqr_prod_code索引需要注意
     *
     * @param param
     * @return 返回内部产品编码
     */
    public String addOrUpdate(ProdEditParam param, String userCode, String orgCode) {
        LoanProduct loanProduct = null;
        if (StringUtils.isBlank(param.getAbssqrProdCode())) {
            // 组装产品参数
            loanProduct = ProdParamConvert.convert2Prod(param);
            int cnt = loanProdRepository.getSameProdCnt(loanProduct);
            VarChecker.checkEquals(cnt, 0, "产品名称：[{0}]重复或该机构存在相同产品编码[{1}]",param.getProdName(),param.getProdCode());
            //审批状态默认为审批中
            loanProduct.setApproveStatus(ApproveStatusEnum.PENDING);
            //添加创建人
            loanProduct.setGmtCreator(userCode);
            loanProdRepository.addProduct(loanProduct);
        }else{
            // 通过产品编码查询
            loanProduct = loanProdRepository.getProductByAbsProductNo(param.getAbssqrProdCode());
            VarChecker.checkNotNull(loanProduct, "未找到对应产品信息，编码：{0}", param.getAbssqrProdCode());
            //产品如果是撤销状态下允许修改  AND添加修改人
            ApproveChangeUtil.bizModification(loanProduct.getApproveStatus(), loanProduct.getProdCode(),loanProduct.getProdName());
            
            loanProduct.setProdName(param.getProdName().trim());
            loanProduct.setManagerOrgCode(param.getManagerOrgCode().trim());
            loanProduct.setProdCode(param.getProdCode().trim());
    
            int cnt = loanProdRepository.getSameProdCnt(loanProduct);
            VarChecker.checkEquals(cnt, 0, "产品名称：[{0}]重复或该机构存在相同产品编码[{1}]", param.getProdName(),param.getProdCode());
            
            loanProduct.setApproveStatus(ApproveStatusEnum.PENDING);
            loanProduct.setProductRiskCls(ProdParamConvert.convertTOProd(param.getProductRiskClList()));
            loanProduct.setGmtModifier(userCode);
            loanProdRepository.update(loanProduct);
        }
        // 处理五级分类
        handleProdRiskCl(loanProduct.getAbssqrProdCode(), loanProduct.getProductRiskCls());
        loanProduct.setManagerOrgName(getOrgManagerNames(loanProduct.getManagerOrgCode()));
        // 产品准入审批流启动
        BizDataChange dataChange = changeManager.addApprove(DataChangeBizTypeEnum.PROD,
                loanProduct.getAbssqrProdCode(), loanProduct.getProdName(), loanProduct, userCode, orgCode);
        return dataChange.getChangeNo();
    }
    
    
    //添加机构
    private String getOrgManagerNames(String managerOrgCode) {
        Set<String> orgCodes = Sets.newHashSet();
        orgCodes.add(managerOrgCode);
        Map<String, String> orgNames = orgRepository.getOrgNames(orgCodes);
        String managerOrgName=orgNames.get(managerOrgCode);
        return managerOrgName;
    }
    
    /**
     * 处理五级分类
     * @param abssqrProdCode
     * @param models
     */
    private void handleProdRiskCl(String abssqrProdCode, List<ProductRiskCl> models){
        //处理五级分类
        loanProdRiskClRepository.deleteProductRiskClByAbssqrProdCode(abssqrProdCode);
        List<ProductRiskClDO> productRiskClDOList = ProdRiskClConvert.convertDOs(models,
                abssqrProdCode);
        loanProdRiskClRepository.insertBatch(productRiskClDOList);
    }
    
    /**
     * 审批完成后回调
     *
     * @param approveStatus
     * @param bizDataChange
     */
    @Override
    public void approveCallBack(BizDataChange<LoanProduct> bizDataChange, ApproveStatusEnum approveStatus) {
        //回调更新五级分类
        LoanProduct loanProduct = bizDataChange.getDatas();
        loanProduct.setApproveStatus(approveStatus);
        //更新产品
        loanProdRepository.update(loanProduct);
        //更新产品五级分类
        handleProdRiskCl(loanProduct.getAbssqrProdCode(), loanProduct.getProductRiskCls());
    }
    
    @Override
    public DataChangeBizTypeEnum getChangeType() {
        return DataChangeBizTypeEnum.PROD;
    }
}
