/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.report;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ArrayUtil;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanAstReportStat;
import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.ReportAstStatTypeEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.convertor.ReportItemStatConvert;
import com.abssqr.plat.common.model.repo.ast.LoanBakRepo;
import com.general.system.tool.util.ToolUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 *
 * @author hanfei
 * @version $Id: LoanBakStatManager.java, v 0.1 2019-12-05 12:48 PM hanfei Exp $ 
 */
@Component
public class LoanBakStatManager {
    
    @Autowired
    private LoanBakRepo loanBakRepo;
    /**
     * 通过LOAN_BAK的统计替代statBal 减少report_entity中余额的记录值
     * @param accountDate
     * @param planBase
     * @return
     */
    public List<PlanAstReportStat> statLoanBakBal(Date accountDate, PlanBase planBase) {
        String planNo = planBase.getPlanNo();
        List<PlanAstReportStat> stats = new ArrayList<>();
        List<PlanAstReportStat> statusStat = loanBakRepo.statLoanBakStatus(accountDate, planNo);
        if (ToolUtil.isEmpty(statusStat)){
            return stats;
        }
        // 根据状态进行合计 总计 "ALL"
        PlanAstReportStat stat = ReportItemStatConvert.combine(ArrayUtil.toArray(statusStat, PlanAstReportStat.class));
        stat.setStatType(ReportAstStatTypeEnum.ALL.getCode());
        stat.setStatVal(AbssqrConstants.COMMON_VAL);
        stats.add(stat);
        // 状态统计 "STT"
        CollectionUtil.addAll(stats, statusStat);
        // 五级分类 "CLZ"
        CollectionUtil.addAll(stats, loanBakRepo.statLoanBakClz(accountDate, planNo));
        // 逾期天数 "OVD"
        CollectionUtil.addAll(stats, loanBakRepo.statLoanBakOvdDays(accountDate, planNo));
        // 91逾期未核销 "OVD_WTF"
        CollectionUtil.addAll(stats, loanBakRepo.statLoanBakOvdWtf(accountDate, planNo,
                AbssqrConstants.COMMON_VAL,ReportAstStatTypeEnum.OVD_WTF_STATUS.getCode(), YNEnum.N.getCode(), AbssqrConstants.WTF_OVD_DAYS));
        // 已核销 "WTF"
        CollectionUtil.addAll(stats, loanBakRepo.statLoanBakOvdWtf(accountDate, planNo,
                YNEnum.Y.getCode(), ReportAstStatTypeEnum.WTF_STATUS.getCode(), YNEnum.Y.getCode(), null));
        // 统计当日备份全量资产年化收益，及合计本金  'ALL' as STAT_TYPE, 'accrued' as STAT_VAL
        if (planBase.getPlanType() == PlanTypeEnum.SPV){
            // 放款池计算disbDate 进行统计
            CollectionUtil.addAll(stats, loanBakRepo.statSpvProfit(null, accountDate, planNo));
        }else{
            // 承接池计算inDate 进行统计
            CollectionUtil.addAll(stats, loanBakRepo.statPlanProfit(null, accountDate, planNo));
        }
        return stats;
    }
    
}
