/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.report.calc.impl;

import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.ReportAstStatTypeEnum;
import com.abssqr.plat.common.facade.model.report.ReportItem;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanLog;
import com.general.system.common.util.DateTimeUtil;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * 报表 到期应还统计
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.report.calc.impl: DueItemCalc.java, v 0.1 2019-06-27 00:59 zhenxuan.luo Exp $
 */
@Component
public class DueItemCalc extends BaseReportItemCalc {
    
    @Override
    protected ReportAstStatTypeEnum getType() {
        return ReportAstStatTypeEnum.DUE;
    }
    
    @Override
    public List<ReportItem> calcByInstal(Loan loan, LoanInstal loanInstal, Date accountDate) {
        return buildCommonItem(loanInstal.getStatus(),loanInstal.getAstPeriod().getEndDate(), accountDate);
    }
    
    @Override
    public List<ReportItem> calcByLog(LoanLog loanLog, LoanInstalLog log) {
        return buildCommonItem(loanLog.getAfStatus(),loanLog.getAfEndDate(), loanLog.getAccountDate());
    }
    
    private List<ReportItem> buildCommonItem(LoanStatusEnum status, Date endDate, Date accountDate) {
        List<ReportItem> reportItems = Lists.newArrayList();
        //还款计划分析会用到只统计当天资产状态正常应还统计
        if (status.isActive()
            && DateTimeUtil.calcDayDiff(accountDate, endDate) == 0) {
            ReportItem reportItem = new ReportItem();
            reportItem.setAstStatType(getType());
            reportItem.setAstStatVal(status.getCode());
            reportItems.add(reportItem);
        }
        return reportItems;
    }
}