/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.wk.ast.sync;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.abssqr.plat.common.dal.sync.auto.dao.WkAbsMfsBillCtrDAO;
import com.abssqr.plat.common.dal.sync.auto.dao.WkMfsDueBillDAO;
import com.abssqr.plat.common.dal.sync.auto.dao.WkMfsLnPactDAO;
import com.abssqr.plat.common.dal.sync.auto.dao.WkMfsOpenRepayDAO;
import com.abssqr.plat.common.dal.sync.auto.dao.WkMfsRepayInfoDAO;
import com.abssqr.plat.common.dal.sync.auto.dao.WkMfsRepayPlanDAO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsDueBillDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsLnPactDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsOpenRepayDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsRepayInfoDO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkMfsRepayPlanDO;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.wk.domain.sync.WkMfsSyncCtr;
import com.abssqr.plat.common.model.wk.repo.WkMfsSyncCtrRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.core.service.wk.ast.sync: WkMfsAstManager.java, v 0.1 2019-12-04 12:54 bangis.wangdf Exp $
 */
@Service
public class WkMfsAstManager {
    private final static Logger LOGGER = LoggerFactory.getLogger(WkMfsAstManager.class);
    @Autowired
    WkAbsMfsBillCtrDAO  wkAbsMfsBillCtrDAO;
    @Autowired
    WkMfsDueBillDAO     wkMfsDueBillDAO;
    @Autowired
    WkMfsLnPactDAO      wkMfsLnPactDAO;
    @Autowired
    WkMfsOpenRepayDAO   wkMfsOpenRepayDAO;
    @Autowired
    WkMfsRepayInfoDAO   wkMfsRepayInfoDAO;
    @Autowired
    WkMfsRepayPlanDAO   wkMfsRepayPlanDAO;
    @Autowired
    WkMfsSyncCtrRepo    wkMfsSyncCtrRepo;
    @Autowired
    AbsConfig           absConfig;
    @Autowired
    TransactionTemplate transactionTemplate;

    public void clearData(String orgCode) {

        boolean hasRemainTask = wkMfsSyncCtrRepo.isHasRemainTask(orgCode);
        VarChecker.checkArgument(!hasRemainTask,"存才为处理完成的账单，停止数据同步");

        wkAbsMfsBillCtrDAO.updateData(orgCode);
        //同步借据表
        wkMfsDueBillDAO.updateData();
        //同步合同
        wkMfsLnPactDAO.updateData();
        //同步放款
        wkMfsOpenRepayDAO.updateData();
        //同步还款
        wkMfsRepayInfoDAO.updateData();
        //同步还款计划
        wkMfsRepayPlanDAO.updateData();
    }

    public void genWkMfsCtl(AstSyncTaskCtr astSyncTaskCtr, Day curDay, Day absAcctDay) {
        String zlAcctDayStr = curDay.getDayString();
        String zlAcctYesterDayStr = curDay.getYesterday().getDayString();
        //同步控制表
        wkAbsMfsBillCtrDAO.insertSync2AbsMfsBillCtr(absAcctDay.getStandardDate(),
            astSyncTaskCtr.getOrgCode(), absConfig.getDbLink(), zlAcctDayStr,
            absConfig.getWkxwDbName(),zlAcctYesterDayStr, absConfig.getWkxwSynonymAft(),
            absConfig.getWkxwSynonymPre(), Lists.newArrayList(astSyncTaskCtr.getExtData().getSpvPlanNos()));
    }

    public Map<String, BigDecimal> calcProjRepayAmt(AstSyncTaskCtr astSyncTaskCtr, Day curDay) {

        Set<String> spvPlanNos = astSyncTaskCtr.getExtData().getSpvPlanNos();
        Map<String, BigDecimal> resultMap = Maps.newHashMap();
        if(CollectionUtils.isNotEmpty(spvPlanNos)){
            Map<String, BigDecimal> wkResultMap = wkMfsRepayInfoDAO.queryStatRepayInfoForOrg(
                astSyncTaskCtr.getOrgCode(), absConfig.getDbLink(), curDay.getDayString(), absConfig.getWkxwDbName(),
                absConfig.getWkxwSynonymAft(),
                absConfig.getWkxwSynonymPre());
            for (String spvPlanNo : spvPlanNos) {
                resultMap.put(spvPlanNo,wkResultMap.getOrDefault(spvPlanNo,new BigDecimal(0)));
            }
        }
        return resultMap;
    }

    public WkMfsDueBillDO getWkMfsDueBillDO(WkMfsSyncCtr ctrl) {
        WkMfsDueBillDO wkMfsDueBillDO = null;
        if (ctrl.getExecTimes() > 1) {
            wkMfsDueBillDO = wkMfsDueBillDAO.getSelfByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(), ctrl.getLoanId());
        }
        if (wkMfsDueBillDO == null) {
            wkMfsDueBillDO = wkMfsDueBillDAO.getByLoanId(ctrl.getAbsSyncDate(), absConfig.getDbLink(),
                ctrl.getLoanId(), absConfig.getWkxwDbName(), absConfig.getWkxwSynonymAft(),
                absConfig.getWkxwSynonymPre());
        }
        return wkMfsDueBillDO;
    }

    public List<WkMfsRepayPlanDO> getWkMfsRepayPlanDOS(WkMfsSyncCtr ctrl) {
        List<WkMfsRepayPlanDO> result = null;
        if (ctrl.getExecTimes() > 1) {
            result = wkMfsRepayPlanDAO.getSelfByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(), ctrl.getLoanId());
        }
        if (CollectionUtils.isEmpty(result)) {
            result = wkMfsRepayPlanDAO
                .getByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(), absConfig.getDbLink(), ctrl.getLoanId(),
                    absConfig.getWkxwDbName(), absConfig.getWkxwSynonymAft(), absConfig.getWkxwSynonymPre());
        }
        return result;
    }

    public WkMfsLnPactDO getWkMfsLnPactDO(WkMfsSyncCtr ctrl) {
        WkMfsLnPactDO wkMfsLnPactDO = null;
        if (ctrl.getExecTimes() > 1) {
            wkMfsLnPactDO = wkMfsLnPactDAO.getSelfByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(), ctrl.getLoanId());
        }
        if (wkMfsLnPactDO == null) {
            wkMfsLnPactDO = wkMfsLnPactDAO.getByLoanId(ctrl.getAbsSyncDate(), absConfig.getDbLink(),
                ctrl.getLoanId(), absConfig.getWkxwDbName(), absConfig.getWkxwSynonymAft(),
                absConfig.getWkxwSynonymPre());
        }
        return wkMfsLnPactDO;
    }

    public WkMfsOpenRepayDO getWkMfsOpenRepayDO(WkMfsSyncCtr ctrl) {
        WkMfsOpenRepayDO wkMfsOpenRepayDO = null;
        if (ctrl.getExecTimes() > 1) {
            wkMfsOpenRepayDO = wkMfsOpenRepayDAO.getSelfByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(),
                ctrl.getLoanId());
        }
        if (wkMfsOpenRepayDO == null) {
            wkMfsOpenRepayDO = wkMfsOpenRepayDAO.getByLoanId(ctrl.getAbsSyncDate(), absConfig.getDbLink(),
                ctrl.getLoanId(), absConfig.getWkxwDbName(), absConfig.getWkxwSynonymAft(),
                absConfig.getWkxwSynonymPre());
        }
        return wkMfsOpenRepayDO;
    }

    public List<WkMfsRepayInfoDO> getWkMfsRepayInfoDOS(WkMfsSyncCtr ctrl) {
        List<WkMfsRepayInfoDO> result = null;
        if (ctrl.getExecTimes() > 1) {
            result = wkMfsRepayInfoDAO.getSelfByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(), ctrl.getLoanId());
        }
        if (CollectionUtils.isEmpty(result)) {
            result = wkMfsRepayInfoDAO
                .getByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(), absConfig.getDbLink(), ctrl.getLoanId(),
                    absConfig.getWkxwDbName(), absConfig.getWkxwSynonymAft(), absConfig.getWkxwSynonymPre());
        }
        return result;
    }

    public void syncData2Local(WkMfsSyncCtr ctrl) {
        transactionTemplate.execute(new TransactionCallbackWithoutResult() {
            @Override
            protected void doInTransactionWithoutResult(TransactionStatus transactionStatus) {

                if (wkMfsDueBillDAO.getSelfByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(), ctrl.getLoanId()) == null) {
                    wkMfsDueBillDAO.insertSyncOne(ctrl.getAbsSyncDate(), absConfig.getDbLink(), ctrl.getLoanId(),
                        absConfig.getWkxwDbName(), absConfig.getWkxwSynonymAft(), absConfig.getWkxwSynonymPre());
                }
                if (wkMfsLnPactDAO.getSelfByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(), ctrl.getLoanId()) == null) {
                    wkMfsLnPactDAO.insertSyncOne(ctrl.getAbsSyncDate(), absConfig.getDbLink(), ctrl.getLoanId(),
                        absConfig.getWkxwDbName(), absConfig.getWkxwSynonymAft(), absConfig.getWkxwSynonymPre());
                }
                if (wkMfsOpenRepayDAO.getSelfByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(), ctrl.getLoanId())
                    == null) {
                    wkMfsOpenRepayDAO.insertSyncOne(ctrl.getAbsSyncDate(), absConfig.getDbLink(), ctrl.getLoanId(),
                        absConfig.getWkxwDbName(), absConfig.getWkxwSynonymAft(), absConfig.getWkxwSynonymPre());
                }
                if (CollectionUtils.isEmpty(wkMfsRepayInfoDAO.getSelfByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(), ctrl.getLoanId()))) {
                    LogUtil.info(LOGGER,"syncData2Local.wkMfsRepayInfo-{0}",ctrl.getLoanId());
                    int i = wkMfsRepayInfoDAO.insertSyncOne(ctrl.getAbsSyncDate(), absConfig.getDbLink(),
                        ctrl.getLoanId(),
                        absConfig.getWkxwDbName(), absConfig.getWkxwSynonymAft(), absConfig.getWkxwSynonymPre());
                    LogUtil.info(LOGGER,"syncData2Local.wkMfsRepayInfo-{0}-{1}",ctrl.getLoanId(),i);
                }
                if (CollectionUtils.isEmpty(wkMfsRepayPlanDAO.getSelfByLoanId(ctrl.getAbsSyncDate(), ctrl.getOrgNo(), ctrl.getLoanId()))) {
                    LogUtil.info(LOGGER,"syncData2Local.wkMfsRepayPlan-{0}",ctrl.getLoanId());
                    int i = wkMfsRepayPlanDAO.insertSyncOne(ctrl.getAbsSyncDate(), absConfig.getDbLink(),
                        ctrl.getLoanId(),
                        absConfig.getWkxwDbName(), absConfig.getWkxwSynonymAft(), absConfig.getWkxwSynonymPre());
                    LogUtil.info(LOGGER,"syncData2Local.wkMfsRepayPlan-{0}-{1}",ctrl.getLoanId(),i);
                }
            }
        });
    }
}
