/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.wk.ast.sync;

import com.abssqr.plat.common.facade.enums.AbssqrNoticeTypeEnum;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.model.wk.repo.WkDhccBatchRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.host.HostManager;
import com.abssqr.plat.core.service.notice.AbssqrNoticeManager;
import com.abssqr.plat.core.service.plan.PlanAstManager;
import com.general.system.common.util.LogUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;

/**
 * 招联资产同步任务
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.core.service.wk.ast.sync: ZLAstSyncTaskManager.java, v 0.1 2019-06-14 13:52 bangis.wangdf
 * Exp $
 */
@Component
public class ZLAstSyncTaskManager {

    protected final Logger LOGGER = LoggerFactory.getLogger(getClass());
    @Autowired
    HostManager         hostManager;
    @Autowired
    PlanAstManager      planAstManager;
    @Autowired
    WkMfsAstManager     wkMfsAstManager;
    @Autowired
    AstSyncTaskCtrRepo  astSyncTaskCtrRepo;
    @Autowired
    WkDhccBatchRepo     wkDhccBatchRepo;
    @Autowired
    TransactionTemplate transactionTemplate;
    @Autowired
    AbssqrNoticeManager abssqrNoticeManager;

    public boolean syncData(AstSyncTaskCtr astSyncTaskCtr, Day orgDay, Day trfDay) {
        if (StringUtils.isNotBlank(astSyncTaskCtr.getExtData().getCtrFlg())) {
            if (CollectionUtils.isEmpty(astSyncTaskCtr.getExtData().getSpvPlanNos())) {
                LogUtil.info(LOGGER, "{0}数据同步未配置待同步的放款计划", astSyncTaskCtr.getOrgCode());
                return true;
            }
            // 校验五矿准备数据完成标记
            if (wkDhccBatchRepo.checkDataReady(astSyncTaskCtr.getExtData().getCtrFlg(), orgDay)) {
                // 插入同步日志
                String taskLogCode = astSyncTaskCtrRepo.insertSyncLog(trfDay.getDate(), orgDay, astSyncTaskCtr,
                    SyncTaskTypeEnum.AST_SYNC, hostManager.getHostInfo().getHostName());
                try {
                    LogUtil.info(LOGGER, "{0}开始同步wkAbsMfsBillCtr", astSyncTaskCtr.getOrgCode());

                    transactionTemplate.execute(status -> {
                        wkMfsAstManager.clearData(astSyncTaskCtr.getOrgCode());
                        wkMfsAstManager.genWkMfsCtl(astSyncTaskCtr, orgDay, trfDay);

                        // 根据招联资产管理机构 统计对应存续中计划 repayInfo 还款金额 对计划水位增加
                        Map<String, BigDecimal> projRepayAmtMap =
                            wkMfsAstManager.calcProjRepayAmt(astSyncTaskCtr, orgDay);
                        planAstManager.syncRepayAddCash(orgDay.getStandardDate(), projRepayAmtMap);

                        // 更新同步任务总控数据为同步完成
                        astSyncTaskCtrRepo.updateSyncLogStatus(hostManager.getHostInfo().getHostName(), taskLogCode,
                            SyncTaskTypeEnum.AST_SYNC,
                            TaskStatusEnum.SUCCESS,
                            TaskStatusEnum.EXECUTING);
                        LogUtil.info(LOGGER, "{0}数据同步完成", astSyncTaskCtr.getOrgCode());
                        //同步完成后发送余额通知
                        if (MapUtils.isNotEmpty(projRepayAmtMap)) {
                            StringBuilder contentSb = new StringBuilder();
                            projRepayAmtMap.forEach((planNO, payAmt) -> {
                                contentSb.append(planNO).append(":").append(payAmt).append("\n");
                            });
                            abssqrNoticeManager.sendSystemNotice(astSyncTaskCtr.getOrgCode() + "当日还款", taskLogCode,
                                AbssqrNoticeTypeEnum.OrgDayRepay, contentSb.toString(),
                                abssqrNoticeManager.getAdminReceivers());
                            LogUtil.info(LOGGER, "{0}数据同步完成发送月通知,{1}", astSyncTaskCtr.getOrgCode(),
                                contentSb.toString());
                        }

                        return null;
                    });

                } catch (Throwable e) {
                    LogUtil.info(LOGGER, "{0}数据同步失败", astSyncTaskCtr.getOrgCode());
                    astSyncTaskCtrRepo.updateSyncLogStatus(hostManager.getHostInfo().getHostName(), taskLogCode,
                        SyncTaskTypeEnum.AST_SYNC,
                        TaskStatusEnum.FAILURE,
                        TaskStatusEnum.EXECUTING);
                    throw e;
                }
                return true;
            }
        }
        return false;
    }
}
