/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.test.controllertest.plan;


import com.abssqr.plat.common.facade.model.plan.Plan;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockHttpServletResponse;
import org.springframework.test.web.servlet.RequestBuilder;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.testng.annotations.Test;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Date;
import java.util.UUID;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

/**
 * @author zhenxuan.luo
 * @version com.abssqr.plat.test.controllertest.plan: PlanAllocControllerTest.java, v 0.1 2019-07-03 18:04 zhenxuan
 * .luo Exp $
 */
public class PlanReportControllerTest extends BaseAutoTests {

    @Autowired
    private PlanManager planManager;
    @Autowired
    private PlanRepo    planRepo;

    @Autowired
    private AstSyncTaskCtrRepo astSyncTaskCtrRepo;

    @Test(dataProvider = "YamlDataProvider", description = "no desc", priority = 0)
    @XTest(relatePath = "controllertest/plan/PlanReportController/calcAstRpt")
    public void calcAstRpt(ICase iCase, String planNo) throws Exception {

        Plan plan = planManager.getPlanFullByNo("jy005");

        PlanBase planBase = planRepo.getPlanByNo("jy005");
        AstSyncTaskCtr astSyncTaskCtr_1009 = astSyncTaskCtrRepo.getAstSyncTaskCtr("1009");
        AstSyncTaskCtr astSyncTaskCtr_cyjg = astSyncTaskCtrRepo.getAstSyncTaskCtr("cyjg");

        Date accountDate = DateTimeUtil.parseFromYMD("20190601");
        Date endDate = DateTimeUtil.parseFromYMD("20190701");

        while (DateTimeUtil.calcDayDiff(endDate, accountDate) >= 0) {
            try {
                accountDate = DateTimeUtil.addDays(accountDate, 1);
//                astSyncTaskCtrRepo
//                    .insertSyncLog(DateTimeUtil.addDays(accountDate, 1), astSyncTaskCtr_1009, SyncTaskTypeEnum.AST_MOD,
//                        TaskStatusEnum.SUCCESS, "testHost");
//                astSyncTaskCtrRepo
//                    .insertSyncLog(DateTimeUtil.addDays(accountDate, 1), astSyncTaskCtr_cyjg, SyncTaskTypeEnum.AST_MOD,
//                        TaskStatusEnum.SUCCESS, "testHost");
                //wkPlanReportJob.buildDailyAstReport(accountDate, plan);
            } catch (Throwable e) {
                LogUtil.error(logger, e, "[{0}]日报生成中断", accountDate);
                throw e;
            }
        }
        return;
    }

    @Test(dataProvider = "YamlDataProvider", description = "no desc", priority = 0)
    @XTest(relatePath = "controllertest/plan/PlanReportController/calcAstRpt")
    public void calcAstRpt1(ICase iCase, String planNo) throws Exception {

        Plan plan = planManager.getPlanFullByNo("plan_org_8897371");
        PlanBase planBase = planRepo.getPlanByNo("plan_org_8897371");

        Date accountDate = DateTimeUtil.parseFromYMD("20190718");
        Date endDate = DateTimeUtil.parseFromYMD("20190718");

        while (DateTimeUtil.calcDayDiff(endDate, accountDate) >= 0) {
            try {
                accountDate = DateTimeUtil.addDays(accountDate, 1);
                //wkPlanReportJob.buildDailyAstReport(accountDate, plan);
            } catch (Throwable e) {
                LogUtil.error(logger, e, "[{0}]日报生成中断", accountDate);
                throw e;
            }
        }
        return;
    }

    @Test(dataProvider = "YamlDataProvider", description = "no desc", priority = 0)
    @XTest(relatePath = "controllertest/plan/PlanReportController/calcPlanRpt")
    public void calcPlanRpt(ICase iCase, String planNo) throws Exception {

        Plan plan = planManager.getPlanFullByNo("jy005");
        Date accountDate = DateTimeUtil.parseFromYMD("20190630");
        //kPlanReportJob.buildMonthPlanReport(accountDate, plan);
        return;
    }

    @Test(dataProvider = "YamlDataProvider", description = "no desc", priority = 0)
    @XTest(relatePath = "controllertest/plan/PlanReportController/downloanAst")
    public void downloanAst(ICase iCase, String planNo) throws Exception {

        MultiValueMap<String, String> params = new LinkedMultiValueMap<>();
        params.add("planNo", "trustPlanNo001");
        params.add("beginDate", "20190610");
        params.add("endDate", "20190611");

        RequestBuilder request = get("/export/reportForm/ast/list.json").contentType(
            MediaType.APPLICATION_FORM_URLENCODED)
            .params(params);

        MockHttpServletResponse response = mvc.perform(request).andExpect(status().isOk())
            .andReturn().getResponse();
        String fileName = UUID.randomUUID() + ".xlsx";
        File file = new File(fileName);
        System.out.println(file.getAbsolutePath());
        FileOutputStream fileOutputStream = new FileOutputStream(fileName);
        fileOutputStream.write(response.getContentAsByteArray());
        fileOutputStream.flush();
        fileOutputStream.close();
    }

    @Test(dataProvider = "YamlDataProvider", description = "no desc", priority = 0)
    @XTest(relatePath = "controllertest/plan/PlanReportController/downloanAst")
    public void downMonthAst(ICase iCase, String planNo) throws Exception {

        MultiValueMap<String, String> params = new LinkedMultiValueMap<>();
        params.add("planNo", "trustPlanNo001");

        RequestBuilder request = get("/export/reportForm/ast/listMonth.json").contentType(
            MediaType.APPLICATION_FORM_URLENCODED)
            .params(params);

        MockHttpServletResponse response = mvc.perform(request).andExpect(status().isOk())
            .andReturn().getResponse();
        String fileName = UUID.randomUUID() + ".xlsx";
        File file = new File(fileName);
        System.out.println(file.getAbsolutePath());
        FileOutputStream fileOutputStream = new FileOutputStream(fileName);
        fileOutputStream.write(response.getContentAsByteArray());
        fileOutputStream.flush();
        fileOutputStream.close();
    }

    @Test(dataProvider = "YamlDataProvider", description = "no desc", priority = 0)
    @XTest(relatePath = "controllertest/plan/PlanReportController/downloanMonths")
    public void downMonthPlan(ICase iCase, String planNo) throws Exception {

        MultiValueMap<String, String> params = new LinkedMultiValueMap<>();
        params.add("planNo", "plan_org_8897371");
        params.add("year", "2019");
        params.add("timeStamp", "7");

        RequestBuilder request = get("/export/reportForm/plan/months").contentType(
            MediaType.APPLICATION_FORM_URLENCODED)
            .params(params);

        MockHttpServletResponse response = mvc.perform(request).andExpect(status().isOk())
            .andReturn().getResponse();
        String fileName = UUID.randomUUID() + ".xlsx";
        File file = new File(fileName);
        System.out.println(file.getAbsolutePath());
        FileOutputStream fileOutputStream = new FileOutputStream(fileName);
        fileOutputStream.write(response.getContentAsByteArray());
        fileOutputStream.flush();
        fileOutputStream.close();
    }
}