package com.abssqr.plat.test.testUtils;

import com.abssqr.plat.common.dal.sync.auto.dao.*;
import com.abssqr.plat.common.dal.sync.auto.dataobject.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

@Component
public class GenWkData extends BaseTestData{

    @Autowired
    private WkMfsDueBillDAO wkMfsDueBillDAO;
    @Autowired
    private WkMfsLnPactDAO wkMfsLnPactDAO;
    @Autowired
    private WkMfsOpenRepayDAO wkMfsOpenRepayDAO;
    @Autowired
    private WkMfsRepayInfoDAO wkMfsRepayInfoDAO;
    @Autowired
    private WkMfsRepayPlanDAO wkMfsRepayPlanDAO;

    public void insertDbData(BaseTestData data){

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");

        // 构建合同数据
        WkMfsLnPactDO lnPactDO = buildMFSLnPact(data);
        wkMfsLnPactDAO.insert(lnPactDO);


        // 构建放款信息
        WkMfsOpenRepayDO openRepay = buildMFSOpenRepay(data);
        wkMfsOpenRepayDAO.insert(openRepay);

        // 构建还款明细
        WkMfsRepayInfoDO repayInfo = buildMFSRepayInfo(data);
        wkMfsRepayInfoDAO.insert(repayInfo);

        // 构建还款计划
        WkMfsRepayPlanDO repayPlan = buildMFSRepayPlan(data);
        wkMfsRepayPlanDAO.insert(repayPlan);

        // 构建借据信息-中间表
        WkMfsDueBillDO dueBill = buildMFSDueBill(data);
        wkMfsDueBillDAO.insert(dueBill);
    }


    /**
     * 构建放款信息
     */
    private WkMfsOpenRepayDO buildMFSOpenRepay(BaseTestData param){
        WkMfsOpenRepayDO result = new WkMfsOpenRepayDO();

        //absSyncDate ABS数据日期
        try {
            result.setAbsSyncDate(sdf.parse(param.getSyncDate()));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        //graceDay 宽限期
        result.setGraceDay(param.getGraceDay());
        //orgNo 机构码
        result.setOrgNo(param.getOrgCode());
        //seqNo 机构放款交易流水
        result.setSeqNo(param.getLoanSeqNo());
//        //certNo 证件号
        result.setCertNo(param.getCertNo());
        //dataId 主键
//        result.setDataId(param.getDataId());
        //leader 牵头方[例如：BAIDU;QUDIAN]
        result.setLeader(param.getLeader());
        //loanId 借据号
        result.setLoanId(param.getAppId());
        //parter 合作方
        result.setParter(null);
        //projNo 项目编号
        result.setProjNo(param.getPlanNo());
        //txDate 入库日期
        result.setTxDate(param.getTxDate());
        //curDate 账务日期
        result.setCurDate(param.getLoanStartDate());
        //dataSts 数据标志（01-新增 02-历史）
        result.setDataSts(param.getDataSts());
        //endDate 到期日期
        result.setEndDate(param.getLoanEndDate());
        //borwType 借款人类型[01-法定代表人 02-自然人股东 04-个体工商户 05-合伙企业合伙人 06-个人独资企业出资人 07-自然人 99-其他 默认07-自然人]
        result.setBorwType(param.getBorwType());
        //certType 证件类型[0-身份证 1-户口簿 2-护照 3-军官证 4-士兵证 5-港澳居民来往内地通行证 6-台湾同胞来往内地通行证 7-临时身份证 8-外国人居留证 9-警官证 A-香港身份证 B-澳门身份证 C-台湾身份证 X-其他证件 默认为身份证0]
        result.setCertType(param.getCertType());
        //currency 币种[默认01	人民币]
        result.setCurrency(param.getCurrency());
        //custName 客户姓名
        result.setCustName(param.getCifName());
        //failType 放款失败类型
        result.setFailType(null);
        //loanType 贷款类型[01-房贷 02-车贷 03-消费贷 04-经营贷 99-其他 默认03-消费贷]
        result.setLoanType(param.getLoanType());
        //rateType 利率类型[10-年利率  20-月利率  30-日利率 注：此字段做为”第39项利率”  的补充说明。  默认10-年利率]
        result.setRateType(param.getRateType());
        //applyDate 申请日期
        result.setApplyDate(param.getApplyDate());
        //deducData 扣款日期
        result.setDeducData(param.getLoanStartDate());
        //deducType 扣款日类型[01-放款日为扣款日 02-固定扣款日]
        result.setDeducType(null);
        //productId 产品编号
        result.setProductId(param.getProductNo());
        //repayMode 还款方式[01--等额本息 02--等额本金]
        result.setRepayMode(param.getRepayMethod());
        //startDate 确认日期
        result.setStartDate(param.getLoanStartDate());
        //deducClass 扣款日类别[01-对月 05-对周  默认01-对月]
        result.setDeducClass(null);
        //fundStatus 放款状态[1-放款中 2-已放款 3-已冲正 4-已撤销]
        result.setFundStatus(param.getLoanStatus());
        //repayCycle 还款周期[M-月]
        result.setRepayCycle(null);
        //accountDesc 对账描述
        result.setAccountDesc(param.getAccountDesc());
        //productName 产品名称
        result.setProductName(param.getProductName());
        //accountState 对账状态[00-未处理 01-对账成功 02-与预授信对账失败 03-与银行流水对账失败 04-与授信对账成功 05-对账失败 默认值 00]
        result.setAccountState(param.getAccountState());
        //openBankName 放款银行名
        result.setOpenBankName(param.getOpenBankName());
        //openAccountname 放款账户名
        result.setOpenAccountname(null);
        //openBankaccount 放款账户卡号
        result.setOpenBankaccount(param.getOpenBankAccount());
        //totalTerms 总期数
        result.setTotalTerms(param.getTerms());
        //intRate 利率
        result.setIntRate(BigDecimal.valueOf(param.getLoanRate()));
        //chargeAmt 手续费
        result.setChargeAmt(null);
        //encashAmt 借据金额
        result.setEncashAmt(BigDecimal.valueOf(param.getEncashAmt()));
        //chargeRate 手续费率
        result.setChargeRate(null);

        return result;
    }

    // 构建合同数据
    private WkMfsLnPactDO buildMFSLnPact(BaseTestData param) {

        WkMfsLnPactDO result = new WkMfsLnPactDO();


        try {
            //absSyncDate ABS数据日期
            result.setAbsSyncDate(sdf.parse(param.getSyncDate()));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        //graceDay 宽限期限（天）
        result.setGraceDay(param.getGraceDay());
        //productId 产品编号
        result.setProductId(param.getProductNo());
        //productName 产品名称
        result.setProductName(param.getProductName());
        //brNo 合作机构号
        result.setBrNo(param.getOrgCode());
        //appId 申请ID
        result.setAppId(param.getAppId());
        //cifNo 客户号码
        result.setCifNo(param.getCifNo());
        //curNo 币种[01人民币]
        result.setCurNo(param.getCurrency());
        //lnUse 贷款用途
        result.setLnUse(param.getLoanType());
        //repDt 债转日期
        result.setRepDt(null);
        //payDay 扣款日期
        result.setPayDay(null);
        //termDay 期限天
        result.setTermDay(null);
        //termMon 期限月
        result.setTermMon(null);
        //payChn 支付渠道
        result.setPayChn(null);
        //appArea 申请地点
        result.setAppArea(null);
        //payKind 扣款日类别
        result.setPayKind(null);
        //payType 扣款日类型
        result.setPayType(null);
        //dataId 主键
//        result.setDataId(param.getDataId());
        //fundNo 资金编号
        result.setFundNo(param.getFundNo());
        //lnType 贷款类型
        result.setLnType(param.getLoanType());
        //pactId 合同ID
        result.setPactId(param.getPactId());
        //pactNo 合同编号
        result.setPactNo(param.getPactNo());
        //projNo 项目编号
        result.setProjNo(param.getPlanNo());
        //txDate 入库日期
        result.setTxDate(param.getTxDate());
        //upTime 更新时间
        result.setUpTime(param.getUpdateDate());
        //begDate 起始日期
        result.setBegDate(param.getContStartDate());
        //cifName 客户名称
        result.setCifName(param.getCifName());
        //dataSts 数据标志（01-新增 02-历史）
        result.setDataSts(param.getDataSts());
        //endDate 到期日期
        result.setEndDate(param.getContEndDate());
        //pactSts 合同状态
        result.setPactSts(param.getPactSts());
        //payMent 支付方式[01自助02受托]
        result.setPayMent(param.getPayMent());
        //vouType 担保方式
        result.setVouType(param.getVouType());
        //brAccType 机构核算类型[A/B]
        result.setBrAccType(param.getBrAccType());
        //repayType 还款方式[01--等额本息 02--等额本金]
        result.setRepayType(param.getRepayMethod());
        //factPactNo 实际贷款合同号
        result.setFactPactNo(null);
        //lnRate 月利率
        result.setLnRate(BigDecimal.valueOf(param.getLoanRate()));
        //putAmt 发放金额
        result.setPutAmt(BigDecimal.valueOf(param.getEncashAmt()));
        //pactAmt 合同金额
        result.setPactAmt(BigDecimal.valueOf(param.getEncashAmt()));
        //overRate 罚息利率
        result.setOverRate(BigDecimal.valueOf(param.getOverRate()));

        return result;

    }

    // 构建还款明细
    private WkMfsRepayInfoDO buildMFSRepayInfo(BaseTestData param){

        if(param.getRepayInfo() == null){
            return null;
        }

        WkMfsRepayInfoDO result = new WkMfsRepayInfoDO();
        try {
            //absSyncDate ABS数据日期
            result.setAbsSyncDate(sdf.parse(param.getSyncDate()));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        //dzSts 对账标志[00未对账01对账成功02对账失败]
        result.setDzSts(param.getAccountState());
        //event 交易事件
        result.setEvent(param.getEvent());
        //orgNo 机构码
        result.setOrgNo(param.getOrgCode());
        //seqNo 交易流水号
        result.setSeqNo(param.getRepaySeqNo());
        //dataId 主键
//        result.setDataId(param.getDataId());
        //fundNo 资金编号
        result.setFundNo(param.getFundNo());
        //loanId 借据号
        result.setLoanId(param.getAppId());
        //projno 项目编号
        result.setProjno(param.getPlanNo());
        //txDate 入库日期
        result.setTxDate(param.getTxDate());
        //curDate 账务日期
        result.setCurDate(param.getRepayDate());
        //dataSts 数据标志（01-新增 02-历史）
        result.setDataSts(param.getDataSts());
        //deduWay 扣款方式
        result.setDeduWay(param.getDeduWay());
        //summary 对账描述
        result.setSummary(param.getAccountDesc());
        //tranDate 交易日期
        result.setTranDate(param.getRepayDate());
        //tranTime 交易时间
        result.setTranTime(param.getRepayDate());
        //productId 产品编号
        result.setProductId(param.getProductNo());
        //dbbankname 扣款银行名
        result.setDbbankname(param.getOpenBankName());
        //productName 产品名称
        result.setProductName(param.getProductName());
        //dbaccountname 扣款账户名
        result.setDbaccountname(param.getOpenAccountName());
        //dbbankaccount 扣款账户号
        result.setDbbankaccount(param.getOpenAccountNo());
        //partnerLoanId 机构贷款标识号
        result.setPartnerLoanId(null);
        //termNo 期数
        result.setTermNo(param.getRepayTermNo());

        //breCut 违约金减免
        result.setBreCut(BigDecimal.valueOf(param.getExpPnltFee()));
        //intAmt 利息发生额
        result.setIntAmt(BigDecimal.valueOf(param.getRepayIntr()));
        //inteCut 利息减免
        result.setInteCut(BigDecimal.valueOf(param.getExpIntr()));
        //prinAmt 本金发生额
        result.setPrinAmt(BigDecimal.valueOf(param.getRepayPrin()));
        //expFinte 实扣罚息
        result.setExpFinte(BigDecimal.valueOf(param.getIncomePnlt()));
        //totalAmt 交易金额
        result.setTotalAmt(BigDecimal.valueOf(param.getRepayAmt()));
        //freeFinte 罚息减免
        result.setFreeFinte(BigDecimal.valueOf(param.getExpPnlt()));
        //incomeAmt 实收金额
        result.setIncomeAmt(BigDecimal.valueOf(param.getIncomeAmt()));
        //chargesAmt 分期手续费发生额
        result.setChargesAmt(null);
        //fundFeeAmt 借款服务费发生额
        result.setFundFeeAmt(null);
        //overdueAmt 逾期费发生额
        result.setOverdueAmt(BigDecimal.valueOf(param.getRepayOverFee()));
        //pnltIntAmt 罚息发生额
        result.setPnltIntAmt(BigDecimal.valueOf(param.getRepayPnlt()));
        //serviceAmt 展期服务费发生额
        result.setServiceAmt(null);
        //intReducedAmt 折扣券抵扣利息金额
        result.setIntReducedAmt(null);
        //pnltReducedAmt 折扣券抵扣罚息金额
        result.setPnltReducedAmt(null);
        //repayViolateAmt 提前还款违约金发生额
        result.setRepayViolateAmt(null);
        //refundViolateAmt 退款违约金发生额
        result.setRefundViolateAmt(null);
        //chargesReducedAmt 折扣券抵扣分期手续费
        result.setChargesReducedAmt(null);
        //fundFeeReducedAmt 折扣券抵扣借款服务费金额
        result.setFundFeeReducedAmt(null);
        //overdueReducedAmt 折扣券抵扣逾期费
        result.setOverdueReducedAmt(null);
        //serviceReducedAmt 折扣券抵扣展期服务费
        result.setServiceReducedAmt(null);
        //repayViolateReducedAmt 折扣券抵扣提前还款违约金
        result.setRepayViolateReducedAmt(null);
        //refundViolateReducedAmt 折扣券抵扣退款违约金
        result.setRefundViolateReducedAmt(null);
        return result;
    }


    // 构建还款计划
    private WkMfsRepayPlanDO buildMFSRepayPlan(BaseTestData param){

        WkMfsRepayPlanDO result = new WkMfsRepayPlanDO();
        try {
            //absSyncDate ABS数据日期
            result.setAbsSyncDate(sdf.parse(param.getSyncDate()));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        //orgNo 机构码
        result.setOrgNo(param.getOrgCode());
        //dataId 主键
//        result.setDataId(param.getDataId());
        //fundNo 资金编号
        result.setFundNo(param.getFundNo());
        //loanId 借据号
        result.setLoanId(param.getAppId());
        //projNo 项目编号
        result.setProjNo(param.getPlanNo());
        //txDate 入库日期
        result.setTxDate(param.getTxDate());
        //curDate 账务日期
        result.setCurDate(param.getRepayDate());
        //dataSts 数据标志（01-新增 02-历史）
        result.setDataSts(param.getDataSts());
        //endDate 到期日期
        //  数据值需要修改
//        result.setEndDate(param.getLoanEndDate());
        //clearDate 结清日期
//        result.setClearDate(param.getClearDate());
        //startDate 开始日期  数据值需要修改
        result.setStartDate(param.getLoanStartDate());
        //termStatus 本期状态[1-正常 2-逾期 3-已冲正 4-已撤销 5-已结清]  数据值需要修改
//        result.setTermStatus(param.getTermStatus());
        //accountState 对账状态[00-未对账 01--通过 02-未通过]
        result.setAccountState(param.getAccountState());
        //termNo 期序
//        result.setTermNo(param.getTermNo());
        //intBal 利息余额
//        result.setIntBal(param.getIntBal());
//        //remPrin 剩余本金
//        result.setRemPrin(param.getRemPrin());
//        //expCount 一次性结清应还金额
//        result.setExpCount(param.getExpCount());
//        //intRepay 已还利息
//        result.setIntRepay(param.getIntRepay());
//        //intTotal 应还利息
//        result.setIntTotal(param.getIntTotal());
//        //prinRepay 已还本金
//        result.setPrinRepay(param.getPrinRepay());
//        //prinTotal 应还本金
//        result.setPrinTotal(param.getPrinTotal());
//        //chargesRepay 已还分期手续费
//        result.setChargesRepay(param.getChargesRepay());
//        //chargesTotal 应还分期手续费
//        result.setChargesTotal(param.getChargesTotal());
//        //fundFeeRepay 已还借款服务费
//        result.setFundFeeRepay(param.getFundFeeRepay());
//        //fundFeeTotal 应还借款服务费
//        result.setFundFeeTotal(param.getFundFeeTotal());
//        //overdueRepay 已还逾期费
//        result.setOverdueRepay(param.getOverdueRepay());
//        //overdueTotal 应还逾期费
//        result.setOverdueTotal(param.getOverdueTotal());
//        //pnltIntRepay 已还罚息
//        result.setPnltIntRepay(param.getPnltIntRepay());
//        //pnltIntTotal 应还罚息
//        result.setPnltIntTotal(param.getPnltIntTotal());
//        //serviceRepay 已还展期服务费
//        result.setServiceRepay(param.getServiceRepay());
//        //serviceTotal 应还展期服务费
//        result.setServiceTotal(param.getServiceTotal());
//        //intReducedAmt 减免利息
//        result.setIntReducedAmt(param.getIntReducedAmt());
//        //pnltReducedAmt 减免罚息
//        result.setPnltReducedAmt(param.getPnltReducedAmt());
//        //fundFeeReducedAmt 减免借款服务费
//        result.setFundFeeReducedAmt(param.getFundFeeReducedAmt());
//        //repayViolateRepay 已还提前还款违约金
//        result.setRepayViolateRepay(param.getRepayViolateRepay());
//        //repayViolateTotal 应还提前还款违约金
//        result.setRepayViolateTotal(param.getRepayViolateTotal());
//        //refundViolateRepay 已还退款违约金
//        result.setRefundViolateRepay(param.getRefundViolateRepay());
//        //refundViolateTotal 应还退款违约金
//        result.setRefundViolateTotal(param.getRefundViolateTotal());
        return result;
    }

    // 构建借据信息-中间表
    private WkMfsDueBillDO buildMFSDueBill(BaseTestData param){

        WkMfsDueBillDO result = new WkMfsDueBillDO();

        try {
            //absSyncDate ABS数据日期
            result.setAbsSyncDate(sdf.parse(param.getSyncDate()));
            //repayDay 还款日
            Date loanStartDate = sdf.parse(param.getLoanStartDate());
            String repayDate = String.format("%td", loanStartDate);
            result.setRepayDay(Long.valueOf(repayDate));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        //graceDay 宽限期
        result.setGraceDay(param.getGraceDay());

//        //lnUse 申请用途
        result.setLnUse(param.getLoanType());
        //orgNo 机构码
        result.setOrgNo(param.getOrgCode());
        //dataId 主键
//        result.setDataId(param.getDataId());
        //fundNo 资金编号
        result.setFundNo(param.getFundNo());
        //loanId 借据号
        result.setLoanId(param.getAppId());
        //projNo 项目编号
        result.setProjNo(param.getPlanNo());
        //txDate 入库日期
        result.setTxDate(param.getTxDate());
        //applyId 预审批申请号
        result.setApplyId(null);
        //curDate 账务日期
        result.setCurDate(param.getLoanStartDate());
        //dataSts 数据标志（01-新增 02-历史）
        result.setDataSts(param.getDataSts());
        //endDate 到期日期
        result.setEndDate(param.getLoanEndDate());
        //traceNo 三方支付流水/银行流水
        result.setTraceNo(null);
        //currency 币种
        result.setCurrency(param.getCurrency());
        //loanForm 贷款形态 1-正常 2-逾期 5-核销
        result.setLoanForm(param.getLoanForm());
        //rateType 利率类型 10-年利率 20-月利率 30-日利率
        result.setRateType(param.getRateType());
        //applyDate 申请日期
        result.setApplyDate(param.getApplyDate());
        //clearDate 结清日期
        result.setClearDate(param.getClearDate());
        //productId 产品编号
        result.setProductId(param.getProductNo());
        //repayMode 还款方式 01-等额本金 02-等额本息
        result.setRepayMode(param.getRepayMethod());
        //startDate 确认日期
        result.setStartDate(param.getLoanStartDate());
        //loanStatus 贷款状态 1-放款中 2-已放款 3-已冲正 4-已撤销 5-已还款（借据已放款且客户已归还过一次款) 6-已结清
        result.setLoanStatus(param.getLoanStatus());
        //payoutDate 起息日期
        result.setPayoutDate(param.getLoanStartDate());
        //repayCycle 还款周期
        result.setRepayCycle(param.getRepayCycle());
        //productName 产品名称
        result.setProductName(param.getProductName());
        //accountState 对账状态 01-通过 02-未通过
        result.setAccountState(param.getAccountState());
        //curTerm 当前期数
        result.setCurTerm(param.getCurTerm());
        //totalTerms 总期数
        result.setTotalTerms(param.getTerms());

        //intBal 应收应计利息余额
//        result.setIntBal(param.getIntBal());
//        //intRate 利率
//        result.setIntRate(param.getIntRate());
//        //prinBal 正常本金余额
//        result.setPrinBal(param.getPrinBal());
//        //intRepay 已还利息
//        result.setIntRepay(param.getIntRepay());
//        //intTotal 应还利息
//        result.setIntTotal(param.getIntTotal());
//        //encashAmt 借据金额
//        result.setEncashAmt(param.getEncashAmt());
//        //ovdIntBal 应收欠息余额
//        result.setOvdIntBal(param.getOvdIntBal());
//        //prinRepay 已还本金
//        result.setPrinRepay(param.getPrinRepay());
//        //prinTotal 应还本金
//        result.setPrinTotal(param.getPrinTotal());
//        //ovdPrinBal 逾期本金余额
//        result.setOvdPrinBal(param.getOvdPrinBal());
//        //pnltIntBal 应收应计罚息余额
//        result.setPnltIntBal(param.getPnltIntBal());
//        //fundFeeRepay 已还借款服务费
//        result.setFundFeeRepay(param.getFundFeeRepay());
//        //fundFeeTotal 应还借款服务费
//        result.setFundFeeTotal(param.getFundFeeTotal());
//        //pnltIntRepay 已还罚息
//        result.setPnltIntRepay(param.getPnltIntRepay());
//        //pnltIntTotal 应还罚息
//        result.setPnltIntTotal(param.getPnltIntTotal());

        return result;
    }
}
