/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */

package com.abssqr.plat.test.unittest.astMatch;

import cn.hutool.core.date.DateUtil;
import com.abssqr.plat.biz.shared.dispatcher.match.PlanAstMatchDispatcherExecutor;
import com.abssqr.plat.common.model.domain.ast.Pack;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.model.repo.tr.TrCtrTaskRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.ast.AstPackManager;
import com.abssqr.plat.core.service.wk.ast.sync.ZLAstSyncTaskManager;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.common.model.exception.BaseCommonException;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.util.List;

/**
 * 资产筛选测试
 *
 * @author xiachenxiang
 * @version com.abssqr.plat.test.unittest.astMatch: AstMatchUnitTest.java, v 0.1 2019-12-12 11:22 AM xiachenxiang Exp $
 */
public class AstMatchUnitTest extends BaseAutoTests {
    @Autowired
    private PlanAstMatchDispatcherExecutor planAstMatchDispatcherExecutor;

    @Autowired
    private AstPackManager astPackManager;

    @Autowired
    private TrCtrTaskRepo       trCtrTaskRepo;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;

    @Autowired
    private ZLAstSyncTaskManager zlAstSyncTaskManager;

    @Autowired
    private AstSyncTaskCtrRepo astSyncTaskCtrRepo;

    @XTest(relatePath = "unittest/astMatch")
    @Test(dataProvider = "YamlDataProvider",description = "资产筛选测试")
    public void astMatch(ICase iCase, String spvPlanNo, String packCode, String strDate) {

        try {
            // 更新会计日期
            super.updateDate(strDate, OrgCodeEnum.ABS.getCode());

            Pack pack = astPackManager.getByPackNo(packCode);
            List<Pack> packList = Lists.newArrayList(pack);
            CommonTask node = new CommonTask();
            node.setContext(spvPlanNo);
            node.setExtObj(packList);
            planAstMatchDispatcherExecutor.doExecute(node);

        } catch(BaseCommonException e){
            if(!iCase.getContext().get("exp").toString().contains("待筛选资产利率不能为null")){
                throw  e;
            }
            Assert.assertTrue(ToolUtil.contains(e.getMessage(), iCase.getContext().get("exp")));
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }
    }

}
