/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */

package com.abssqr.plat.test.unittest.planBuyBack;

import java.util.List;

import com.abssqr.plat.biz.shared.dispatcher.buyback.LoanAstBuyBackExecutor;
import com.abssqr.plat.biz.shared.dispatcher.buyback.LoanAstBuyBackLoader;
import com.abssqr.plat.biz.shared.dispatcher.buyback.LoanAstBuyBackStarter;
import com.abssqr.plat.biz.shared.dispatcher.wkAstSync.mfs.WkMfsAstSyncDispatcherExecutor;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.wk.domain.sync.WkMfsSyncCtr;
import com.abssqr.plat.common.model.wk.repo.WkMfsSyncCtrRepo;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.core.service.host.HostManager;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.SystemDateUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.Test;

/**
 * 资产回购交易单元测试
 *
 * @author xiachenxiang
 * @version com.abssqr.plat.test.unittest.planBuyBack: PlanBuyBackTest.java, v 0.1 2020-02-13 2:28 PM xiachenxiang Exp $
 */
//@ActiveProfiles("dev")
public class PlanBuyBackTest extends BaseAutoTests {
    @Autowired
    private LoanAstBuyBackExecutor         loanAstBuyBackExecutor;
    @Autowired
    private LoanAstBuyBackLoader           loanAstBuyBackLoader;
    @Autowired
    private LoanAstBuyBackStarter          loanAstBuyBackStarter;
    @Autowired
    private AbsConfig                      config;
    @Autowired
    private WkMfsAstSyncDispatcherExecutor astSyncExecutor;
    @Autowired
    private WkMfsSyncCtrRepo               wkMfsSyncCtrRepo;
    @Autowired
    private HostManager                    hostManager;

    @XTest(relatePath = "unittest/planBuyBack/tr")
//    @XTest(relatePath = "unittest/planBuyBack/tr")
    @Test(dataProvider = "YamlDataProvider", description = "资产回购")
    public void buyback(ICase iCase, String acctDate) {
        try {
            // 停止任务
            super.invalidJobCtr(loanAstBuyBackStarter.getName());
            // 更新会计日期
            super.updateDate(acctDate, OrgCodeEnum.ABS.getCode());
            List<TrTask> trTasks = loanAstBuyBackLoader.loadToDoNodes(1);
            loanAstBuyBackExecutor.execute(trTasks.get(0));
            // ctr_task 更新succ
            loanAstBuyBackLoader.loadToDoNodes(1);
        } catch (Exception e) {
            throw e;
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }

    }
    

    @XTest(relatePath = "unittest/planBuyBack/trRepo")
    @Test(dataProvider = "YamlDataProvider", description = "资产回购后还款同步")
    public void buybackRepo(ICase iCase, String buybackDate, WkMfsSyncCtr syncNode) {
        try {
            // 停止任务
            super.invalidJobCtr(loanAstBuyBackStarter.getName());

            // 更新回购会计日期
            super.updateDate(buybackDate, OrgCodeEnum.ABS.getCode());
            /**
             * 先进行回购处理
             */
            List<TrTask> trTasks = loanAstBuyBackLoader.loadToDoNodes(1);
            loanAstBuyBackExecutor.execute(trTasks.get(0));
            // ctr_task 更新succ
            loanAstBuyBackLoader.loadToDoNodes(1);

            /**
             * 进行同步处理 包含结息 与 还款
             */
            config.setWkxwDbName("mfsbaid");
            config.setWkxwSynonymPre(".");
            // 执行同步任务
            syncNode.setId(1L);
            syncNode.setAbsSyncDate(DateTimeUtil.getStandardDate(syncNode.getAbsSyncDate()));
            // 更新到同步日期
            super.updateDate(syncNode.getAbsSyncDate(), OrgCodeEnum.ABS.getCode());
            // 更新执行中
            int upCnt = wkMfsSyncCtrRepo.updateTaskExec(SystemDateUtil.getSystemDate(),
                    hostManager.getHostInfo().getHostName(), Lists.newArrayList(syncNode.getId()));
            // 执行同步
            astSyncExecutor.execute(syncNode);

        } catch (Exception e) {
            throw e;
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }

    }
}
