package com.abssqr.plat.test.unittest.report;

import com.abssqr.plat.common.facade.model.analyse.PlanFundFile;
import com.abssqr.plat.common.facade.param.analyse.AnalyseBaseParam;
import com.abssqr.plat.common.facade.param.analyse.FundPlanFileQprParam;
import com.abssqr.plat.common.facade.param.analyse.FundPlanFileQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.service.analyse.AnalyseFundSpvService;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import org.junit.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.testng.annotations.Test;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

/**
 * 资金计划文件
 */
public class FundPlanFileTest extends BaseAutoTests {

    @Autowired
    private AnalyseFundSpvService analyseFundSpvService;
    
    @XTest(relatePath = "unittest/plan/fund/upload")
    @Test(dataProvider = "YamlDataProvider", description = "上传资金计划文件", priority = 1)
    public void testAnalyseUploadPlanFile(ICase iCase, FundPlanFileQprParam param) {
        try {
            // 读取文件
            InputStream inputStream = Thread.currentThread().getContextClassLoader()
                                              .getResourceAsStream("tmpl/export/tmpl_ast_plan_rpt.xlsx");
            MockMultipartFile file = new MockMultipartFile(
                    "测试文件.xlsx", "测试文件.xlsx", "multipart/form-data;", inputStream);
            param.setFile(file);
            // 上传文件
            String fileId = analyseFundSpvService.uploadPlanFile(param);
            Assert.assertNotNull(fileId);
        } catch (IOException e) {
        
        }finally {
            super.checkDB(iCase);
        }
    }
    
    @XTest(relatePath = "unittest/plan/fund/file", testOnly = { "analyse_base_param"})
    @Test(dataProvider = "YamlDataProvider", description = "获取资金计划文件页面数据", priority = 2)
    public void testQueryFundFile(ICase iCase, AnalyseBaseParam param) {
        try {
            // 查询列表
            BasePage<PlanFundFile> result = analyseFundSpvService.queryFundFile(param);
            // 获取文件
            List<PlanFundFile> list = result.getDatas();
            Assert.assertNotNull(list);
            // 获取资金计划文件
            FundPlanFileQryParam qryParam = new FundPlanFileQryParam();
            qryParam.setFileId(list.get(0).getFileId());
            PlanFundFile fundFile = analyseFundSpvService.getFundPlanFile(qryParam);
        
            Assert.assertNotNull(fundFile);
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }
    }
    
    
}
