/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.test.unittest.split;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.abssqr.plat.biz.shared.dispatcher.TaskGroupService;
import com.abssqr.plat.biz.shared.dispatcher.wkAstSync.mfs.WkMfsAstSyncDispatcherLoader;
import com.abssqr.plat.biz.shared.dispatcher.wkAstSync.mfs.WkMfsAstSyncDispatcherSplitor;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.TaskGroup;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.wk.domain.sync.WkMfsSyncCtr;
import com.abssqr.plat.common.model.wk.repo.WkMfsSyncCtrRepo;
import com.abssqr.plat.core.service.host.HostManager;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.common.util.DateTimeUtil;
import org.junit.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.testng.annotations.Test;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.test.unittest.split: DispatcherSplitorTest.java, v 0.1 2019-12-11 10:49 bangis.wangdf Exp $
 */
//@ActiveProfiles("dev")
public class DispatcherSplitorTest extends BaseAutoTests {

    @Autowired
    WkMfsAstSyncDispatcherSplitor wkMfsAstSyncDispatcherSplitor;

    @Autowired
    WkMfsAstSyncDispatcherLoader wkMfsAstSyncDispatcherLoader;

    @Autowired
    WkMfsSyncCtrRepo wkMfsSyncCtrRepo;

    @Autowired
    HostManager hostManager;

    @XTest(relatePath = "unittest/dispatcherSplitor")
    @Test(dataProvider = "YamlDataProvider", description = "加载数据测试")
    public void testSplit(ICase iCase, String acctDate) {
        validJobCtr(wkMfsAstSyncDispatcherSplitor.getJobName());
        try {
            hostManager.afterPropertiesSet();
        } catch (Exception e) {
            throw new RuntimeException("插入当前心跳失败");
        }

        try {
            super.updateDate(acctDate, OrgCodeEnum.WK1009.getCode());

            super.updateDate(DateTimeUtil.addDays(DateTimeUtil.parseFromYMD(acctDate), 1), OrgCodeEnum.ABS.getCode());

            wkMfsAstSyncDispatcherSplitor.execute(null);

            List<TaskGroup> taskGroups = wkMfsAstSyncDispatcherSplitor.getTask();
            TaskGroup taskGroup = taskGroups.get(0);
            Assert.assertNotNull(taskGroup);
            List<WkMfsSyncCtr> wkMfsSyncCtrs = wkMfsAstSyncDispatcherLoader
                .loadToDoNodes(new TaskGroupService() {
                    @Override
                    public List getTasks() {
                        return taskGroup.getTasks();
                    }

                    @Override
                    public void finish() {
                        wkMfsAstSyncDispatcherSplitor.finish(taskGroup.getId());
                    }
                }, new TaskGroupService() {
                    @Override
                    public List getTasks() {
                        return new ArrayList();
                    }

                    @Override
                    public void finish() {

                    }
                }, 2);
            Assert.assertEquals(wkMfsSyncCtrs.size(), 2);

            wkMfsSyncCtrs.forEach(v -> {
                wkMfsSyncCtrRepo.updateTaskExep(new Date(), v.getId(), TaskStatusEnum.SUCCESS);
            });
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        } finally {
            // 校验数据
            super.checkDB(iCase);
            //super.clearDB(iCase);
            //把任务重置为不开启
            super.invalidJobCtr(wkMfsAstSyncDispatcherSplitor.getJobName());
        }
    }
}
