/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.test.unittest.util;

import com.abssqr.plat.common.model.utils.DecimalCalcUtil;
import com.abssqr.plat.common.util.utils.BigDecimalUtil;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.common.util.MessageUtil;
import com.google.common.collect.Maps;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.math.BigDecimal;
import java.util.Map;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.test.unittest.util: DecimalCalcUtilUnitTest.java, v 0.1 2019-11-17 上午08:39 zhenxuan.luo Exp $
 */
public class DecimalCalcUtilUnitTest extends BaseAutoTests {
    
    @XTest(relatePath = "unittest/decimalCalcUtil/calcSplitFactor")
    @Test(dataProvider = "YamlDataProvider", description = "金额拆分", priority = 1)
    public void calcSplitFactorTest(ICase iCase, Map<String, String> params, Boolean isAvgWhileZero,
                                Map<String, String> expects) {
        try{
            Map<String, BigDecimal> paramVals = Maps.newHashMap();
            for (String key : params.keySet()) {
                paramVals.put(key, new BigDecimal(params.get(key)));
            }
            Map<String, BigDecimal> expectVals = Maps.newHashMap();
            for (String key : expects.keySet()) {
                expectVals.put(key, new BigDecimal(expects.get(key)));
            }
            Map<String, BigDecimal> ret = Maps.newHashMap();
            DecimalCalcUtil.calcSplitFactor(ret, paramVals, isAvgWhileZero);
            
            Assert.assertEquals(expectVals.size(), ret.size());
            
            for (String key : expectVals.keySet()) {
                BigDecimal expectVal = expectVals.get(key);
                BigDecimal resVal = ret.get(key);
                Assert.assertTrue(BigDecimalUtil.isEQ(expectVal, resVal),
                        MessageUtil.formatMsg("exp[{0}], act[{1}]", expectVal, resVal));
            }
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }
    }
    
    @XTest(relatePath = "unittest/decimalCalcUtil/calcSplitFactorToOne")
    @Test(dataProvider = "YamlDataProvider", description = "金额拆分", priority = 1)
    public void calcSplitFactorToOneTest(ICase iCase, Map<String, String> params, Boolean isAvgWhileZero,
                                     Map<String, String> expects) {
        try{
            Map<String, BigDecimal> paramVals = Maps.newHashMap();
            for (String key : params.keySet()) {
                paramVals.put(key, new BigDecimal(params.get(key)));
            }
            Map<String, BigDecimal> expectVals = Maps.newHashMap();
            for (String key : expects.keySet()) {
                expectVals.put(key, new BigDecimal(expects.get(key)));
            }
            Map<String, BigDecimal> ret = Maps.newHashMap();
            DecimalCalcUtil.calcSplitFactorToOne(ret, paramVals, isAvgWhileZero);
            
            Assert.assertEquals(expectVals.size(), ret.size());
            
            for (String key : expectVals.keySet()) {
                BigDecimal expectVal = expectVals.get(key);
                BigDecimal resVal = ret.get(key);
                Assert.assertTrue(BigDecimalUtil.isEQ(expectVal, resVal),
                        MessageUtil.formatMsg("exp[{0}], act[{1}]", expectVal, resVal));
            }
        
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }
    }
}