/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.web.controller.analyse;

import com.abssqr.plat.common.facade.param.analyse.AnalyseBaseParam;
import com.abssqr.plat.common.facade.param.analyse.CalcFundParam;
import com.abssqr.plat.common.facade.param.analyse.FundPlanFileQprParam;
import com.abssqr.plat.common.facade.param.plan.PlanPageQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.analyse.CalcFundDefaultParam;
import com.abssqr.plat.common.facade.result.analyse.CalcFundResultVO;
import com.abssqr.plat.common.facade.result.analyse.PlanAnalyseVO;
import com.abssqr.plat.common.facade.service.analyse.AnalyseFundSpvService;
import com.general.enhanced.controller.annotation.RestJsonController;
import com.general.system.common.model.BaseResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

/**
 * 资金分析
 * @author hanfei
 * @version $Id: AnalyseFundPlanController.java, v 0.1 2019-09-18 6:14 PM hanfei Exp $
 */
@RestJsonController
@RequestMapping("/analyse")
public class AnalyseFundSpvController {
    
    @Autowired
    private AnalyseFundSpvService analyseFundSpvService;
    
    /**
     * 项目分析列表
     * @return
     */
    @RequestMapping(path = "/plan/page.json", method = RequestMethod.GET)
    public BasePage queryAnalysePlan(PlanPageQryParam param){
        return analyseFundSpvService.queryAnalyseSpvPlan(param);
    }
    
    /**
     * 资金计划文件数据
     */
    @RequestMapping(path = "/fund/file.json", method = RequestMethod.GET)
    public BasePage queryFundFile(AnalyseBaseParam param){
        return analyseFundSpvService.queryFundFile(param);
    }
    
    /**
     * 上传资金计划
     * @param param
     * @return
     */
    @RequestMapping(path = "/fund/import.json", method = RequestMethod.POST)
    public BaseResult uploadFundFile(FundPlanFileQprParam param){
        // 上传资金计划文件
        String fileId = analyseFundSpvService.uploadPlanFile(param);
        return BaseResult.success(fileId);
    }
    
    /**
     * 短期资金需求 默认显示数据
     */
    @RequestMapping(path = "/fund/calc.json", method = RequestMethod.GET)
    public CalcFundDefaultParam fundCalcView(AnalyseBaseParam param){
        return analyseFundSpvService.fundCalcView(param);
    }
    
    /**
     * 短期资金需求计算器
     */
    @RequestMapping(path = "/fund/calc.json", method = RequestMethod.POST)
    public CalcFundResultVO fundCalc(@RequestBody CalcFundParam param){
        return analyseFundSpvService.fundCalc(param);
    }
    
    /**
     * 项目计划简要信息
     * @return
     */
    @RequestMapping(path = "/plan.json", method = RequestMethod.GET)
    public PlanAnalyseVO simplePlan(AnalyseBaseParam param){
        return analyseFundSpvService.getAnalyseSpvPlan(param);
    }
    
}
