/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.web.controller.analyse;

import cn.hutool.core.io.FastByteArrayOutputStream;
import cn.hutool.core.io.IoUtil;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.model.analyse.PlanFundFile;
import com.abssqr.plat.common.facade.param.analyse.FundPlanFileQryParam;
import com.abssqr.plat.common.facade.service.analyse.AnalyseFundSpvService;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.general.system.common.util.MessageUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletResponse;

/**
 * 资金计划文件下载
 * @author hanfei
 * @version $Id: FundFileExportController.java, v 0.1 2019-09-18 4:21 PM hanfei Exp $
 */
@Controller
@RequestMapping(value = "/export")
public class FundFileExportController {
    
    @Autowired
    private AnalyseFundSpvService analyseFundSpvService;
    /**
     * 下载资金计划文件
     *
     * @param param
     * @param response
     */
    @RequestMapping(value = "/fund/file.json", method = RequestMethod.GET)
    public void downloadFundFile(FundPlanFileQryParam param, HttpServletResponse response) {
        PlanFundFile entity = analyseFundSpvService.getFundPlanFile(param);
        // 获取附件文件流
        setResponseStream(entity.getFileOutStream(), response, entity.getFileName());
    }
    
    /**
     * 导出文件流
     *
     * @param stream
     * @param response
     * @param fileName
     */
    private void setResponseStream(FastByteArrayOutputStream stream, HttpServletResponse response, String fileName){
        try {
            String outfileName = new String(fileName.getBytes(), "ISO-8859-1");
            response.reset();
            response.setCharacterEncoding("UTF-8");
            //1.设置文件ContentType类型，这样设置，会自动判断下载文件类型
            response.setContentType("multipart/form-data");
            //2.设置文件头：最后一个参数是设置下载文件名
            response.setHeader("Content-Disposition", "attachment;fileName=" + outfileName);
            // 把数据写入到输出流中
            IoUtil.write(response.getOutputStream(), true, stream.toByteArray());
        } catch (Exception e) {
            throw new AbssqrBizException(MessageUtil.formatMsg("下载资金计划文件[{0}]失败", fileName),
                    CommonErrorCodeEnum.FILE_EXPORT_ERROR, e);
        } finally {
            IoUtil.close(stream);
        }
    }
}
