/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.web.controller.ast;

import cn.hutool.core.io.IoUtil;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.OfflineDebugService;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.wk.ast.sync.ZLAstSyncTaskManager;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;

/**
 * 全量同步招联数据
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.web.controller.ast: ZLAstSyncController.java, v 0.1 2019-07-10 15:02 bangis.wangdf Exp $
 */
@Controller
public class ZLAstSyncController {

    @Autowired
    private ZLAstSyncTaskManager zlAstSyncTaskManager;
    @Autowired
    private AstSyncTaskCtrRepo   astSyncTaskCtrRepo;
    @Autowired
    private OfflineDebugService  offlineDebugService;
    @Autowired
    private OrgAcctDayComponent  orgAcctDayComponent;
    @Value("${wkabs.astsync.zhaolian:1009}")
    private String               zhaolianOrgCode;

    /**
     * 分页查询计划下的资产信息
     *
     * @return
     */
    @RequestMapping(path = "/zlast/sync.json", method = RequestMethod.GET)
    public Boolean syncZlAst(String orgCode) {
        OrgCodeEnum orgCodeEnum = OrgCodeEnum.getByCode(orgCode);
        VarChecker.checkNotNull(orgCodeEnum, "机构编码不正确");
        
        AstSyncTaskCtr astSyncTaskCtr = astSyncTaskCtrRepo.getAstSyncTaskCtr(orgCode);
        astSyncTaskCtr.getExtData().setSyncAll(true);
        //ABS转让会计日
        Day absAcctDay = orgAcctDayComponent.getTrfAcctDay();
        //五矿资产机构会计日
        Day orgDay = orgAcctDayComponent.getAstAcctDay(orgCodeEnum);

        return zlAstSyncTaskManager.syncData(astSyncTaskCtr, orgDay, absAcctDay);
    }

    /**
     * 分页查询计划下的资产信息
     *
     * @return
     */
    @RequestMapping(path = "/zl/data", method = RequestMethod.GET)
    public void syncData(String loanId, HttpServletResponse response) {
        File file = offlineDebugService.generateAstSyncData(loanId);
        FileInputStream fis = null;
        BufferedInputStream buff = null;
        ServletOutputStream myout = null;
        try {
            response.setContentType("application/x-msdownload");
            response.setContentLength((int) file.length());
            response.setHeader("Content-Disposition", "attachment;filename=" + loanId + ".zip");
            //读出文件到i/o流
            fis = new FileInputStream(file);
            buff = new BufferedInputStream(fis);
            byte[] b = new byte[1024];
            long k = 0;
            //从response对象中得到输出流,准备下载
            myout = response.getOutputStream();
            //开始循环下载
            while (k < file.length()) {
                int j = buff.read(b, 0, 1024);
                k += j;
                //将b中的数据写到客户端的内存
                myout.write(b, 0, j);
            }
            //将写入到客户端的内存的数据,刷新到磁盘
            myout.flush();
        } catch (Throwable e) {
            throw new AbssqrBizException(MessageUtil.formatMsg("文件[{0}]导出失败", loanId),
                    CommonErrorCodeEnum.FILE_EXPORT_ERROR, e);
        } finally {
            IoUtil.close(myout);
            IoUtil.close(buff);
            IoUtil.close(fis);
        }
    }

}
