/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.web.controller.job;

//import com.abssqr.plat.biz.shared.scheduler.job.plan.LenderOrgAstClearingOrderJob;
//import com.abssqr.plat.biz.shared.scheduler.job.plan.LenderOrgAstStatJob;

import java.util.List;

import com.abssqr.plat.biz.shared.scheduler.job.ast.ZLFinSerFeeJob;
import com.abssqr.plat.biz.shared.scheduler.job.capitalAssertMatch.CapitalAssertMatchJob;
import com.abssqr.plat.biz.shared.scheduler.job.keeper.node.WkDataCheckKeeper;
import com.abssqr.plat.biz.shared.scheduler.job.plan.PlanDailyJob;
import com.abssqr.plat.biz.shared.scheduler.job.plan.PlanPackJob;
import com.abssqr.plat.biz.shared.scheduler.job.sync.ast.ZLOriginalAstSyncJob;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.service.AccountService;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.plan.PlanManager;

import cn.hutool.core.date.DateUtil;
import com.general.enhanced.controller.annotation.RestJsonController;
import com.general.system.common.model.BaseResult;
import com.general.system.common.util.VarChecker;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * job 手工出发
 *
 * @author yaphet
 * @version com.abssqr.admin.web.controller.product: ProductController.java, v 0.1 2018-03-02 下午3:32 yaphet Exp $
 */
@RestJsonController
public class JobController {

    @Autowired
    private     AstSyncTaskCtrRepo     astSyncTaskCtrRepo;
    @Autowired
    private     ZLOriginalAstSyncJob   zlOriginalAstSyncJob;
    @Autowired
    private PlanManager             planManager;
    @Autowired
    private PlanRepo                planRepo;
    @Autowired
    private     PlanDailyJob           planDailyJob;
    @Autowired
    private     PlanPackJob            planPackJob;
    @Autowired
    private     ZLFinSerFeeJob         zlFinSerFeeJob;
    @Autowired
    private     PlanReportRepo         planReportRepo;
    @Autowired
    private     AccountService         accountService;
    @Autowired
    private     WkDataCheckKeeper      wkDataCheckKeeper;
    @Autowired
    private     CapitalAssertMatchJob  capitalAssertMatchJob;
    @Autowired
    private     OrgAcctDayComponent    orgAcctDayComponent;
    /**
     * checkData
     *
     * @return
     */
    @RequestMapping(path = "/checkData.json", method = RequestMethod.GET)
    public BaseResult<Boolean> checkData(String orgCode) {
        OrgCodeEnum orgCodeEnum = OrgCodeEnum.getByCode(orgCode);
        VarChecker.checkNotNull(orgCodeEnum, "机构编码不正确");
        Day curDate = orgAcctDayComponent.getTrfAcctDay();
        StringBuilder sb= new StringBuilder();
        
        Day orgDate = orgAcctDayComponent.getAstAcctDay(orgCodeEnum);
        wkDataCheckKeeper.checkData(curDate, orgCode, orgDate, sb);
        BaseResult result = BaseResult.success(true);
        result.setDataObject(sb.toString());
        return result;
    }

    /**
     * 计划资产包生成任务
     *
     * @return
     * @description
     */
    @RequestMapping(path = "/job/planPackJob.json", method = RequestMethod.GET)
    public BaseResult<Boolean> planPackJob() {
        planPackJob.doExecute();
        return BaseResult.success(null);
    }

    /**
     * 每日处理
     *
     * @return
     * @description
     */
    @RequestMapping(path = "/job/planDailyJob.json", method = RequestMethod.GET)
    public BaseResult<Boolean> planDailyJob() {
        planDailyJob.doExecute();
        return BaseResult.success(null);
    }



    /**
     * 招联资产数据同步
     *
     * @return
     * @description
     */
    @RequestMapping(path = "/job/zlOriginalAstSyncJob.json", method = RequestMethod.GET)
    public BaseResult<Boolean> zlOriginalAstSyncJob(String all, String orgCode) {
        AstSyncTaskCtr astSyncTaskCtr = astSyncTaskCtrRepo.getAstSyncTaskCtr(orgCode);
        if (StringUtils.equals(all, "Y")) {
            astSyncTaskCtr.getExtData().setSyncAll(true);
        } else {
            astSyncTaskCtr.getExtData().setSyncAll(false);
        }
        zlOriginalAstSyncJob.doExecute(astSyncTaskCtr);
        return BaseResult.success(null);
    }



    /**
     * 资金资产任务Task生成Job
     *
     * @return
     * @description
     */
    @RequestMapping(path = "/job/capitalAssetMatchJob.json", method = RequestMethod.GET)
    public BaseResult<Boolean> capitalAssetMatchJob() {
        capitalAssertMatchJob.doExecute();
        return BaseResult.success(null);
    }

    /**
     * 查询账户余额
     *
     * @return
     * @description
     */
    @RequestMapping(path = "/acct/balance/all.json", method = RequestMethod.GET)
    public List<Account> allBalance() {
        return accountService.queryBalance();
    }

    /**
     * 查询账户余额
     *
     * @return
     * @description
     */
    @RequestMapping(path = "/acct/zlFinSerFeeJob.json", method = RequestMethod.GET)
    public BaseResult<Boolean> zlFinSerFeeJob() {
        zlFinSerFeeJob.doExecute();
        return BaseResult.success(null);
    }

    /**
     * 初始化计划 日报基础数据
     *
     * @param planNo
     * @param date
     * @return
     */
    @RequestMapping(path = "/report/default.json", method = RequestMethod.GET)
    public BaseResult<Boolean> initDefaultReport001(@RequestParam String planNo, @RequestParam String date) {
        // 校验计划
        PlanBase planBase = planManager.getPlanFullByNo(planNo);
        VarChecker.checkNotNull(planBase, "错误的计划编号");
        // 初始化上一日空值报表基础数据
        int cnt = planReportRepo.defaultRPT001Init(planNo, DateUtil.parseDate(date));
        return BaseResult.success(cnt);
    }



}
