/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.web.controller.plan;

import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocAssignedOprParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocCfrmCalcParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocDelParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocDetailQryParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocEditParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocQryParam;
import com.abssqr.plat.common.facade.param.plan.alloc.TitleNameQryParam;
import com.abssqr.plat.common.facade.result.plan.PlanAllocVO;
import com.abssqr.plat.common.facade.result.plan.TitleNameVO;
import com.abssqr.plat.common.facade.service.plan.PlanAllocService;
import com.general.enhanced.controller.annotation.RestJsonController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import java.util.List;

/**
 * 分配计划
 * @author bangis.wangdf
 * @version com.abssqr.plat.web.controller.plan: PlanAllocController.java, v 0.1 2019-06-03 14:11 bangis.wangdf Exp $
 */
@RestJsonController
public class PlanAllocController {

    @Autowired
    private PlanAllocService planAllocService;

    /**
     * 查询分配计划列表
     * @param param
     * @return
     */
    @RequestMapping(path = "/plan/alloc/get.json", method = RequestMethod.GET)
    public List get(PlanAllocQryParam param) {
        return planAllocService.getPlanAlloc(param);
    }

    /**
     * 查询分配计划详情
     * @param param
     * @return
     */
    @RequestMapping(path = "/plan/allocDetail/get.json", method = RequestMethod.GET)
    public PlanAllocVO getPlanAlloc(PlanAllocDetailQryParam param) {
        return planAllocService.getPlanAllocDetail(param);
    }
    /**
     * 分配方案调整为已分配
     * @param param
     * @return
     */
    @RequestMapping(path = "/plan/allocDetail/assigned.json", method = RequestMethod.POST)
    public Boolean updatePlanAlloc(PlanAllocAssignedOprParam param) {
        return planAllocService.planAllocDetailAssigned(param);
    }
    /**
     * 受让池保存分配计划
     *
     * @param param
     * @return
     */
    @RequestMapping(path = "/plan/allocDetail/{type}/edit.json", method = RequestMethod.POST)
    public PlanAllocVO edit(@RequestBody PlanAllocEditParam param,@PathVariable String type) {
        PlanTypeEnum planType = PlanTypeEnum.getByCode(type.toUpperCase());
        return planAllocService.edit(param,planType,false); }
    /**
     * 受让池提交审批分配计划
     *
     * @param param
     * @return
     */
    @RequestMapping(path = "/plan/allocDetail/{type}/submit.json", method = RequestMethod.POST)
    public PlanAllocVO submit(@RequestBody PlanAllocEditParam param,@PathVariable String type) {
        PlanTypeEnum planType = PlanTypeEnum.getByCode(type.toUpperCase());
        return planAllocService.edit(param, planType, true); }
    
    /**
     * 删除分配计划
     *
     * @param param
     * @return
     */
    @RequestMapping(path = "/plan/allocDetail/del.json", method = RequestMethod.DELETE)
    public Boolean delete(PlanAllocDelParam param) {
        return planAllocService.delete(param);
    }

    /**
     * 分配计划详情编辑是查询分配事项接口
     *
     * @param param
     */
    @RequestMapping(path = "/plan/titleName/get.json", method = RequestMethod.GET)
    public List<TitleNameVO> getTitleNames(TitleNameQryParam param) {
        return planAllocService.getTitleNames(param);
    }

    /**
     * 根据分配金额进行试算
     * @param param
     * @return
     */
    @RequestMapping(path = "/plan/allocDetail/{type}/calcCfrm.json", method = RequestMethod.GET)
    public PlanAllocVO calcCfrmSpv(PlanAllocCfrmCalcParam param, @PathVariable String type) {
        PlanTypeEnum entityType = PlanTypeEnum.getByCode(type.toUpperCase());
        return planAllocService.calcCfrmByAmt(param,entityType);
    }
}
