/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.web.controller.plan;

import com.abssqr.plat.common.facade.param.plan.PlanDelOprParam;
import com.abssqr.plat.common.facade.param.plan.PlanOprParam;
import com.abssqr.plat.common.facade.param.plan.PlanPageQryParam;
import com.abssqr.plat.common.facade.param.plan.PlanQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.plan.PlanListVO;
import com.abssqr.plat.common.facade.result.plan.PlanVO;
import com.abssqr.plat.common.facade.service.plan.PlanService;
import com.general.enhanced.controller.annotation.RestJsonController;
import com.general.system.common.model.BaseResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.validation.Valid;

/**
 * 资产转让项目计划管理
 * @author yaphet
 * @version com.abssqr.admin.web.controller: PlanController.java, v 0.1 2017-12-09 上午12:15 yaphet Exp $
 */
@RestJsonController
public class PlanController {

    @Autowired
    private PlanService planService;

    /**
     * @param param
     * @return
     * @description 查询计划列表
     */
    @RequestMapping(path = "/plan/list.json", method = RequestMethod.GET)
    public BasePage<PlanListVO> getOrgList(PlanPageQryParam param) {
        return planService.getPlanList(param, false);
    }

    /**
     * @param param
     * @return
     * @description 资产管理查询计划列表
     */
    @RequestMapping(path = "/plan/ast/list.json", method = RequestMethod.GET)
    public BasePage<PlanListVO> getAstList(PlanPageQryParam param) {
        return planService.getPlanList(param, true);
    }

    /**
     * 创建计划
     *
     * @param param
     * @return
     */
    @RequestMapping(path = "/plan/edit.json", method = RequestMethod.POST)
    public BaseResult<String> edit(@RequestBody @Valid PlanOprParam param) {
        return BaseResult.success(planService.edit(param));
    }

    /**
     * 查询计划详情
     *
     * @param param
     * @return
     */
    @RequestMapping(path = "/plan/get.json", method = RequestMethod.GET)
    public PlanVO get(PlanQryParam param) {
        return planService.getPlan(param);
    }


//    /**
//     * 查询计划详情
//     *
//     * @return
//     */
//    @RequestMapping(path = "/plan/initPlan.json", method = RequestMethod.GET)
//    public BaseResult<String> initPlanFor20191030() {
//        planService.initPlanFor20191030();
//        return BaseResult.success("万");
//    }


    /**
     * 删除草稿箱
     *
     * @return
     */
    @RequestMapping(path = "/plan/draft/del.json", method = RequestMethod.DELETE)
    public BaseResult deleteDraftPlan(@Valid PlanDelOprParam param){
        planService.deleteDraftPlan(param);
        return BaseResult.success(null);
    }

}